/*
  ZynAddSubFX - a software synthesizer

  config.h - options configurable by CMake
  Copyright (C) 2018 Johannes Lorenz
  Author: Johannes Lorenz

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
*/

#ifndef ZYN_CONFIG_H_IN
#define ZYN_CONFIG_H_IN

namespace zyn {

constexpr const char* fusion_dir = "";

}

#endif
