/*
 * This file contains a list of all
 * initializing methods which are
 * necessary to bootstrap all gems.
 *
 * IMPORTANT:
 *   This file was generated!
 *   All manual changes will get lost.
 */

#include <mruby.h>

void GENERATED_TMP_mrb_mruby_print_gem_init(mrb_state*);
void GENERATED_TMP_mrb_mruby_print_gem_final(mrb_state*);
void GENERATED_TMP_mrb_mruby_sprintf_gem_init(mrb_state*);
void GENERATED_TMP_mrb_mruby_sprintf_gem_final(mrb_state*);
void GENERATED_TMP_mrb_mruby_math_gem_init(mrb_state*);
void GENERATED_TMP_mrb_mruby_math_gem_final(mrb_state*);
void GENERATED_TMP_mrb_mruby_time_gem_init(mrb_state*);
void GENERATED_TMP_mrb_mruby_time_gem_final(mrb_state*);
void GENERATED_TMP_mrb_mruby_string_ext_gem_init(mrb_state*);
void GENERATED_TMP_mrb_mruby_string_ext_gem_final(mrb_state*);
void GENERATED_TMP_mrb_mruby_proc_ext_gem_init(mrb_state*);
void GENERATED_TMP_mrb_mruby_proc_ext_gem_final(mrb_state*);
void GENERATED_TMP_mrb_mruby_random_gem_init(mrb_state*);
void GENERATED_TMP_mrb_mruby_random_gem_final(mrb_state*);
void GENERATED_TMP_mrb_mruby_object_ext_gem_init(mrb_state*);
void GENERATED_TMP_mrb_mruby_object_ext_gem_final(mrb_state*);
void GENERATED_TMP_mrb_mruby_toplevel_ext_gem_init(mrb_state*);
void GENERATED_TMP_mrb_mruby_toplevel_ext_gem_final(mrb_state*);
void GENERATED_TMP_mrb_mruby_eval_gem_init(mrb_state*);
void GENERATED_TMP_mrb_mruby_eval_gem_final(mrb_state*);
void GENERATED_TMP_mrb_mruby_io_gem_init(mrb_state*);
void GENERATED_TMP_mrb_mruby_io_gem_final(mrb_state*);
void GENERATED_TMP_mrb_mruby_errno_gem_init(mrb_state*);
void GENERATED_TMP_mrb_mruby_errno_gem_final(mrb_state*);
void GENERATED_TMP_mrb_mruby_nanovg_gem_init(mrb_state*);
void GENERATED_TMP_mrb_mruby_nanovg_gem_final(mrb_state*);
void GENERATED_TMP_mrb_mruby_regexp_pcre_gem_init(mrb_state*);
void GENERATED_TMP_mrb_mruby_regexp_pcre_gem_final(mrb_state*);
void GENERATED_TMP_mrb_mruby_array_ext_gem_init(mrb_state*);
void GENERATED_TMP_mrb_mruby_array_ext_gem_final(mrb_state*);
void GENERATED_TMP_mrb_mruby_hash_ext_gem_init(mrb_state*);
void GENERATED_TMP_mrb_mruby_hash_ext_gem_final(mrb_state*);
void GENERATED_TMP_mrb_mruby_fiber_gem_init(mrb_state*);
void GENERATED_TMP_mrb_mruby_fiber_gem_final(mrb_state*);
void GENERATED_TMP_mrb_mruby_enumerator_gem_init(mrb_state*);
void GENERATED_TMP_mrb_mruby_enumerator_gem_final(mrb_state*);
void GENERATED_TMP_mrb_mruby_set_gem_init(mrb_state*);
void GENERATED_TMP_mrb_mruby_set_gem_final(mrb_state*);
void GENERATED_TMP_mrb_mruby_file_stat_gem_init(mrb_state*);
void GENERATED_TMP_mrb_mruby_file_stat_gem_final(mrb_state*);
void GENERATED_TMP_mrb_mruby_dir_gem_init(mrb_state*);
void GENERATED_TMP_mrb_mruby_dir_gem_final(mrb_state*);
void GENERATED_TMP_mrb_mruby_process_gem_init(mrb_state*);
void GENERATED_TMP_mrb_mruby_process_gem_final(mrb_state*);
void GENERATED_TMP_mrb_mruby_pack_gem_init(mrb_state*);
void GENERATED_TMP_mrb_mruby_pack_gem_final(mrb_state*);
void GENERATED_TMP_mrb_mruby_dir_glob_gem_init(mrb_state*);
void GENERATED_TMP_mrb_mruby_dir_glob_gem_final(mrb_state*);
void GENERATED_TMP_mrb_mruby_qml_spawn_gem_init(mrb_state*);
void GENERATED_TMP_mrb_mruby_qml_spawn_gem_final(mrb_state*);
void GENERATED_TMP_mrb_mruby_qml_parse_gem_init(mrb_state*);
void GENERATED_TMP_mrb_mruby_qml_parse_gem_final(mrb_state*);
void GENERATED_TMP_mrb_mruby_zest_gem_init(mrb_state*);
void GENERATED_TMP_mrb_mruby_zest_gem_final(mrb_state*);
void GENERATED_TMP_mrb_mruby_widget_lib_gem_init(mrb_state*);
void GENERATED_TMP_mrb_mruby_widget_lib_gem_final(mrb_state*);

static void
mrb_final_mrbgems(mrb_state *mrb) {
  GENERATED_TMP_mrb_mruby_widget_lib_gem_final(mrb);
  GENERATED_TMP_mrb_mruby_zest_gem_final(mrb);
  GENERATED_TMP_mrb_mruby_qml_parse_gem_final(mrb);
  GENERATED_TMP_mrb_mruby_qml_spawn_gem_final(mrb);
  GENERATED_TMP_mrb_mruby_dir_glob_gem_final(mrb);
  GENERATED_TMP_mrb_mruby_pack_gem_final(mrb);
  GENERATED_TMP_mrb_mruby_process_gem_final(mrb);
  GENERATED_TMP_mrb_mruby_dir_gem_final(mrb);
  GENERATED_TMP_mrb_mruby_file_stat_gem_final(mrb);
  GENERATED_TMP_mrb_mruby_set_gem_final(mrb);
  GENERATED_TMP_mrb_mruby_enumerator_gem_final(mrb);
  GENERATED_TMP_mrb_mruby_fiber_gem_final(mrb);
  GENERATED_TMP_mrb_mruby_hash_ext_gem_final(mrb);
  GENERATED_TMP_mrb_mruby_array_ext_gem_final(mrb);
  GENERATED_TMP_mrb_mruby_regexp_pcre_gem_final(mrb);
  GENERATED_TMP_mrb_mruby_nanovg_gem_final(mrb);
  GENERATED_TMP_mrb_mruby_errno_gem_final(mrb);
  GENERATED_TMP_mrb_mruby_io_gem_final(mrb);
  GENERATED_TMP_mrb_mruby_eval_gem_final(mrb);
  GENERATED_TMP_mrb_mruby_toplevel_ext_gem_final(mrb);
  GENERATED_TMP_mrb_mruby_object_ext_gem_final(mrb);
  GENERATED_TMP_mrb_mruby_random_gem_final(mrb);
  GENERATED_TMP_mrb_mruby_proc_ext_gem_final(mrb);
  GENERATED_TMP_mrb_mruby_string_ext_gem_final(mrb);
  GENERATED_TMP_mrb_mruby_time_gem_final(mrb);
  GENERATED_TMP_mrb_mruby_math_gem_final(mrb);
  GENERATED_TMP_mrb_mruby_sprintf_gem_final(mrb);
  GENERATED_TMP_mrb_mruby_print_gem_final(mrb);
}

void
mrb_init_mrbgems(mrb_state *mrb) {
  GENERATED_TMP_mrb_mruby_print_gem_init(mrb);
  GENERATED_TMP_mrb_mruby_sprintf_gem_init(mrb);
  GENERATED_TMP_mrb_mruby_math_gem_init(mrb);
  GENERATED_TMP_mrb_mruby_time_gem_init(mrb);
  GENERATED_TMP_mrb_mruby_string_ext_gem_init(mrb);
  GENERATED_TMP_mrb_mruby_proc_ext_gem_init(mrb);
  GENERATED_TMP_mrb_mruby_random_gem_init(mrb);
  GENERATED_TMP_mrb_mruby_object_ext_gem_init(mrb);
  GENERATED_TMP_mrb_mruby_toplevel_ext_gem_init(mrb);
  GENERATED_TMP_mrb_mruby_eval_gem_init(mrb);
  GENERATED_TMP_mrb_mruby_io_gem_init(mrb);
  GENERATED_TMP_mrb_mruby_errno_gem_init(mrb);
  GENERATED_TMP_mrb_mruby_nanovg_gem_init(mrb);
  GENERATED_TMP_mrb_mruby_regexp_pcre_gem_init(mrb);
  GENERATED_TMP_mrb_mruby_array_ext_gem_init(mrb);
  GENERATED_TMP_mrb_mruby_hash_ext_gem_init(mrb);
  GENERATED_TMP_mrb_mruby_fiber_gem_init(mrb);
  GENERATED_TMP_mrb_mruby_enumerator_gem_init(mrb);
  GENERATED_TMP_mrb_mruby_set_gem_init(mrb);
  GENERATED_TMP_mrb_mruby_file_stat_gem_init(mrb);
  GENERATED_TMP_mrb_mruby_dir_gem_init(mrb);
  GENERATED_TMP_mrb_mruby_process_gem_init(mrb);
  GENERATED_TMP_mrb_mruby_pack_gem_init(mrb);
  GENERATED_TMP_mrb_mruby_dir_glob_gem_init(mrb);
  GENERATED_TMP_mrb_mruby_qml_spawn_gem_init(mrb);
  GENERATED_TMP_mrb_mruby_qml_parse_gem_init(mrb);
  GENERATED_TMP_mrb_mruby_zest_gem_init(mrb);
  GENERATED_TMP_mrb_mruby_widget_lib_gem_init(mrb);
  mrb_state_atexit(mrb, mrb_final_mrbgems);
}
