/*
  ZynAddSubFX - a software synthesizer

  Engine.cpp - Audio Driver base class
  Copyright (C) 2009-2010 Mark McCurry
  Author: Mark McCurry

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
*/
#include "Engine.h"

namespace zyn {

Engine::Engine()
{}

Engine::~Engine()
{}

}
