/*
 * This file is loading the irep
 * Ruby GEM code.
 *
 * IMPORTANT:
 *   This file was generated!
 *   All manual changes will get lost.
 */
#include <stdlib.h>
#include <mruby.h>
#include <mruby/irep.h>
#include <mruby/debug.h>
#include <mruby/proc.h>
#include <mruby/presym.h>

#define mrb_BRACED(...) {__VA_ARGS__}
#define mrb_DEFINE_SYMS_VAR(name, len, syms, qualifier) \
  static qualifier mrb_sym name[len] = mrb_BRACED syms

static const mrb_pool_value gem_mrblib_mruby_print_proc_pool_2[1] = {
{IREP_TT_STR|(1<<2), {"\x0a"}},
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_print_proc_syms_2, 5, (MRB_SYM(size), MRB_OPSYM(aref), MRB_SYM(inspect), MRB_SYM(__printstr__), MRB_SYM(__svalue), ), const);
static const mrb_code gem_mrblib_mruby_print_proc_iseq_2[72] = {
0x33,0x00,0x10,0x00,0x06,0x03,0x01,0x05,0x01,0x2d,0x05,0x00,0x00,0x01,0x04,0x05,0x01,0x05,0x03,0x01,
0x06,0x04,0x42,0x05,0x25,0x05,0x00,0x23,0x12,0x05,0x01,0x06,0x01,0x01,0x07,0x03,0x2d,0x06,0x01,0x01,
0x2d,0x06,0x02,0x00,0x2d,0x05,0x03,0x01,0x12,0x05,0x50,0x06,0x00,0x2d,0x05,0x03,0x01,0x3c,0x03,0x01,
0x23,0xff,0xd1,0x01,0x05,0x01,0x2d,0x05,0x04,0x00,0x37,0x05,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_print_proc_lv_2, 4, (MRB_SYM(args), MRB_OPSYM(and), MRB_SYM(i), MRB_SYM(len), ), const);
static char gem_mrblib_mruby_print_proc_debug_lines_2[] = "\x00\x0b\x04\x01\x02\x01\x0a\x01\x0a\x03\x02\xfe\xff\xff\xff\x0f\x14\x01\x09\x01\x06\x02";
static mrb_irep_debug_info_file gem_mrblib_mruby_print_proc_debug_file_2 = {
0, 173, 22, mrb_debug_line_packed_map, {gem_mrblib_mruby_print_proc_debug_lines_2}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_print_proc_debug_file_2_ = &gem_mrblib_mruby_print_proc_debug_file_2;
static mrb_irep_debug_info gem_mrblib_mruby_print_proc_debug_2 = {
72, 1, &gem_mrblib_mruby_print_proc_debug_file_2_};
static const mrb_irep gem_mrblib_mruby_print_proc_irep_2 = {
  5,9,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_print_proc_iseq_2,
  gem_mrblib_mruby_print_proc_pool_2,gem_mrblib_mruby_print_proc_syms_2,NULL,
  gem_mrblib_mruby_print_proc_lv_2,
  &gem_mrblib_mruby_print_proc_debug_2,
  72,1,5,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_print_proc_syms_3, 4, (MRB_SYM(size), MRB_OPSYM(aref), MRB_SYM(to_s), MRB_SYM(__printstr__), ), const);
static const mrb_code gem_mrblib_mruby_print_proc_iseq_3[58] = {
0x33,0x00,0x10,0x00,0x06,0x03,0x01,0x05,0x01,0x2d,0x05,0x00,0x00,0x01,0x04,0x05,0x01,0x05,0x03,0x01,
0x06,0x04,0x42,0x05,0x25,0x05,0x00,0x1a,0x12,0x05,0x01,0x06,0x01,0x01,0x07,0x03,0x2d,0x06,0x01,0x01,
0x2d,0x06,0x02,0x00,0x2d,0x05,0x03,0x01,0x3c,0x03,0x01,0x23,0xff,0xda,0x11,0x05,0x37,0x05,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_print_proc_lv_3, 4, (MRB_SYM(args), MRB_OPSYM(and), MRB_SYM(i), MRB_SYM(len), ), const);
static char gem_mrblib_mruby_print_proc_debug_lines_3[] = "\x00\x18\x04\x01\x02\x01\x0a\x01\x0a\x02\x02\xff\xff\xff\xff\x0f\x14\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_print_proc_debug_file_3 = {
0, 173, 18, mrb_debug_line_packed_map, {gem_mrblib_mruby_print_proc_debug_lines_3}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_print_proc_debug_file_3_ = &gem_mrblib_mruby_print_proc_debug_file_3;
static mrb_irep_debug_info gem_mrblib_mruby_print_proc_debug_3 = {
58, 1, &gem_mrblib_mruby_print_proc_debug_file_3_};
static const mrb_irep gem_mrblib_mruby_print_proc_irep_3 = {
  5,9,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_print_proc_iseq_3,
  NULL,gem_mrblib_mruby_print_proc_syms_3,NULL,
  gem_mrblib_mruby_print_proc_lv_3,
  &gem_mrblib_mruby_print_proc_debug_3,
  58,0,4,0,0
};
static const mrb_pool_value gem_mrblib_mruby_print_proc_pool_4[1] = {
{IREP_TT_STR|(1<<2), {"\x0a"}},
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_print_proc_syms_4, 8, (MRB_SYM(size), MRB_OPSYM(aref), MRB_SYM(Array), MRB_SYM_Q(kind_of), MRB_SYM(puts), MRB_SYM(to_s), MRB_SYM(__printstr__), MRB_OPSYM(neq), ), const);
static const mrb_code gem_mrblib_mruby_print_proc_iseq_4[148] = {
0x33,0x00,0x10,0x00,0x06,0x03,0x01,0x06,0x01,0x2d,0x06,0x00,0x00,0x01,0x04,0x06,0x01,0x06,0x03,0x01,
0x07,0x04,0x42,0x06,0x25,0x06,0x00,0x60,0x01,0x06,0x01,0x01,0x07,0x03,0x2d,0x06,0x01,0x01,0x01,0x05,
0x06,0x01,0x06,0x05,0x1d,0x07,0x02,0x2d,0x06,0x03,0x01,0x25,0x06,0x00,0x0f,0x12,0x06,0x11,0x07,0x01,
0x08,0x05,0x48,0x07,0x2b,0x06,0x04,0x23,0x00,0x30,0x01,0x06,0x05,0x2d,0x06,0x05,0x00,0x01,0x05,0x06,
0x12,0x06,0x01,0x07,0x05,0x2d,0x06,0x06,0x01,0x01,0x06,0x05,0x05,0x07,0x2d,0x06,0x01,0x01,0x50,0x07,
0x00,0x2d,0x06,0x07,0x01,0x25,0x06,0x00,0x09,0x12,0x06,0x50,0x07,0x00,0x2d,0x06,0x06,0x01,0x3c,0x03,
0x01,0x23,0xff,0x94,0x01,0x06,0x04,0x06,0x07,0x41,0x06,0x25,0x06,0x00,0x09,0x12,0x06,0x50,0x07,0x00,
0x2d,0x06,0x06,0x01,0x11,0x06,0x37,0x06,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_print_proc_lv_4, 5, (MRB_SYM(args), MRB_OPSYM(and), MRB_SYM(i), MRB_SYM(len), MRB_SYM(s), ), const);
static char gem_mrblib_mruby_print_proc_debug_lines_4[] = "\x00\x23\x04\x01\x02\x01\x0a\x01\x0a\x09\x02\xf8\xff\xff\xff\x0f\x0d\x01\x0c\x01\x0f\x04\x02\xfe\xff\xff\xff\x0f\x0a\x01\x09\x01\x1d\x02\x06\x02\x14\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_print_proc_debug_file_4 = {
0, 173, 38, mrb_debug_line_packed_map, {gem_mrblib_mruby_print_proc_debug_lines_4}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_print_proc_debug_file_4_ = &gem_mrblib_mruby_print_proc_debug_file_4;
static mrb_irep_debug_info gem_mrblib_mruby_print_proc_debug_4 = {
148, 1, &gem_mrblib_mruby_print_proc_debug_file_4_};
static const mrb_irep gem_mrblib_mruby_print_proc_irep_4 = {
  6,9,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_print_proc_iseq_4,
  gem_mrblib_mruby_print_proc_pool_4,gem_mrblib_mruby_print_proc_syms_4,NULL,
  gem_mrblib_mruby_print_proc_lv_4,
  &gem_mrblib_mruby_print_proc_debug_4,
  148,1,8,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_print_proc_syms_5, 2, (MRB_SYM(sprintf), MRB_SYM(__printstr__), ), const);
static const mrb_code gem_mrblib_mruby_print_proc_iseq_5[26] = {
0x33,0x00,0x10,0x00,0x12,0x03,0x12,0x04,0x11,0x05,0x01,0x06,0x01,0x48,0x05,0x2b,0x04,0x00,0x2d,0x03,
0x01,0x01,0x11,0x03,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_print_proc_lv_5, 2, (MRB_SYM(args), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_print_proc_debug_lines_5[] = "\x00\x35\x04\x01\x12\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_print_proc_debug_file_5 = {
0, 173, 6, mrb_debug_line_packed_map, {gem_mrblib_mruby_print_proc_debug_lines_5}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_print_proc_debug_file_5_ = &gem_mrblib_mruby_print_proc_debug_file_5;
static mrb_irep_debug_info gem_mrblib_mruby_print_proc_debug_5 = {
26, 1, &gem_mrblib_mruby_print_proc_debug_file_5_};
static const mrb_irep gem_mrblib_mruby_print_proc_irep_5 = {
  3,7,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_print_proc_iseq_5,
  NULL,gem_mrblib_mruby_print_proc_syms_5,NULL,
  gem_mrblib_mruby_print_proc_lv_5,
  &gem_mrblib_mruby_print_proc_debug_5,
  26,0,2,0,0
};
static const mrb_irep *gem_mrblib_mruby_print_proc_reps_1[4] = {
  &gem_mrblib_mruby_print_proc_irep_2,
  &gem_mrblib_mruby_print_proc_irep_3,
  &gem_mrblib_mruby_print_proc_irep_4,
  &gem_mrblib_mruby_print_proc_irep_5,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_print_proc_syms_1, 4, (MRB_SYM(p), MRB_SYM(print), MRB_SYM(puts), MRB_SYM(printf), ), const);
static const mrb_code gem_mrblib_mruby_print_proc_iseq_1[34] = {
0x62,0x01,0x57,0x02,0x00,0x5e,0x01,0x00,0x62,0x01,0x57,0x02,0x01,0x5e,0x01,0x01,0x62,0x01,0x57,0x02,
0x02,0x5e,0x01,0x02,0x62,0x01,0x57,0x02,0x03,0x5e,0x01,0x03,0x37,0x01,};
static char gem_mrblib_mruby_print_proc_debug_lines_1[] = "\x00\x0b\x08\x0d\x08\x0b\x08\x12";
static mrb_irep_debug_info_file gem_mrblib_mruby_print_proc_debug_file_1 = {
0, 173, 8, mrb_debug_line_packed_map, {gem_mrblib_mruby_print_proc_debug_lines_1}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_print_proc_debug_file_1_ = &gem_mrblib_mruby_print_proc_debug_file_1;
static mrb_irep_debug_info gem_mrblib_mruby_print_proc_debug_1 = {
34, 1, &gem_mrblib_mruby_print_proc_debug_file_1_};
static const mrb_irep gem_mrblib_mruby_print_proc_irep_1 = {
  1,3,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_print_proc_iseq_1,
  NULL,gem_mrblib_mruby_print_proc_syms_1,gem_mrblib_mruby_print_proc_reps_1,
  NULL,					/* lv */
  &gem_mrblib_mruby_print_proc_debug_1,
  34,0,4,4,0
};
static const mrb_irep *gem_mrblib_mruby_print_proc_reps_0[1] = {
  &gem_mrblib_mruby_print_proc_irep_1,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_print_proc_syms_0, 1, (MRB_SYM(Kernel), ), const);
static const mrb_code gem_mrblib_mruby_print_proc_iseq_0[11] = {
0x11,0x01,0x5c,0x01,0x00,0x5d,0x01,0x00,0x37,0x01,0x68,};
static char gem_mrblib_mruby_print_proc_debug_lines_0[] = "\x00\x05";
static mrb_irep_debug_info_file gem_mrblib_mruby_print_proc_debug_file_0 = {
0, 173, 2, mrb_debug_line_packed_map, {gem_mrblib_mruby_print_proc_debug_lines_0}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_print_proc_debug_file_0_ = &gem_mrblib_mruby_print_proc_debug_file_0;
static mrb_irep_debug_info gem_mrblib_mruby_print_proc_debug_0 = {
11, 1, &gem_mrblib_mruby_print_proc_debug_file_0_};
static const mrb_irep gem_mrblib_mruby_print_proc_irep_0 = {
  1,2,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_print_proc_iseq_0,
  NULL,gem_mrblib_mruby_print_proc_syms_0,gem_mrblib_mruby_print_proc_reps_0,
  NULL,					/* lv */
  &gem_mrblib_mruby_print_proc_debug_0,
  11,0,1,1,0
};
static
const struct RProc gem_mrblib_mruby_print_proc[] = {{
NULL,NULL,MRB_TT_PROC,MRB_GC_RED,0,{&gem_mrblib_mruby_print_proc_irep_0},NULL,{NULL},
}};
static void
gem_mrblib_mruby_print_proc_init_syms(mrb_state *mrb)
{
  gem_mrblib_mruby_print_proc_debug_file_2.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-print/mrblib/print.rb");
  gem_mrblib_mruby_print_proc_debug_file_3.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-print/mrblib/print.rb");
  gem_mrblib_mruby_print_proc_debug_file_4.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-print/mrblib/print.rb");
  gem_mrblib_mruby_print_proc_debug_file_5.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-print/mrblib/print.rb");
  gem_mrblib_mruby_print_proc_debug_file_1.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-print/mrblib/print.rb");
  gem_mrblib_mruby_print_proc_debug_file_0.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-print/mrblib/print.rb");
}
void mrb_mruby_print_gem_init(mrb_state *mrb);
void mrb_mruby_print_gem_final(mrb_state *mrb);

void GENERATED_TMP_mrb_mruby_print_gem_init(mrb_state *mrb) {
  int ai = mrb_gc_arena_save(mrb);
  gem_mrblib_mruby_print_proc_init_syms(mrb);
  mrb_mruby_print_gem_init(mrb);
  mrb_load_proc(mrb, gem_mrblib_mruby_print_proc);
  if (mrb->exc) {
    mrb_print_error(mrb);
    mrb_close(mrb);
    exit(EXIT_FAILURE);
  }
  struct REnv *e = mrb_vm_ci_env(mrb->c->cibase);
  mrb_vm_ci_env_set(mrb->c->cibase, NULL);
  mrb_env_unshare(mrb, e);
  mrb_gc_arena_restore(mrb, ai);
}

void GENERATED_TMP_mrb_mruby_print_gem_final(mrb_state *mrb) {
  mrb_mruby_print_gem_final(mrb);
}
