/*
 * This file is loading the irep
 * Ruby GEM code.
 *
 * IMPORTANT:
 *   This file was generated!
 *   All manual changes will get lost.
 */
#include <stdlib.h>
#include <mruby.h>
#include <mruby/irep.h>
#include <mruby/debug.h>
#include <mruby/proc.h>
#include <mruby/presym.h>

#define mrb_BRACED(...) {__VA_ARGS__}
#define mrb_DEFINE_SYMS_VAR(name, len, syms, qualifier) \
  static qualifier mrb_sym name[len] = mrb_BRACED syms

mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_process_proc_syms_3, 3, (MRB_SYM(waitpid), 0, MRB_SYM(dup), ), );
static const mrb_code gem_mrblib_mruby_process_proc_iseq_3[54] = {
0x33,0x04,0x20,0x00,0x23,0x00,0x03,0x23,0x00,0x02,0x06,0x02,0x12,0x05,0x01,0x06,0x01,0x01,0x07,0x02,
0x2d,0x05,0x00,0x02,0x01,0x04,0x05,0x01,0x05,0x04,0x25,0x05,0x00,0x10,0x01,0x05,0x04,0x15,0x06,0x01,
0x2d,0x06,0x02,0x00,0x46,0x05,0x02,0x23,0x00,0x02,0x11,0x05,0x37,0x05,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_process_proc_lv_3, 4, (MRB_SYM(pid), MRB_SYM(flags), MRB_OPSYM(and), MRB_SYM(i), ), const);
static char gem_mrblib_mruby_process_proc_debug_lines_3[] = "\x00\x02\x0c\x01\x0f\x01\x05\x01\x10\x02";
static mrb_irep_debug_info_file gem_mrblib_mruby_process_proc_debug_file_3 = {
0, 173, 10, mrb_debug_line_packed_map, {gem_mrblib_mruby_process_proc_debug_lines_3}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_process_proc_debug_file_3_ = &gem_mrblib_mruby_process_proc_debug_file_3;
static mrb_irep_debug_info gem_mrblib_mruby_process_proc_debug_3 = {
54, 1, &gem_mrblib_mruby_process_proc_debug_file_3_};
static const mrb_irep gem_mrblib_mruby_process_proc_irep_3 = {
  5,9,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_process_proc_iseq_3,
  NULL,gem_mrblib_mruby_process_proc_syms_3,NULL,
  gem_mrblib_mruby_process_proc_lv_3,
  &gem_mrblib_mruby_process_proc_debug_3,
  54,0,3,0,0
};
static const mrb_irep *gem_mrblib_mruby_process_proc_reps_1[1] = {
  &gem_mrblib_mruby_process_proc_irep_3,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_process_proc_syms_1, 1, (MRB_SYM(waitpid2), ), const);
static const mrb_code gem_mrblib_mruby_process_proc_iseq_1[12] = {
0x12,0x01,0x61,0x01,0x57,0x02,0x00,0x5e,0x01,0x00,0x37,0x01,};
static char gem_mrblib_mruby_process_proc_debug_lines_1[] = "\x00\x02";
static mrb_irep_debug_info_file gem_mrblib_mruby_process_proc_debug_file_1 = {
0, 173, 2, mrb_debug_line_packed_map, {gem_mrblib_mruby_process_proc_debug_lines_1}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_process_proc_debug_file_1_ = &gem_mrblib_mruby_process_proc_debug_file_1;
static mrb_irep_debug_info gem_mrblib_mruby_process_proc_debug_1 = {
12, 1, &gem_mrblib_mruby_process_proc_debug_file_1_};
static const mrb_irep gem_mrblib_mruby_process_proc_irep_1 = {
  1,2,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_process_proc_iseq_1,
  NULL,gem_mrblib_mruby_process_proc_syms_1,gem_mrblib_mruby_process_proc_reps_1,
  NULL,					/* lv */
  &gem_mrblib_mruby_process_proc_debug_1,
  12,0,1,1,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_process_proc_syms_7, 2, (MRB_IVSYM(pid), MRB_IVSYM(status), ), const);
static const mrb_code gem_mrblib_mruby_process_proc_iseq_7[15] = {
0x33,0x08,0x00,0x00,0x1a,0x01,0x00,0x01,0x04,0x02,0x1a,0x04,0x01,0x37,0x04,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_process_proc_lv_7, 3, (MRB_SYM(pid), MRB_SYM(status), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_process_proc_debug_lines_7[] = "\x00\x03\x04\x01\x03\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_process_proc_debug_file_7 = {
0, 178, 6, mrb_debug_line_packed_map, {gem_mrblib_mruby_process_proc_debug_lines_7}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_process_proc_debug_file_7_ = &gem_mrblib_mruby_process_proc_debug_file_7;
static mrb_irep_debug_info gem_mrblib_mruby_process_proc_debug_7 = {
15, 1, &gem_mrblib_mruby_process_proc_debug_file_7_};
static const mrb_irep gem_mrblib_mruby_process_proc_irep_7 = {
  4,5,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_process_proc_iseq_7,
  NULL,gem_mrblib_mruby_process_proc_syms_7,NULL,
  gem_mrblib_mruby_process_proc_lv_7,
  &gem_mrblib_mruby_process_proc_debug_7,
  15,0,2,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_process_proc_syms_8, 1, (MRB_SYM(to_i), ), const);
static const mrb_code gem_mrblib_mruby_process_proc_iseq_8[21] = {
0x33,0x04,0x00,0x00,0x12,0x03,0x2d,0x03,0x00,0x00,0x01,0x04,0x01,0x2d,0x04,0x00,0x00,0x41,0x03,0x37,
0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_process_proc_lv_8, 2, (MRB_SYM(other), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_process_proc_debug_lines_8[] = "\x00\x08\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_process_proc_debug_file_8 = {
0, 178, 4, mrb_debug_line_packed_map, {gem_mrblib_mruby_process_proc_debug_lines_8}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_process_proc_debug_file_8_ = &gem_mrblib_mruby_process_proc_debug_file_8;
static mrb_irep_debug_info gem_mrblib_mruby_process_proc_debug_8 = {
21, 1, &gem_mrblib_mruby_process_proc_debug_file_8_};
static const mrb_irep gem_mrblib_mruby_process_proc_irep_8 = {
  3,6,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_process_proc_iseq_8,
  NULL,gem_mrblib_mruby_process_proc_syms_8,NULL,
  gem_mrblib_mruby_process_proc_lv_8,
  &gem_mrblib_mruby_process_proc_debug_8,
  21,0,1,0,0
};
static const mrb_pool_value gem_mrblib_mruby_process_proc_pool_9[10] = {
{IREP_TT_STR|(7<<2), {"\x65\x78\x69\x74\x65\x64\x28"}},
{IREP_TT_STR|(1<<2), {"\x29"}},
{IREP_TT_STR|(8<<2), {"\x73\x74\x6f\x70\x70\x65\x64\x28"}},
{IREP_TT_STR|(10<<2), {"\x63\x6f\x72\x65\x64\x75\x6d\x70\x65\x64"}},
{IREP_TT_STR|(9<<2), {"\x73\x69\x67\x6e\x61\x6c\x65\x64\x28"}},
{IREP_TT_STR|(7<<2), {"\x73\x74\x61\x74\x75\x73\x3d"}},
{IREP_TT_STR|(0<<2), {""}},
{IREP_TT_STR|(23<<2), {"\x23\x3c\x50\x72\x6f\x63\x65\x73\x73\x3a\x3a\x53\x74\x61\x74\x75\x73\x3a\x20\x70\x69\x64\x3d"}},
{IREP_TT_STR|(1<<2), {"\x2c"}},
{IREP_TT_STR|(1<<2), {"\x3e"}},
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_process_proc_syms_9, 9, (MRB_SYM_Q(exited), MRB_SYM(exitstatus), MRB_SYM_Q(stopped), MRB_SYM(stopsig), MRB_SYM_Q(signaled), MRB_SYM_Q(coredump), MRB_SYM(termsig), MRB_IVSYM(status), MRB_IVSYM(pid), ), const);
static const mrb_code gem_mrblib_mruby_process_proc_iseq_9[157] = {
0x33,0x00,0x00,0x00,0x12,0x03,0x2d,0x03,0x00,0x00,0x25,0x03,0x00,0x16,0x50,0x03,0x00,0x12,0x04,0x2d,
0x04,0x01,0x00,0x51,0x03,0x50,0x04,0x01,0x51,0x03,0x01,0x02,0x03,0x23,0x00,0x60,0x12,0x03,0x2d,0x03,
0x02,0x00,0x25,0x03,0x00,0x16,0x50,0x03,0x02,0x12,0x04,0x2d,0x04,0x03,0x00,0x51,0x03,0x50,0x04,0x01,
0x51,0x03,0x01,0x02,0x03,0x23,0x00,0x40,0x12,0x03,0x2d,0x03,0x04,0x00,0x25,0x03,0x00,0x26,0x12,0x03,
0x2d,0x03,0x05,0x00,0x25,0x03,0x00,0x06,0x50,0x02,0x03,0x23,0x00,0x13,0x50,0x03,0x04,0x12,0x04,0x2d,
0x04,0x06,0x00,0x51,0x03,0x50,0x04,0x01,0x51,0x03,0x01,0x02,0x03,0x23,0x00,0x10,0x50,0x03,0x05,0x19,
0x04,0x07,0x51,0x03,0x50,0x04,0x06,0x51,0x03,0x01,0x02,0x03,0x50,0x03,0x07,0x19,0x04,0x08,0x51,0x03,
0x50,0x04,0x08,0x51,0x03,0x01,0x04,0x02,0x51,0x03,0x50,0x04,0x09,0x51,0x03,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_process_proc_lv_9, 2, (MRB_OPSYM(and), MRB_SYM(s), ), const);
static char gem_mrblib_mruby_process_proc_debug_lines_9[] = "\x00\x0c\x04\x01\x08\x01\x16\x0a\x02\xf7\xff\xff\xff\x0f\x08\x01\x16\x08\x02\xf9\xff\xff\xff\x0f\x08\x04\x02\xfd\xff\xff\xff\x0f\x08\x01\x06\x02\x16\x03\x12\x02";
static mrb_irep_debug_info_file gem_mrblib_mruby_process_proc_debug_file_9 = {
0, 178, 40, mrb_debug_line_packed_map, {gem_mrblib_mruby_process_proc_debug_lines_9}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_process_proc_debug_file_9_ = &gem_mrblib_mruby_process_proc_debug_file_9;
static mrb_irep_debug_info gem_mrblib_mruby_process_proc_debug_9 = {
157, 1, &gem_mrblib_mruby_process_proc_debug_file_9_};
static const mrb_irep gem_mrblib_mruby_process_proc_irep_9 = {
  3,6,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_process_proc_iseq_9,
  gem_mrblib_mruby_process_proc_pool_9,gem_mrblib_mruby_process_proc_syms_9,NULL,
  gem_mrblib_mruby_process_proc_lv_9,
  &gem_mrblib_mruby_process_proc_debug_9,
  157,10,9,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_process_proc_syms_10, 1, (MRB_SYM(exitstatus), ), const);
static const mrb_code gem_mrblib_mruby_process_proc_iseq_10[16] = {
0x33,0x00,0x00,0x00,0x12,0x02,0x2d,0x02,0x00,0x00,0x06,0x03,0x41,0x02,0x37,0x02,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_process_proc_lv_10, 1, (MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_process_proc_debug_lines_10[] = "\x00\x1f\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_process_proc_debug_file_10 = {
0, 178, 4, mrb_debug_line_packed_map, {gem_mrblib_mruby_process_proc_debug_lines_10}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_process_proc_debug_file_10_ = &gem_mrblib_mruby_process_proc_debug_file_10;
static mrb_irep_debug_info gem_mrblib_mruby_process_proc_debug_10 = {
16, 1, &gem_mrblib_mruby_process_proc_debug_file_10_};
static const mrb_irep gem_mrblib_mruby_process_proc_irep_10 = {
  2,5,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_process_proc_iseq_10,
  NULL,gem_mrblib_mruby_process_proc_syms_10,NULL,
  gem_mrblib_mruby_process_proc_lv_10,
  &gem_mrblib_mruby_process_proc_debug_10,
  16,0,1,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_process_proc_syms_11, 1, (MRB_IVSYM(status), ), const);
static const mrb_code gem_mrblib_mruby_process_proc_iseq_11[9] = {
0x33,0x00,0x00,0x00,0x19,0x02,0x00,0x37,0x02,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_process_proc_lv_11, 1, (MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_process_proc_debug_lines_11[] = "\x00\x23\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_process_proc_debug_file_11 = {
0, 178, 4, mrb_debug_line_packed_map, {gem_mrblib_mruby_process_proc_debug_lines_11}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_process_proc_debug_file_11_ = &gem_mrblib_mruby_process_proc_debug_file_11;
static mrb_irep_debug_info gem_mrblib_mruby_process_proc_debug_11 = {
9, 1, &gem_mrblib_mruby_process_proc_debug_file_11_};
static const mrb_irep gem_mrblib_mruby_process_proc_irep_11 = {
  2,3,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_process_proc_iseq_11,
  NULL,gem_mrblib_mruby_process_proc_syms_11,NULL,
  gem_mrblib_mruby_process_proc_lv_11,
  &gem_mrblib_mruby_process_proc_debug_11,
  9,0,1,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_process_proc_syms_12, 2, (MRB_SYM(to_i), MRB_SYM(to_s), ), const);
static const mrb_code gem_mrblib_mruby_process_proc_iseq_12[16] = {
0x33,0x00,0x00,0x00,0x12,0x02,0x2d,0x02,0x00,0x00,0x2d,0x02,0x01,0x00,0x37,0x02,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_process_proc_lv_12, 1, (MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_process_proc_debug_lines_12[] = "\x00\x28\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_process_proc_debug_file_12 = {
0, 178, 4, mrb_debug_line_packed_map, {gem_mrblib_mruby_process_proc_debug_lines_12}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_process_proc_debug_file_12_ = &gem_mrblib_mruby_process_proc_debug_file_12;
static mrb_irep_debug_info gem_mrblib_mruby_process_proc_debug_12 = {
16, 1, &gem_mrblib_mruby_process_proc_debug_file_12_};
static const mrb_irep gem_mrblib_mruby_process_proc_irep_12 = {
  2,4,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_process_proc_iseq_12,
  NULL,gem_mrblib_mruby_process_proc_syms_12,NULL,
  gem_mrblib_mruby_process_proc_lv_12,
  &gem_mrblib_mruby_process_proc_debug_12,
  16,0,2,0,0
};
static const mrb_irep *gem_mrblib_mruby_process_proc_reps_6[6] = {
  &gem_mrblib_mruby_process_proc_irep_7,
  &gem_mrblib_mruby_process_proc_irep_8,
  &gem_mrblib_mruby_process_proc_irep_9,
  &gem_mrblib_mruby_process_proc_irep_10,
  &gem_mrblib_mruby_process_proc_irep_11,
  &gem_mrblib_mruby_process_proc_irep_12,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_process_proc_syms_6, 9, (MRB_SYM(initialize), MRB_OPSYM(eq), MRB_SYM(inspect), MRB_SYM(pid), MRB_SYM(attr_reader), MRB_SYM_Q(success), MRB_SYM(to_i), MRB_SYM(to_int), MRB_SYM(to_s), ), const);
static const mrb_code gem_mrblib_mruby_process_proc_iseq_6[62] = {
0x62,0x01,0x57,0x02,0x00,0x5e,0x01,0x00,0x62,0x01,0x57,0x02,0x01,0x5e,0x01,0x01,0x62,0x01,0x57,0x02,
0x02,0x5e,0x01,0x02,0x12,0x01,0x10,0x02,0x03,0x2d,0x01,0x04,0x01,0x62,0x01,0x57,0x02,0x03,0x5e,0x01,
0x05,0x62,0x01,0x57,0x02,0x04,0x5e,0x01,0x06,0x5f,0x07,0x06,0x62,0x01,0x57,0x02,0x05,0x5e,0x01,0x08,
0x37,0x01,};
static char gem_mrblib_mruby_process_proc_debug_lines_6[] = "\x00\x03\x08\x05\x08\x04\x08\x11\x09\x02\x08\x04\x08\x03\x03\x02";
static mrb_irep_debug_info_file gem_mrblib_mruby_process_proc_debug_file_6 = {
0, 178, 16, mrb_debug_line_packed_map, {gem_mrblib_mruby_process_proc_debug_lines_6}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_process_proc_debug_file_6_ = &gem_mrblib_mruby_process_proc_debug_file_6;
static mrb_irep_debug_info gem_mrblib_mruby_process_proc_debug_6 = {
62, 1, &gem_mrblib_mruby_process_proc_debug_file_6_};
static const mrb_irep gem_mrblib_mruby_process_proc_irep_6 = {
  1,4,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_process_proc_iseq_6,
  NULL,gem_mrblib_mruby_process_proc_syms_6,gem_mrblib_mruby_process_proc_reps_6,
  NULL,					/* lv */
  &gem_mrblib_mruby_process_proc_debug_6,
  62,0,9,6,0
};
static const mrb_irep *gem_mrblib_mruby_process_proc_reps_2[1] = {
  &gem_mrblib_mruby_process_proc_irep_6,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_process_proc_syms_2, 1, (MRB_SYM(Status), ), const);
static const mrb_code gem_mrblib_mruby_process_proc_iseq_2[12] = {
0x11,0x01,0x11,0x02,0x5b,0x01,0x00,0x5d,0x01,0x00,0x37,0x01,};
static char gem_mrblib_mruby_process_proc_debug_lines_2[] = "\x00\x02";
static mrb_irep_debug_info_file gem_mrblib_mruby_process_proc_debug_file_2 = {
0, 178, 2, mrb_debug_line_packed_map, {gem_mrblib_mruby_process_proc_debug_lines_2}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_process_proc_debug_file_2_ = &gem_mrblib_mruby_process_proc_debug_file_2;
static mrb_irep_debug_info gem_mrblib_mruby_process_proc_debug_2 = {
12, 1, &gem_mrblib_mruby_process_proc_debug_file_2_};
static const mrb_irep gem_mrblib_mruby_process_proc_irep_2 = {
  1,3,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_process_proc_iseq_2,
  NULL,gem_mrblib_mruby_process_proc_syms_2,gem_mrblib_mruby_process_proc_reps_2,
  NULL,					/* lv */
  &gem_mrblib_mruby_process_proc_debug_2,
  12,0,1,1,0
};
static const mrb_irep *gem_mrblib_mruby_process_proc_reps_0[2] = {
  &gem_mrblib_mruby_process_proc_irep_1,
  &gem_mrblib_mruby_process_proc_irep_2,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_process_proc_syms_0, 1, (MRB_SYM(Process), ), const);
static const mrb_code gem_mrblib_mruby_process_proc_iseq_0[19] = {
0x11,0x01,0x5c,0x01,0x00,0x5d,0x01,0x00,0x11,0x01,0x5c,0x01,0x00,0x5d,0x01,0x01,0x37,0x01,0x68,};
static const mrb_irep gem_mrblib_mruby_process_proc_irep_0 = {
  1,2,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_process_proc_iseq_0,
  NULL,gem_mrblib_mruby_process_proc_syms_0,gem_mrblib_mruby_process_proc_reps_0,
  NULL,					/* lv */
  NULL,					/* debug_info */
  19,0,1,2,0
};
static
const struct RProc gem_mrblib_mruby_process_proc[] = {{
NULL,NULL,MRB_TT_PROC,MRB_GC_RED,0,{&gem_mrblib_mruby_process_proc_irep_0},NULL,{NULL},
}};
static void
gem_mrblib_mruby_process_proc_init_syms(mrb_state *mrb)
{
  gem_mrblib_mruby_process_proc_syms_3[1] = mrb_intern_lit(mrb, "$?");
  gem_mrblib_mruby_process_proc_debug_file_3.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-process/mrblib/process.rb");
  gem_mrblib_mruby_process_proc_debug_file_1.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-process/mrblib/process.rb");
  gem_mrblib_mruby_process_proc_debug_file_7.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-process/mrblib/status.rb");
  gem_mrblib_mruby_process_proc_debug_file_8.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-process/mrblib/status.rb");
  gem_mrblib_mruby_process_proc_debug_file_9.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-process/mrblib/status.rb");
  gem_mrblib_mruby_process_proc_debug_file_10.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-process/mrblib/status.rb");
  gem_mrblib_mruby_process_proc_debug_file_11.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-process/mrblib/status.rb");
  gem_mrblib_mruby_process_proc_debug_file_12.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-process/mrblib/status.rb");
  gem_mrblib_mruby_process_proc_debug_file_6.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-process/mrblib/status.rb");
  gem_mrblib_mruby_process_proc_debug_file_2.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-process/mrblib/status.rb");
}
void mrb_mruby_process_gem_init(mrb_state *mrb);
void mrb_mruby_process_gem_final(mrb_state *mrb);

void GENERATED_TMP_mrb_mruby_process_gem_init(mrb_state *mrb) {
  int ai = mrb_gc_arena_save(mrb);
  gem_mrblib_mruby_process_proc_init_syms(mrb);
  mrb_mruby_process_gem_init(mrb);
  mrb_load_proc(mrb, gem_mrblib_mruby_process_proc);
  if (mrb->exc) {
    mrb_print_error(mrb);
    mrb_close(mrb);
    exit(EXIT_FAILURE);
  }
  struct REnv *e = mrb_vm_ci_env(mrb->c->cibase);
  mrb_vm_ci_env_set(mrb->c->cibase, NULL);
  mrb_env_unshare(mrb, e);
  mrb_gc_arena_restore(mrb, ai);
}

void GENERATED_TMP_mrb_mruby_process_gem_final(mrb_state *mrb) {
  mrb_mruby_process_gem_final(mrb);
}
