/*
 * This file is loading the irep
 * Ruby GEM code.
 *
 * IMPORTANT:
 *   This file was generated!
 *   All manual changes will get lost.
 */
#include <stdlib.h>
#include <mruby.h>
#include <mruby/irep.h>
#include <mruby/debug.h>
#include <mruby/proc.h>
#include <mruby/presym.h>

#define mrb_BRACED(...) {__VA_ARGS__}
#define mrb_DEFINE_SYMS_VAR(name, len, syms, qualifier) \
  static qualifier mrb_sym name[len] = mrb_BRACED syms

mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_syms_4, 4, (MRB_SYM(Glob), MRB_SYM(Dir), MRB_SYM(glob), MRB_SYM(size), ), const);
static const mrb_code gem_mrblib_mruby_dir_glob_proc_iseq_4[45] = {
0x33,0x04,0x00,0x00,0x1d,0x03,0x01,0x1f,0x03,0x00,0x01,0x04,0x01,0x21,0x05,0x02,0x00,0x21,0x06,0x05,
0x00,0x2d,0x03,0x02,0x03,0x21,0x03,0x06,0x00,0x21,0x04,0x05,0x00,0x2d,0x04,0x03,0x00,0x3b,0x03,0x22,
0x03,0x06,0x00,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_lv_4, 2, (MRB_SYM(pat), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_dir_glob_proc_debug_lines_4[] = "\x00\x0e\x04\x01\x15\x02";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_glob_proc_debug_file_4 = {
0, 173, 6, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_glob_proc_debug_lines_4}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_glob_proc_debug_file_4_ = &gem_mrblib_mruby_dir_glob_proc_debug_file_4;
static mrb_irep_debug_info gem_mrblib_mruby_dir_glob_proc_debug_4 = {
45, 1, &gem_mrblib_mruby_dir_glob_proc_debug_file_4_};
static const mrb_irep gem_mrblib_mruby_dir_glob_proc_irep_4 = {
  3,8,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_glob_proc_iseq_4,
  NULL,gem_mrblib_mruby_dir_glob_proc_syms_4,NULL,
  gem_mrblib_mruby_dir_glob_proc_lv_4,
  &gem_mrblib_mruby_dir_glob_proc_debug_4,
  45,0,4,0,0
};
static const mrb_irep *gem_mrblib_mruby_dir_glob_proc_reps_3[1] = {
  &gem_mrblib_mruby_dir_glob_proc_irep_4,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_syms_3, 4, (MRB_SYM(Array), MRB_SYM_Q(kind_of), MRB_SYM_Q(empty), MRB_SYM(each), ), const);
static const mrb_code gem_mrblib_mruby_dir_glob_proc_iseq_3[101] = {
0x33,0x04,0x20,0x01,0x23,0x00,0x03,0x23,0x00,0x02,0x06,0x02,0x01,0x07,0x01,0x1d,0x08,0x00,0x2d,0x07,
0x01,0x01,0x25,0x07,0x00,0x09,0x01,0x07,0x01,0x01,0x04,0x07,0x23,0x00,0x19,0x01,0x07,0x01,0x2d,0x07,
0x02,0x00,0x25,0x07,0x00,0x05,0x46,0x07,0x00,0x37,0x07,0x01,0x07,0x01,0x46,0x07,0x01,0x01,0x04,0x07,
0x46,0x07,0x00,0x01,0x05,0x07,0x06,0x06,0x01,0x07,0x04,0x56,0x08,0x00,0x2e,0x07,0x03,0x00,0x01,0x07,
0x03,0x25,0x07,0x00,0x0e,0x01,0x07,0x05,0x01,0x08,0x03,0x2e,0x07,0x03,0x00,0x11,0x07,0x37,0x07,0x37,
0x05,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_lv_3, 6, (MRB_SYM(pattern), MRB_SYM(flags), MRB_SYM(block), MRB_SYM(patterns), MRB_SYM(matches), MRB_SYM(index), ), const);
static char gem_mrblib_mruby_dir_glob_proc_debug_lines_3[] = "\x00\x03\x0c\x01\x0c\x01\x09\x03\x02\xff\xff\xff\xff\x0f\x10\x01\x09\x03\x06\x01\x02\x02\x0a\x06\x05\x02\x02\xff\xff\xff\xff\x0f\x0a\x01\x04\x03";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_glob_proc_debug_file_3 = {
0, 173, 36, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_glob_proc_debug_lines_3}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_glob_proc_debug_file_3_ = &gem_mrblib_mruby_dir_glob_proc_debug_file_3;
static mrb_irep_debug_info gem_mrblib_mruby_dir_glob_proc_debug_3 = {
101, 1, &gem_mrblib_mruby_dir_glob_proc_debug_file_3_};
static const mrb_irep gem_mrblib_mruby_dir_glob_proc_irep_3 = {
  7,10,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_glob_proc_iseq_3,
  NULL,gem_mrblib_mruby_dir_glob_proc_syms_3,gem_mrblib_mruby_dir_glob_proc_reps_3,
  gem_mrblib_mruby_dir_glob_proc_lv_3,
  &gem_mrblib_mruby_dir_glob_proc_debug_3,
  101,0,4,1,0
};
static const mrb_irep *gem_mrblib_mruby_dir_glob_proc_reps_1[1] = {
  &gem_mrblib_mruby_dir_glob_proc_irep_3,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_syms_1, 1, (MRB_SYM(glob), ), const);
static const mrb_code gem_mrblib_mruby_dir_glob_proc_iseq_1[12] = {
0x12,0x01,0x61,0x01,0x57,0x02,0x00,0x5e,0x01,0x00,0x37,0x01,};
static char gem_mrblib_mruby_dir_glob_proc_debug_lines_1[] = "\x00\x03";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_glob_proc_debug_file_1 = {
0, 173, 2, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_glob_proc_debug_lines_1}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_glob_proc_debug_file_1_ = &gem_mrblib_mruby_dir_glob_proc_debug_file_1;
static mrb_irep_debug_info gem_mrblib_mruby_dir_glob_proc_debug_1 = {
12, 1, &gem_mrblib_mruby_dir_glob_proc_debug_file_1_};
static const mrb_irep gem_mrblib_mruby_dir_glob_proc_irep_1 = {
  1,2,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_glob_proc_iseq_1,
  NULL,gem_mrblib_mruby_dir_glob_proc_syms_1,gem_mrblib_mruby_dir_glob_proc_reps_1,
  NULL,					/* lv */
  &gem_mrblib_mruby_dir_glob_proc_debug_1,
  12,0,1,1,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_syms_24, 3, (MRB_IVSYM(flags), MRB_IVSYM(next), MRB_IVSYM(separator), ), const);
static const mrb_code gem_mrblib_mruby_dir_glob_proc_iseq_24[17] = {
0x33,0x08,0x00,0x00,0x1a,0x02,0x00,0x1a,0x01,0x01,0x11,0x04,0x1a,0x04,0x02,0x37,0x04,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_lv_24, 3, (MRB_SYM(nxt), MRB_SYM(flags), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_dir_glob_proc_debug_lines_24[] = "\x00\x04\x04\x01\x03\x01\x03\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_glob_proc_debug_file_24 = {
0, 177, 8, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_glob_proc_debug_lines_24}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_glob_proc_debug_file_24_ = &gem_mrblib_mruby_dir_glob_proc_debug_file_24;
static mrb_irep_debug_info gem_mrblib_mruby_dir_glob_proc_debug_24 = {
17, 1, &gem_mrblib_mruby_dir_glob_proc_debug_file_24_};
static const mrb_irep gem_mrblib_mruby_dir_glob_proc_irep_24 = {
  4,5,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_glob_proc_iseq_24,
  NULL,gem_mrblib_mruby_dir_glob_proc_syms_24,NULL,
  gem_mrblib_mruby_dir_glob_proc_lv_24,
  &gem_mrblib_mruby_dir_glob_proc_debug_24,
  17,0,3,0,0
};
static const mrb_pool_value gem_mrblib_mruby_dir_glob_proc_pool_25[1] = {
{IREP_TT_STR|(1<<2), {"\x2f"}},
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_syms_25, 1, (MRB_IVSYM(separator), ), const);
static const mrb_code gem_mrblib_mruby_dir_glob_proc_iseq_25[16] = {
0x33,0x00,0x00,0x00,0x19,0x02,0x00,0x24,0x02,0x00,0x03,0x50,0x02,0x00,0x37,0x02,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_lv_25, 1, (MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_dir_glob_proc_debug_lines_25[] = "\x00\x0c\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_glob_proc_debug_file_25 = {
0, 177, 4, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_glob_proc_debug_lines_25}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_glob_proc_debug_file_25_ = &gem_mrblib_mruby_dir_glob_proc_debug_file_25;
static mrb_irep_debug_info gem_mrblib_mruby_dir_glob_proc_debug_25 = {
16, 1, &gem_mrblib_mruby_dir_glob_proc_debug_file_25_};
static const mrb_irep gem_mrblib_mruby_dir_glob_proc_irep_25 = {
  2,3,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_glob_proc_iseq_25,
  gem_mrblib_mruby_dir_glob_proc_pool_25,gem_mrblib_mruby_dir_glob_proc_syms_25,NULL,
  gem_mrblib_mruby_dir_glob_proc_lv_25,
  &gem_mrblib_mruby_dir_glob_proc_debug_25,
  16,1,1,0,0
};
static const mrb_pool_value gem_mrblib_mruby_dir_glob_proc_pool_26[2] = {
{IREP_TT_STR|(1<<2), {"\x2f"}},
{IREP_TT_STR|(0<<2), {""}},
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_syms_26, 1, (MRB_SYM(separator), ), const);
static const mrb_code gem_mrblib_mruby_dir_glob_proc_iseq_26[79] = {
0x33,0x08,0x00,0x00,0x01,0x04,0x01,0x24,0x04,0x00,0x02,0x37,0x02,0x01,0x04,0x01,0x50,0x05,0x00,0x41,
0x04,0x25,0x04,0x00,0x10,0x50,0x04,0x00,0x01,0x05,0x02,0x51,0x04,0x50,0x05,0x01,0x51,0x04,0x23,0x00,
0x24,0x50,0x04,0x01,0x01,0x05,0x01,0x51,0x04,0x50,0x05,0x01,0x51,0x04,0x12,0x05,0x2d,0x05,0x00,0x00,
0x51,0x04,0x50,0x05,0x01,0x51,0x04,0x01,0x05,0x02,0x51,0x04,0x50,0x05,0x01,0x51,0x04,0x37,0x04,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_lv_26, 3, (MRB_SYM(parent), MRB_SYM(ent), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_dir_glob_proc_debug_lines_26[] = "\x00\x10\x04\x01\x09\x01\x0a\x01\x10\x02";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_glob_proc_debug_file_26 = {
0, 177, 10, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_glob_proc_debug_lines_26}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_glob_proc_debug_file_26_ = &gem_mrblib_mruby_dir_glob_proc_debug_file_26;
static mrb_irep_debug_info gem_mrblib_mruby_dir_glob_proc_debug_26 = {
79, 1, &gem_mrblib_mruby_dir_glob_proc_debug_file_26_};
static const mrb_irep gem_mrblib_mruby_dir_glob_proc_irep_26 = {
  4,7,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_glob_proc_iseq_26,
  gem_mrblib_mruby_dir_glob_proc_pool_26,gem_mrblib_mruby_dir_glob_proc_syms_26,NULL,
  gem_mrblib_mruby_dir_glob_proc_lv_26,
  &gem_mrblib_mruby_dir_glob_proc_debug_26,
  79,2,1,0,0
};
static const mrb_irep *gem_mrblib_mruby_dir_glob_proc_reps_8[3] = {
  &gem_mrblib_mruby_dir_glob_proc_irep_24,
  &gem_mrblib_mruby_dir_glob_proc_irep_25,
  &gem_mrblib_mruby_dir_glob_proc_irep_26,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_syms_8, 4, (MRB_SYM(initialize), MRB_SYM(separator), MRB_SYM(attr_writer), MRB_SYM(path_join), ), const);
static const mrb_code gem_mrblib_mruby_dir_glob_proc_iseq_8[35] = {
0x62,0x01,0x57,0x02,0x00,0x5e,0x01,0x00,0x12,0x01,0x10,0x02,0x01,0x2d,0x01,0x02,0x01,0x62,0x01,0x57,
0x02,0x01,0x5e,0x01,0x01,0x62,0x01,0x57,0x02,0x02,0x5e,0x01,0x03,0x37,0x01,};
static char gem_mrblib_mruby_dir_glob_proc_debug_lines_8[] = "\x00\x04\x08\x06\x09\x02\x08\x04";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_glob_proc_debug_file_8 = {
0, 177, 8, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_glob_proc_debug_lines_8}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_glob_proc_debug_file_8_ = &gem_mrblib_mruby_dir_glob_proc_debug_file_8;
static mrb_irep_debug_info gem_mrblib_mruby_dir_glob_proc_debug_8 = {
35, 1, &gem_mrblib_mruby_dir_glob_proc_debug_file_8_};
static const mrb_irep gem_mrblib_mruby_dir_glob_proc_irep_8 = {
  1,4,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_glob_proc_iseq_8,
  NULL,gem_mrblib_mruby_dir_glob_proc_syms_8,gem_mrblib_mruby_dir_glob_proc_reps_8,
  NULL,					/* lv */
  &gem_mrblib_mruby_dir_glob_proc_debug_8,
  35,0,4,3,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_syms_49, 1, (MRB_IVSYM(dir), ), const);
static const mrb_code gem_mrblib_mruby_dir_glob_proc_iseq_49[27] = {
0x33,0x0c,0x00,0x00,0x32,0x06,0x00,0x00,0x01,0x06,0x01,0x01,0x07,0x02,0x11,0x08,0x31,0x05,0x02,0x01,
0x05,0x03,0x1a,0x05,0x00,0x37,0x05,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_lv_49, 4, (MRB_SYM(nxt), MRB_SYM(flags), MRB_SYM(dir), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_dir_glob_proc_debug_lines_49[] = "\x00\x1b\x04\x01\x0f\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_glob_proc_debug_file_49 = {
0, 177, 6, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_glob_proc_debug_lines_49}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_glob_proc_debug_file_49_ = &gem_mrblib_mruby_dir_glob_proc_debug_file_49;
static mrb_irep_debug_info gem_mrblib_mruby_dir_glob_proc_debug_49 = {
27, 1, &gem_mrblib_mruby_dir_glob_proc_debug_file_49_};
static const mrb_irep gem_mrblib_mruby_dir_glob_proc_irep_49 = {
  5,9,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_glob_proc_iseq_49,
  NULL,gem_mrblib_mruby_dir_glob_proc_syms_49,NULL,
  gem_mrblib_mruby_dir_glob_proc_lv_49,
  &gem_mrblib_mruby_dir_glob_proc_debug_49,
  27,0,1,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_syms_50, 4, (MRB_IVSYM(dir), MRB_SYM(path_join), MRB_IVSYM(next), MRB_SYM(call), ), const);
static const mrb_code gem_mrblib_mruby_dir_glob_proc_iseq_50[34] = {
0x33,0x08,0x00,0x00,0x12,0x05,0x01,0x06,0x02,0x19,0x07,0x00,0x2d,0x05,0x01,0x02,0x01,0x04,0x05,0x19,
0x05,0x02,0x01,0x06,0x01,0x01,0x07,0x04,0x2d,0x05,0x03,0x02,0x37,0x05,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_lv_50, 4, (MRB_SYM(env), MRB_SYM(path), MRB_OPSYM(and), MRB_SYM(full), ), const);
static char gem_mrblib_mruby_dir_glob_proc_debug_lines_50[] = "\x00\x20\x04\x01\x0f\x04";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_glob_proc_debug_file_50 = {
0, 177, 6, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_glob_proc_debug_lines_50}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_glob_proc_debug_file_50_ = &gem_mrblib_mruby_dir_glob_proc_debug_file_50;
static mrb_irep_debug_info gem_mrblib_mruby_dir_glob_proc_debug_50 = {
34, 1, &gem_mrblib_mruby_dir_glob_proc_debug_file_50_};
static const mrb_irep gem_mrblib_mruby_dir_glob_proc_irep_50 = {
  5,9,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_glob_proc_iseq_50,
  NULL,gem_mrblib_mruby_dir_glob_proc_syms_50,NULL,
  gem_mrblib_mruby_dir_glob_proc_lv_50,
  &gem_mrblib_mruby_dir_glob_proc_debug_50,
  34,0,4,0,0
};
static const mrb_irep *gem_mrblib_mruby_dir_glob_proc_reps_9[2] = {
  &gem_mrblib_mruby_dir_glob_proc_irep_49,
  &gem_mrblib_mruby_dir_glob_proc_irep_50,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_syms_9, 2, (MRB_SYM(initialize), MRB_SYM(call), ), const);
static const mrb_code gem_mrblib_mruby_dir_glob_proc_iseq_9[18] = {
0x62,0x01,0x57,0x02,0x00,0x5e,0x01,0x00,0x62,0x01,0x57,0x02,0x01,0x5e,0x01,0x01,0x37,0x01,};
static char gem_mrblib_mruby_dir_glob_proc_debug_lines_9[] = "\x00\x1b\x08\x05";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_glob_proc_debug_file_9 = {
0, 177, 4, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_glob_proc_debug_lines_9}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_glob_proc_debug_file_9_ = &gem_mrblib_mruby_dir_glob_proc_debug_file_9;
static mrb_irep_debug_info gem_mrblib_mruby_dir_glob_proc_debug_9 = {
18, 1, &gem_mrblib_mruby_dir_glob_proc_debug_file_9_};
static const mrb_irep gem_mrblib_mruby_dir_glob_proc_irep_9 = {
  1,3,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_glob_proc_iseq_9,
  NULL,gem_mrblib_mruby_dir_glob_proc_syms_9,gem_mrblib_mruby_dir_glob_proc_reps_9,
  NULL,					/* lv */
  &gem_mrblib_mruby_dir_glob_proc_debug_9,
  18,0,2,2,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_syms_69, 1, (MRB_IVSYM(name), ), const);
static const mrb_code gem_mrblib_mruby_dir_glob_proc_iseq_69[27] = {
0x33,0x0c,0x00,0x00,0x32,0x06,0x00,0x00,0x01,0x06,0x01,0x01,0x07,0x02,0x11,0x08,0x31,0x05,0x02,0x01,
0x05,0x03,0x1a,0x05,0x00,0x37,0x05,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_lv_69, 4, (MRB_SYM(nxt), MRB_SYM(flags), MRB_SYM(name), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_dir_glob_proc_debug_lines_69[] = "\x00\x2a\x04\x01\x0f\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_glob_proc_debug_file_69 = {
0, 177, 6, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_glob_proc_debug_lines_69}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_glob_proc_debug_file_69_ = &gem_mrblib_mruby_dir_glob_proc_debug_file_69;
static mrb_irep_debug_info gem_mrblib_mruby_dir_glob_proc_debug_69 = {
27, 1, &gem_mrblib_mruby_dir_glob_proc_debug_file_69_};
static const mrb_irep gem_mrblib_mruby_dir_glob_proc_irep_69 = {
  5,9,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_glob_proc_iseq_69,
  NULL,gem_mrblib_mruby_dir_glob_proc_syms_69,NULL,
  gem_mrblib_mruby_dir_glob_proc_lv_69,
  &gem_mrblib_mruby_dir_glob_proc_debug_69,
  27,0,1,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_syms_70, 6, (MRB_IVSYM(name), MRB_SYM(path_join), MRB_SYM(File), MRB_SYM_Q(exists), MRB_SYM(matches), MRB_OPSYM(lshift), ), const);
static const mrb_code gem_mrblib_mruby_dir_glob_proc_iseq_70[54] = {
0x33,0x08,0x00,0x00,0x12,0x05,0x01,0x06,0x02,0x19,0x07,0x00,0x2d,0x05,0x01,0x02,0x01,0x04,0x05,0x1d,
0x05,0x02,0x01,0x06,0x04,0x2d,0x05,0x03,0x01,0x25,0x05,0x00,0x11,0x01,0x05,0x01,0x2d,0x05,0x04,0x00,
0x01,0x06,0x04,0x2d,0x05,0x05,0x01,0x23,0x00,0x02,0x11,0x05,0x37,0x05,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_lv_70, 4, (MRB_SYM(env), MRB_SYM(parent), MRB_OPSYM(and), MRB_SYM(path), ), const);
static char gem_mrblib_mruby_dir_glob_proc_debug_lines_70[] = "\x00\x2f\x04\x01\x0f\x02\x0c\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_glob_proc_debug_file_70 = {
0, 177, 8, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_glob_proc_debug_lines_70}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_glob_proc_debug_file_70_ = &gem_mrblib_mruby_dir_glob_proc_debug_file_70;
static mrb_irep_debug_info gem_mrblib_mruby_dir_glob_proc_debug_70 = {
54, 1, &gem_mrblib_mruby_dir_glob_proc_debug_file_70_};
static const mrb_irep gem_mrblib_mruby_dir_glob_proc_irep_70 = {
  5,9,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_glob_proc_iseq_70,
  NULL,gem_mrblib_mruby_dir_glob_proc_syms_70,NULL,
  gem_mrblib_mruby_dir_glob_proc_lv_70,
  &gem_mrblib_mruby_dir_glob_proc_debug_70,
  54,0,6,0,0
};
static const mrb_irep *gem_mrblib_mruby_dir_glob_proc_reps_10[2] = {
  &gem_mrblib_mruby_dir_glob_proc_irep_69,
  &gem_mrblib_mruby_dir_glob_proc_irep_70,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_syms_10, 2, (MRB_SYM(initialize), MRB_SYM(call), ), const);
static const mrb_code gem_mrblib_mruby_dir_glob_proc_iseq_10[18] = {
0x62,0x01,0x57,0x02,0x00,0x5e,0x01,0x00,0x62,0x01,0x57,0x02,0x01,0x5e,0x01,0x01,0x37,0x01,};
static char gem_mrblib_mruby_dir_glob_proc_debug_lines_10[] = "\x00\x2a\x08\x05";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_glob_proc_debug_file_10 = {
0, 177, 4, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_glob_proc_debug_lines_10}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_glob_proc_debug_file_10_ = &gem_mrblib_mruby_dir_glob_proc_debug_file_10;
static mrb_irep_debug_info gem_mrblib_mruby_dir_glob_proc_debug_10 = {
18, 1, &gem_mrblib_mruby_dir_glob_proc_debug_file_10_};
static const mrb_irep gem_mrblib_mruby_dir_glob_proc_irep_10 = {
  1,3,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_glob_proc_iseq_10,
  NULL,gem_mrblib_mruby_dir_glob_proc_syms_10,gem_mrblib_mruby_dir_glob_proc_reps_10,
  NULL,					/* lv */
  &gem_mrblib_mruby_dir_glob_proc_debug_10,
  18,0,2,2,0
};
static const mrb_pool_value gem_mrblib_mruby_dir_glob_proc_pool_89[1] = {
{IREP_TT_STR|(1<<2), {"\x2f"}},
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_syms_89, 2, (MRB_IVSYM(next), MRB_SYM(call), ), const);
static const mrb_code gem_mrblib_mruby_dir_glob_proc_iseq_89[19] = {
0x33,0x08,0x00,0x00,0x19,0x04,0x00,0x01,0x05,0x01,0x50,0x06,0x00,0x2d,0x04,0x01,0x02,0x37,0x04,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_lv_89, 3, (MRB_SYM(env), MRB_SYM(path), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_dir_glob_proc_debug_lines_89[] = "\x00\x39\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_glob_proc_debug_file_89 = {
0, 177, 4, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_glob_proc_debug_lines_89}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_glob_proc_debug_file_89_ = &gem_mrblib_mruby_dir_glob_proc_debug_file_89;
static mrb_irep_debug_info gem_mrblib_mruby_dir_glob_proc_debug_89 = {
19, 1, &gem_mrblib_mruby_dir_glob_proc_debug_file_89_};
static const mrb_irep gem_mrblib_mruby_dir_glob_proc_irep_89 = {
  4,8,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_glob_proc_iseq_89,
  gem_mrblib_mruby_dir_glob_proc_pool_89,gem_mrblib_mruby_dir_glob_proc_syms_89,NULL,
  gem_mrblib_mruby_dir_glob_proc_lv_89,
  &gem_mrblib_mruby_dir_glob_proc_debug_89,
  19,1,2,0,0
};
static const mrb_irep *gem_mrblib_mruby_dir_glob_proc_reps_11[1] = {
  &gem_mrblib_mruby_dir_glob_proc_irep_89,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_syms_11, 1, (MRB_SYM(call), ), const);
static const mrb_code gem_mrblib_mruby_dir_glob_proc_iseq_11[10] = {
0x62,0x01,0x57,0x02,0x00,0x5e,0x01,0x00,0x37,0x01,};
static char gem_mrblib_mruby_dir_glob_proc_debug_lines_11[] = "\x00\x39";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_glob_proc_debug_file_11 = {
0, 177, 2, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_glob_proc_debug_lines_11}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_glob_proc_debug_file_11_ = &gem_mrblib_mruby_dir_glob_proc_debug_file_11;
static mrb_irep_debug_info gem_mrblib_mruby_dir_glob_proc_debug_11 = {
10, 1, &gem_mrblib_mruby_dir_glob_proc_debug_file_11_};
static const mrb_irep gem_mrblib_mruby_dir_glob_proc_irep_11 = {
  1,3,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_glob_proc_iseq_11,
  NULL,gem_mrblib_mruby_dir_glob_proc_syms_11,gem_mrblib_mruby_dir_glob_proc_reps_11,
  NULL,					/* lv */
  &gem_mrblib_mruby_dir_glob_proc_debug_11,
  10,0,1,1,0
};
static const mrb_pool_value gem_mrblib_mruby_dir_glob_proc_pool_106[2] = {
{IREP_TT_STR|(1<<2), {"\x2e"}},
{IREP_TT_STR|(2<<2), {"\x2e\x2e"}},
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_syms_106, 25, (MRB_OPSYM(not), MRB_SYM(File), MRB_SYM_Q(exists), MRB_IVSYM(next), MRB_SYM(dup), MRB_IVSYM(separator), MRB_SYM_E(separator), MRB_SYM(call), MRB_IVSYM(flags), MRB_SYM(FNM_DOTMATCH), MRB_OPSYM(and), MRB_OPSYM(neq), MRB_SYM_Q(empty), MRB_SYM(pop), MRB_SYM(Dir), MRB_SYM(new), MRB_SYM(ENOTDIR), MRB_SYM(Errno), MRB_SYM(read), MRB_SYM(path_join), MRB_SYM(lstat), MRB_SYM_Q(directory), MRB_SYM(getbyte), MRB_OPSYM(lshift), MRB_SYM(close), ), const);
static const mrb_code gem_mrblib_mruby_dir_glob_proc_iseq_106[328] = {
0x33,0x08,0x00,0x00,0x01,0x0c,0x02,0x2d,0x0c,0x00,0x00,0x24,0x0c,0x00,0x0e,0x1d,0x0c,0x01,0x01,0x0d,
0x02,0x2d,0x0c,0x02,0x01,0x2d,0x0c,0x00,0x00,0x25,0x0c,0x00,0x04,0x11,0x0c,0x37,0x0c,0x19,0x0c,0x03,
0x2d,0x0c,0x04,0x00,0x01,0x04,0x0c,0x19,0x0c,0x05,0x01,0x0d,0x04,0x01,0x0e,0x0c,0x2d,0x0d,0x06,0x01,
0x01,0x0c,0x04,0x01,0x0d,0x01,0x01,0x0e,0x02,0x2d,0x0c,0x07,0x02,0x01,0x0c,0x02,0x46,0x0c,0x01,0x01,
0x05,0x0c,0x19,0x0c,0x08,0x1d,0x0d,0x01,0x1f,0x0d,0x09,0x2d,0x0c,0x0a,0x01,0x06,0x0d,0x2d,0x0c,0x0b,
0x01,0x01,0x06,0x0c,0x01,0x0c,0x05,0x2d,0x0c,0x0c,0x00,0x24,0x0c,0x00,0xc4,0x01,0x0c,0x05,0x2d,0x0c,
0x0d,0x00,0x01,0x07,0x0c,0x1d,0x0c,0x0e,0x01,0x0d,0x07,0x2d,0x0c,0x0f,0x01,0x01,0x08,0x0c,0x23,0x00,
0x1b,0x28,0x0c,0x1d,0x0d,0x11,0x1f,0x0d,0x10,0x29,0x0c,0x0d,0x24,0x0d,0x00,0x03,0x23,0x00,0x07,0x11,
0x0c,0x37,0x0c,0x23,0x00,0x02,0x2a,0x0c,0x01,0x0c,0x08,0x2d,0x0c,0x12,0x00,0x01,0x09,0x0c,0x25,0x0c,
0x00,0x77,0x01,0x0c,0x09,0x50,0x0d,0x00,0x41,0x0c,0x24,0x0c,0x00,0x08,0x01,0x0c,0x09,0x50,0x0d,0x01,
0x41,0x0c,0x25,0x0c,0x00,0x03,0x27,0xff,0xd7,0x12,0x0c,0x01,0x0d,0x07,0x01,0x0e,0x09,0x2d,0x0c,0x13,
0x02,0x01,0x0a,0x0c,0x1d,0x0c,0x01,0x01,0x0d,0x0a,0x2d,0x0c,0x14,0x01,0x01,0x0b,0x0c,0x01,0x0c,0x0b,
0x2d,0x0c,0x15,0x00,0x25,0x0c,0x00,0x17,0x01,0x0c,0x06,0x24,0x0c,0x00,0x10,0x01,0x0c,0x09,0x06,0x0d,
0x2d,0x0c,0x16,0x01,0x03,0x0d,0x2e,0x2d,0x0c,0x0b,0x01,0x25,0x0c,0x00,0x17,0x01,0x0c,0x05,0x01,0x0d,
0x0a,0x2d,0x0c,0x17,0x01,0x19,0x0c,0x03,0x01,0x0d,0x01,0x01,0x0e,0x0a,0x2d,0x0c,0x07,0x02,0x23,0xff,
0x7b,0x01,0x0c,0x08,0x2d,0x0c,0x18,0x00,0x23,0xff,0x31,0x11,0x0c,0x37,0x0c,0x00,0x00,0x00,0x00,0x7d,
0x00,0x00,0x00,0x8a,0x00,0x00,0x00,0x8d,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_lv_106, 11, (MRB_SYM(env), MRB_SYM(start), MRB_OPSYM(and), MRB_SYM(switched), MRB_SYM(stack), MRB_SYM(allow_dots), MRB_SYM(path), MRB_SYM(dir), MRB_SYM(ent), MRB_SYM(full), MRB_SYM(stat), ), const);
static char gem_mrblib_mruby_dir_glob_proc_debug_lines_106[] = "\x00\x3f\x04\x01\x21\x05\x0a\x01\x0d\x01\x0d\x02\x09\x02\x16\x02\x09\x12\x02\xef\xff\xff\xff\x0f\x0a\x02\x0e\x02\x02\xfe\xff\xff\xff\x0f\x02\x01\x0e\x01\x0b\x03\x0c\x07\x02\xfa\xff\xff\xff\x0f\x1b\x01\x0f\x01\x0d\x02\x24\x02\x02\xff\xff\xff\xff\x0f\x0a\x01\x10\x03";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_glob_proc_debug_file_106 = {
0, 177, 66, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_glob_proc_debug_lines_106}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_glob_proc_debug_file_106_ = &gem_mrblib_mruby_dir_glob_proc_debug_file_106;
static mrb_irep_debug_info gem_mrblib_mruby_dir_glob_proc_debug_106 = {
315, 1, &gem_mrblib_mruby_dir_glob_proc_debug_file_106_};
static const mrb_irep gem_mrblib_mruby_dir_glob_proc_irep_106 = {
  12,16,1,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_glob_proc_iseq_106,
  gem_mrblib_mruby_dir_glob_proc_pool_106,gem_mrblib_mruby_dir_glob_proc_syms_106,NULL,
  gem_mrblib_mruby_dir_glob_proc_lv_106,
  &gem_mrblib_mruby_dir_glob_proc_debug_106,
  315,2,25,0,0
};
static const mrb_irep *gem_mrblib_mruby_dir_glob_proc_reps_12[1] = {
  &gem_mrblib_mruby_dir_glob_proc_irep_106,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_syms_12, 1, (MRB_SYM(call), ), const);
static const mrb_code gem_mrblib_mruby_dir_glob_proc_iseq_12[10] = {
0x62,0x01,0x57,0x02,0x00,0x5e,0x01,0x00,0x37,0x01,};
static char gem_mrblib_mruby_dir_glob_proc_debug_lines_12[] = "\x00\x3f";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_glob_proc_debug_file_12 = {
0, 177, 2, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_glob_proc_debug_lines_12}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_glob_proc_debug_file_12_ = &gem_mrblib_mruby_dir_glob_proc_debug_file_12;
static mrb_irep_debug_info gem_mrblib_mruby_dir_glob_proc_debug_12 = {
10, 1, &gem_mrblib_mruby_dir_glob_proc_debug_file_12_};
static const mrb_irep gem_mrblib_mruby_dir_glob_proc_irep_12 = {
  1,3,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_glob_proc_iseq_12,
  NULL,gem_mrblib_mruby_dir_glob_proc_syms_12,gem_mrblib_mruby_dir_glob_proc_reps_12,
  NULL,					/* lv */
  &gem_mrblib_mruby_dir_glob_proc_debug_12,
  10,0,1,1,0
};
static const mrb_pool_value gem_mrblib_mruby_dir_glob_proc_pool_123[3] = {
{IREP_TT_STR|(13<<2), {"\x69\x6e\x76\x61\x6c\x69\x64\x20\x75\x73\x61\x67\x65"}},
{IREP_TT_STR|(1<<2), {"\x2e"}},
{IREP_TT_STR|(2<<2), {"\x2e\x2e"}},
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_syms_123, 22, (MRB_SYM(raise), MRB_IVSYM(separator), MRB_IVSYM(next), MRB_SYM(dup), MRB_SYM_E(separator), MRB_SYM(call), MRB_IVSYM(flags), MRB_SYM(FNM_DOTMATCH), MRB_SYM(File), MRB_OPSYM(and), MRB_OPSYM(neq), MRB_SYM(Dir), MRB_SYM(new), MRB_SYM(read), MRB_SYM(lstat), MRB_SYM_Q(directory), MRB_SYM(getbyte), MRB_OPSYM(lshift), MRB_SYM(close), MRB_SYM_Q(empty), MRB_SYM(pop), MRB_SYM(path_join), ), const);
static const mrb_code gem_mrblib_mruby_dir_glob_proc_iseq_123[419] = {
0x33,0x08,0x00,0x00,0x01,0x0c,0x02,0x25,0x0c,0x00,0x09,0x12,0x0c,0x50,0x0d,0x00,0x2d,0x0c,0x00,0x01,
0x19,0x0c,0x01,0x25,0x0c,0x00,0x27,0x19,0x0c,0x02,0x2d,0x0c,0x03,0x00,0x01,0x04,0x0c,0x19,0x0c,0x01,
0x01,0x0d,0x04,0x01,0x0e,0x0c,0x2d,0x0d,0x04,0x01,0x01,0x0c,0x04,0x01,0x0d,0x01,0x01,0x0e,0x02,0x2d,
0x0c,0x05,0x02,0x23,0x00,0x0d,0x19,0x0c,0x02,0x01,0x0d,0x01,0x01,0x0e,0x02,0x2d,0x0c,0x05,0x02,0x46,
0x0c,0x00,0x01,0x05,0x0c,0x19,0x0c,0x06,0x1d,0x0d,0x08,0x1f,0x0d,0x07,0x2d,0x0c,0x09,0x01,0x06,0x0d,
0x2d,0x0c,0x0a,0x01,0x01,0x06,0x0c,0x1d,0x0c,0x0b,0x50,0x0d,0x01,0x2d,0x0c,0x0c,0x01,0x01,0x07,0x0c,
0x01,0x0c,0x07,0x2d,0x0c,0x0d,0x00,0x01,0x08,0x0c,0x25,0x0c,0x00,0x68,0x01,0x0c,0x08,0x50,0x0d,0x01,
0x41,0x0c,0x24,0x0c,0x00,0x08,0x01,0x0c,0x08,0x50,0x0d,0x02,0x41,0x0c,0x25,0x0c,0x00,0x03,0x27,0xff,
0xd7,0x1d,0x0c,0x08,0x01,0x0d,0x08,0x2d,0x0c,0x0e,0x01,0x01,0x09,0x0c,0x01,0x0c,0x09,0x2d,0x0c,0x0f,
0x00,0x25,0x0c,0x00,0x17,0x01,0x0c,0x06,0x24,0x0c,0x00,0x10,0x01,0x0c,0x08,0x06,0x0d,0x2d,0x0c,0x10,
0x01,0x03,0x0d,0x2e,0x2d,0x0c,0x0a,0x01,0x25,0x0c,0x00,0x17,0x01,0x0c,0x05,0x01,0x0d,0x08,0x2d,0x0c,
0x11,0x01,0x19,0x0c,0x02,0x01,0x0d,0x01,0x01,0x0e,0x08,0x2d,0x0c,0x05,0x02,0x23,0xff,0x8a,0x01,0x0c,
0x07,0x2d,0x0c,0x12,0x00,0x01,0x0c,0x05,0x2d,0x0c,0x13,0x00,0x24,0x0c,0x00,0x9f,0x01,0x0c,0x05,0x2d,
0x0c,0x14,0x00,0x01,0x0a,0x0c,0x1d,0x0c,0x0b,0x01,0x0d,0x0a,0x2d,0x0c,0x0c,0x01,0x01,0x07,0x0c,0x01,
0x0c,0x07,0x2d,0x0c,0x0d,0x00,0x01,0x08,0x0c,0x25,0x0c,0x00,0x70,0x01,0x0c,0x08,0x50,0x0d,0x01,0x41,
0x0c,0x24,0x0c,0x00,0x08,0x01,0x0c,0x08,0x50,0x0d,0x02,0x41,0x0c,0x25,0x0c,0x00,0x03,0x27,0xff,0xd7,
0x12,0x0c,0x01,0x0d,0x0a,0x01,0x0e,0x08,0x2d,0x0c,0x15,0x02,0x01,0x0b,0x0c,0x1d,0x0c,0x08,0x01,0x0d,
0x0b,0x2d,0x0c,0x0e,0x01,0x01,0x09,0x0c,0x01,0x0c,0x09,0x2d,0x0c,0x0f,0x00,0x25,0x0c,0x00,0x10,0x01,
0x0c,0x08,0x06,0x0d,0x2d,0x0c,0x10,0x01,0x03,0x0d,0x2e,0x2d,0x0c,0x0a,0x01,0x25,0x0c,0x00,0x17,0x01,
0x0c,0x05,0x01,0x0d,0x0b,0x2d,0x0c,0x11,0x01,0x19,0x0c,0x02,0x01,0x0d,0x01,0x01,0x0e,0x0b,0x2d,0x0c,
0x05,0x02,0x23,0xff,0x82,0x01,0x0c,0x07,0x2d,0x0c,0x12,0x00,0x23,0xff,0x56,0x11,0x0c,0x37,0x0c,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_lv_123, 11, (MRB_SYM(env), MRB_SYM(start), MRB_OPSYM(and), MRB_SYM(switched), MRB_SYM(stack), MRB_SYM(allow_dots), MRB_SYM(dir), MRB_SYM(ent), MRB_SYM(stat), MRB_SYM(path), MRB_SYM(full), ), const);
static char gem_mrblib_mruby_dir_glob_proc_debug_lines_123[] = "\x00\x65\x04\x01\x10\x05\x05\x03\x02\xfe\xff\xff\xff\x0f\x0a\x01\x0d\x01\x0e\x02\x0f\x03\x06\x02\x16\x02\x0d\x01\x0c\x06\x02\xfb\xff\xff\xff\x0f\x1b\x01\x0d\x02\x24\x02\x02\xff\xff\xff\xff\x0f\x0a\x01\x10\x03\x07\x02\x09\x0d\x02\xf4\xff\xff\xff\x0f\x0a\x01\x0d\x01\x0c\x07\x02\xfa\xff\xff\xff\x0f\x1b\x01\x0f\x01\x0d\x02\x1d\x02\x02\xff\xff\xff\xff\x0f\x0a\x01\x10\x03";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_glob_proc_debug_file_123 = {
0, 177, 92, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_glob_proc_debug_lines_123}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_glob_proc_debug_file_123_ = &gem_mrblib_mruby_dir_glob_proc_debug_file_123;
static mrb_irep_debug_info gem_mrblib_mruby_dir_glob_proc_debug_123 = {
419, 1, &gem_mrblib_mruby_dir_glob_proc_debug_file_123_};
static const mrb_irep gem_mrblib_mruby_dir_glob_proc_irep_123 = {
  12,16,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_glob_proc_iseq_123,
  gem_mrblib_mruby_dir_glob_proc_pool_123,gem_mrblib_mruby_dir_glob_proc_syms_123,NULL,
  gem_mrblib_mruby_dir_glob_proc_lv_123,
  &gem_mrblib_mruby_dir_glob_proc_debug_123,
  419,3,22,0,0
};
static const mrb_irep *gem_mrblib_mruby_dir_glob_proc_reps_13[1] = {
  &gem_mrblib_mruby_dir_glob_proc_irep_123,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_syms_13, 1, (MRB_SYM(call), ), const);
static const mrb_code gem_mrblib_mruby_dir_glob_proc_iseq_13[10] = {
0x62,0x01,0x57,0x02,0x00,0x5e,0x01,0x00,0x37,0x01,};
static char gem_mrblib_mruby_dir_glob_proc_debug_lines_13[] = "\x00\x65";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_glob_proc_debug_file_13 = {
0, 177, 2, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_glob_proc_debug_lines_13}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_glob_proc_debug_file_13_ = &gem_mrblib_mruby_dir_glob_proc_debug_file_13;
static mrb_irep_debug_info gem_mrblib_mruby_dir_glob_proc_debug_13 = {
10, 1, &gem_mrblib_mruby_dir_glob_proc_debug_file_13_};
static const mrb_irep gem_mrblib_mruby_dir_glob_proc_irep_13 = {
  1,3,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_glob_proc_iseq_13,
  NULL,gem_mrblib_mruby_dir_glob_proc_syms_13,gem_mrblib_mruby_dir_glob_proc_reps_13,
  NULL,					/* lv */
  &gem_mrblib_mruby_dir_glob_proc_debug_13,
  10,0,1,1,0
};
static const mrb_pool_value gem_mrblib_mruby_dir_glob_proc_pool_140[1] = {
{IREP_TT_STR|(0<<2), {""}},
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_syms_140, 1, (MRB_IVSYM(glob), ), const);
static const mrb_code gem_mrblib_mruby_dir_glob_proc_iseq_140[34] = {
0x33,0x0c,0x00,0x00,0x32,0x06,0x00,0x00,0x01,0x06,0x01,0x01,0x07,0x02,0x11,0x08,0x31,0x05,0x02,0x01,
0x05,0x03,0x24,0x05,0x00,0x03,0x50,0x05,0x00,0x1a,0x05,0x00,0x37,0x05,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_lv_140, 4, (MRB_SYM(nxt), MRB_SYM(flags), MRB_SYM(glob), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_dir_glob_proc_debug_lines_140[] = "\x00\x96\x01\x04\x01\x0f\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_glob_proc_debug_file_140 = {
0, 177, 7, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_glob_proc_debug_lines_140}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_glob_proc_debug_file_140_ = &gem_mrblib_mruby_dir_glob_proc_debug_file_140;
static mrb_irep_debug_info gem_mrblib_mruby_dir_glob_proc_debug_140 = {
34, 1, &gem_mrblib_mruby_dir_glob_proc_debug_file_140_};
static const mrb_irep gem_mrblib_mruby_dir_glob_proc_irep_140 = {
  5,9,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_glob_proc_iseq_140,
  gem_mrblib_mruby_dir_glob_proc_pool_140,gem_mrblib_mruby_dir_glob_proc_syms_140,NULL,
  gem_mrblib_mruby_dir_glob_proc_lv_140,
  &gem_mrblib_mruby_dir_glob_proc_debug_140,
  34,1,1,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_syms_141, 4, (MRB_SYM(File), MRB_IVSYM(glob), MRB_IVSYM(flags), MRB_SYM(fnmatch), ), const);
static const mrb_code gem_mrblib_mruby_dir_glob_proc_iseq_141[22] = {
0x33,0x04,0x00,0x00,0x1d,0x03,0x00,0x19,0x04,0x01,0x01,0x05,0x01,0x19,0x06,0x02,0x2d,0x03,0x03,0x03,
0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_lv_141, 2, (MRB_SYM(str), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_dir_glob_proc_debug_lines_141[] = "\x00\x9b\x01\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_glob_proc_debug_file_141 = {
0, 177, 5, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_glob_proc_debug_lines_141}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_glob_proc_debug_file_141_ = &gem_mrblib_mruby_dir_glob_proc_debug_file_141;
static mrb_irep_debug_info gem_mrblib_mruby_dir_glob_proc_debug_141 = {
22, 1, &gem_mrblib_mruby_dir_glob_proc_debug_file_141_};
static const mrb_irep gem_mrblib_mruby_dir_glob_proc_irep_141 = {
  3,8,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_glob_proc_iseq_141,
  NULL,gem_mrblib_mruby_dir_glob_proc_syms_141,NULL,
  gem_mrblib_mruby_dir_glob_proc_lv_141,
  &gem_mrblib_mruby_dir_glob_proc_debug_141,
  22,0,4,0,0
};
static const mrb_irep *gem_mrblib_mruby_dir_glob_proc_reps_14[2] = {
  &gem_mrblib_mruby_dir_glob_proc_irep_140,
  &gem_mrblib_mruby_dir_glob_proc_irep_141,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_syms_14, 2, (MRB_SYM(initialize), MRB_SYM_Q(match), ), const);
static const mrb_code gem_mrblib_mruby_dir_glob_proc_iseq_14[18] = {
0x62,0x01,0x57,0x02,0x00,0x5e,0x01,0x00,0x62,0x01,0x57,0x02,0x01,0x5e,0x01,0x01,0x37,0x01,};
static char gem_mrblib_mruby_dir_glob_proc_debug_lines_14[] = "\x00\x96\x01\x08\x05";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_glob_proc_debug_file_14 = {
0, 177, 5, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_glob_proc_debug_lines_14}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_glob_proc_debug_file_14_ = &gem_mrblib_mruby_dir_glob_proc_debug_file_14;
static mrb_irep_debug_info gem_mrblib_mruby_dir_glob_proc_debug_14 = {
18, 1, &gem_mrblib_mruby_dir_glob_proc_debug_file_14_};
static const mrb_irep gem_mrblib_mruby_dir_glob_proc_irep_14 = {
  1,3,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_glob_proc_iseq_14,
  NULL,gem_mrblib_mruby_dir_glob_proc_syms_14,gem_mrblib_mruby_dir_glob_proc_reps_14,
  NULL,					/* lv */
  &gem_mrblib_mruby_dir_glob_proc_debug_14,
  18,0,2,2,0
};
static const mrb_pool_value gem_mrblib_mruby_dir_glob_proc_pool_160[2] = {
{IREP_TT_STR|(2<<2), {"\x2a\x2a"}},
{IREP_TT_STR|(1<<2), {"\x2a"}},
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_syms_160, 2, (MRB_IVSYM(glob), MRB_SYM_B(gsub), ), const);
static const mrb_code gem_mrblib_mruby_dir_glob_proc_iseq_160[26] = {
0x33,0x0c,0x00,0x00,0x32,0x06,0x18,0x00,0x31,0x05,0x7f,0x19,0x05,0x00,0x50,0x06,0x00,0x50,0x07,0x01,
0x2d,0x05,0x01,0x02,0x37,0x05,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_lv_160, 4, (MRB_SYM(nxt), MRB_SYM(flags), MRB_SYM(glob), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_dir_glob_proc_debug_lines_160[] = "\x00\xa1\x01\x04\x01\x07\x02";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_glob_proc_debug_file_160 = {
0, 177, 7, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_glob_proc_debug_lines_160}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_glob_proc_debug_file_160_ = &gem_mrblib_mruby_dir_glob_proc_debug_file_160;
static mrb_irep_debug_info gem_mrblib_mruby_dir_glob_proc_debug_160 = {
26, 1, &gem_mrblib_mruby_dir_glob_proc_debug_file_160_};
static const mrb_irep gem_mrblib_mruby_dir_glob_proc_irep_160 = {
  5,9,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_glob_proc_iseq_160,
  gem_mrblib_mruby_dir_glob_proc_pool_160,gem_mrblib_mruby_dir_glob_proc_syms_160,NULL,
  gem_mrblib_mruby_dir_glob_proc_lv_160,
  &gem_mrblib_mruby_dir_glob_proc_debug_160,
  26,2,2,0,0
};
static const mrb_pool_value gem_mrblib_mruby_dir_glob_proc_pool_161[3] = {
{IREP_TT_STR|(0<<2), {""}},
{IREP_TT_STR|(2<<2), {"\x2f\x2e"}},
{IREP_TT_STR|(1<<2), {"\x2e"}},
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_syms_161, 12, (MRB_SYM(File), MRB_SYM_Q(exists), MRB_OPSYM(not), MRB_SYM(Dir), MRB_SYM(new), MRB_SYM(read), MRB_SYM_Q(match), MRB_SYM(path_join), MRB_SYM_Q(directory), MRB_IVSYM(next), MRB_SYM(call), MRB_SYM(close), ), const);
static const mrb_code gem_mrblib_mruby_dir_glob_proc_iseq_161[150] = {
0x33,0x08,0x00,0x00,0x01,0x07,0x02,0x25,0x07,0x00,0x18,0x1d,0x07,0x00,0x50,0x08,0x00,0x01,0x09,0x02,
0x51,0x08,0x50,0x09,0x01,0x51,0x08,0x2d,0x07,0x01,0x01,0x2d,0x07,0x02,0x00,0x25,0x07,0x00,0x04,0x11,
0x07,0x37,0x07,0x1d,0x07,0x03,0x01,0x08,0x02,0x25,0x08,0x00,0x06,0x01,0x08,0x02,0x23,0x00,0x03,0x50,
0x08,0x02,0x2d,0x07,0x04,0x01,0x01,0x04,0x07,0x01,0x07,0x04,0x2d,0x07,0x05,0x00,0x01,0x05,0x07,0x25,
0x07,0x00,0x3a,0x12,0x07,0x01,0x08,0x05,0x2d,0x07,0x06,0x01,0x25,0x07,0x00,0x2a,0x12,0x07,0x01,0x08,
0x02,0x01,0x09,0x05,0x2d,0x07,0x07,0x02,0x01,0x06,0x07,0x1d,0x07,0x00,0x01,0x08,0x06,0x2d,0x07,0x08,
0x01,0x25,0x07,0x00,0x0d,0x19,0x07,0x09,0x01,0x08,0x01,0x01,0x09,0x06,0x2d,0x07,0x0a,0x02,0x23,0xff,
0xb8,0x01,0x07,0x04,0x2d,0x07,0x0b,0x00,0x37,0x07,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_lv_161, 6, (MRB_SYM(env), MRB_SYM(path), MRB_OPSYM(and), MRB_SYM(dir), MRB_SYM(ent), MRB_SYM(full), ), const);
static char gem_mrblib_mruby_dir_glob_proc_debug_lines_161[] = "\x00\xa7\x01\x04\x01\x27\x02\x1a\x01\x0c\x05\x02\xfc\xff\xff\xff\x0f\x0b\x04\x02\xfd\xff\xff\xff\x0f\x0f\x02\x0c\x01\x12\x04";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_glob_proc_debug_file_161 = {
0, 177, 31, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_glob_proc_debug_lines_161}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_glob_proc_debug_file_161_ = &gem_mrblib_mruby_dir_glob_proc_debug_file_161;
static mrb_irep_debug_info gem_mrblib_mruby_dir_glob_proc_debug_161 = {
150, 1, &gem_mrblib_mruby_dir_glob_proc_debug_file_161_};
static const mrb_irep gem_mrblib_mruby_dir_glob_proc_irep_161 = {
  7,11,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_glob_proc_iseq_161,
  gem_mrblib_mruby_dir_glob_proc_pool_161,gem_mrblib_mruby_dir_glob_proc_syms_161,NULL,
  gem_mrblib_mruby_dir_glob_proc_lv_161,
  &gem_mrblib_mruby_dir_glob_proc_debug_161,
  150,3,12,0,0
};
static const mrb_irep *gem_mrblib_mruby_dir_glob_proc_reps_15[2] = {
  &gem_mrblib_mruby_dir_glob_proc_irep_160,
  &gem_mrblib_mruby_dir_glob_proc_irep_161,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_syms_15, 2, (MRB_SYM(initialize), MRB_SYM(call), ), const);
static const mrb_code gem_mrblib_mruby_dir_glob_proc_iseq_15[18] = {
0x62,0x01,0x57,0x02,0x00,0x5e,0x01,0x00,0x62,0x01,0x57,0x02,0x01,0x5e,0x01,0x01,0x37,0x01,};
static char gem_mrblib_mruby_dir_glob_proc_debug_lines_15[] = "\x00\xa1\x01\x08\x06";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_glob_proc_debug_file_15 = {
0, 177, 5, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_glob_proc_debug_lines_15}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_glob_proc_debug_file_15_ = &gem_mrblib_mruby_dir_glob_proc_debug_file_15;
static mrb_irep_debug_info gem_mrblib_mruby_dir_glob_proc_debug_15 = {
18, 1, &gem_mrblib_mruby_dir_glob_proc_debug_file_15_};
static const mrb_irep gem_mrblib_mruby_dir_glob_proc_irep_15 = {
  1,3,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_glob_proc_iseq_15,
  NULL,gem_mrblib_mruby_dir_glob_proc_syms_15,gem_mrblib_mruby_dir_glob_proc_reps_15,
  NULL,					/* lv */
  &gem_mrblib_mruby_dir_glob_proc_debug_15,
  18,0,2,2,0
};
static const mrb_pool_value gem_mrblib_mruby_dir_glob_proc_pool_180[3] = {
{IREP_TT_STR|(0<<2), {""}},
{IREP_TT_STR|(2<<2), {"\x2f\x2e"}},
{IREP_TT_STR|(1<<2), {"\x2e"}},
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_syms_180, 12, (MRB_SYM(File), MRB_SYM_Q(exists), MRB_OPSYM(not), MRB_SYM(Dir), MRB_SYM(new), MRB_SYM(SystemCallError), MRB_SYM(read), MRB_SYM_Q(match), MRB_SYM(matches), MRB_SYM(path_join), MRB_OPSYM(lshift), MRB_SYM(close), ), const);
static const mrb_code gem_mrblib_mruby_dir_glob_proc_iseq_180[171] = {
0x33,0x08,0x00,0x00,0x01,0x06,0x02,0x25,0x06,0x00,0x18,0x1d,0x06,0x00,0x50,0x07,0x00,0x01,0x08,0x02,
0x51,0x07,0x50,0x08,0x01,0x51,0x07,0x2d,0x06,0x01,0x01,0x2d,0x06,0x02,0x00,0x25,0x06,0x00,0x04,0x11,
0x06,0x37,0x06,0x1d,0x06,0x03,0x01,0x07,0x02,0x25,0x07,0x00,0x06,0x01,0x07,0x02,0x23,0x00,0x03,0x50,
0x07,0x02,0x2d,0x06,0x04,0x01,0x01,0x04,0x06,0x23,0x00,0x18,0x28,0x06,0x1d,0x07,0x05,0x29,0x06,0x07,
0x24,0x07,0x00,0x03,0x23,0x00,0x07,0x11,0x06,0x38,0x06,0x23,0x00,0x02,0x2a,0x06,0x01,0x06,0x04,0x2d,
0x06,0x06,0x00,0x01,0x05,0x06,0x25,0x06,0x00,0x27,0x12,0x06,0x01,0x07,0x05,0x2d,0x06,0x07,0x01,0x25,
0x06,0x00,0x17,0x01,0x06,0x01,0x2d,0x06,0x08,0x00,0x12,0x07,0x01,0x08,0x02,0x01,0x09,0x05,0x2d,0x07,
0x09,0x02,0x2d,0x06,0x0a,0x01,0x23,0xff,0xcb,0x01,0x06,0x04,0x2d,0x06,0x0b,0x00,0x37,0x06,0x00,0x00,
0x00,0x00,0x2b,0x00,0x00,0x00,0x45,0x00,0x00,0x00,0x48,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_lv_180, 5, (MRB_SYM(env), MRB_SYM(path), MRB_OPSYM(and), MRB_SYM(dir), MRB_SYM(ent), ), const);
static char gem_mrblib_mruby_dir_glob_proc_debug_lines_180[] = "\x00\xb9\x01\x04\x01\x27\x03\x1b\x02\x02\xfe\xff\xff\xff\x0f\x02\x01\x0b\x01\x0b\x03\x0c\x02\x02\xff\xff\xff\xff\x0f\x0b\x01\x1c\x03";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_glob_proc_debug_file_180 = {
0, 177, 33, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_glob_proc_debug_lines_180}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_glob_proc_debug_file_180_ = &gem_mrblib_mruby_dir_glob_proc_debug_file_180;
static mrb_irep_debug_info gem_mrblib_mruby_dir_glob_proc_debug_180 = {
158, 1, &gem_mrblib_mruby_dir_glob_proc_debug_file_180_};
static const mrb_irep gem_mrblib_mruby_dir_glob_proc_irep_180 = {
  6,11,1,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_glob_proc_iseq_180,
  gem_mrblib_mruby_dir_glob_proc_pool_180,gem_mrblib_mruby_dir_glob_proc_syms_180,NULL,
  gem_mrblib_mruby_dir_glob_proc_lv_180,
  &gem_mrblib_mruby_dir_glob_proc_debug_180,
  158,3,12,0,0
};
static const mrb_irep *gem_mrblib_mruby_dir_glob_proc_reps_16[1] = {
  &gem_mrblib_mruby_dir_glob_proc_irep_180,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_syms_16, 1, (MRB_SYM(call), ), const);
static const mrb_code gem_mrblib_mruby_dir_glob_proc_iseq_16[10] = {
0x62,0x01,0x57,0x02,0x00,0x5e,0x01,0x00,0x37,0x01,};
static char gem_mrblib_mruby_dir_glob_proc_debug_lines_16[] = "\x00\xb9\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_glob_proc_debug_file_16 = {
0, 177, 3, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_glob_proc_debug_lines_16}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_glob_proc_debug_file_16_ = &gem_mrblib_mruby_dir_glob_proc_debug_file_16;
static mrb_irep_debug_info gem_mrblib_mruby_dir_glob_proc_debug_16 = {
10, 1, &gem_mrblib_mruby_dir_glob_proc_debug_file_16_};
static const mrb_irep gem_mrblib_mruby_dir_glob_proc_irep_16 = {
  1,3,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_glob_proc_iseq_16,
  NULL,gem_mrblib_mruby_dir_glob_proc_syms_16,gem_mrblib_mruby_dir_glob_proc_reps_16,
  NULL,					/* lv */
  &gem_mrblib_mruby_dir_glob_proc_debug_16,
  10,0,1,1,0
};
static const mrb_pool_value gem_mrblib_mruby_dir_glob_proc_pool_197[3] = {
{IREP_TT_STR|(0<<2), {""}},
{IREP_TT_STR|(2<<2), {"\x2f\x2e"}},
{IREP_TT_STR|(1<<2), {"\x2f"}},
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_syms_197, 4, (MRB_SYM(File), MRB_SYM_Q(exists), MRB_SYM(matches), MRB_OPSYM(lshift), ), const);
static const mrb_code gem_mrblib_mruby_dir_glob_proc_iseq_197[66] = {
0x33,0x08,0x00,0x00,0x01,0x04,0x02,0x25,0x04,0x00,0x14,0x1d,0x04,0x00,0x50,0x05,0x00,0x01,0x06,0x02,
0x51,0x05,0x50,0x06,0x01,0x51,0x05,0x2d,0x04,0x01,0x01,0x25,0x04,0x00,0x1b,0x01,0x04,0x01,0x2d,0x04,
0x02,0x00,0x50,0x05,0x00,0x01,0x06,0x02,0x51,0x05,0x50,0x06,0x02,0x51,0x05,0x2d,0x04,0x03,0x01,0x23,
0x00,0x02,0x11,0x04,0x37,0x04,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_lv_197, 3, (MRB_SYM(env), MRB_SYM(path), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_dir_glob_proc_debug_lines_197[] = "\x00\xcc\x01\x04\x01\x1d\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_glob_proc_debug_file_197 = {
0, 177, 7, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_glob_proc_debug_lines_197}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_glob_proc_debug_file_197_ = &gem_mrblib_mruby_dir_glob_proc_debug_file_197;
static mrb_irep_debug_info gem_mrblib_mruby_dir_glob_proc_debug_197 = {
66, 1, &gem_mrblib_mruby_dir_glob_proc_debug_file_197_};
static const mrb_irep gem_mrblib_mruby_dir_glob_proc_irep_197 = {
  4,7,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_glob_proc_iseq_197,
  gem_mrblib_mruby_dir_glob_proc_pool_197,gem_mrblib_mruby_dir_glob_proc_syms_197,NULL,
  gem_mrblib_mruby_dir_glob_proc_lv_197,
  &gem_mrblib_mruby_dir_glob_proc_debug_197,
  66,3,4,0,0
};
static const mrb_irep *gem_mrblib_mruby_dir_glob_proc_reps_17[1] = {
  &gem_mrblib_mruby_dir_glob_proc_irep_197,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_syms_17, 1, (MRB_SYM(call), ), const);
static const mrb_code gem_mrblib_mruby_dir_glob_proc_iseq_17[10] = {
0x62,0x01,0x57,0x02,0x00,0x5e,0x01,0x00,0x37,0x01,};
static char gem_mrblib_mruby_dir_glob_proc_debug_lines_17[] = "\x00\xcc\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_glob_proc_debug_file_17 = {
0, 177, 3, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_glob_proc_debug_lines_17}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_glob_proc_debug_file_17_ = &gem_mrblib_mruby_dir_glob_proc_debug_file_17;
static mrb_irep_debug_info gem_mrblib_mruby_dir_glob_proc_debug_17 = {
10, 1, &gem_mrblib_mruby_dir_glob_proc_debug_file_17_};
static const mrb_irep gem_mrblib_mruby_dir_glob_proc_irep_17 = {
  1,3,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_glob_proc_iseq_17,
  NULL,gem_mrblib_mruby_dir_glob_proc_syms_17,gem_mrblib_mruby_dir_glob_proc_reps_17,
  NULL,					/* lv */
  &gem_mrblib_mruby_dir_glob_proc_debug_17,
  10,0,1,1,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_syms_214, 1, (MRB_IVSYM(matches), ), const);
static const mrb_code gem_mrblib_mruby_dir_glob_proc_iseq_214[24] = {
0x33,0x00,0x20,0x00,0x23,0x00,0x03,0x23,0x00,0x06,0x46,0x03,0x00,0x01,0x01,0x03,0x01,0x03,0x01,0x1a,
0x03,0x00,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_lv_214, 2, (MRB_SYM(matches), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_dir_glob_proc_debug_lines_214[] = "\x00\xd6\x01\x10\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_glob_proc_debug_file_214 = {
0, 177, 5, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_glob_proc_debug_lines_214}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_glob_proc_debug_file_214_ = &gem_mrblib_mruby_dir_glob_proc_debug_file_214;
static mrb_irep_debug_info gem_mrblib_mruby_dir_glob_proc_debug_214 = {
24, 1, &gem_mrblib_mruby_dir_glob_proc_debug_file_214_};
static const mrb_irep gem_mrblib_mruby_dir_glob_proc_irep_214 = {
  3,4,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_glob_proc_iseq_214,
  NULL,gem_mrblib_mruby_dir_glob_proc_syms_214,NULL,
  gem_mrblib_mruby_dir_glob_proc_lv_214,
  &gem_mrblib_mruby_dir_glob_proc_debug_214,
  24,0,1,0,0
};
static const mrb_irep *gem_mrblib_mruby_dir_glob_proc_reps_18[1] = {
  &gem_mrblib_mruby_dir_glob_proc_irep_214,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_syms_18, 3, (MRB_SYM(matches), MRB_SYM(attr_reader), MRB_SYM(initialize), ), const);
static const mrb_code gem_mrblib_mruby_dir_glob_proc_iseq_18[19] = {
0x12,0x01,0x10,0x02,0x00,0x2d,0x01,0x01,0x01,0x62,0x01,0x57,0x02,0x00,0x5e,0x01,0x02,0x37,0x01,};
static char gem_mrblib_mruby_dir_glob_proc_debug_lines_18[] = "\x00\xd4\x01\x09\x02";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_glob_proc_debug_file_18 = {
0, 177, 5, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_glob_proc_debug_lines_18}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_glob_proc_debug_file_18_ = &gem_mrblib_mruby_dir_glob_proc_debug_file_18;
static mrb_irep_debug_info gem_mrblib_mruby_dir_glob_proc_debug_18 = {
19, 1, &gem_mrblib_mruby_dir_glob_proc_debug_file_18_};
static const mrb_irep gem_mrblib_mruby_dir_glob_proc_irep_18 = {
  1,4,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_glob_proc_iseq_18,
  NULL,gem_mrblib_mruby_dir_glob_proc_syms_18,gem_mrblib_mruby_dir_glob_proc_reps_18,
  NULL,					/* lv */
  &gem_mrblib_mruby_dir_glob_proc_debug_18,
  19,0,3,1,0
};
static const mrb_pool_value gem_mrblib_mruby_dir_glob_proc_pool_19[1] = {
{IREP_TT_STR|(2<<2), {"\x2f\x2b"}},
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_syms_19, 14, (MRB_SYM(dup), MRB_SYM(Regexp), MRB_SYM(compile), MRB_SYM(match), MRB_SYM(begin), MRB_SYM(end), MRB_SYM(slice), MRB_OPSYM(lshift), MRB_OPSYM(aref), MRB_SYM(post_match), MRB_SYM_Q(empty), MRB_OPSYM(not), MRB_SYM(last), MRB_SYM(pop), ), const);
static const mrb_code gem_mrblib_mruby_dir_glob_proc_iseq_19[231] = {
0x33,0x04,0x00,0x00,0x46,0x0a,0x00,0x01,0x03,0x0a,0x11,0x04,0x01,0x0a,0x01,0x2d,0x0a,0x00,0x00,0x01,
0x05,0x0a,0x01,0x0a,0x05,0x5a,0x0b,0x1f,0x0b,0x01,0x50,0x0c,0x00,0x2d,0x0b,0x02,0x01,0x2d,0x0a,0x03,
0x01,0x01,0x06,0x0a,0x25,0x0a,0x00,0x61,0x01,0x0a,0x06,0x06,0x0b,0x2d,0x0a,0x04,0x01,0x01,0x0b,0x06,
0x06,0x0c,0x2d,0x0b,0x05,0x01,0x01,0x07,0x0a,0x01,0x08,0x0b,0x01,0x0a,0x03,0x01,0x0b,0x05,0x06,0x0c,
0x01,0x0d,0x08,0x3e,0x0d,0x01,0x2d,0x0b,0x06,0x02,0x2d,0x0a,0x07,0x01,0x01,0x0a,0x03,0x01,0x0b,0x05,
0x01,0x0c,0x07,0x01,0x0d,0x08,0x01,0x0e,0x07,0x3d,0x0d,0x2d,0x0b,0x06,0x02,0x2d,0x0a,0x07,0x01,0x01,
0x0a,0x05,0x01,0x0b,0x08,0x05,0x0c,0x58,0x0b,0x2d,0x0a,0x08,0x01,0x01,0x05,0x0a,0x01,0x0a,0x06,0x01,
0x04,0x0a,0x23,0xff,0x85,0x01,0x0a,0x04,0x25,0x0a,0x00,0x11,0x01,0x0a,0x03,0x01,0x0b,0x04,0x2d,0x0b,
0x09,0x00,0x2d,0x0a,0x07,0x01,0x23,0x00,0x0a,0x01,0x0a,0x03,0x01,0x0b,0x01,0x2d,0x0a,0x07,0x01,0x01,
0x0a,0x03,0x2d,0x0a,0x0a,0x00,0x2d,0x0a,0x0b,0x00,0x25,0x0a,0x00,0x23,0x01,0x0a,0x03,0x2d,0x0a,0x0c,
0x00,0x01,0x09,0x0a,0x25,0x0a,0x00,0x07,0x01,0x0a,0x09,0x2d,0x0a,0x0a,0x00,0x25,0x0a,0x00,0x0a,0x01,
0x0a,0x03,0x2d,0x0a,0x0d,0x00,0x23,0xff,0xdd,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_lv_19, 9, (MRB_SYM(str), MRB_OPSYM(and), MRB_SYM(ret), MRB_SYM(last_match), MRB_SYM(str2), MRB_SYM(match), MRB_SYM(cur_start), MRB_SYM(cur_end), MRB_SYM(s), ), const);
static char gem_mrblib_mruby_dir_glob_proc_debug_lines_19[] = "\x00\xdb\x01\x04\x01\x06\x01\x02\x02\x0a\x01\x18\x07\x02\xfa\xff\xff\xff\x0f\x18\x01\x16\x01\x19\x02\x11\x02\x09\x03\x05\x01\x11\x02\x0c\x04\x0d\x02\x02\xff\xff\xff\xff\x0f\x17\x01\x0c\x04";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_glob_proc_debug_file_19 = {
0, 177, 47, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_glob_proc_debug_lines_19}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_glob_proc_debug_file_19_ = &gem_mrblib_mruby_dir_glob_proc_debug_file_19;
static mrb_irep_debug_info gem_mrblib_mruby_dir_glob_proc_debug_19 = {
231, 1, &gem_mrblib_mruby_dir_glob_proc_debug_file_19_};
static const mrb_irep gem_mrblib_mruby_dir_glob_proc_irep_19 = {
  10,16,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_glob_proc_iseq_19,
  gem_mrblib_mruby_dir_glob_proc_pool_19,gem_mrblib_mruby_dir_glob_proc_syms_19,NULL,
  gem_mrblib_mruby_dir_glob_proc_lv_19,
  &gem_mrblib_mruby_dir_glob_proc_debug_19,
  231,1,14,0,0
};
static const mrb_pool_value gem_mrblib_mruby_dir_glob_proc_pool_20[3] = {
{IREP_TT_STR|(16<<2), {"\x5e\x5b\x61\x2d\x7a\x41\x2d\x5a\x30\x2d\x39\x2e\x5f\x5d\x2b\x24"}},
{IREP_TT_STR|(2<<2), {"\x2a\x2a"}},
{IREP_TT_STR|(12<<2), {"\x5e\x5b\x5e\x5c\x2a\x5c\x3f\x5c\x5d\x5d\x2b\x24"}},
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_syms_20, 20, (MRB_SYM(path_split), MRB_SYM(getbyte), MRB_SYM(DirectoriesOnly), MRB_SYM(new), MRB_SYM(pop), MRB_SYM(Regexp), MRB_SYM(compile), MRB_SYM(match), MRB_SYM(ConstantEntry), MRB_SYM(EntryMatch), MRB_SYM_Q(empty), MRB_SYM_E(separator), MRB_SYM(StartRecursiveDirectories), MRB_SYM(RecursiveDirectories), MRB_OPSYM(aref), MRB_OPSYM(lshift), MRB_SYM(ConstantDirectory), MRB_OPSYM(not), MRB_SYM(DirectoryMatch), MRB_SYM(RootDirectory), ), const);
static const mrb_code gem_mrblib_mruby_dir_glob_proc_iseq_20[436] = {
0x33,0x04,0x20,0x00,0x23,0x00,0x03,0x23,0x00,0x02,0x06,0x02,0x12,0x0a,0x01,0x0b,0x01,0x2d,0x0a,0x00,
0x01,0x01,0x04,0x0a,0x01,0x0a,0x01,0x05,0x0b,0x2d,0x0a,0x01,0x01,0x03,0x0b,0x2f,0x41,0x0a,0x25,0x0a,
0x00,0x12,0x1d,0x0a,0x02,0x11,0x0b,0x01,0x0c,0x02,0x2d,0x0a,0x03,0x02,0x01,0x05,0x0a,0x23,0x00,0x48,
0x01,0x0a,0x04,0x2d,0x0a,0x04,0x00,0x01,0x06,0x0a,0x5a,0x0a,0x1f,0x0a,0x05,0x50,0x0b,0x00,0x2d,0x0a,
0x06,0x01,0x01,0x0b,0x06,0x2d,0x0a,0x07,0x01,0x25,0x0a,0x00,0x15,0x1d,0x0a,0x08,0x11,0x0b,0x01,0x0c,
0x02,0x01,0x0d,0x06,0x2d,0x0a,0x03,0x03,0x01,0x05,0x0a,0x23,0x00,0x12,0x1d,0x0a,0x09,0x11,0x0b,0x01,
0x0c,0x02,0x01,0x0d,0x06,0x2d,0x0a,0x03,0x03,0x01,0x05,0x0a,0x01,0x0a,0x04,0x2d,0x0a,0x0a,0x00,0x24,
0x0a,0x01,0x01,0x01,0x0a,0x04,0x2d,0x0a,0x04,0x00,0x01,0x0b,0x05,0x01,0x0c,0x0a,0x2d,0x0b,0x0b,0x01,
0x01,0x0a,0x04,0x2d,0x0a,0x04,0x00,0x01,0x07,0x0a,0x01,0x0a,0x07,0x50,0x0b,0x01,0x41,0x0a,0x25,0x0a,
0x00,0x31,0x01,0x0a,0x04,0x2d,0x0a,0x0a,0x00,0x25,0x0a,0x00,0x13,0x1d,0x0a,0x0c,0x01,0x0b,0x05,0x01,
0x0c,0x02,0x2d,0x0a,0x03,0x02,0x01,0x05,0x0a,0x23,0x00,0x10,0x1d,0x0a,0x0d,0x01,0x0b,0x05,0x01,0x0c,
0x02,0x2d,0x0a,0x03,0x02,0x01,0x05,0x0a,0x23,0x00,0xa6,0x5a,0x0a,0x1f,0x0a,0x05,0x50,0x0b,0x02,0x2d,
0x0a,0x06,0x01,0x01,0x0b,0x07,0x2d,0x0a,0x07,0x01,0x25,0x0a,0x00,0x6d,0x01,0x0a,0x04,0x04,0x0b,0x02,
0x2d,0x0a,0x0e,0x01,0x25,0x0a,0x00,0x1a,0x5a,0x0a,0x1f,0x0a,0x05,0x50,0x0b,0x02,0x2d,0x0a,0x06,0x01,
0x01,0x0b,0x04,0x04,0x0c,0x02,0x2d,0x0b,0x0e,0x01,0x2d,0x0a,0x07,0x01,0x25,0x0a,0x00,0x2b,0x01,0x0a,
0x04,0x2d,0x0a,0x04,0x00,0x01,0x08,0x0a,0x01,0x0a,0x04,0x2d,0x0a,0x04,0x00,0x01,0x09,0x0a,0x01,0x0a,
0x09,0x01,0x0b,0x08,0x2d,0x0a,0x0f,0x01,0x01,0x0b,0x07,0x2d,0x0a,0x0f,0x01,0x01,0x07,0x0a,0x23,0xff,
0xa9,0x1d,0x0a,0x10,0x01,0x0b,0x05,0x01,0x0c,0x02,0x01,0x0d,0x07,0x2d,0x0a,0x03,0x03,0x01,0x05,0x0a,
0x23,0x00,0x22,0x01,0x0a,0x07,0x2d,0x0a,0x0a,0x00,0x2d,0x0a,0x11,0x00,0x25,0x0a,0x00,0x13,0x1d,0x0a,
0x12,0x01,0x0b,0x05,0x01,0x0c,0x02,0x01,0x0d,0x07,0x2d,0x0a,0x03,0x03,0x01,0x05,0x0a,0x23,0xfe,0xf4,
0x01,0x0a,0x01,0x06,0x0b,0x2d,0x0a,0x01,0x01,0x03,0x0b,0x2f,0x41,0x0a,0x25,0x0a,0x00,0x10,0x1d,0x0a,
0x13,0x01,0x0b,0x05,0x01,0x0c,0x02,0x2d,0x0a,0x03,0x02,0x01,0x05,0x0a,0x37,0x05,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_lv_20, 9, (MRB_SYM(glob), MRB_SYM(flags), MRB_OPSYM(and), MRB_SYM(parts), MRB_SYM(last), MRB_SYM(file), MRB_SYM(dir), MRB_SYM(next_sep), MRB_SYM(next_sect), ), const);
static char gem_mrblib_mruby_dir_glob_proc_debug_lines_20[] = "\x00\xfa\x01\x0c\x01\x0c\x02\x10\x01\x12\x06\x02\xfc\xff\xff\xff\x0f\x0a\x01\x15\x01\x15\x02\x14\x04\x09\x14\x02\xed\xff\xff\xff\x0f\x11\x01\x0a\x02\x0a\x04\x02\xfd\xff\xff\xff\x0f\x09\x01\x13\x02\x13\x0c\x02\xf6\xff\xff\xff\x0f\x15\x08\x02\xf9\xff\xff\xff\x0f\x2a\x04\x02\xfd\xff\xff\xff\x0f\x0a\x01\x0a\x02\x17\x03\x14\x02\x02\xff\xff\xff\xff\x0f\x0d\x01\x18\x04\x10\x01\x12\x03";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_glob_proc_debug_file_20 = {
0, 177, 95, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_glob_proc_debug_lines_20}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_glob_proc_debug_file_20_ = &gem_mrblib_mruby_dir_glob_proc_debug_file_20;
static mrb_irep_debug_info gem_mrblib_mruby_dir_glob_proc_debug_20 = {
436, 1, &gem_mrblib_mruby_dir_glob_proc_debug_file_20_};
static const mrb_irep gem_mrblib_mruby_dir_glob_proc_irep_20 = {
  10,15,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_glob_proc_iseq_20,
  gem_mrblib_mruby_dir_glob_proc_pool_20,gem_mrblib_mruby_dir_glob_proc_syms_20,NULL,
  gem_mrblib_mruby_dir_glob_proc_lv_20,
  &gem_mrblib_mruby_dir_glob_proc_debug_20,
  436,3,20,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_syms_21, 4, (MRB_SYM(Environment), MRB_SYM(new), MRB_SYM(call), MRB_SYM(matches), ), const);
static const mrb_code gem_mrblib_mruby_dir_glob_proc_iseq_21[50] = {
0x33,0x04,0x20,0x00,0x23,0x00,0x03,0x23,0x00,0x06,0x46,0x05,0x00,0x01,0x02,0x05,0x1d,0x05,0x00,0x01,
0x06,0x02,0x2d,0x05,0x01,0x01,0x01,0x04,0x05,0x01,0x05,0x01,0x01,0x06,0x04,0x11,0x07,0x2d,0x05,0x02,
0x02,0x01,0x05,0x04,0x2d,0x05,0x03,0x00,0x37,0x05,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_lv_21, 4, (MRB_SYM(node), MRB_SYM(matches), MRB_OPSYM(and), MRB_SYM(env), ), const);
static char gem_mrblib_mruby_dir_glob_proc_debug_lines_21[] = "\x00\xa7\x02\x10\x01\x0d\x01\x0c\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_glob_proc_debug_file_21 = {
0, 177, 9, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_glob_proc_debug_lines_21}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_glob_proc_debug_file_21_ = &gem_mrblib_mruby_dir_glob_proc_debug_file_21;
static mrb_irep_debug_info gem_mrblib_mruby_dir_glob_proc_debug_21 = {
50, 1, &gem_mrblib_mruby_dir_glob_proc_debug_file_21_};
static const mrb_irep gem_mrblib_mruby_dir_glob_proc_irep_21 = {
  5,9,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_glob_proc_iseq_21,
  NULL,gem_mrblib_mruby_dir_glob_proc_syms_21,NULL,
  gem_mrblib_mruby_dir_glob_proc_lv_21,
  &gem_mrblib_mruby_dir_glob_proc_debug_21,
  50,0,4,0,0
};
static const mrb_pool_value gem_mrblib_mruby_dir_glob_proc_pool_279[1] = {
{IREP_TT_STR|(0<<2), {""}},
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_syms_279, 3, (MRB_SYM(File), MRB_SYM_Q(exists), MRB_OPSYM(lshift), ), const);
static const mrb_code gem_mrblib_mruby_dir_glob_proc_iseq_279[63] = {
0x33,0x04,0x00,0x00,0x50,0x04,0x00,0x21,0x05,0x07,0x00,0x51,0x04,0x50,0x05,0x00,0x51,0x04,0x01,0x05,
0x01,0x51,0x04,0x50,0x05,0x00,0x51,0x04,0x01,0x03,0x04,0x1d,0x04,0x00,0x01,0x05,0x03,0x2d,0x04,0x01,
0x01,0x25,0x04,0x00,0x0e,0x21,0x04,0x03,0x00,0x01,0x05,0x03,0x2d,0x04,0x02,0x01,0x23,0x00,0x02,0x11,
0x04,0x37,0x04,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_lv_279, 3, (MRB_SYM(s), MRB_OPSYM(and), MRB_SYM(path), ), const);
static char gem_mrblib_mruby_dir_glob_proc_debug_lines_279[] = "\x00\xbb\x02\x04\x01\x1b\x01\x0c\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_glob_proc_debug_file_279 = {
0, 177, 9, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_glob_proc_debug_lines_279}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_glob_proc_debug_file_279_ = &gem_mrblib_mruby_dir_glob_proc_debug_file_279;
static mrb_irep_debug_info gem_mrblib_mruby_dir_glob_proc_debug_279 = {
63, 1, &gem_mrblib_mruby_dir_glob_proc_debug_file_279_};
static const mrb_irep gem_mrblib_mruby_dir_glob_proc_irep_279 = {
  4,7,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_glob_proc_iseq_279,
  gem_mrblib_mruby_dir_glob_proc_pool_279,gem_mrblib_mruby_dir_glob_proc_syms_279,NULL,
  gem_mrblib_mruby_dir_glob_proc_lv_279,
  &gem_mrblib_mruby_dir_glob_proc_debug_279,
  63,1,3,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_syms_280, 1, (MRB_SYM(run), ), const);
static const mrb_code gem_mrblib_mruby_dir_glob_proc_iseq_280[19] = {
0x33,0x04,0x00,0x00,0x12,0x03,0x01,0x04,0x01,0x21,0x05,0x03,0x00,0x2d,0x03,0x00,0x02,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_lv_280, 2, (MRB_SYM(node), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_dir_glob_proc_debug_lines_280[] = "\x00\xd4\x02\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_glob_proc_debug_file_280 = {
0, 177, 5, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_glob_proc_debug_lines_280}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_glob_proc_debug_file_280_ = &gem_mrblib_mruby_dir_glob_proc_debug_file_280;
static mrb_irep_debug_info gem_mrblib_mruby_dir_glob_proc_debug_280 = {
19, 1, &gem_mrblib_mruby_dir_glob_proc_debug_file_280_};
static const mrb_irep gem_mrblib_mruby_dir_glob_proc_irep_280 = {
  3,7,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_glob_proc_iseq_280,
  NULL,gem_mrblib_mruby_dir_glob_proc_syms_280,NULL,
  gem_mrblib_mruby_dir_glob_proc_lv_280,
  &gem_mrblib_mruby_dir_glob_proc_debug_280,
  19,0,1,0,0
};
static const mrb_irep *gem_mrblib_mruby_dir_glob_proc_reps_22[2] = {
  &gem_mrblib_mruby_dir_glob_proc_irep_279,
  &gem_mrblib_mruby_dir_glob_proc_irep_280,
};
static const mrb_pool_value gem_mrblib_mruby_dir_glob_proc_pool_22[3] = {
{IREP_TT_STR|(56<<2), {"\x5e\x28\x5b\x61\x2d\x7a\x41\x2d\x5a\x30\x2d\x39\x5f\x2e\x2f\x5c\x73\x5d\x2a\x5b\x61\x2d\x7a\x41\x2d\x5a\x30\x2d\x39\x5f\x2e\x5d\x29\x28\x3f\x3a\x5c\x7b\x28\x5b\x5e\x7b\x7d\x2f\x5c\x2a\x5c\x3f\x5d\x2a\x29\x5c\x7d\x29\x3f\x24"}},
{IREP_TT_STR|(1<<2), {"\x2c"}},
{IREP_TT_STR|(1<<2), {"\x7b"}},
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_syms_22, 15, (MRB_SYM(Regexp), MRB_SYM(compile), MRB_SYM(match), MRB_OPSYM(aref), MRB_SYM(split), MRB_SYM(each), MRB_SYM(getbyte), MRB_SYM(File), MRB_SYM_Q(exists), MRB_OPSYM(lshift), MRB_SYM_Q(include), MRB_SYM(gc), MRB_OPSYM(aset), MRB_SYM(single_compile), MRB_SYM(run), ), const);
static const mrb_code gem_mrblib_mruby_dir_glob_proc_iseq_22[307] = {
0x33,0x08,0x20,0x00,0x23,0x00,0x03,0x23,0x00,0x06,0x46,0x0b,0x00,0x01,0x03,0x0b,0x01,0x0b,0x02,0x06,
0x0c,0x41,0x0b,0x25,0x0b,0x00,0x16,0x5a,0x0b,0x1f,0x0b,0x00,0x50,0x0c,0x00,0x2d,0x0b,0x01,0x01,0x01,
0x0c,0x01,0x2d,0x0b,0x02,0x01,0x01,0x05,0x0b,0x25,0x0b,0x00,0x74,0x01,0x0b,0x05,0x08,0x0c,0x2d,0x0b,
0x03,0x01,0x01,0x06,0x0b,0x25,0x0b,0x00,0x4a,0x01,0x0b,0x05,0x07,0x0c,0x2d,0x0b,0x03,0x01,0x01,0x07,
0x0b,0x01,0x0b,0x06,0x50,0x0c,0x01,0x2d,0x0b,0x04,0x01,0x56,0x0c,0x00,0x2e,0x0b,0x05,0x00,0x01,0x0b,
0x06,0x05,0x0c,0x2d,0x0b,0x06,0x01,0x03,0x0c,0x2c,0x41,0x0b,0x25,0x0b,0x00,0x18,0x1d,0x0b,0x07,0x01,
0x0c,0x07,0x2d,0x0b,0x08,0x01,0x25,0x0b,0x00,0x0a,0x01,0x0b,0x03,0x01,0x0c,0x07,0x2d,0x0b,0x09,0x01,
0x23,0x00,0x18,0x1d,0x0b,0x07,0x01,0x0c,0x01,0x2d,0x0b,0x08,0x01,0x25,0x0b,0x00,0x0a,0x01,0x0b,0x03,
0x01,0x0c,0x01,0x2d,0x0b,0x09,0x01,0x37,0x03,0x11,0x08,0x01,0x0b,0x01,0x50,0x0c,0x02,0x2d,0x0b,0x0a,
0x01,0x25,0x0b,0x00,0x36,0x12,0x0b,0x01,0x0c,0x01,0x01,0x0d,0x02,0x2d,0x0b,0x01,0x02,0x01,0x09,0x0b,
0x01,0x0b,0x08,0x25,0x0b,0x00,0x13,0x01,0x0b,0x09,0x12,0x0c,0x2d,0x0c,0x0b,0x00,0x01,0x0d,0x08,0x01,
0x0e,0x0b,0x2d,0x0c,0x0c,0x02,0x01,0x0b,0x09,0x56,0x0c,0x01,0x2e,0x0b,0x05,0x00,0x23,0x00,0x42,0x12,
0x0b,0x01,0x0c,0x01,0x01,0x0d,0x02,0x2d,0x0b,0x0d,0x02,0x01,0x0a,0x0b,0x25,0x0b,0x00,0x2c,0x01,0x0b,
0x08,0x25,0x0b,0x00,0x16,0x01,0x0b,0x0a,0x46,0x0b,0x01,0x12,0x0c,0x2d,0x0c,0x0b,0x00,0x01,0x0d,0x08,
0x01,0x0e,0x0b,0x2d,0x0c,0x0c,0x02,0x12,0x0b,0x01,0x0c,0x0a,0x01,0x0d,0x03,0x2d,0x0b,0x0e,0x02,0x23,
0x00,0x03,0x01,0x0b,0x03,0x37,0x0b,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_lv_22, 10, (MRB_SYM(pattern), MRB_SYM(flags), MRB_SYM(matches), MRB_OPSYM(and), MRB_SYM(m), MRB_SYM(braces), MRB_SYM(stem), MRB_SYM(ec_key), MRB_SYM(patterns), MRB_SYM(node), ), const);
static char gem_mrblib_mruby_dir_glob_proc_debug_lines_22[] = "\x00\xad\x02\x10\x05\x09\x01\x1a\x17\x02\xee\xff\xff\xff\x0f\x0e\x0c\x02\xf5\xff\xff\xff\x0f\x0c\x02\x11\x08\x10\x01\x1b\x03\x1a\x03\x02\x03\x02\x02\x0c\x05\x02\xfc\xff\xff\xff\x0f\x0f\x02\x1a\x02\x0b\x08\x02\xfb\xff\xff\xff\x0f\x11\x03\x02\xfe\xff\xff\xff\x0f\x1d\x02\x0d\x02";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_glob_proc_debug_file_22 = {
0, 177, 69, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_glob_proc_debug_lines_22}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_glob_proc_debug_file_22_ = &gem_mrblib_mruby_dir_glob_proc_debug_file_22;
static mrb_irep_debug_info gem_mrblib_mruby_dir_glob_proc_debug_22 = {
307, 1, &gem_mrblib_mruby_dir_glob_proc_debug_file_22_};
static const mrb_irep gem_mrblib_mruby_dir_glob_proc_irep_22 = {
  11,16,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_glob_proc_iseq_22,
  gem_mrblib_mruby_dir_glob_proc_pool_22,gem_mrblib_mruby_dir_glob_proc_syms_22,gem_mrblib_mruby_dir_glob_proc_reps_22,
  gem_mrblib_mruby_dir_glob_proc_lv_22,
  &gem_mrblib_mruby_dir_glob_proc_debug_22,
  307,3,15,2,0
};
static const mrb_pool_value gem_mrblib_mruby_dir_glob_proc_pool_23[5] = {
{IREP_TT_STR|(1<<2), {"\x7b"}},
{IREP_TT_STR|(1<<2), {"\x7d"}},
{IREP_TT_STR|(1<<2), {"\x5c"}},
{IREP_TT_STR|(1<<2), {"\x2c"}},
{IREP_TT_STR|(0<<2), {""}},
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_syms_23, 10, (MRB_SYM(FNM_NOESCAPE), MRB_SYM(File), MRB_OPSYM(and), MRB_SYM(index), MRB_SYM(size), MRB_OPSYM(aref), MRB_OPSYM(not), MRB_SYM(compile), MRB_SYM(single_compile), MRB_OPSYM(lshift), ), const);
static const mrb_code gem_mrblib_mruby_dir_glob_proc_iseq_23[511] = {
0x33,0x04,0x40,0x00,0x23,0x00,0x06,0x23,0x00,0x05,0x23,0x00,0x08,0x06,0x02,0x46,0x11,0x00,0x01,0x03,
0x11,0x01,0x11,0x02,0x1d,0x12,0x01,0x1f,0x12,0x00,0x2d,0x11,0x02,0x01,0x06,0x12,0x41,0x11,0x01,0x05,
0x11,0x11,0x06,0x11,0x07,0x01,0x11,0x01,0x50,0x12,0x00,0x2d,0x11,0x03,0x01,0x01,0x08,0x11,0x01,0x11,
0x08,0x25,0x11,0x00,0x7e,0x06,0x09,0x01,0x11,0x08,0x01,0x12,0x01,0x2d,0x12,0x04,0x00,0x42,0x11,0x25,
0x11,0x00,0x6c,0x01,0x11,0x01,0x01,0x12,0x08,0x2d,0x11,0x05,0x01,0x01,0x0a,0x11,0x01,0x11,0x0a,0x50,
0x12,0x00,0x41,0x11,0x25,0x11,0x00,0x14,0x01,0x11,0x09,0x06,0x12,0x41,0x11,0x25,0x11,0x00,0x06,0x01,
0x11,0x08,0x01,0x07,0x11,0x3c,0x09,0x01,0x01,0x11,0x0a,0x50,0x12,0x01,0x41,0x11,0x25,0x11,0x00,0x03,
0x3e,0x09,0x01,0x01,0x11,0x09,0x06,0x12,0x41,0x11,0x25,0x11,0x00,0x09,0x01,0x11,0x08,0x01,0x06,0x11,
0x27,0x00,0x1c,0x01,0x11,0x0a,0x50,0x12,0x02,0x41,0x11,0x25,0x11,0x00,0x03,0x01,0x11,0x05,0x25,0x11,
0x00,0x03,0x3c,0x08,0x01,0x3c,0x08,0x01,0x23,0xff,0x84,0x01,0x11,0x07,0x25,0x11,0x00,0x03,0x01,0x11,
0x06,0x25,0x11,0x01,0x13,0x01,0x11,0x07,0x01,0x0b,0x11,0x01,0x11,0x01,0x06,0x12,0x01,0x13,0x07,0x59,
0x12,0x2d,0x11,0x05,0x01,0x01,0x0c,0x11,0x01,0x11,0x01,0x01,0x12,0x06,0x3c,0x12,0x01,0x05,0x13,0x58,
0x12,0x2d,0x11,0x05,0x01,0x01,0x0d,0x11,0x01,0x11,0x0b,0x01,0x12,0x06,0x42,0x11,0x25,0x11,0x00,0xd9,
0x06,0x09,0x3c,0x0b,0x01,0x01,0x11,0x0b,0x01,0x0e,0x11,0x01,0x11,0x0b,0x01,0x12,0x06,0x42,0x11,0x25,
0x11,0x00,0x1e,0x01,0x11,0x01,0x01,0x12,0x0b,0x2d,0x11,0x05,0x01,0x50,0x12,0x03,0x41,0x11,0x25,0x11,
0x00,0x07,0x01,0x11,0x09,0x06,0x12,0x41,0x11,0x2d,0x11,0x06,0x00,0x25,0x11,0x00,0x5e,0x01,0x11,0x01,
0x01,0x12,0x0b,0x2d,0x11,0x05,0x01,0x50,0x12,0x00,0x41,0x11,0x25,0x11,0x00,0x03,0x3c,0x09,0x01,0x01,
0x11,0x01,0x01,0x12,0x0b,0x2d,0x11,0x05,0x01,0x50,0x12,0x01,0x41,0x11,0x25,0x11,0x00,0x03,0x3e,0x09,
0x01,0x01,0x11,0x01,0x01,0x12,0x0b,0x2d,0x11,0x05,0x01,0x50,0x12,0x02,0x41,0x11,0x25,0x11,0x00,0x03,
0x01,0x11,0x05,0x25,0x11,0x00,0x12,0x3c,0x0b,0x01,0x01,0x11,0x0b,0x01,0x12,0x06,0x41,0x11,0x25,0x11,
0x00,0x03,0x27,0x00,0x06,0x3c,0x0b,0x01,0x23,0xff,0x74,0x50,0x11,0x04,0x01,0x12,0x0c,0x51,0x11,0x50,
0x12,0x04,0x51,0x11,0x01,0x12,0x01,0x01,0x13,0x0e,0x01,0x14,0x0b,0x59,0x13,0x2d,0x12,0x05,0x01,0x51,
0x11,0x50,0x12,0x04,0x51,0x11,0x01,0x12,0x0d,0x51,0x11,0x50,0x12,0x04,0x51,0x11,0x01,0x0f,0x11,0x12,
0x11,0x01,0x12,0x0f,0x01,0x13,0x02,0x01,0x14,0x03,0x2d,0x11,0x07,0x03,0x23,0xff,0x1b,0x23,0x00,0x1d,
0x12,0x11,0x01,0x12,0x01,0x01,0x13,0x02,0x2d,0x11,0x08,0x02,0x01,0x10,0x11,0x25,0x11,0x00,0x0a,0x01,
0x11,0x03,0x01,0x12,0x10,0x2d,0x11,0x09,0x01,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_lv_23, 16, (MRB_SYM(pattern), MRB_SYM(flags), MRB_SYM(patterns), MRB_OPSYM(and), MRB_SYM(escape), MRB_SYM(rbrace), MRB_SYM(lbrace), MRB_SYM(i), MRB_SYM(nest), MRB_SYM(char), MRB_SYM(pos), MRB_SYM(front), MRB_SYM(back), MRB_SYM(last), MRB_SYM(brace_pattern), MRB_SYM(node), ), const);
static char gem_mrblib_mruby_dir_glob_proc_debug_lines_23[] = "\x00\xe0\x02\x15\x01\x14\x02\x02\x01\x02\x03\x0d\x03\x05\x18\x02\xe9\xff\xff\xff\x0f\x02\x02\x0e\x15\x02\xec\xff\xff\xff\x0f\x0d\x02\x0a\x02\x02\xff\xff\xff\xff\x0f\x11\x01\x03\x03\x0a\x01\x05\x03\x09\x02\x02\xff\xff\xff\xff\x0f\x06\x01\x01\x07\x02\xfc\xff\xff\xff\x0f\x11\x01\x05\x03\x06\x06\x0c\x18\x02\xe9\xff\xff\xff\x0f\x06\x01\x11\x01\x14\x02\x0a\x13\x02\xee\xff\xff\xff\x0f\x02\x01\x03\x01\x06\x02\x2c\x09\x02\xf8\xff\xff\xff\x0f\x16\x01\x16\x02\x18\x02\x02\xff\xff\xff\xff\x0f\x03\x01\x0d\x03\x08\x03\x30\x02\x13\x0a\x02\xff\xff\xff\xff\x0f\x11\x01\x0c\x04";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_glob_proc_debug_file_23 = {
0, 177, 141, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_glob_proc_debug_lines_23}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_glob_proc_debug_file_23_ = &gem_mrblib_mruby_dir_glob_proc_debug_file_23;
static mrb_irep_debug_info gem_mrblib_mruby_dir_glob_proc_debug_23 = {
511, 1, &gem_mrblib_mruby_dir_glob_proc_debug_file_23_};
static const mrb_irep gem_mrblib_mruby_dir_glob_proc_irep_23 = {
  17,22,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_glob_proc_iseq_23,
  gem_mrblib_mruby_dir_glob_proc_pool_23,gem_mrblib_mruby_dir_glob_proc_syms_23,NULL,
  gem_mrblib_mruby_dir_glob_proc_lv_23,
  &gem_mrblib_mruby_dir_glob_proc_debug_23,
  511,5,10,0,0
};
static const mrb_irep *gem_mrblib_mruby_dir_glob_proc_reps_7[16] = {
  &gem_mrblib_mruby_dir_glob_proc_irep_8,
  &gem_mrblib_mruby_dir_glob_proc_irep_9,
  &gem_mrblib_mruby_dir_glob_proc_irep_10,
  &gem_mrblib_mruby_dir_glob_proc_irep_11,
  &gem_mrblib_mruby_dir_glob_proc_irep_12,
  &gem_mrblib_mruby_dir_glob_proc_irep_13,
  &gem_mrblib_mruby_dir_glob_proc_irep_14,
  &gem_mrblib_mruby_dir_glob_proc_irep_15,
  &gem_mrblib_mruby_dir_glob_proc_irep_16,
  &gem_mrblib_mruby_dir_glob_proc_irep_17,
  &gem_mrblib_mruby_dir_glob_proc_irep_18,
  &gem_mrblib_mruby_dir_glob_proc_irep_19,
  &gem_mrblib_mruby_dir_glob_proc_irep_20,
  &gem_mrblib_mruby_dir_glob_proc_irep_21,
  &gem_mrblib_mruby_dir_glob_proc_irep_22,
  &gem_mrblib_mruby_dir_glob_proc_irep_23,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_syms_7, 16, (MRB_SYM(Node), MRB_SYM(ConstantDirectory), MRB_SYM(ConstantEntry), MRB_SYM(RootDirectory), MRB_SYM(RecursiveDirectories), MRB_SYM(StartRecursiveDirectories), MRB_SYM(Match), MRB_SYM(DirectoryMatch), MRB_SYM(EntryMatch), MRB_SYM(DirectoriesOnly), MRB_SYM(Environment), MRB_SYM(path_split), MRB_SYM(single_compile), MRB_SYM(run), MRB_SYM(glob), MRB_SYM(compile), ), const);
static const mrb_code gem_mrblib_mruby_dir_glob_proc_iseq_7[171] = {
0x11,0x01,0x11,0x02,0x5b,0x01,0x00,0x5d,0x01,0x00,0x11,0x01,0x1d,0x02,0x00,0x5b,0x01,0x01,0x5d,0x01,
0x01,0x11,0x01,0x1d,0x02,0x00,0x5b,0x01,0x02,0x5d,0x01,0x02,0x11,0x01,0x1d,0x02,0x00,0x5b,0x01,0x03,
0x5d,0x01,0x03,0x11,0x01,0x1d,0x02,0x00,0x5b,0x01,0x04,0x5d,0x01,0x04,0x11,0x01,0x1d,0x02,0x00,0x5b,
0x01,0x05,0x5d,0x01,0x05,0x11,0x01,0x1d,0x02,0x00,0x5b,0x01,0x06,0x5d,0x01,0x06,0x11,0x01,0x1d,0x02,
0x06,0x5b,0x01,0x07,0x5d,0x01,0x07,0x11,0x01,0x1d,0x02,0x06,0x5b,0x01,0x08,0x5d,0x01,0x08,0x11,0x01,
0x1d,0x02,0x00,0x5b,0x01,0x09,0x5d,0x01,0x09,0x11,0x01,0x11,0x02,0x5b,0x01,0x0a,0x5d,0x01,0x0a,0x12,
0x01,0x61,0x01,0x57,0x02,0x0b,0x5e,0x01,0x0b,0x12,0x01,0x61,0x01,0x57,0x02,0x0c,0x5e,0x01,0x0c,0x12,
0x01,0x61,0x01,0x57,0x02,0x0d,0x5e,0x01,0x0d,0x12,0x01,0x61,0x01,0x57,0x02,0x0e,0x5e,0x01,0x0e,0x12,
0x01,0x61,0x01,0x57,0x02,0x0f,0x5e,0x01,0x0f,0x37,0x01,};
static char gem_mrblib_mruby_dir_glob_proc_debug_lines_7[] = "\x00\x03\x0a\x17\x0b\x0f\x0b\x0f\x0b\x06\x0b\x26\x0b\x31\x0b\x0b\x0b\x18\x0b\x13\x0b\x08\x0a\x08\x0a\x1f\x0a\x2d\x0a\x06\x0a\x33";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_glob_proc_debug_file_7 = {
0, 177, 32, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_glob_proc_debug_lines_7}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_glob_proc_debug_file_7_ = &gem_mrblib_mruby_dir_glob_proc_debug_file_7;
static mrb_irep_debug_info gem_mrblib_mruby_dir_glob_proc_debug_7 = {
171, 1, &gem_mrblib_mruby_dir_glob_proc_debug_file_7_};
static const mrb_irep gem_mrblib_mruby_dir_glob_proc_irep_7 = {
  1,3,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_glob_proc_iseq_7,
  NULL,gem_mrblib_mruby_dir_glob_proc_syms_7,gem_mrblib_mruby_dir_glob_proc_reps_7,
  NULL,					/* lv */
  &gem_mrblib_mruby_dir_glob_proc_debug_7,
  171,0,16,16,0
};
static const mrb_irep *gem_mrblib_mruby_dir_glob_proc_reps_2[1] = {
  &gem_mrblib_mruby_dir_glob_proc_irep_7,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_syms_2, 1, (MRB_SYM(Glob), ), const);
static const mrb_code gem_mrblib_mruby_dir_glob_proc_iseq_2[10] = {
0x11,0x01,0x5c,0x01,0x00,0x5d,0x01,0x00,0x37,0x01,};
static char gem_mrblib_mruby_dir_glob_proc_debug_lines_2[] = "\x00\x02";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_glob_proc_debug_file_2 = {
0, 177, 2, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_glob_proc_debug_lines_2}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_glob_proc_debug_file_2_ = &gem_mrblib_mruby_dir_glob_proc_debug_file_2;
static mrb_irep_debug_info gem_mrblib_mruby_dir_glob_proc_debug_2 = {
10, 1, &gem_mrblib_mruby_dir_glob_proc_debug_file_2_};
static const mrb_irep gem_mrblib_mruby_dir_glob_proc_irep_2 = {
  1,2,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_glob_proc_iseq_2,
  NULL,gem_mrblib_mruby_dir_glob_proc_syms_2,gem_mrblib_mruby_dir_glob_proc_reps_2,
  NULL,					/* lv */
  &gem_mrblib_mruby_dir_glob_proc_debug_2,
  10,0,1,1,0
};
static const mrb_irep *gem_mrblib_mruby_dir_glob_proc_reps_0[2] = {
  &gem_mrblib_mruby_dir_glob_proc_irep_1,
  &gem_mrblib_mruby_dir_glob_proc_irep_2,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_glob_proc_syms_0, 1, (MRB_SYM(Dir), ), const);
static const mrb_code gem_mrblib_mruby_dir_glob_proc_iseq_0[23] = {
0x11,0x01,0x11,0x02,0x5b,0x01,0x00,0x5d,0x01,0x00,0x11,0x01,0x11,0x02,0x5b,0x01,0x00,0x5d,0x01,0x01,
0x37,0x01,0x68,};
static const mrb_irep gem_mrblib_mruby_dir_glob_proc_irep_0 = {
  1,3,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_glob_proc_iseq_0,
  NULL,gem_mrblib_mruby_dir_glob_proc_syms_0,gem_mrblib_mruby_dir_glob_proc_reps_0,
  NULL,					/* lv */
  NULL,					/* debug_info */
  23,0,1,2,0
};
static
const struct RProc gem_mrblib_mruby_dir_glob_proc[] = {{
NULL,NULL,MRB_TT_PROC,MRB_GC_RED,0,{&gem_mrblib_mruby_dir_glob_proc_irep_0},NULL,{NULL},
}};
static void
gem_mrblib_mruby_dir_glob_proc_init_syms(mrb_state *mrb)
{
  gem_mrblib_mruby_dir_glob_proc_debug_file_4.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir-glob/mrblib/dir.rb");
  gem_mrblib_mruby_dir_glob_proc_debug_file_3.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir-glob/mrblib/dir.rb");
  gem_mrblib_mruby_dir_glob_proc_debug_file_1.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir-glob/mrblib/dir.rb");
  gem_mrblib_mruby_dir_glob_proc_debug_file_24.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir-glob/mrblib/dir_glob.rb");
  gem_mrblib_mruby_dir_glob_proc_debug_file_25.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir-glob/mrblib/dir_glob.rb");
  gem_mrblib_mruby_dir_glob_proc_debug_file_26.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir-glob/mrblib/dir_glob.rb");
  gem_mrblib_mruby_dir_glob_proc_debug_file_8.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir-glob/mrblib/dir_glob.rb");
  gem_mrblib_mruby_dir_glob_proc_debug_file_49.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir-glob/mrblib/dir_glob.rb");
  gem_mrblib_mruby_dir_glob_proc_debug_file_50.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir-glob/mrblib/dir_glob.rb");
  gem_mrblib_mruby_dir_glob_proc_debug_file_9.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir-glob/mrblib/dir_glob.rb");
  gem_mrblib_mruby_dir_glob_proc_debug_file_69.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir-glob/mrblib/dir_glob.rb");
  gem_mrblib_mruby_dir_glob_proc_debug_file_70.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir-glob/mrblib/dir_glob.rb");
  gem_mrblib_mruby_dir_glob_proc_debug_file_10.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir-glob/mrblib/dir_glob.rb");
  gem_mrblib_mruby_dir_glob_proc_debug_file_89.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir-glob/mrblib/dir_glob.rb");
  gem_mrblib_mruby_dir_glob_proc_debug_file_11.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir-glob/mrblib/dir_glob.rb");
  gem_mrblib_mruby_dir_glob_proc_debug_file_106.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir-glob/mrblib/dir_glob.rb");
  gem_mrblib_mruby_dir_glob_proc_debug_file_12.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir-glob/mrblib/dir_glob.rb");
  gem_mrblib_mruby_dir_glob_proc_debug_file_123.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir-glob/mrblib/dir_glob.rb");
  gem_mrblib_mruby_dir_glob_proc_debug_file_13.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir-glob/mrblib/dir_glob.rb");
  gem_mrblib_mruby_dir_glob_proc_debug_file_140.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir-glob/mrblib/dir_glob.rb");
  gem_mrblib_mruby_dir_glob_proc_debug_file_141.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir-glob/mrblib/dir_glob.rb");
  gem_mrblib_mruby_dir_glob_proc_debug_file_14.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir-glob/mrblib/dir_glob.rb");
  gem_mrblib_mruby_dir_glob_proc_debug_file_160.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir-glob/mrblib/dir_glob.rb");
  gem_mrblib_mruby_dir_glob_proc_debug_file_161.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir-glob/mrblib/dir_glob.rb");
  gem_mrblib_mruby_dir_glob_proc_debug_file_15.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir-glob/mrblib/dir_glob.rb");
  gem_mrblib_mruby_dir_glob_proc_debug_file_180.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir-glob/mrblib/dir_glob.rb");
  gem_mrblib_mruby_dir_glob_proc_debug_file_16.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir-glob/mrblib/dir_glob.rb");
  gem_mrblib_mruby_dir_glob_proc_debug_file_197.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir-glob/mrblib/dir_glob.rb");
  gem_mrblib_mruby_dir_glob_proc_debug_file_17.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir-glob/mrblib/dir_glob.rb");
  gem_mrblib_mruby_dir_glob_proc_debug_file_214.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir-glob/mrblib/dir_glob.rb");
  gem_mrblib_mruby_dir_glob_proc_debug_file_18.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir-glob/mrblib/dir_glob.rb");
  gem_mrblib_mruby_dir_glob_proc_debug_file_19.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir-glob/mrblib/dir_glob.rb");
  gem_mrblib_mruby_dir_glob_proc_debug_file_20.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir-glob/mrblib/dir_glob.rb");
  gem_mrblib_mruby_dir_glob_proc_debug_file_21.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir-glob/mrblib/dir_glob.rb");
  gem_mrblib_mruby_dir_glob_proc_debug_file_279.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir-glob/mrblib/dir_glob.rb");
  gem_mrblib_mruby_dir_glob_proc_debug_file_280.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir-glob/mrblib/dir_glob.rb");
  gem_mrblib_mruby_dir_glob_proc_debug_file_22.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir-glob/mrblib/dir_glob.rb");
  gem_mrblib_mruby_dir_glob_proc_debug_file_23.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir-glob/mrblib/dir_glob.rb");
  gem_mrblib_mruby_dir_glob_proc_debug_file_7.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir-glob/mrblib/dir_glob.rb");
  gem_mrblib_mruby_dir_glob_proc_debug_file_2.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir-glob/mrblib/dir_glob.rb");
}
void mrb_mruby_dir_glob_gem_init(mrb_state *mrb);
void mrb_mruby_dir_glob_gem_final(mrb_state *mrb);

void GENERATED_TMP_mrb_mruby_dir_glob_gem_init(mrb_state *mrb) {
  int ai = mrb_gc_arena_save(mrb);
  gem_mrblib_mruby_dir_glob_proc_init_syms(mrb);
  mrb_mruby_dir_glob_gem_init(mrb);
  mrb_load_proc(mrb, gem_mrblib_mruby_dir_glob_proc);
  if (mrb->exc) {
    mrb_print_error(mrb);
    mrb_close(mrb);
    exit(EXIT_FAILURE);
  }
  struct REnv *e = mrb_vm_ci_env(mrb->c->cibase);
  mrb_vm_ci_env_set(mrb->c->cibase, NULL);
  mrb_env_unshare(mrb, e);
  mrb_gc_arena_restore(mrb, ai);
}

void GENERATED_TMP_mrb_mruby_dir_glob_gem_final(mrb_state *mrb) {
  mrb_mruby_dir_glob_gem_final(mrb);
}
