/*
 * This file is loading the irep
 * Ruby GEM code.
 *
 * IMPORTANT:
 *   This file was generated!
 *   All manual changes will get lost.
 */
#include <stdlib.h>
#include <mruby.h>
#include <mruby/irep.h>
#include <mruby/debug.h>
#include <mruby/proc.h>
#include <mruby/presym.h>

#define mrb_BRACED(...) {__VA_ARGS__}
#define mrb_DEFINE_SYMS_VAR(name, len, syms, qualifier) \
  static qualifier mrb_sym name[len] = mrb_BRACED syms

mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_proc_syms_2, 2, (MRB_SYM(read), MRB_SYM(call), ), const);
static const mrb_code gem_mrblib_mruby_dir_proc_iseq_2[34] = {
0x33,0x00,0x00,0x01,0x12,0x03,0x2d,0x03,0x00,0x00,0x01,0x02,0x03,0x25,0x03,0x00,0x0d,0x01,0x03,0x01,
0x01,0x04,0x02,0x2d,0x03,0x01,0x01,0x23,0xff,0xe6,0x12,0x03,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_proc_lv_2, 2, (MRB_SYM(block), MRB_SYM(s), ), const);
static char gem_mrblib_mruby_dir_proc_debug_lines_2[] = "\x00\x02\x04\x01\x0b\x01\x0f\x02";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_proc_debug_file_2 = {
0, 173, 8, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_proc_debug_lines_2}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_proc_debug_file_2_ = &gem_mrblib_mruby_dir_proc_debug_file_2;
static mrb_irep_debug_info gem_mrblib_mruby_dir_proc_debug_2 = {
34, 1, &gem_mrblib_mruby_dir_proc_debug_file_2_};
static const mrb_irep gem_mrblib_mruby_dir_proc_irep_2 = {
  3,6,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_proc_iseq_2,
  NULL,gem_mrblib_mruby_dir_proc_syms_2,NULL,
  gem_mrblib_mruby_dir_proc_lv_2,
  &gem_mrblib_mruby_dir_proc_debug_2,
  34,0,2,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_proc_syms_14, 2, (MRB_SYM(read), MRB_OPSYM(lshift), ), const);
static const mrb_code gem_mrblib_mruby_dir_proc_iseq_14[36] = {
0x33,0x04,0x00,0x00,0x01,0x04,0x01,0x2d,0x04,0x00,0x00,0x01,0x03,0x04,0x25,0x04,0x00,0x0e,0x21,0x04,
0x03,0x00,0x01,0x05,0x03,0x2d,0x04,0x01,0x01,0x23,0xff,0xe4,0x11,0x04,0x37,0x04,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_proc_lv_14, 3, (MRB_SYM(d), MRB_OPSYM(and), MRB_SYM(s), ), const);
static char gem_mrblib_mruby_dir_proc_debug_lines_14[] = "\x00\x0e\x04\x01\x0c\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_proc_debug_file_14 = {
0, 173, 6, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_proc_debug_lines_14}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_proc_debug_file_14_ = &gem_mrblib_mruby_dir_proc_debug_file_14;
static mrb_irep_debug_info gem_mrblib_mruby_dir_proc_debug_14 = {
36, 1, &gem_mrblib_mruby_dir_proc_debug_file_14_};
static const mrb_irep gem_mrblib_mruby_dir_proc_irep_14 = {
  4,7,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_proc_iseq_14,
  NULL,gem_mrblib_mruby_dir_proc_syms_14,NULL,
  gem_mrblib_mruby_dir_proc_lv_14,
  &gem_mrblib_mruby_dir_proc_debug_14,
  36,0,2,0,0
};
static const mrb_irep *gem_mrblib_mruby_dir_proc_reps_3[1] = {
  &gem_mrblib_mruby_dir_proc_irep_14,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_proc_syms_3, 1, (MRB_SYM(open), ), const);
static const mrb_code gem_mrblib_mruby_dir_proc_iseq_3[24] = {
0x33,0x04,0x00,0x00,0x46,0x04,0x00,0x01,0x03,0x04,0x12,0x04,0x01,0x05,0x01,0x56,0x06,0x00,0x2e,0x04,
0x00,0x01,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_proc_lv_3, 3, (MRB_SYM(path), MRB_OPSYM(and), MRB_SYM(a), ), const);
static char gem_mrblib_mruby_dir_proc_debug_lines_3[] = "\x00\x0c\x04\x01\x06\x01\x0c\x05";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_proc_debug_file_3 = {
0, 173, 8, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_proc_debug_lines_3}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_proc_debug_file_3_ = &gem_mrblib_mruby_dir_proc_debug_file_3;
static mrb_irep_debug_info gem_mrblib_mruby_dir_proc_debug_3 = {
24, 1, &gem_mrblib_mruby_dir_proc_debug_file_3_};
static const mrb_irep gem_mrblib_mruby_dir_proc_irep_3 = {
  4,7,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_proc_iseq_3,
  NULL,gem_mrblib_mruby_dir_proc_syms_3,gem_mrblib_mruby_dir_proc_reps_3,
  gem_mrblib_mruby_dir_proc_lv_3,
  &gem_mrblib_mruby_dir_proc_debug_3,
  24,0,1,1,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_proc_syms_21, 1, (MRB_SYM(call), ), const);
static const mrb_code gem_mrblib_mruby_dir_proc_iseq_21[17] = {
0x33,0x04,0x00,0x00,0x21,0x03,0x02,0x00,0x01,0x04,0x01,0x2d,0x03,0x00,0x01,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_proc_lv_21, 2, (MRB_SYM(f), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_dir_proc_debug_lines_21[] = "\x00\x18";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_proc_debug_file_21 = {
0, 173, 2, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_proc_debug_lines_21}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_proc_debug_file_21_ = &gem_mrblib_mruby_dir_proc_debug_file_21;
static mrb_irep_debug_info gem_mrblib_mruby_dir_proc_debug_21 = {
17, 1, &gem_mrblib_mruby_dir_proc_debug_file_21_};
static const mrb_irep gem_mrblib_mruby_dir_proc_irep_21 = {
  3,6,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_proc_iseq_21,
  NULL,gem_mrblib_mruby_dir_proc_syms_21,NULL,
  gem_mrblib_mruby_dir_proc_lv_21,
  &gem_mrblib_mruby_dir_proc_debug_21,
  17,0,1,0,0
};
static const mrb_irep *gem_mrblib_mruby_dir_proc_reps_4[1] = {
  &gem_mrblib_mruby_dir_proc_irep_21,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_proc_syms_4, 2, (MRB_SYM(open), MRB_SYM(each), ), const);
static const mrb_code gem_mrblib_mruby_dir_proc_iseq_4[45] = {
0x33,0x04,0x00,0x01,0x01,0x03,0x02,0x25,0x03,0x00,0x13,0x12,0x03,0x01,0x04,0x01,0x2d,0x03,0x00,0x01,
0x56,0x04,0x00,0x2e,0x03,0x01,0x00,0x23,0x00,0x0d,0x12,0x03,0x01,0x04,0x01,0x2d,0x03,0x00,0x01,0x2d,
0x03,0x01,0x00,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_proc_lv_4, 2, (MRB_SYM(path), MRB_SYM(block), ), const);
static char gem_mrblib_mruby_dir_proc_debug_lines_4[] = "\x00\x16\x04\x01\x05\x01\x13\x02";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_proc_debug_file_4 = {
0, 173, 8, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_proc_debug_lines_4}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_proc_debug_file_4_ = &gem_mrblib_mruby_dir_proc_debug_file_4;
static mrb_irep_debug_info gem_mrblib_mruby_dir_proc_debug_4 = {
45, 1, &gem_mrblib_mruby_dir_proc_debug_file_4_};
static const mrb_irep gem_mrblib_mruby_dir_proc_irep_4 = {
  3,6,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_proc_iseq_4,
  NULL,gem_mrblib_mruby_dir_proc_syms_4,gem_mrblib_mruby_dir_proc_reps_4,
  gem_mrblib_mruby_dir_proc_lv_4,
  &gem_mrblib_mruby_dir_proc_debug_4,
  45,0,2,1,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_proc_syms_5, 3, (MRB_SYM(new), MRB_SYM(call), MRB_SYM(close), ), const);
static const mrb_code gem_mrblib_mruby_dir_proc_iseq_5[71] = {
0x33,0x04,0x00,0x01,0x01,0x04,0x02,0x25,0x04,0x00,0x24,0x12,0x04,0x01,0x05,0x01,0x2d,0x04,0x00,0x01,
0x01,0x03,0x04,0x01,0x04,0x02,0x01,0x05,0x03,0x2d,0x04,0x01,0x01,0x28,0x06,0x01,0x07,0x03,0x2d,0x07,
0x02,0x00,0x2a,0x06,0x23,0x00,0x09,0x12,0x04,0x01,0x05,0x01,0x2d,0x04,0x00,0x01,0x37,0x04,0x01,0x00,
0x00,0x00,0x17,0x00,0x00,0x00,0x21,0x00,0x00,0x00,0x21,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_proc_lv_5, 3, (MRB_SYM(path), MRB_SYM(block), MRB_SYM(d), ), const);
static char gem_mrblib_mruby_dir_proc_debug_lines_5[] = "\x00\x1e\x04\x01\x05\x05\x02\xfc\xff\xff\xff\x0f\x0c\x02\x0c\x02\x0a\x03";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_proc_debug_file_5 = {
0, 173, 18, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_proc_debug_lines_5}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_proc_debug_file_5_ = &gem_mrblib_mruby_dir_proc_debug_file_5;
static mrb_irep_debug_info gem_mrblib_mruby_dir_proc_debug_5 = {
58, 1, &gem_mrblib_mruby_dir_proc_debug_file_5_};
static const mrb_irep gem_mrblib_mruby_dir_proc_irep_5 = {
  4,9,1,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_proc_iseq_5,
  NULL,gem_mrblib_mruby_dir_proc_syms_5,NULL,
  gem_mrblib_mruby_dir_proc_lv_5,
  &gem_mrblib_mruby_dir_proc_debug_5,
  58,0,3,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_proc_syms_6, 3, (MRB_SYM(getwd), MRB_SYM(_chdir), MRB_SYM(call), ), const);
static const mrb_code gem_mrblib_mruby_dir_proc_iseq_6[82] = {
0x33,0x04,0x00,0x01,0x12,0x03,0x01,0x05,0x02,0x25,0x05,0x00,0x2d,0x12,0x05,0x2d,0x05,0x00,0x00,0x01,
0x04,0x05,0x12,0x05,0x01,0x06,0x01,0x2d,0x05,0x01,0x01,0x01,0x05,0x02,0x01,0x06,0x01,0x2d,0x05,0x02,
0x01,0x28,0x07,0x01,0x08,0x03,0x01,0x09,0x04,0x2d,0x08,0x01,0x01,0x2a,0x07,0x23,0x00,0x09,0x12,0x05,
0x01,0x06,0x01,0x2d,0x05,0x01,0x01,0x37,0x05,0x01,0x00,0x00,0x00,0x16,0x00,0x00,0x00,0x29,0x00,0x00,
0x00,0x29,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_proc_lv_6, 4, (MRB_SYM(path), MRB_SYM(block), MRB_SYM(my), MRB_SYM(wd), ), const);
static char gem_mrblib_mruby_dir_proc_debug_lines_6[] = "\x00\x2b\x04\x01\x02\x01\x05\x06\x02\xfb\xff\xff\xff\x0f\x09\x02\x09\x01\x0c\x02\x0d\x03";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_proc_debug_file_6 = {
0, 173, 22, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_proc_debug_lines_6}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_proc_debug_file_6_ = &gem_mrblib_mruby_dir_proc_debug_file_6;
static mrb_irep_debug_info gem_mrblib_mruby_dir_proc_debug_6 = {
69, 1, &gem_mrblib_mruby_dir_proc_debug_file_6_};
static const mrb_irep gem_mrblib_mruby_dir_proc_irep_6 = {
  5,11,1,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_proc_iseq_6,
  NULL,gem_mrblib_mruby_dir_proc_syms_6,NULL,
  gem_mrblib_mruby_dir_proc_lv_6,
  &gem_mrblib_mruby_dir_proc_debug_6,
  69,0,3,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_proc_syms_7, 7, (MRB_SYM_Q(exists), MRB_SYM_Q(exist), MRB_SYM(pwd), MRB_SYM(getwd), MRB_SYM(rmdir), MRB_SYM(delete), MRB_SYM(unlink), ), const);
static const mrb_code gem_mrblib_mruby_dir_proc_iseq_7[16] = {
0x5f,0x00,0x01,0x5f,0x02,0x03,0x5f,0x04,0x05,0x5f,0x06,0x05,0x11,0x01,0x37,0x01,};
static char gem_mrblib_mruby_dir_proc_debug_lines_7[] = "\x00\x3b\x03\x01\x03\x01\x03\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_proc_debug_file_7 = {
0, 173, 8, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_proc_debug_lines_7}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_proc_debug_file_7_ = &gem_mrblib_mruby_dir_proc_debug_file_7;
static mrb_irep_debug_info gem_mrblib_mruby_dir_proc_debug_7 = {
16, 1, &gem_mrblib_mruby_dir_proc_debug_file_7_};
static const mrb_irep gem_mrblib_mruby_dir_proc_irep_7 = {
  1,2,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_proc_iseq_7,
  NULL,gem_mrblib_mruby_dir_proc_syms_7,NULL,
  NULL,					/* lv */
  &gem_mrblib_mruby_dir_proc_debug_7,
  16,0,7,0,0
};
static const mrb_irep *gem_mrblib_mruby_dir_proc_reps_1[6] = {
  &gem_mrblib_mruby_dir_proc_irep_2,
  &gem_mrblib_mruby_dir_proc_irep_3,
  &gem_mrblib_mruby_dir_proc_irep_4,
  &gem_mrblib_mruby_dir_proc_irep_5,
  &gem_mrblib_mruby_dir_proc_irep_6,
  &gem_mrblib_mruby_dir_proc_irep_7,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_proc_syms_1, 9, (MRB_SYM(each), MRB_SYM(pos), MRB_SYM(tell), MRB_SYM_E(pos), MRB_SYM(seek), MRB_SYM(entries), MRB_SYM(foreach), MRB_SYM(open), MRB_SYM(chdir), ), const);
static const mrb_code gem_mrblib_mruby_dir_proc_iseq_1[63] = {
0x62,0x01,0x57,0x02,0x00,0x5e,0x01,0x00,0x5f,0x01,0x02,0x5f,0x03,0x04,0x12,0x01,0x61,0x01,0x57,0x02,
0x01,0x5e,0x01,0x05,0x12,0x01,0x61,0x01,0x57,0x02,0x02,0x5e,0x01,0x06,0x12,0x01,0x61,0x01,0x57,0x02,
0x03,0x5e,0x01,0x07,0x12,0x01,0x61,0x01,0x57,0x02,0x04,0x5e,0x01,0x08,0x12,0x01,0x61,0x01,0x5d,0x01,
0x05,0x37,0x01,};
static char gem_mrblib_mruby_dir_proc_debug_lines_1[] = "\x00\x02\x08\x07\x03\x01\x03\x02\x0a\x0a\x0a\x08\x0a\x0d\x0a\x0f";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_proc_debug_file_1 = {
0, 173, 16, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_proc_debug_lines_1}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_proc_debug_file_1_ = &gem_mrblib_mruby_dir_proc_debug_file_1;
static mrb_irep_debug_info gem_mrblib_mruby_dir_proc_debug_1 = {
63, 1, &gem_mrblib_mruby_dir_proc_debug_file_1_};
static const mrb_irep gem_mrblib_mruby_dir_proc_irep_1 = {
  1,3,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_proc_iseq_1,
  NULL,gem_mrblib_mruby_dir_proc_syms_1,gem_mrblib_mruby_dir_proc_reps_1,
  NULL,					/* lv */
  &gem_mrblib_mruby_dir_proc_debug_1,
  63,0,9,6,0
};
static const mrb_irep *gem_mrblib_mruby_dir_proc_reps_0[1] = {
  &gem_mrblib_mruby_dir_proc_irep_1,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_dir_proc_syms_0, 1, (MRB_SYM(Dir), ), const);
static const mrb_code gem_mrblib_mruby_dir_proc_iseq_0[13] = {
0x11,0x01,0x11,0x02,0x5b,0x01,0x00,0x5d,0x01,0x00,0x37,0x01,0x68,};
static char gem_mrblib_mruby_dir_proc_debug_lines_0[] = "\x00\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_dir_proc_debug_file_0 = {
0, 173, 2, mrb_debug_line_packed_map, {gem_mrblib_mruby_dir_proc_debug_lines_0}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_dir_proc_debug_file_0_ = &gem_mrblib_mruby_dir_proc_debug_file_0;
static mrb_irep_debug_info gem_mrblib_mruby_dir_proc_debug_0 = {
13, 1, &gem_mrblib_mruby_dir_proc_debug_file_0_};
static const mrb_irep gem_mrblib_mruby_dir_proc_irep_0 = {
  1,3,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_dir_proc_iseq_0,
  NULL,gem_mrblib_mruby_dir_proc_syms_0,gem_mrblib_mruby_dir_proc_reps_0,
  NULL,					/* lv */
  &gem_mrblib_mruby_dir_proc_debug_0,
  13,0,1,1,0
};
static
const struct RProc gem_mrblib_mruby_dir_proc[] = {{
NULL,NULL,MRB_TT_PROC,MRB_GC_RED,0,{&gem_mrblib_mruby_dir_proc_irep_0},NULL,{NULL},
}};
static void
gem_mrblib_mruby_dir_proc_init_syms(mrb_state *mrb)
{
  gem_mrblib_mruby_dir_proc_debug_file_2.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir/mrblib/dir.rb");
  gem_mrblib_mruby_dir_proc_debug_file_14.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir/mrblib/dir.rb");
  gem_mrblib_mruby_dir_proc_debug_file_3.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir/mrblib/dir.rb");
  gem_mrblib_mruby_dir_proc_debug_file_21.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir/mrblib/dir.rb");
  gem_mrblib_mruby_dir_proc_debug_file_4.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir/mrblib/dir.rb");
  gem_mrblib_mruby_dir_proc_debug_file_5.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir/mrblib/dir.rb");
  gem_mrblib_mruby_dir_proc_debug_file_6.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir/mrblib/dir.rb");
  gem_mrblib_mruby_dir_proc_debug_file_7.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir/mrblib/dir.rb");
  gem_mrblib_mruby_dir_proc_debug_file_1.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir/mrblib/dir.rb");
  gem_mrblib_mruby_dir_proc_debug_file_0.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-dir/mrblib/dir.rb");
}
void mrb_mruby_dir_gem_init(mrb_state *mrb);
void mrb_mruby_dir_gem_final(mrb_state *mrb);

void GENERATED_TMP_mrb_mruby_dir_gem_init(mrb_state *mrb) {
  int ai = mrb_gc_arena_save(mrb);
  gem_mrblib_mruby_dir_proc_init_syms(mrb);
  mrb_mruby_dir_gem_init(mrb);
  mrb_load_proc(mrb, gem_mrblib_mruby_dir_proc);
  if (mrb->exc) {
    mrb_print_error(mrb);
    mrb_close(mrb);
    exit(EXIT_FAILURE);
  }
  struct REnv *e = mrb_vm_ci_env(mrb->c->cibase);
  mrb_vm_ci_env_set(mrb->c->cibase, NULL);
  mrb_env_unshare(mrb, e);
  mrb_gc_arena_restore(mrb, ai);
}

void GENERATED_TMP_mrb_mruby_dir_gem_final(mrb_state *mrb) {
  mrb_mruby_dir_gem_final(mrb);
}
