/*
 * This file is loading the irep
 * Ruby GEM code.
 *
 * IMPORTANT:
 *   This file was generated!
 *   All manual changes will get lost.
 */
#include <stdlib.h>
#include <mruby.h>
#include <mruby/irep.h>
#include <mruby/debug.h>
#include <mruby/proc.h>
#include <mruby/presym.h>

#define mrb_BRACED(...) {__VA_ARGS__}
#define mrb_DEFINE_SYMS_VAR(name, len, syms, qualifier) \
  static qualifier mrb_sym name[len] = mrb_BRACED syms

mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_syms_24, 1, (MRB_OPSYM(aset), ), const);
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_24[23] = {
0x33,0x08,0x00,0x00,0x01,0x04,0x02,0x21,0x05,0x05,0x00,0x01,0x06,0x01,0x01,0x07,0x04,0x2d,0x05,0x00,
0x02,0x37,0x04,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_lv_24, 3, (MRB_SYM(k), MRB_SYM(v), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_24[] = "\x00\x20";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_24 = {
0, 173, 2, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_24}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_24_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_24;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_24 = {
23, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_24_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_24 = {
  4,9,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_24,
  NULL,gem_mrblib_mruby_hash_ext_proc_syms_24,NULL,
  gem_mrblib_mruby_hash_ext_proc_lv_24,
  &gem_mrblib_mruby_hash_ext_proc_debug_24,
  23,0,1,0,0
};
static const mrb_pool_value gem_mrblib_mruby_hash_ext_proc_pool_25[4] = {
{IREP_TT_STR|(19<<2), {"\x77\x72\x6f\x6e\x67\x20\x65\x6c\x65\x6d\x65\x6e\x74\x20\x74\x79\x70\x65\x20"}},
{IREP_TT_STR|(17<<2), {"\x20\x28\x65\x78\x70\x65\x63\x74\x65\x64\x20\x61\x72\x72\x61\x79\x29"}},
{IREP_TT_STR|(28<<2), {"\x69\x6e\x76\x61\x6c\x69\x64\x20\x6e\x75\x6d\x62\x65\x72\x20\x6f\x66\x20\x65\x6c\x65\x6d\x65\x6e\x74\x73\x20\x28"}},
{IREP_TT_STR|(10<<2), {"\x20\x66\x6f\x72\x20\x31\x2e\x2e\x32\x29"}},
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_syms_25, 9, (MRB_SYM(to_a), MRB_SYM_Q(respond_to), MRB_SYM(ArgumentError), MRB_SYM(class), MRB_SYM(raise), MRB_SYM(size), MRB_OPSYM(eqq), MRB_OPSYM(aref), MRB_OPSYM(aset), ), const);
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_25[189] = {
0x33,0x04,0x00,0x00,0x01,0x05,0x01,0x10,0x06,0x00,0x2d,0x05,0x01,0x01,0x24,0x05,0x00,0x1a,0x12,0x05,
0x1d,0x06,0x02,0x50,0x07,0x00,0x01,0x08,0x01,0x2d,0x08,0x03,0x00,0x51,0x07,0x50,0x08,0x01,0x51,0x07,
0x2d,0x05,0x04,0x02,0x11,0x05,0x4b,0x06,0x05,0x00,0x01,0x03,0x06,0x4b,0x06,0x05,0x01,0x01,0x04,0x06,
0x01,0x05,0x01,0x2d,0x05,0x05,0x00,0x08,0x06,0x01,0x07,0x05,0x2d,0x06,0x06,0x01,0x24,0x06,0x00,0x03,
0x23,0x00,0x1b,0x01,0x06,0x01,0x06,0x07,0x2d,0x06,0x07,0x01,0x01,0x03,0x06,0x01,0x06,0x01,0x07,0x07,
0x2d,0x06,0x07,0x01,0x01,0x04,0x06,0x23,0x00,0x3c,0x07,0x06,0x01,0x07,0x05,0x2d,0x06,0x06,0x01,0x24,
0x06,0x00,0x03,0x23,0x00,0x0f,0x01,0x06,0x01,0x06,0x07,0x2d,0x06,0x07,0x01,0x01,0x03,0x06,0x23,0x00,
0x1d,0x12,0x06,0x1d,0x07,0x02,0x50,0x08,0x02,0x01,0x09,0x01,0x2d,0x09,0x05,0x00,0x51,0x08,0x50,0x09,
0x03,0x51,0x08,0x2d,0x06,0x04,0x02,0x23,0x00,0x00,0x01,0x05,0x04,0x21,0x06,0x05,0x00,0x01,0x07,0x03,
0x01,0x08,0x05,0x2d,0x06,0x08,0x02,0x37,0x05,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_lv_25, 4, (MRB_SYM(i), MRB_OPSYM(and), MRB_SYM(k), MRB_SYM(v), ), const);
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_25[] = "\x00\x24\x04\x01\x28\x01\x10\x01\x07\x01\x0e\x02\x02\xff\xff\xff\xff\x0f\x0c\x01\x0d\x04\x02\xfd\xff\xff\xff\x0f\x0e\x01\x0f\x02\x1f\x02";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_25 = {
0, 173, 34, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_25}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_25_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_25;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_25 = {
189, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_25_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_25 = {
  5,11,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_25,
  gem_mrblib_mruby_hash_ext_proc_pool_25,gem_mrblib_mruby_hash_ext_proc_syms_25,NULL,
  gem_mrblib_mruby_hash_ext_proc_lv_25,
  &gem_mrblib_mruby_hash_ext_proc_debug_25,
  189,4,9,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_syms_26, 2, (MRB_OPSYM(aref), MRB_OPSYM(aset), ), const);
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_26[42] = {
0x33,0x04,0x00,0x00,0x21,0x03,0x01,0x00,0x01,0x04,0x01,0x3c,0x04,0x01,0x2d,0x03,0x00,0x01,0x21,0x04,
0x05,0x00,0x21,0x05,0x01,0x00,0x01,0x06,0x01,0x2d,0x05,0x00,0x01,0x01,0x06,0x03,0x2d,0x04,0x01,0x02,
0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_lv_26, 2, (MRB_SYM(i), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_26[] = "\x00\x39\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_26 = {
0, 173, 4, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_26}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_26_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_26;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_26 = {
42, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_26_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_26 = {
  3,8,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_26,
  NULL,gem_mrblib_mruby_hash_ext_proc_syms_26,NULL,
  gem_mrblib_mruby_hash_ext_proc_lv_26,
  &gem_mrblib_mruby_hash_ext_proc_debug_26,
  42,0,2,0,0
};
static const mrb_irep *gem_mrblib_mruby_hash_ext_proc_reps_2[3] = {
  &gem_mrblib_mruby_hash_ext_proc_irep_24,
  &gem_mrblib_mruby_hash_ext_proc_irep_25,
  &gem_mrblib_mruby_hash_ext_proc_irep_26,
};
static const mrb_pool_value gem_mrblib_mruby_hash_ext_proc_pool_2[1] = {
{IREP_TT_STR|(32<<2), {"\x6f\x64\x64\x20\x6e\x75\x6d\x62\x65\x72\x20\x6f\x66\x20\x61\x72\x67\x75\x6d\x65\x6e\x74\x73\x20\x66\x6f\x72\x20\x48\x61\x73\x68"}},
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_syms_2, 12, (MRB_SYM(length), MRB_OPSYM(aref), MRB_SYM(Hash), MRB_OPSYM(eqq), MRB_SYM(new), MRB_SYM(each), MRB_SYM(to_a), MRB_SYM_Q(respond_to), MRB_OPSYM(mod), MRB_SYM(ArgumentError), MRB_SYM(raise), MRB_SYM(step), ), const);
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_2[171] = {
0x33,0x00,0x10,0x00,0x01,0x06,0x01,0x2d,0x06,0x00,0x00,0x01,0x03,0x06,0x01,0x06,0x03,0x07,0x07,0x41,
0x06,0x25,0x06,0x00,0x59,0x01,0x06,0x01,0x06,0x07,0x2d,0x06,0x01,0x01,0x01,0x04,0x06,0x1d,0x06,0x02,
0x01,0x07,0x04,0x2d,0x06,0x03,0x01,0x25,0x06,0x00,0x18,0x12,0x06,0x2d,0x06,0x04,0x00,0x01,0x05,0x06,
0x01,0x06,0x04,0x56,0x07,0x00,0x2e,0x06,0x05,0x00,0x37,0x05,0x23,0x00,0x27,0x01,0x06,0x04,0x10,0x07,
0x06,0x2d,0x06,0x07,0x01,0x25,0x06,0x00,0x19,0x12,0x06,0x2d,0x06,0x04,0x00,0x01,0x05,0x06,0x01,0x06,
0x04,0x2d,0x06,0x06,0x00,0x56,0x07,0x01,0x2e,0x06,0x05,0x00,0x37,0x05,0x01,0x06,0x03,0x08,0x07,0x2d,
0x06,0x08,0x01,0x06,0x07,0x41,0x06,0x24,0x06,0x00,0x0c,0x12,0x06,0x1d,0x07,0x09,0x50,0x08,0x00,0x2d,
0x06,0x0a,0x02,0x12,0x06,0x2d,0x06,0x04,0x00,0x01,0x05,0x06,0x06,0x06,0x01,0x07,0x03,0x3e,0x07,0x02,
0x08,0x08,0x56,0x09,0x02,0x2e,0x06,0x0b,0x02,0x37,0x05,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_lv_2, 5, (MRB_SYM(object), MRB_OPSYM(and), MRB_SYM(length), MRB_SYM(o), MRB_SYM(h), ), const);
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_2[] = "\x00\x1a\x04\x01\x0a\x01\x09\x16\x02\xeb\xff\xff\xff\x0f\x0c\x01\x0c\x03\x02\xfe\xff\xff\xff\x0f\x09\x01\x0a\x01\x03\x11\x02\xf0\xff\xff\xff\x0f\x0c\x10\x02\xf1\xff\xff\xff\x0f\x09\x01\x0e\x0e\x02\x03\x0f\x01\x0e\x02\x09\x01\x11\x03";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_2 = {
0, 173, 58, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_2}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_2_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_2;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_2 = {
171, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_2_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_2 = {
  6,10,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_2,
  gem_mrblib_mruby_hash_ext_proc_pool_2,gem_mrblib_mruby_hash_ext_proc_syms_2,gem_mrblib_mruby_hash_ext_proc_reps_2,
  gem_mrblib_mruby_hash_ext_proc_lv_2,
  &gem_mrblib_mruby_hash_ext_proc_debug_2,
  171,1,12,3,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_syms_55, 4, (MRB_SYM_Q(has_key), MRB_OPSYM(aref), MRB_SYM(call), MRB_OPSYM(aset), ), const);
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_55[76] = {
0x33,0x04,0x00,0x00,0x12,0x03,0x01,0x04,0x01,0x2d,0x03,0x00,0x01,0x25,0x03,0x00,0x22,0x21,0x03,0x02,
0x00,0x01,0x04,0x01,0x12,0x05,0x01,0x06,0x01,0x2d,0x05,0x01,0x01,0x21,0x06,0x01,0x00,0x01,0x07,0x01,
0x2d,0x06,0x01,0x01,0x2d,0x03,0x02,0x03,0x23,0x00,0x0b,0x21,0x03,0x01,0x00,0x01,0x04,0x01,0x2d,0x03,
0x01,0x01,0x12,0x04,0x01,0x05,0x01,0x01,0x06,0x03,0x2d,0x04,0x03,0x02,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_lv_55, 2, (MRB_SYM(k), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_55[] = "\x00\x57\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_55 = {
0, 173, 4, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_55}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_55_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_55;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_55 = {
76, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_55_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_55 = {
  3,9,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_55,
  NULL,gem_mrblib_mruby_hash_ext_proc_syms_55,NULL,
  gem_mrblib_mruby_hash_ext_proc_lv_55,
  &gem_mrblib_mruby_hash_ext_proc_debug_55,
  76,0,4,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_syms_56, 2, (MRB_OPSYM(aref), MRB_OPSYM(aset), ), const);
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_56[29] = {
0x33,0x04,0x00,0x00,0x21,0x03,0x01,0x00,0x01,0x04,0x01,0x2d,0x03,0x00,0x01,0x12,0x04,0x01,0x05,0x01,
0x01,0x06,0x03,0x2d,0x04,0x01,0x02,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_lv_56, 2, (MRB_SYM(k), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_56[] = "\x00\x5b";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_56 = {
0, 173, 2, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_56}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_56_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_56;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_56 = {
29, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_56_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_56 = {
  3,8,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_56,
  NULL,gem_mrblib_mruby_hash_ext_proc_syms_56,NULL,
  gem_mrblib_mruby_hash_ext_proc_lv_56,
  &gem_mrblib_mruby_hash_ext_proc_debug_56,
  29,0,2,0,0
};
static const mrb_irep *gem_mrblib_mruby_hash_ext_proc_reps_3[2] = {
  &gem_mrblib_mruby_hash_ext_proc_irep_55,
  &gem_mrblib_mruby_hash_ext_proc_irep_56,
};
static const mrb_pool_value gem_mrblib_mruby_hash_ext_proc_pool_3[2] = {
{IREP_TT_STR|(15<<2), {"\x48\x61\x73\x68\x20\x72\x65\x71\x75\x69\x72\x65\x64\x20\x28"}},
{IREP_TT_STR|(7<<2), {"\x20\x67\x69\x76\x65\x6e\x29"}},
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_syms_3, 6, (MRB_SYM(Hash), MRB_OPSYM(eqq), MRB_SYM(TypeError), MRB_SYM(class), MRB_SYM(raise), MRB_SYM(each_key), ), const);
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_3[78] = {
0x33,0x04,0x00,0x01,0x1d,0x03,0x00,0x01,0x04,0x01,0x2d,0x03,0x01,0x01,0x24,0x03,0x00,0x1a,0x12,0x03,
0x1d,0x04,0x02,0x50,0x05,0x00,0x01,0x06,0x01,0x2d,0x06,0x03,0x00,0x51,0x05,0x50,0x06,0x01,0x51,0x05,
0x2d,0x03,0x04,0x02,0x01,0x03,0x02,0x25,0x03,0x00,0x0d,0x01,0x03,0x01,0x56,0x04,0x00,0x2e,0x03,0x05,
0x00,0x23,0x00,0x0a,0x01,0x03,0x01,0x56,0x04,0x01,0x2e,0x03,0x05,0x00,0x12,0x03,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_lv_3, 2, (MRB_SYM(other), MRB_SYM(block), ), const);
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_3[] = "\x00\x54\x04\x01\x28\x01\x05\x01\x0d\x04\x0c\x02";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_3 = {
0, 173, 12, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_3}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_3_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_3;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_3 = {
78, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_3_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_3 = {
  3,8,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_3,
  gem_mrblib_mruby_hash_ext_proc_pool_3,gem_mrblib_mruby_hash_ext_proc_syms_3,gem_mrblib_mruby_hash_ext_proc_reps_3,
  gem_mrblib_mruby_hash_ext_proc_lv_3,
  &gem_mrblib_mruby_hash_ext_proc_debug_3,
  78,2,6,2,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_syms_81, 2, (MRB_OPSYM(aref), MRB_OPSYM(neq), ), const);
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_81[21] = {
0x33,0x04,0x00,0x00,0x12,0x03,0x01,0x04,0x01,0x2d,0x03,0x00,0x01,0x11,0x04,0x2d,0x03,0x01,0x01,0x37,
0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_lv_81, 2, (MRB_SYM(k), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_81[] = "\x00\x6f\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_81 = {
0, 173, 4, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_81}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_81_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_81;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_81 = {
21, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_81_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_81 = {
  3,6,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_81,
  NULL,gem_mrblib_mruby_hash_ext_proc_syms_81,NULL,
  gem_mrblib_mruby_hash_ext_proc_lv_81,
  &gem_mrblib_mruby_hash_ext_proc_debug_81,
  21,0,2,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_syms_82, 2, (MRB_OPSYM(aref), MRB_OPSYM(aset), ), const);
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_82[29] = {
0x33,0x04,0x00,0x00,0x12,0x03,0x01,0x04,0x01,0x2d,0x03,0x00,0x01,0x21,0x04,0x04,0x00,0x01,0x05,0x01,
0x01,0x06,0x03,0x2d,0x04,0x01,0x02,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_lv_82, 2, (MRB_SYM(k), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_82[] = "\x00\x74\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_82 = {
0, 173, 4, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_82}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_82_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_82;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_82 = {
29, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_82_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_82 = {
  3,8,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_82,
  NULL,gem_mrblib_mruby_hash_ext_proc_syms_82,NULL,
  gem_mrblib_mruby_hash_ext_proc_lv_82,
  &gem_mrblib_mruby_hash_ext_proc_debug_82,
  29,0,2,0,0
};
static const mrb_irep *gem_mrblib_mruby_hash_ext_proc_reps_4[2] = {
  &gem_mrblib_mruby_hash_ext_proc_irep_81,
  &gem_mrblib_mruby_hash_ext_proc_irep_82,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_syms_4, 5, (MRB_SYM(keys), MRB_SYM(select), MRB_SYM(size), MRB_SYM(each), MRB_SYM(replace), ), const);
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_4[77] = {
0x33,0x00,0x00,0x00,0x12,0x05,0x2d,0x05,0x00,0x00,0x01,0x02,0x05,0x01,0x05,0x02,0x56,0x06,0x00,0x2e,
0x05,0x01,0x00,0x01,0x03,0x05,0x01,0x05,0x02,0x2d,0x05,0x02,0x00,0x01,0x06,0x03,0x2d,0x06,0x02,0x00,
0x41,0x05,0x25,0x05,0x00,0x04,0x11,0x05,0x37,0x05,0x52,0x05,0x00,0x01,0x04,0x05,0x01,0x05,0x03,0x56,
0x06,0x01,0x2e,0x05,0x03,0x00,0x12,0x05,0x01,0x06,0x04,0x2d,0x05,0x04,0x01,0x37,0x05,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_lv_4, 4, (MRB_OPSYM(and), MRB_SYM(keys), MRB_SYM(nk), MRB_SYM(h), ), const);
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_4[] = "\x00\x6d\x04\x01\x09\x01\x0d\x03\x18\x01\x06\x01\x0a\x03";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_4 = {
0, 173, 14, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_4}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_4_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_4;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_4 = {
77, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_4_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_4 = {
  5,8,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_4,
  NULL,gem_mrblib_mruby_hash_ext_proc_syms_4,gem_mrblib_mruby_hash_ext_proc_reps_4,
  gem_mrblib_mruby_hash_ext_proc_lv_4,
  &gem_mrblib_mruby_hash_ext_proc_debug_4,
  77,0,5,2,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_syms_107, 2, (MRB_OPSYM(aref), MRB_OPSYM(neq), ), const);
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_107[21] = {
0x33,0x04,0x00,0x00,0x12,0x03,0x01,0x04,0x01,0x2d,0x03,0x00,0x01,0x11,0x04,0x2d,0x03,0x01,0x01,0x37,
0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_lv_107, 2, (MRB_SYM(k), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_107[] = "\x00\x86\x01\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_107 = {
0, 173, 5, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_107}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_107_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_107;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_107 = {
21, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_107_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_107 = {
  3,6,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_107,
  NULL,gem_mrblib_mruby_hash_ext_proc_syms_107,NULL,
  gem_mrblib_mruby_hash_ext_proc_lv_107,
  &gem_mrblib_mruby_hash_ext_proc_debug_107,
  21,0,2,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_syms_108, 2, (MRB_OPSYM(aref), MRB_OPSYM(aset), ), const);
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_108[29] = {
0x33,0x04,0x00,0x00,0x12,0x03,0x01,0x04,0x01,0x2d,0x03,0x00,0x01,0x21,0x04,0x02,0x00,0x01,0x05,0x01,
0x01,0x06,0x03,0x2d,0x04,0x01,0x02,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_lv_108, 2, (MRB_SYM(k), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_108[] = "\x00\x88\x01\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_108 = {
0, 173, 5, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_108}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_108_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_108;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_108 = {
29, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_108_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_108 = {
  3,8,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_108,
  NULL,gem_mrblib_mruby_hash_ext_proc_syms_108,NULL,
  gem_mrblib_mruby_hash_ext_proc_lv_108,
  &gem_mrblib_mruby_hash_ext_proc_debug_108,
  29,0,2,0,0
};
static const mrb_irep *gem_mrblib_mruby_hash_ext_proc_reps_5[2] = {
  &gem_mrblib_mruby_hash_ext_proc_irep_107,
  &gem_mrblib_mruby_hash_ext_proc_irep_108,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_syms_5, 3, (MRB_SYM(keys), MRB_SYM(select), MRB_SYM(each), ), const);
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_5[32] = {
0x33,0x00,0x00,0x00,0x52,0x03,0x00,0x01,0x02,0x03,0x12,0x03,0x2d,0x03,0x00,0x00,0x56,0x04,0x00,0x2e,
0x03,0x01,0x00,0x56,0x04,0x01,0x2e,0x03,0x02,0x00,0x37,0x02,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_lv_5, 2, (MRB_OPSYM(and), MRB_SYM(h), ), const);
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_5[] = "\x00\x84\x01\x04\x01\x06\x01\x0d\x02\x07\x03";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_5 = {
0, 173, 11, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_5}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_5_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_5;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_5 = {
32, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_5_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_5 = {
  3,5,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_5,
  NULL,gem_mrblib_mruby_hash_ext_proc_syms_5,gem_mrblib_mruby_hash_ext_proc_reps_5,
  gem_mrblib_mruby_hash_ext_proc_lv_5,
  &gem_mrblib_mruby_hash_ext_proc_debug_5,
  32,0,3,2,0
};
static const mrb_pool_value gem_mrblib_mruby_hash_ext_proc_pool_6[2] = {
{IREP_TT_STR|(15<<2), {"\x4b\x65\x79\x20\x6e\x6f\x74\x20\x66\x6f\x75\x6e\x64\x3a\x20"}},
{IREP_TT_STR|(0<<2), {""}},
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_syms_6, 8, (MRB_SYM(NONE), MRB_SYM_Q(key), MRB_OPSYM(aref), MRB_SYM(call), MRB_OPSYM(neq), MRB_SYM(KeyError), MRB_SYM(inspect), MRB_SYM(raise), ), const);
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_6[106] = {
0x33,0x04,0x20,0x01,0x23,0x00,0x03,0x23,0x00,0x03,0x1d,0x02,0x00,0x12,0x04,0x01,0x05,0x01,0x2d,0x04,
0x01,0x01,0x25,0x04,0x00,0x0c,0x12,0x04,0x01,0x05,0x01,0x2d,0x04,0x02,0x01,0x23,0x00,0x42,0x01,0x04,
0x03,0x25,0x04,0x00,0x0d,0x01,0x04,0x03,0x01,0x05,0x01,0x2d,0x04,0x03,0x01,0x23,0x00,0x2e,0x01,0x04,
0x02,0x1d,0x05,0x00,0x2d,0x04,0x04,0x01,0x25,0x04,0x00,0x06,0x01,0x04,0x02,0x23,0x00,0x1a,0x12,0x04,
0x1d,0x05,0x05,0x50,0x06,0x00,0x01,0x07,0x01,0x2d,0x07,0x06,0x00,0x51,0x06,0x50,0x07,0x01,0x51,0x06,
0x2d,0x04,0x07,0x02,0x37,0x04,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_lv_6, 3, (MRB_SYM(key), MRB_SYM(none), MRB_SYM(block), ), const);
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_6[] = "\x00\xaa\x01\x0d\x01\x0b\x09\x0c\xfd\xff\xff\xff\x0f\x02\xfb\xff\xff\xff\x0f\x05\x01\x0d\x04\x02\xfd\xff\xff\xff\x0f\x0c\x01\x06\x02";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_6 = {
0, 173, 33, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_6}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_6_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_6;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_6 = {
106, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_6_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_6 = {
  4,9,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_6,
  gem_mrblib_mruby_hash_ext_proc_pool_6,gem_mrblib_mruby_hash_ext_proc_syms_6,NULL,
  gem_mrblib_mruby_hash_ext_proc_lv_6,
  &gem_mrblib_mruby_hash_ext_proc_debug_6,
  106,2,8,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_syms_155, 2, (MRB_SYM(call), MRB_SYM(delete), ), const);
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_155[38] = {
0x33,0x08,0x00,0x00,0x21,0x04,0x01,0x00,0x01,0x05,0x01,0x01,0x06,0x02,0x2d,0x04,0x00,0x02,0x25,0x04,
0x00,0x0c,0x12,0x04,0x01,0x05,0x01,0x2d,0x04,0x01,0x01,0x23,0x00,0x02,0x11,0x04,0x37,0x04,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_lv_155, 3, (MRB_SYM(k), MRB_SYM(v), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_155[] = "\x00\xc9\x01\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_155 = {
0, 173, 5, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_155}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_155_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_155;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_155 = {
38, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_155_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_155 = {
  4,8,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_155,
  NULL,gem_mrblib_mruby_hash_ext_proc_syms_155,NULL,
  gem_mrblib_mruby_hash_ext_proc_lv_155,
  &gem_mrblib_mruby_hash_ext_proc_debug_155,
  38,0,2,0,0
};
static const mrb_irep *gem_mrblib_mruby_hash_ext_proc_reps_7[1] = {
  &gem_mrblib_mruby_hash_ext_proc_irep_155,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_syms_7, 3, (MRB_SYM(delete_if), MRB_SYM(to_enum), MRB_SYM(each), ), const);
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_7[35] = {
0x33,0x00,0x00,0x01,0x01,0x02,0x01,0x24,0x02,0x00,0x0b,0x12,0x02,0x10,0x03,0x00,0x2d,0x02,0x01,0x01,
0x37,0x02,0x12,0x02,0x56,0x03,0x00,0x2e,0x02,0x02,0x00,0x12,0x02,0x37,0x02,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_lv_7, 1, (MRB_SYM(block), ), const);
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_7[] = "\x00\xc6\x01\x04\x01\x12\x02\x09\x03";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_7 = {
0, 173, 9, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_7}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_7_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_7;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_7 = {
35, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_7_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_7 = {
  2,5,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_7,
  NULL,gem_mrblib_mruby_hash_ext_proc_syms_7,gem_mrblib_mruby_hash_ext_proc_reps_7,
  gem_mrblib_mruby_hash_ext_proc_lv_7,
  &gem_mrblib_mruby_hash_ext_proc_debug_7,
  35,0,3,1,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_syms_8, 2, (MRB_SYM(to_a), MRB_SYM(flatten), ), const);
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_8[27] = {
0x33,0x00,0x20,0x00,0x23,0x00,0x03,0x23,0x00,0x02,0x07,0x01,0x12,0x03,0x2d,0x03,0x00,0x00,0x01,0x04,
0x01,0x2d,0x03,0x01,0x01,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_lv_8, 2, (MRB_SYM(level), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_8[] = "\x00\xdf\x01\x0c\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_8 = {
0, 173, 5, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_8}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_8_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_8;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_8 = {
27, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_8_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_8 = {
  3,6,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_8,
  NULL,gem_mrblib_mruby_hash_ext_proc_syms_8,NULL,
  gem_mrblib_mruby_hash_ext_proc_lv_8,
  &gem_mrblib_mruby_hash_ext_proc_debug_8,
  27,0,2,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_syms_200, 1, (MRB_OPSYM(aset), ), const);
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_200[23] = {
0x33,0x08,0x00,0x00,0x01,0x04,0x01,0x21,0x05,0x02,0x00,0x01,0x06,0x02,0x01,0x07,0x04,0x2d,0x05,0x00,
0x02,0x37,0x04,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_lv_200, 3, (MRB_SYM(k), MRB_SYM(v), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_200[] = "\x00\xf0\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_200 = {
0, 173, 3, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_200}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_200_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_200;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_200 = {
23, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_200_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_200 = {
  4,9,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_200,
  NULL,gem_mrblib_mruby_hash_ext_proc_syms_200,NULL,
  gem_mrblib_mruby_hash_ext_proc_lv_200,
  &gem_mrblib_mruby_hash_ext_proc_debug_200,
  23,0,1,0,0
};
static const mrb_irep *gem_mrblib_mruby_hash_ext_proc_reps_9[1] = {
  &gem_mrblib_mruby_hash_ext_proc_irep_200,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_syms_9, 3, (MRB_SYM(class), MRB_SYM(new), MRB_SYM(each), ), const);
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_9[28] = {
0x33,0x00,0x00,0x00,0x12,0x03,0x2d,0x03,0x00,0x00,0x2d,0x03,0x01,0x00,0x01,0x02,0x03,0x12,0x03,0x56,
0x04,0x00,0x2e,0x03,0x02,0x00,0x37,0x02,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_lv_9, 2, (MRB_OPSYM(and), MRB_SYM(h), ), const);
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_9[] = "\x00\xee\x01\x04\x01\x0d\x01\x09\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_9 = {
0, 173, 9, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_9}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_9_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_9;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_9 = {
28, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_9_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_9 = {
  3,5,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_9,
  NULL,gem_mrblib_mruby_hash_ext_proc_syms_9,gem_mrblib_mruby_hash_ext_proc_reps_9,
  gem_mrblib_mruby_hash_ext_proc_lv_9,
  &gem_mrblib_mruby_hash_ext_proc_debug_9,
  28,0,3,1,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_syms_223, 2, (MRB_SYM(call), MRB_SYM(delete), ), const);
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_223[41] = {
0x33,0x08,0x00,0x00,0x21,0x04,0x01,0x00,0x01,0x05,0x01,0x01,0x06,0x02,0x46,0x05,0x02,0x2d,0x04,0x00,
0x01,0x25,0x04,0x00,0x05,0x11,0x04,0x23,0x00,0x09,0x12,0x04,0x01,0x05,0x01,0x2d,0x04,0x01,0x01,0x37,
0x04,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_lv_223, 3, (MRB_SYM(k), MRB_SYM(v), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_223[] = "\x00\x82\x02\x04\x01\x18\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_223 = {
0, 173, 7, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_223}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_223_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_223;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_223 = {
41, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_223_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_223 = {
  4,7,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_223,
  NULL,gem_mrblib_mruby_hash_ext_proc_syms_223,NULL,
  gem_mrblib_mruby_hash_ext_proc_lv_223,
  &gem_mrblib_mruby_hash_ext_proc_debug_223,
  41,0,2,0,0
};
static const mrb_irep *gem_mrblib_mruby_hash_ext_proc_reps_10[1] = {
  &gem_mrblib_mruby_hash_ext_proc_irep_223,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_syms_10, 3, (MRB_SYM(keep_if), MRB_SYM(to_enum), MRB_SYM(each), ), const);
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_10[35] = {
0x33,0x00,0x00,0x01,0x01,0x02,0x01,0x24,0x02,0x00,0x0b,0x12,0x02,0x10,0x03,0x00,0x2d,0x02,0x01,0x01,
0x37,0x02,0x12,0x02,0x56,0x03,0x00,0x2e,0x02,0x02,0x00,0x12,0x02,0x37,0x02,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_lv_10, 1, (MRB_SYM(block), ), const);
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_10[] = "\x00\xff\x01\x04\x01\x12\x02\x09\x05";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_10 = {
0, 173, 9, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_10}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_10_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_10;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_10 = {
35, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_10_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_10 = {
  2,5,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_10,
  NULL,gem_mrblib_mruby_hash_ext_proc_syms_10,gem_mrblib_mruby_hash_ext_proc_reps_10,
  gem_mrblib_mruby_hash_ext_proc_lv_10,
  &gem_mrblib_mruby_hash_ext_proc_debug_10,
  35,0,3,1,0
};
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_246[26] = {
0x33,0x08,0x00,0x00,0x01,0x04,0x02,0x21,0x05,0x01,0x00,0x41,0x04,0x25,0x04,0x00,0x05,0x38,0x01,0x23,
0x00,0x02,0x11,0x04,0x37,0x04,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_lv_246, 3, (MRB_SYM(k), MRB_SYM(v), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_246[] = "\x00\x98\x02\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_246 = {
0, 173, 5, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_246}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_246_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_246;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_246 = {
26, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_246_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_246 = {
  4,7,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_246,
  NULL,NULL,NULL,
  gem_mrblib_mruby_hash_ext_proc_lv_246,
  &gem_mrblib_mruby_hash_ext_proc_debug_246,
  26,0,0,0,0
};
static const mrb_irep *gem_mrblib_mruby_hash_ext_proc_reps_11[1] = {
  &gem_mrblib_mruby_hash_ext_proc_irep_246,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_syms_11, 1, (MRB_SYM(each), ), const);
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_11[17] = {
0x33,0x04,0x00,0x00,0x12,0x03,0x56,0x04,0x00,0x2e,0x03,0x00,0x00,0x11,0x03,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_lv_11, 2, (MRB_SYM(val), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_11[] = "\x00\x97\x02\x04\x01\x09\x03";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_11 = {
0, 173, 7, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_11}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_11_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_11;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_11 = {
17, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_11_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_11 = {
  3,5,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_11,
  NULL,gem_mrblib_mruby_hash_ext_proc_syms_11,gem_mrblib_mruby_hash_ext_proc_reps_11,
  gem_mrblib_mruby_hash_ext_proc_lv_11,
  &gem_mrblib_mruby_hash_ext_proc_debug_11,
  17,0,1,1,0
};
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_12[8] = {
0x33,0x00,0x00,0x00,0x12,0x02,0x37,0x02,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_lv_12, 1, (MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_12[] = "\x00\xa5\x02\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_12 = {
0, 173, 5, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_12}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_12_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_12;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_12 = {
8, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_12_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_12 = {
  2,3,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_12,
  NULL,NULL,NULL,
  gem_mrblib_mruby_hash_ext_proc_lv_12,
  &gem_mrblib_mruby_hash_ext_proc_debug_12,
  8,0,0,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_syms_291, 2, (MRB_SYM_Q(key), MRB_OPSYM(aref), ), const);
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_291[37] = {
0x33,0x08,0x00,0x00,0x21,0x04,0x01,0x00,0x01,0x05,0x01,0x2d,0x04,0x00,0x01,0x25,0x04,0x00,0x10,0x21,
0x04,0x01,0x00,0x01,0x05,0x01,0x2d,0x04,0x01,0x01,0x01,0x05,0x02,0x41,0x04,0x37,0x04,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_lv_291, 3, (MRB_SYM(key), MRB_SYM(val), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_291[] = "\x00\xb8\x02\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_291 = {
0, 173, 5, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_291}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_291_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_291;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_291 = {
37, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_291_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_291 = {
  4,7,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_291,
  NULL,gem_mrblib_mruby_hash_ext_proc_syms_291,NULL,
  gem_mrblib_mruby_hash_ext_proc_lv_291,
  &gem_mrblib_mruby_hash_ext_proc_debug_291,
  37,0,2,0,0
};
static const mrb_irep *gem_mrblib_mruby_hash_ext_proc_reps_13[1] = {
  &gem_mrblib_mruby_hash_ext_proc_irep_291,
};
static const mrb_pool_value gem_mrblib_mruby_hash_ext_proc_pool_13[2] = {
{IREP_TT_STR|(14<<2), {"\x63\x61\x6e\x27\x74\x20\x63\x6f\x6e\x76\x65\x72\x74\x20"}},
{IREP_TT_STR|(8<<2), {"\x20\x74\x6f\x20\x48\x61\x73\x68"}},
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_syms_13, 7, (MRB_SYM(Hash), MRB_OPSYM(eqq), MRB_SYM(TypeError), MRB_SYM(class), MRB_SYM(raise), MRB_SYM(size), MRB_SYM_Q(all), ), const);
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_13[74] = {
0x33,0x04,0x00,0x00,0x1d,0x03,0x00,0x01,0x04,0x01,0x2d,0x03,0x01,0x01,0x24,0x03,0x00,0x1a,0x12,0x03,
0x1d,0x04,0x02,0x50,0x05,0x00,0x01,0x06,0x01,0x2d,0x06,0x03,0x00,0x51,0x05,0x50,0x06,0x01,0x51,0x05,
0x2d,0x03,0x04,0x02,0x12,0x03,0x2d,0x03,0x05,0x00,0x01,0x04,0x01,0x2d,0x04,0x05,0x00,0x42,0x03,0x25,
0x03,0x00,0x09,0x12,0x03,0x56,0x04,0x00,0x2e,0x03,0x06,0x00,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_lv_13, 2, (MRB_SYM(hash), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_13[] = "\x00\xb6\x02\x04\x01\x28\x01\x13\x02\x02\xfe\xff\xff\xff\x0f";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_13 = {
0, 173, 15, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_13}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_13_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_13;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_13 = {
74, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_13_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_13 = {
  3,8,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_13,
  gem_mrblib_mruby_hash_ext_proc_pool_13,gem_mrblib_mruby_hash_ext_proc_syms_13,gem_mrblib_mruby_hash_ext_proc_reps_13,
  gem_mrblib_mruby_hash_ext_proc_lv_13,
  &gem_mrblib_mruby_hash_ext_proc_debug_13,
  74,2,7,1,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_syms_314, 2, (MRB_SYM_Q(key), MRB_OPSYM(aref), ), const);
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_314[37] = {
0x33,0x08,0x00,0x00,0x21,0x04,0x01,0x00,0x01,0x05,0x01,0x2d,0x04,0x00,0x01,0x25,0x04,0x00,0x10,0x21,
0x04,0x01,0x00,0x01,0x05,0x01,0x2d,0x04,0x01,0x01,0x01,0x05,0x02,0x41,0x04,0x37,0x04,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_lv_314, 3, (MRB_SYM(key), MRB_SYM(val), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_314[] = "\x00\xcc\x02\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_314 = {
0, 173, 5, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_314}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_314_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_314;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_314 = {
37, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_314_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_314 = {
  4,7,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_314,
  NULL,gem_mrblib_mruby_hash_ext_proc_syms_314,NULL,
  gem_mrblib_mruby_hash_ext_proc_lv_314,
  &gem_mrblib_mruby_hash_ext_proc_debug_314,
  37,0,2,0,0
};
static const mrb_irep *gem_mrblib_mruby_hash_ext_proc_reps_14[1] = {
  &gem_mrblib_mruby_hash_ext_proc_irep_314,
};
static const mrb_pool_value gem_mrblib_mruby_hash_ext_proc_pool_14[2] = {
{IREP_TT_STR|(14<<2), {"\x63\x61\x6e\x27\x74\x20\x63\x6f\x6e\x76\x65\x72\x74\x20"}},
{IREP_TT_STR|(8<<2), {"\x20\x74\x6f\x20\x48\x61\x73\x68"}},
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_syms_14, 7, (MRB_SYM(Hash), MRB_OPSYM(eqq), MRB_SYM(TypeError), MRB_SYM(class), MRB_SYM(raise), MRB_SYM(size), MRB_SYM_Q(all), ), const);
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_14[74] = {
0x33,0x04,0x00,0x00,0x1d,0x03,0x00,0x01,0x04,0x01,0x2d,0x03,0x01,0x01,0x24,0x03,0x00,0x1a,0x12,0x03,
0x1d,0x04,0x02,0x50,0x05,0x00,0x01,0x06,0x01,0x2d,0x06,0x03,0x00,0x51,0x05,0x50,0x06,0x01,0x51,0x05,
0x2d,0x03,0x04,0x02,0x12,0x03,0x2d,0x03,0x05,0x00,0x01,0x04,0x01,0x2d,0x04,0x05,0x00,0x43,0x03,0x25,
0x03,0x00,0x09,0x12,0x03,0x56,0x04,0x00,0x2e,0x03,0x06,0x00,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_lv_14, 2, (MRB_SYM(hash), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_14[] = "\x00\xca\x02\x04\x01\x28\x01\x13\x02\x02\xfe\xff\xff\xff\x0f";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_14 = {
0, 173, 15, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_14}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_14_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_14;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_14 = {
74, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_14_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_14 = {
  3,8,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_14,
  gem_mrblib_mruby_hash_ext_proc_pool_14,gem_mrblib_mruby_hash_ext_proc_syms_14,gem_mrblib_mruby_hash_ext_proc_reps_14,
  gem_mrblib_mruby_hash_ext_proc_lv_14,
  &gem_mrblib_mruby_hash_ext_proc_debug_14,
  74,2,7,1,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_syms_337, 2, (MRB_SYM_Q(key), MRB_OPSYM(aref), ), const);
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_337[33] = {
0x33,0x08,0x00,0x00,0x12,0x04,0x01,0x05,0x01,0x2d,0x04,0x00,0x01,0x25,0x04,0x00,0x0e,0x12,0x04,0x01,
0x05,0x01,0x2d,0x04,0x01,0x01,0x01,0x05,0x02,0x41,0x04,0x37,0x04,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_lv_337, 3, (MRB_SYM(key), MRB_SYM(val), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_337[] = "\x00\xe0\x02\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_337 = {
0, 173, 5, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_337}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_337_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_337;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_337 = {
33, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_337_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_337 = {
  4,7,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_337,
  NULL,gem_mrblib_mruby_hash_ext_proc_syms_337,NULL,
  gem_mrblib_mruby_hash_ext_proc_lv_337,
  &gem_mrblib_mruby_hash_ext_proc_debug_337,
  33,0,2,0,0
};
static const mrb_irep *gem_mrblib_mruby_hash_ext_proc_reps_15[1] = {
  &gem_mrblib_mruby_hash_ext_proc_irep_337,
};
static const mrb_pool_value gem_mrblib_mruby_hash_ext_proc_pool_15[2] = {
{IREP_TT_STR|(14<<2), {"\x63\x61\x6e\x27\x74\x20\x63\x6f\x6e\x76\x65\x72\x74\x20"}},
{IREP_TT_STR|(8<<2), {"\x20\x74\x6f\x20\x48\x61\x73\x68"}},
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_syms_15, 7, (MRB_SYM(Hash), MRB_OPSYM(eqq), MRB_SYM(TypeError), MRB_SYM(class), MRB_SYM(raise), MRB_SYM(size), MRB_SYM_Q(all), ), const);
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_15[75] = {
0x33,0x04,0x00,0x00,0x1d,0x03,0x00,0x01,0x04,0x01,0x2d,0x03,0x01,0x01,0x24,0x03,0x00,0x1a,0x12,0x03,
0x1d,0x04,0x02,0x50,0x05,0x00,0x01,0x06,0x01,0x2d,0x06,0x03,0x00,0x51,0x05,0x50,0x06,0x01,0x51,0x05,
0x2d,0x03,0x04,0x02,0x12,0x03,0x2d,0x03,0x05,0x00,0x01,0x04,0x01,0x2d,0x04,0x05,0x00,0x44,0x03,0x25,
0x03,0x00,0x0a,0x01,0x03,0x01,0x56,0x04,0x00,0x2e,0x03,0x06,0x00,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_lv_15, 2, (MRB_SYM(hash), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_15[] = "\x00\xde\x02\x04\x01\x28\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_15 = {
0, 173, 7, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_15}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_15_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_15;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_15 = {
75, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_15_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_15 = {
  3,8,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_15,
  gem_mrblib_mruby_hash_ext_proc_pool_15,gem_mrblib_mruby_hash_ext_proc_syms_15,gem_mrblib_mruby_hash_ext_proc_reps_15,
  gem_mrblib_mruby_hash_ext_proc_lv_15,
  &gem_mrblib_mruby_hash_ext_proc_debug_15,
  75,2,7,1,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_syms_360, 2, (MRB_SYM_Q(key), MRB_OPSYM(aref), ), const);
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_360[33] = {
0x33,0x08,0x00,0x00,0x12,0x04,0x01,0x05,0x01,0x2d,0x04,0x00,0x01,0x25,0x04,0x00,0x0e,0x12,0x04,0x01,
0x05,0x01,0x2d,0x04,0x01,0x01,0x01,0x05,0x02,0x41,0x04,0x37,0x04,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_lv_360, 3, (MRB_SYM(key), MRB_SYM(val), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_360[] = "\x00\xf4\x02\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_360 = {
0, 173, 5, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_360}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_360_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_360;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_360 = {
33, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_360_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_360 = {
  4,7,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_360,
  NULL,gem_mrblib_mruby_hash_ext_proc_syms_360,NULL,
  gem_mrblib_mruby_hash_ext_proc_lv_360,
  &gem_mrblib_mruby_hash_ext_proc_debug_360,
  33,0,2,0,0
};
static const mrb_irep *gem_mrblib_mruby_hash_ext_proc_reps_16[1] = {
  &gem_mrblib_mruby_hash_ext_proc_irep_360,
};
static const mrb_pool_value gem_mrblib_mruby_hash_ext_proc_pool_16[2] = {
{IREP_TT_STR|(14<<2), {"\x63\x61\x6e\x27\x74\x20\x63\x6f\x6e\x76\x65\x72\x74\x20"}},
{IREP_TT_STR|(8<<2), {"\x20\x74\x6f\x20\x48\x61\x73\x68"}},
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_syms_16, 7, (MRB_SYM(Hash), MRB_OPSYM(eqq), MRB_SYM(TypeError), MRB_SYM(class), MRB_SYM(raise), MRB_SYM(size), MRB_SYM_Q(all), ), const);
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_16[75] = {
0x33,0x04,0x00,0x00,0x1d,0x03,0x00,0x01,0x04,0x01,0x2d,0x03,0x01,0x01,0x24,0x03,0x00,0x1a,0x12,0x03,
0x1d,0x04,0x02,0x50,0x05,0x00,0x01,0x06,0x01,0x2d,0x06,0x03,0x00,0x51,0x05,0x50,0x06,0x01,0x51,0x05,
0x2d,0x03,0x04,0x02,0x12,0x03,0x2d,0x03,0x05,0x00,0x01,0x04,0x01,0x2d,0x04,0x05,0x00,0x45,0x03,0x25,
0x03,0x00,0x0a,0x01,0x03,0x01,0x56,0x04,0x00,0x2e,0x03,0x06,0x00,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_lv_16, 2, (MRB_SYM(hash), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_16[] = "\x00\xf2\x02\x04\x01\x28\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_16 = {
0, 173, 7, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_16}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_16_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_16;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_16 = {
75, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_16_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_16 = {
  3,8,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_16,
  gem_mrblib_mruby_hash_ext_proc_pool_16,gem_mrblib_mruby_hash_ext_proc_syms_16,gem_mrblib_mruby_hash_ext_proc_reps_16,
  gem_mrblib_mruby_hash_ext_proc_lv_16,
  &gem_mrblib_mruby_hash_ext_proc_debug_16,
  75,2,7,1,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_syms_17, 3, (MRB_OPSYM(aref), MRB_SYM(size), MRB_SYM(dig), ), const);
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_17[59] = {
0x33,0x04,0x10,0x00,0x12,0x05,0x01,0x06,0x01,0x2d,0x05,0x00,0x01,0x01,0x04,0x05,0x01,0x05,0x02,0x2d,
0x05,0x01,0x00,0x06,0x06,0x44,0x05,0x25,0x05,0x00,0x17,0x01,0x05,0x04,0x01,0x06,0x05,0x26,0x06,0x00,
0x0a,0x11,0x06,0x01,0x07,0x02,0x48,0x06,0x2b,0x05,0x02,0x23,0x00,0x03,0x01,0x05,0x04,0x37,0x05,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_lv_17, 4, (MRB_SYM(idx), MRB_SYM(args), MRB_OPSYM(and), MRB_SYM(n), ), const);
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_17[] = "\x00\x81\x03\x04\x01\x0c\x01\x0d\x01\x17\x02";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_17 = {
0, 173, 11, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_17}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_17_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_17;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_17 = {
59, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_17_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_17 = {
  5,8,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_17,
  NULL,gem_mrblib_mruby_hash_ext_proc_syms_17,NULL,
  gem_mrblib_mruby_hash_ext_proc_lv_17,
  &gem_mrblib_mruby_hash_ext_proc_debug_17,
  59,0,3,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_syms_405, 3, (MRB_SYM(call), MRB_OPSYM(aref), MRB_OPSYM(aset), ), const);
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_405[43] = {
0x33,0x04,0x00,0x00,0x21,0x04,0x01,0x00,0x01,0x05,0x01,0x2d,0x04,0x00,0x01,0x01,0x03,0x04,0x12,0x04,
0x01,0x05,0x01,0x2d,0x04,0x01,0x01,0x21,0x05,0x02,0x00,0x01,0x06,0x03,0x01,0x07,0x04,0x2d,0x05,0x02,
0x02,0x37,0x04,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_lv_405, 3, (MRB_SYM(k), MRB_OPSYM(and), MRB_SYM(new_key), ), const);
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_405[] = "\x00\x97\x03\x04\x01\x0e\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_405 = {
0, 173, 7, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_405}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_405_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_405;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_405 = {
43, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_405_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_405 = {
  4,9,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_405,
  NULL,gem_mrblib_mruby_hash_ext_proc_syms_405,NULL,
  gem_mrblib_mruby_hash_ext_proc_lv_405,
  &gem_mrblib_mruby_hash_ext_proc_debug_405,
  43,0,3,0,0
};
static const mrb_irep *gem_mrblib_mruby_hash_ext_proc_reps_18[1] = {
  &gem_mrblib_mruby_hash_ext_proc_irep_405,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_syms_18, 4, (MRB_SYM(transform_keys), MRB_SYM(to_enum), MRB_SYM(keys), MRB_SYM(each), ), const);
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_18[43] = {
0x33,0x00,0x00,0x01,0x01,0x03,0x01,0x24,0x03,0x00,0x0b,0x12,0x03,0x10,0x04,0x00,0x2d,0x03,0x01,0x01,
0x37,0x03,0x52,0x03,0x00,0x01,0x02,0x03,0x12,0x03,0x2d,0x03,0x02,0x00,0x56,0x04,0x00,0x2e,0x03,0x03,
0x00,0x37,0x02,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_lv_18, 2, (MRB_SYM(block), MRB_SYM(hash), ), const);
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_18[] = "\x00\x94\x03\x04\x01\x12\x01\x06\x01\x0d\x04";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_18 = {
0, 173, 11, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_18}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_18_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_18;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_18 = {
43, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_18_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_18 = {
  3,6,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_18,
  NULL,gem_mrblib_mruby_hash_ext_proc_syms_18,gem_mrblib_mruby_hash_ext_proc_reps_18,
  gem_mrblib_mruby_hash_ext_proc_lv_18,
  &gem_mrblib_mruby_hash_ext_proc_debug_18,
  43,0,4,1,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_syms_428, 4, (MRB_OPSYM(aref), MRB_SYM(__delete), MRB_SYM(call), MRB_OPSYM(aset), ), const);
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_428[64] = {
0x33,0x04,0x00,0x00,0x12,0x04,0x01,0x05,0x01,0x2d,0x04,0x00,0x01,0x01,0x03,0x04,0x12,0x04,0x01,0x05,
0x01,0x2d,0x04,0x01,0x01,0x21,0x04,0x01,0x00,0x25,0x04,0x00,0x0e,0x21,0x04,0x01,0x00,0x01,0x05,0x01,
0x2d,0x04,0x02,0x01,0x01,0x01,0x04,0x01,0x04,0x03,0x12,0x05,0x01,0x06,0x01,0x01,0x07,0x04,0x2d,0x05,
0x03,0x02,0x37,0x04,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_lv_428, 3, (MRB_SYM(k), MRB_OPSYM(and), MRB_SYM(value), ), const);
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_428[] = "\x00\xa9\x03\x04\x01\x0c\x01\x09\x01\x16\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_428 = {
0, 173, 11, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_428}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_428_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_428;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_428 = {
64, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_428_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_428 = {
  4,9,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_428,
  NULL,gem_mrblib_mruby_hash_ext_proc_syms_428,NULL,
  gem_mrblib_mruby_hash_ext_proc_lv_428,
  &gem_mrblib_mruby_hash_ext_proc_debug_428,
  64,0,4,0,0
};
static const mrb_irep *gem_mrblib_mruby_hash_ext_proc_reps_19[1] = {
  &gem_mrblib_mruby_hash_ext_proc_irep_428,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_syms_19, 4, (MRB_SYM_B(transform_keys), MRB_SYM(to_enum), MRB_SYM(keys), MRB_SYM(each), ), const);
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_19[39] = {
0x33,0x00,0x00,0x01,0x01,0x02,0x01,0x24,0x02,0x00,0x0b,0x12,0x02,0x10,0x03,0x00,0x2d,0x02,0x01,0x01,
0x37,0x02,0x12,0x02,0x2d,0x02,0x02,0x00,0x56,0x03,0x00,0x2e,0x02,0x03,0x00,0x12,0x02,0x37,0x02,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_lv_19, 1, (MRB_SYM(block), ), const);
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_19[] = "\x00\xa7\x03\x04\x01\x12\x01\x0d\x06";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_19 = {
0, 173, 9, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_19}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_19_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_19;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_19 = {
39, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_19_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_19 = {
  2,5,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_19,
  NULL,gem_mrblib_mruby_hash_ext_proc_syms_19,gem_mrblib_mruby_hash_ext_proc_reps_19,
  gem_mrblib_mruby_hash_ext_proc_lv_19,
  &gem_mrblib_mruby_hash_ext_proc_debug_19,
  39,0,4,1,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_syms_451, 3, (MRB_OPSYM(aref), MRB_SYM(call), MRB_OPSYM(aset), ), const);
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_451[37] = {
0x33,0x04,0x00,0x00,0x12,0x04,0x01,0x05,0x01,0x2d,0x04,0x00,0x01,0x3a,0x03,0x00,0x01,0x2d,0x03,0x01,
0x01,0x21,0x04,0x02,0x00,0x01,0x05,0x01,0x01,0x06,0x03,0x2d,0x04,0x02,0x02,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_lv_451, 2, (MRB_SYM(k), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_451[] = "\x00\xbf\x03\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_451 = {
0, 173, 5, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_451}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_451_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_451;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_451 = {
37, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_451_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_451 = {
  3,8,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_451,
  NULL,gem_mrblib_mruby_hash_ext_proc_syms_451,NULL,
  gem_mrblib_mruby_hash_ext_proc_lv_451,
  &gem_mrblib_mruby_hash_ext_proc_debug_451,
  37,0,3,0,0
};
static const mrb_irep *gem_mrblib_mruby_hash_ext_proc_reps_20[1] = {
  &gem_mrblib_mruby_hash_ext_proc_irep_451,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_syms_20, 5, (MRB_SYM_Q(block_given), MRB_SYM(transform_values), MRB_SYM(to_enum), MRB_SYM(keys), MRB_SYM(each), ), const);
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_20[46] = {
0x33,0x00,0x00,0x01,0x12,0x03,0x2d,0x03,0x00,0x00,0x24,0x03,0x00,0x0b,0x12,0x03,0x10,0x04,0x01,0x2d,
0x03,0x02,0x01,0x37,0x03,0x52,0x03,0x00,0x01,0x02,0x03,0x12,0x03,0x2d,0x03,0x03,0x00,0x56,0x04,0x00,
0x2e,0x03,0x04,0x00,0x37,0x02,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_lv_20, 2, (MRB_SYM(b), MRB_SYM(hash), ), const);
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_20[] = "\x00\xbc\x03\x04\x01\x15\x01\x06\x01\x0d\x03";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_20 = {
0, 173, 11, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_20}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_20_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_20;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_20 = {
46, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_20_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_20 = {
  3,6,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_20,
  NULL,gem_mrblib_mruby_hash_ext_proc_syms_20,gem_mrblib_mruby_hash_ext_proc_reps_20,
  gem_mrblib_mruby_hash_ext_proc_lv_20,
  &gem_mrblib_mruby_hash_ext_proc_debug_20,
  46,0,5,1,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_syms_474, 3, (MRB_OPSYM(aref), MRB_SYM(call), MRB_OPSYM(aset), ), const);
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_474[35] = {
0x33,0x04,0x00,0x00,0x12,0x04,0x01,0x05,0x01,0x2d,0x04,0x00,0x01,0x3a,0x03,0x00,0x01,0x2d,0x03,0x01,
0x01,0x12,0x04,0x01,0x05,0x01,0x01,0x06,0x03,0x2d,0x04,0x02,0x02,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_lv_474, 2, (MRB_SYM(k), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_474[] = "\x00\xd1\x03\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_474 = {
0, 173, 5, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_474}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_474_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_474;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_474 = {
35, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_474_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_474 = {
  3,8,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_474,
  NULL,gem_mrblib_mruby_hash_ext_proc_syms_474,NULL,
  gem_mrblib_mruby_hash_ext_proc_lv_474,
  &gem_mrblib_mruby_hash_ext_proc_debug_474,
  35,0,3,0,0
};
static const mrb_irep *gem_mrblib_mruby_hash_ext_proc_reps_21[1] = {
  &gem_mrblib_mruby_hash_ext_proc_irep_474,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_syms_21, 5, (MRB_SYM_Q(block_given), MRB_SYM_B(transform_values), MRB_SYM(to_enum), MRB_SYM(keys), MRB_SYM(each), ), const);
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_21[42] = {
0x33,0x00,0x00,0x01,0x12,0x02,0x2d,0x02,0x00,0x00,0x24,0x02,0x00,0x0b,0x12,0x02,0x10,0x03,0x01,0x2d,
0x02,0x02,0x01,0x37,0x02,0x12,0x02,0x2d,0x02,0x03,0x00,0x56,0x03,0x00,0x2e,0x02,0x04,0x00,0x12,0x02,
0x37,0x02,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_lv_21, 1, (MRB_SYM(b), ), const);
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_21[] = "\x00\xcf\x03\x04\x01\x15\x01\x0d\x03";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_21 = {
0, 173, 9, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_21}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_21_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_21;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_21 = {
42, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_21_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_21 = {
  2,5,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_21,
  NULL,gem_mrblib_mruby_hash_ext_proc_syms_21,gem_mrblib_mruby_hash_ext_proc_reps_21,
  gem_mrblib_mruby_hash_ext_proc_lv_21,
  &gem_mrblib_mruby_hash_ext_proc_debug_21,
  42,0,5,1,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_syms_497, 1, (MRB_OPSYM(aref), ), const);
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_497[15] = {
0x33,0x04,0x00,0x00,0x12,0x03,0x01,0x04,0x01,0x2d,0x03,0x00,0x01,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_lv_497, 2, (MRB_SYM(x), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_497[] = "\x00\xd8\x03";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_497 = {
0, 173, 3, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_497}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_497_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_497;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_497 = {
15, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_497_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_497 = {
  3,6,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_497,
  NULL,gem_mrblib_mruby_hash_ext_proc_syms_497,NULL,
  gem_mrblib_mruby_hash_ext_proc_lv_497,
  &gem_mrblib_mruby_hash_ext_proc_debug_497,
  15,0,1,0,0
};
static const mrb_irep *gem_mrblib_mruby_hash_ext_proc_reps_22[1] = {
  &gem_mrblib_mruby_hash_ext_proc_irep_497,
};
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_22[9] = {
0x33,0x00,0x00,0x00,0x55,0x02,0x00,0x37,0x02,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_lv_22, 1, (MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_22[] = "\x00\xd7\x03\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_22 = {
0, 173, 5, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_22}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_22_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_22;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_22 = {
9, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_22_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_22 = {
  2,3,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_22,
  NULL,NULL,gem_mrblib_mruby_hash_ext_proc_reps_22,
  gem_mrblib_mruby_hash_ext_proc_lv_22,
  &gem_mrblib_mruby_hash_ext_proc_debug_22,
  9,0,0,1,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_syms_520, 1, (MRB_SYM(fetch), ), const);
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_520[19] = {
0x33,0x04,0x00,0x00,0x12,0x03,0x01,0x04,0x01,0x21,0x05,0x02,0x00,0x2e,0x03,0x00,0x01,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_lv_520, 2, (MRB_SYM(k), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_520[] = "\x00\xeb\x03\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_520 = {
0, 173, 5, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_520}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_520_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_520;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_520 = {
19, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_520_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_520 = {
  3,6,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_520,
  NULL,gem_mrblib_mruby_hash_ext_proc_syms_520,NULL,
  gem_mrblib_mruby_hash_ext_proc_lv_520,
  &gem_mrblib_mruby_hash_ext_proc_debug_520,
  19,0,1,0,0
};
static const mrb_irep *gem_mrblib_mruby_hash_ext_proc_reps_23[1] = {
  &gem_mrblib_mruby_hash_ext_proc_irep_520,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_syms_23, 1, (MRB_SYM(map), ), const);
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_23[16] = {
0x33,0x00,0x10,0x01,0x01,0x03,0x01,0x56,0x04,0x00,0x2e,0x03,0x00,0x00,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_lv_23, 2, (MRB_SYM(keys), MRB_SYM(block), ), const);
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_23[] = "\x00\xea\x03\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_23 = {
0, 173, 5, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_23}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_23_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_23;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_23 = {
16, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_23_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_23 = {
  3,5,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_23,
  NULL,gem_mrblib_mruby_hash_ext_proc_syms_23,gem_mrblib_mruby_hash_ext_proc_reps_23,
  gem_mrblib_mruby_hash_ext_proc_lv_23,
  &gem_mrblib_mruby_hash_ext_proc_debug_23,
  16,0,1,1,0
};
static const mrb_irep *gem_mrblib_mruby_hash_ext_proc_reps_1[22] = {
  &gem_mrblib_mruby_hash_ext_proc_irep_2,
  &gem_mrblib_mruby_hash_ext_proc_irep_3,
  &gem_mrblib_mruby_hash_ext_proc_irep_4,
  &gem_mrblib_mruby_hash_ext_proc_irep_5,
  &gem_mrblib_mruby_hash_ext_proc_irep_6,
  &gem_mrblib_mruby_hash_ext_proc_irep_7,
  &gem_mrblib_mruby_hash_ext_proc_irep_8,
  &gem_mrblib_mruby_hash_ext_proc_irep_9,
  &gem_mrblib_mruby_hash_ext_proc_irep_10,
  &gem_mrblib_mruby_hash_ext_proc_irep_11,
  &gem_mrblib_mruby_hash_ext_proc_irep_12,
  &gem_mrblib_mruby_hash_ext_proc_irep_13,
  &gem_mrblib_mruby_hash_ext_proc_irep_14,
  &gem_mrblib_mruby_hash_ext_proc_irep_15,
  &gem_mrblib_mruby_hash_ext_proc_irep_16,
  &gem_mrblib_mruby_hash_ext_proc_irep_17,
  &gem_mrblib_mruby_hash_ext_proc_irep_18,
  &gem_mrblib_mruby_hash_ext_proc_irep_19,
  &gem_mrblib_mruby_hash_ext_proc_irep_20,
  &gem_mrblib_mruby_hash_ext_proc_irep_21,
  &gem_mrblib_mruby_hash_ext_proc_irep_22,
  &gem_mrblib_mruby_hash_ext_proc_irep_23,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_syms_1, 29, (MRB_SYM(each_pair), MRB_SYM(each), MRB_OPSYM(aref), MRB_SYM_B(merge), MRB_SYM(update), MRB_SYM_B(compact), MRB_SYM(compact), MRB_SYM(fetch), MRB_SYM(delete_if), MRB_SYM(flatten), MRB_SYM(invert), MRB_SYM(keep_if), MRB_SYM(key), MRB_SYM(to_h), MRB_OPSYM(lt), MRB_OPSYM(le), MRB_OPSYM(gt), MRB_OPSYM(ge), MRB_SYM(dig), MRB_SYM(transform_keys), MRB_SYM_B(transform_keys), MRB_SYM(transform_values), MRB_SYM_B(transform_values), MRB_SYM(to_proc), MRB_SYM(fetch_values), MRB_SYM(filter), MRB_SYM(select), MRB_SYM_B(filter), MRB_SYM_B(select), ), const);
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_1[194] = {
0x5f,0x00,0x01,0x12,0x01,0x61,0x01,0x57,0x02,0x00,0x5e,0x01,0x02,0x62,0x01,0x57,0x02,0x01,0x5e,0x01,
0x03,0x5f,0x04,0x03,0x62,0x01,0x57,0x02,0x02,0x5e,0x01,0x05,0x62,0x01,0x57,0x02,0x03,0x5e,0x01,0x06,
0x62,0x01,0x57,0x02,0x04,0x5e,0x01,0x07,0x62,0x01,0x57,0x02,0x05,0x5e,0x01,0x08,0x62,0x01,0x57,0x02,
0x06,0x5e,0x01,0x09,0x62,0x01,0x57,0x02,0x07,0x5e,0x01,0x0a,0x62,0x01,0x57,0x02,0x08,0x5e,0x01,0x0b,
0x62,0x01,0x57,0x02,0x09,0x5e,0x01,0x0c,0x62,0x01,0x57,0x02,0x0a,0x5e,0x01,0x0d,0x62,0x01,0x57,0x02,
0x0b,0x5e,0x01,0x0e,0x62,0x01,0x57,0x02,0x0c,0x5e,0x01,0x0f,0x62,0x01,0x57,0x02,0x0d,0x5e,0x01,0x10,
0x62,0x01,0x57,0x02,0x0e,0x5e,0x01,0x11,0x62,0x01,0x57,0x02,0x0f,0x5e,0x01,0x12,0x62,0x01,0x57,0x02,
0x10,0x5e,0x01,0x13,0x62,0x01,0x57,0x02,0x11,0x5e,0x01,0x14,0x62,0x01,0x57,0x02,0x12,0x5e,0x01,0x15,
0x62,0x01,0x57,0x02,0x13,0x5e,0x01,0x16,0x62,0x01,0x57,0x02,0x14,0x5e,0x01,0x17,0x62,0x01,0x57,0x02,
0x15,0x5e,0x01,0x18,0x5f,0x19,0x1a,0x5f,0x1b,0x1c,0x11,0x01,0x37,0x01,};
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_1[] = "\x00\x04\x03\x16\x0a\x3a\x08\x0c\x03\x0d\x08\x17\x08\x26\x08\x1c\x08\x19\x08\x0f\x08\x11\x08\x18\x08\x0e\x08\x11\x08\x14\x08\x14\x08\x14\x08\x0f\x08\x13\x08\x13\x08\x15\x08\x13\x08\x08\x08\x13\x08\x06\x03\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_1 = {
0, 173, 52, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_1}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_1_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_1;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_1 = {
194, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_1_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_1 = {
  1,3,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_1,
  NULL,gem_mrblib_mruby_hash_ext_proc_syms_1,gem_mrblib_mruby_hash_ext_proc_reps_1,
  NULL,					/* lv */
  &gem_mrblib_mruby_hash_ext_proc_debug_1,
  194,0,29,22,0
};
static const mrb_irep *gem_mrblib_mruby_hash_ext_proc_reps_0[1] = {
  &gem_mrblib_mruby_hash_ext_proc_irep_1,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_hash_ext_proc_syms_0, 1, (MRB_SYM(Hash), ), const);
static const mrb_code gem_mrblib_mruby_hash_ext_proc_iseq_0[13] = {
0x11,0x01,0x11,0x02,0x5b,0x01,0x00,0x5d,0x01,0x00,0x37,0x01,0x68,};
static char gem_mrblib_mruby_hash_ext_proc_debug_lines_0[] = "\x00\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_hash_ext_proc_debug_file_0 = {
0, 173, 2, mrb_debug_line_packed_map, {gem_mrblib_mruby_hash_ext_proc_debug_lines_0}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_hash_ext_proc_debug_file_0_ = &gem_mrblib_mruby_hash_ext_proc_debug_file_0;
static mrb_irep_debug_info gem_mrblib_mruby_hash_ext_proc_debug_0 = {
13, 1, &gem_mrblib_mruby_hash_ext_proc_debug_file_0_};
static const mrb_irep gem_mrblib_mruby_hash_ext_proc_irep_0 = {
  1,3,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_hash_ext_proc_iseq_0,
  NULL,gem_mrblib_mruby_hash_ext_proc_syms_0,gem_mrblib_mruby_hash_ext_proc_reps_0,
  NULL,					/* lv */
  &gem_mrblib_mruby_hash_ext_proc_debug_0,
  13,0,1,1,0
};
static
const struct RProc gem_mrblib_mruby_hash_ext_proc[] = {{
NULL,NULL,MRB_TT_PROC,MRB_GC_RED,0,{&gem_mrblib_mruby_hash_ext_proc_irep_0},NULL,{NULL},
}};
static void
gem_mrblib_mruby_hash_ext_proc_init_syms(mrb_state *mrb)
{
  gem_mrblib_mruby_hash_ext_proc_debug_file_24.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_25.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_26.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_2.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_55.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_56.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_3.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_81.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_82.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_4.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_107.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_108.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_5.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_6.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_155.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_7.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_8.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_200.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_9.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_223.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_10.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_246.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_11.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_12.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_291.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_13.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_314.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_14.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_337.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_15.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_360.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_16.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_17.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_405.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_18.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_428.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_19.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_451.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_20.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_474.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_21.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_497.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_22.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_520.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_23.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_1.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
  gem_mrblib_mruby_hash_ext_proc_debug_file_0.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-hash-ext/mrblib/hash.rb");
}
void mrb_mruby_hash_ext_gem_init(mrb_state *mrb);
void mrb_mruby_hash_ext_gem_final(mrb_state *mrb);

void GENERATED_TMP_mrb_mruby_hash_ext_gem_init(mrb_state *mrb) {
  int ai = mrb_gc_arena_save(mrb);
  gem_mrblib_mruby_hash_ext_proc_init_syms(mrb);
  mrb_mruby_hash_ext_gem_init(mrb);
  mrb_load_proc(mrb, gem_mrblib_mruby_hash_ext_proc);
  if (mrb->exc) {
    mrb_print_error(mrb);
    mrb_close(mrb);
    exit(EXIT_FAILURE);
  }
  struct REnv *e = mrb_vm_ci_env(mrb->c->cibase);
  mrb_vm_ci_env_set(mrb->c->cibase, NULL);
  mrb_env_unshare(mrb, e);
  mrb_gc_arena_restore(mrb, ai);
}

void GENERATED_TMP_mrb_mruby_hash_ext_gem_final(mrb_state *mrb) {
  mrb_mruby_hash_ext_gem_final(mrb);
}
