/*
 * This file is loading the irep
 * Ruby GEM code.
 *
 * IMPORTANT:
 *   This file was generated!
 *   All manual changes will get lost.
 */
#include <stdlib.h>
#include <mruby.h>
#include <mruby/irep.h>
#include <mruby/debug.h>
#include <mruby/proc.h>
#include <mruby/presym.h>

#define mrb_BRACED(...) {__VA_ARGS__}
#define mrb_DEFINE_SYMS_VAR(name, len, syms, qualifier) \
  static qualifier mrb_sym name[len] = mrb_BRACED syms

static const mrb_pool_value gem_mrblib_mruby_string_ext_proc_pool_2[1] = {
{IREP_TT_STR|(0<<2), {""}},
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_syms_2, 1, (MRB_SYM(replace), ), const);
static const mrb_code gem_mrblib_mruby_string_ext_proc_iseq_2[15] = {
0x33,0x00,0x00,0x00,0x12,0x02,0x50,0x03,0x00,0x2d,0x02,0x00,0x01,0x37,0x02,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_lv_2, 1, (MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_string_ext_proc_debug_lines_2[] = "\x00\x0c\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_string_ext_proc_debug_file_2 = {
0, 173, 4, mrb_debug_line_packed_map, {gem_mrblib_mruby_string_ext_proc_debug_lines_2}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_string_ext_proc_debug_file_2_ = &gem_mrblib_mruby_string_ext_proc_debug_file_2;
static mrb_irep_debug_info gem_mrblib_mruby_string_ext_proc_debug_2 = {
15, 1, &gem_mrblib_mruby_string_ext_proc_debug_file_2_};
static const mrb_irep gem_mrblib_mruby_string_ext_proc_irep_2 = {
  2,5,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_string_ext_proc_iseq_2,
  gem_mrblib_mruby_string_ext_proc_pool_2,gem_mrblib_mruby_string_ext_proc_syms_2,NULL,
  gem_mrblib_mruby_string_ext_proc_lv_2,
  &gem_mrblib_mruby_string_ext_proc_debug_2,
  15,1,1,0,0
};
static const mrb_pool_value gem_mrblib_mruby_string_ext_proc_pool_3[2] = {
{IREP_TT_STR|(6<<2), {"\x20\x0c\x0a\x0d\x09\x0b"}},
{IREP_TT_STR|(0<<2), {""}},
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_syms_3, 3, (MRB_SYM(size), MRB_OPSYM(aref), MRB_SYM_Q(include), ), const);
static const mrb_code gem_mrblib_mruby_string_ext_proc_iseq_3[89] = {
0x33,0x00,0x00,0x00,0x06,0x02,0x12,0x04,0x2d,0x04,0x00,0x00,0x3e,0x04,0x01,0x01,0x03,0x04,0x01,0x04,
0x02,0x01,0x05,0x03,0x43,0x04,0x25,0x04,0x00,0x10,0x50,0x04,0x00,0x12,0x05,0x01,0x06,0x02,0x2d,0x05,
0x01,0x01,0x2d,0x04,0x02,0x01,0x25,0x04,0x00,0x06,0x3c,0x02,0x01,0x23,0xff,0xda,0x01,0x04,0x03,0x06,
0x05,0x45,0x04,0x25,0x04,0x00,0x11,0x12,0x04,0x01,0x05,0x02,0x01,0x06,0x03,0x58,0x05,0x2d,0x04,0x01,
0x01,0x23,0x00,0x03,0x50,0x04,0x01,0x37,0x04,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_lv_3, 3, (MRB_OPSYM(and), MRB_SYM(a), MRB_SYM(z), ), const);
static char gem_mrblib_mruby_string_ext_proc_debug_lines_3[] = "\x00\x1a\x04\x01\x02\x01\x0c\x01\x26\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_string_ext_proc_debug_file_3 = {
0, 173, 10, mrb_debug_line_packed_map, {gem_mrblib_mruby_string_ext_proc_debug_lines_3}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_string_ext_proc_debug_file_3_ = &gem_mrblib_mruby_string_ext_proc_debug_file_3;
static mrb_irep_debug_info gem_mrblib_mruby_string_ext_proc_debug_3 = {
89, 1, &gem_mrblib_mruby_string_ext_proc_debug_file_3_};
static const mrb_irep gem_mrblib_mruby_string_ext_proc_irep_3 = {
  4,8,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_string_ext_proc_iseq_3,
  gem_mrblib_mruby_string_ext_proc_pool_3,gem_mrblib_mruby_string_ext_proc_syms_3,NULL,
  gem_mrblib_mruby_string_ext_proc_lv_3,
  &gem_mrblib_mruby_string_ext_proc_debug_3,
  89,2,3,0,0
};
static const mrb_pool_value gem_mrblib_mruby_string_ext_proc_pool_4[2] = {
{IREP_TT_STR|(7<<2), {"\x20\x0c\x0a\x0d\x09\x0b\x00"}},
{IREP_TT_STR|(0<<2), {""}},
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_syms_4, 3, (MRB_SYM(size), MRB_OPSYM(aref), MRB_SYM_Q(include), ), const);
static const mrb_code gem_mrblib_mruby_string_ext_proc_iseq_4[89] = {
0x33,0x00,0x00,0x00,0x06,0x02,0x12,0x04,0x2d,0x04,0x00,0x00,0x3e,0x04,0x01,0x01,0x03,0x04,0x01,0x04,
0x02,0x01,0x05,0x03,0x43,0x04,0x25,0x04,0x00,0x10,0x50,0x04,0x00,0x12,0x05,0x01,0x06,0x03,0x2d,0x05,
0x01,0x01,0x2d,0x04,0x02,0x01,0x25,0x04,0x00,0x06,0x3e,0x03,0x01,0x23,0xff,0xda,0x01,0x04,0x03,0x06,
0x05,0x45,0x04,0x25,0x04,0x00,0x11,0x12,0x04,0x01,0x05,0x02,0x01,0x06,0x03,0x58,0x05,0x2d,0x04,0x01,
0x01,0x23,0x00,0x03,0x50,0x04,0x01,0x37,0x04,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_lv_4, 3, (MRB_OPSYM(and), MRB_SYM(a), MRB_SYM(z), ), const);
static char gem_mrblib_mruby_string_ext_proc_debug_lines_4[] = "\x00\x2b\x04\x01\x02\x01\x0c\x01\x26\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_string_ext_proc_debug_file_4 = {
0, 173, 10, mrb_debug_line_packed_map, {gem_mrblib_mruby_string_ext_proc_debug_lines_4}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_string_ext_proc_debug_file_4_ = &gem_mrblib_mruby_string_ext_proc_debug_file_4;
static mrb_irep_debug_info gem_mrblib_mruby_string_ext_proc_debug_4 = {
89, 1, &gem_mrblib_mruby_string_ext_proc_debug_file_4_};
static const mrb_irep gem_mrblib_mruby_string_ext_proc_irep_4 = {
  4,8,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_string_ext_proc_iseq_4,
  gem_mrblib_mruby_string_ext_proc_pool_4,gem_mrblib_mruby_string_ext_proc_syms_4,NULL,
  gem_mrblib_mruby_string_ext_proc_lv_4,
  &gem_mrblib_mruby_string_ext_proc_debug_4,
  89,2,3,0,0
};
static const mrb_pool_value gem_mrblib_mruby_string_ext_proc_pool_5[3] = {
{IREP_TT_STR|(6<<2), {"\x20\x0c\x0a\x0d\x09\x0b"}},
{IREP_TT_STR|(7<<2), {"\x20\x0c\x0a\x0d\x09\x0b\x00"}},
{IREP_TT_STR|(0<<2), {""}},
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_syms_5, 3, (MRB_SYM(size), MRB_OPSYM(aref), MRB_SYM_Q(include), ), const);
static const mrb_code gem_mrblib_mruby_string_ext_proc_iseq_5[127] = {
0x33,0x00,0x00,0x00,0x06,0x02,0x12,0x04,0x2d,0x04,0x00,0x00,0x3e,0x04,0x01,0x01,0x03,0x04,0x01,0x04,
0x02,0x01,0x05,0x03,0x43,0x04,0x25,0x04,0x00,0x10,0x50,0x04,0x00,0x12,0x05,0x01,0x06,0x02,0x2d,0x05,
0x01,0x01,0x2d,0x04,0x02,0x01,0x25,0x04,0x00,0x06,0x3c,0x02,0x01,0x23,0xff,0xda,0x01,0x04,0x02,0x01,
0x05,0x03,0x43,0x04,0x25,0x04,0x00,0x10,0x50,0x04,0x01,0x12,0x05,0x01,0x06,0x03,0x2d,0x05,0x01,0x01,
0x2d,0x04,0x02,0x01,0x25,0x04,0x00,0x06,0x3e,0x03,0x01,0x23,0xff,0xda,0x01,0x04,0x03,0x06,0x05,0x45,
0x04,0x25,0x04,0x00,0x11,0x12,0x04,0x01,0x05,0x02,0x01,0x06,0x03,0x58,0x05,0x2d,0x04,0x01,0x01,0x23,
0x00,0x03,0x50,0x04,0x02,0x37,0x04,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_lv_5, 3, (MRB_OPSYM(and), MRB_SYM(a), MRB_SYM(z), ), const);
static char gem_mrblib_mruby_string_ext_proc_debug_lines_5[] = "\x00\x3b\x04\x01\x02\x01\x0c\x01\x26\x01\x26\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_string_ext_proc_debug_file_5 = {
0, 173, 12, mrb_debug_line_packed_map, {gem_mrblib_mruby_string_ext_proc_debug_lines_5}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_string_ext_proc_debug_file_5_ = &gem_mrblib_mruby_string_ext_proc_debug_file_5;
static mrb_irep_debug_info gem_mrblib_mruby_string_ext_proc_debug_5 = {
127, 1, &gem_mrblib_mruby_string_ext_proc_debug_file_5_};
static const mrb_irep gem_mrblib_mruby_string_ext_proc_irep_5 = {
  4,8,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_string_ext_proc_iseq_5,
  gem_mrblib_mruby_string_ext_proc_pool_5,gem_mrblib_mruby_string_ext_proc_syms_5,NULL,
  gem_mrblib_mruby_string_ext_proc_lv_5,
  &gem_mrblib_mruby_string_ext_proc_debug_5,
  127,3,3,0,0
};
static const mrb_pool_value gem_mrblib_mruby_string_ext_proc_pool_6[1] = {
{IREP_TT_STR|(26<<2), {"\x63\x61\x6e\x27\x74\x20\x6d\x6f\x64\x69\x66\x79\x20\x66\x72\x6f\x7a\x65\x6e\x20\x53\x74\x72\x69\x6e\x67"}},
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_syms_6, 5, (MRB_SYM_Q(frozen), MRB_SYM(FrozenError), MRB_SYM(raise), MRB_SYM(lstrip), MRB_SYM(replace), ), const);
static const mrb_code gem_mrblib_mruby_string_ext_proc_iseq_6[62] = {
0x33,0x00,0x00,0x00,0x12,0x03,0x2d,0x03,0x00,0x00,0x25,0x03,0x00,0x0c,0x12,0x03,0x1d,0x04,0x01,0x50,
0x05,0x00,0x2d,0x03,0x02,0x02,0x12,0x03,0x2d,0x03,0x03,0x00,0x01,0x02,0x03,0x01,0x03,0x02,0x12,0x04,
0x41,0x03,0x25,0x03,0x00,0x05,0x11,0x03,0x23,0x00,0x09,0x12,0x03,0x01,0x04,0x02,0x2d,0x03,0x04,0x01,
0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_lv_6, 2, (MRB_OPSYM(and), MRB_SYM(s), ), const);
static char gem_mrblib_mruby_string_ext_proc_debug_lines_6[] = "\x00\x4e\x04\x01\x16\x01\x09\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_string_ext_proc_debug_file_6 = {
0, 173, 8, mrb_debug_line_packed_map, {gem_mrblib_mruby_string_ext_proc_debug_lines_6}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_string_ext_proc_debug_file_6_ = &gem_mrblib_mruby_string_ext_proc_debug_file_6;
static mrb_irep_debug_info gem_mrblib_mruby_string_ext_proc_debug_6 = {
62, 1, &gem_mrblib_mruby_string_ext_proc_debug_file_6_};
static const mrb_irep gem_mrblib_mruby_string_ext_proc_irep_6 = {
  3,7,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_string_ext_proc_iseq_6,
  gem_mrblib_mruby_string_ext_proc_pool_6,gem_mrblib_mruby_string_ext_proc_syms_6,NULL,
  gem_mrblib_mruby_string_ext_proc_lv_6,
  &gem_mrblib_mruby_string_ext_proc_debug_6,
  62,1,5,0,0
};
static const mrb_pool_value gem_mrblib_mruby_string_ext_proc_pool_7[1] = {
{IREP_TT_STR|(26<<2), {"\x63\x61\x6e\x27\x74\x20\x6d\x6f\x64\x69\x66\x79\x20\x66\x72\x6f\x7a\x65\x6e\x20\x53\x74\x72\x69\x6e\x67"}},
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_syms_7, 5, (MRB_SYM_Q(frozen), MRB_SYM(FrozenError), MRB_SYM(raise), MRB_SYM(rstrip), MRB_SYM(replace), ), const);
static const mrb_code gem_mrblib_mruby_string_ext_proc_iseq_7[62] = {
0x33,0x00,0x00,0x00,0x12,0x03,0x2d,0x03,0x00,0x00,0x25,0x03,0x00,0x0c,0x12,0x03,0x1d,0x04,0x01,0x50,
0x05,0x00,0x2d,0x03,0x02,0x02,0x12,0x03,0x2d,0x03,0x03,0x00,0x01,0x02,0x03,0x01,0x03,0x02,0x12,0x04,
0x41,0x03,0x25,0x03,0x00,0x05,0x11,0x03,0x23,0x00,0x09,0x12,0x03,0x01,0x04,0x02,0x2d,0x03,0x04,0x01,
0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_lv_7, 2, (MRB_OPSYM(and), MRB_SYM(s), ), const);
static char gem_mrblib_mruby_string_ext_proc_debug_lines_7[] = "\x00\x5f\x04\x01\x16\x01\x09\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_string_ext_proc_debug_file_7 = {
0, 173, 8, mrb_debug_line_packed_map, {gem_mrblib_mruby_string_ext_proc_debug_lines_7}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_string_ext_proc_debug_file_7_ = &gem_mrblib_mruby_string_ext_proc_debug_file_7;
static mrb_irep_debug_info gem_mrblib_mruby_string_ext_proc_debug_7 = {
62, 1, &gem_mrblib_mruby_string_ext_proc_debug_file_7_};
static const mrb_irep gem_mrblib_mruby_string_ext_proc_irep_7 = {
  3,7,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_string_ext_proc_iseq_7,
  gem_mrblib_mruby_string_ext_proc_pool_7,gem_mrblib_mruby_string_ext_proc_syms_7,NULL,
  gem_mrblib_mruby_string_ext_proc_lv_7,
  &gem_mrblib_mruby_string_ext_proc_debug_7,
  62,1,5,0,0
};
static const mrb_pool_value gem_mrblib_mruby_string_ext_proc_pool_8[1] = {
{IREP_TT_STR|(26<<2), {"\x63\x61\x6e\x27\x74\x20\x6d\x6f\x64\x69\x66\x79\x20\x66\x72\x6f\x7a\x65\x6e\x20\x53\x74\x72\x69\x6e\x67"}},
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_syms_8, 5, (MRB_SYM_Q(frozen), MRB_SYM(FrozenError), MRB_SYM(raise), MRB_SYM(strip), MRB_SYM(replace), ), const);
static const mrb_code gem_mrblib_mruby_string_ext_proc_iseq_8[62] = {
0x33,0x00,0x00,0x00,0x12,0x03,0x2d,0x03,0x00,0x00,0x25,0x03,0x00,0x0c,0x12,0x03,0x1d,0x04,0x01,0x50,
0x05,0x00,0x2d,0x03,0x02,0x02,0x12,0x03,0x2d,0x03,0x03,0x00,0x01,0x02,0x03,0x01,0x03,0x02,0x12,0x04,
0x41,0x03,0x25,0x03,0x00,0x05,0x11,0x03,0x23,0x00,0x09,0x12,0x03,0x01,0x04,0x02,0x2d,0x03,0x04,0x01,
0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_lv_8, 2, (MRB_OPSYM(and), MRB_SYM(s), ), const);
static char gem_mrblib_mruby_string_ext_proc_debug_lines_8[] = "\x00\x6c\x04\x01\x16\x01\x09\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_string_ext_proc_debug_file_8 = {
0, 173, 8, mrb_debug_line_packed_map, {gem_mrblib_mruby_string_ext_proc_debug_lines_8}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_string_ext_proc_debug_file_8_ = &gem_mrblib_mruby_string_ext_proc_debug_file_8;
static mrb_irep_debug_info gem_mrblib_mruby_string_ext_proc_debug_8 = {
62, 1, &gem_mrblib_mruby_string_ext_proc_debug_file_8_};
static const mrb_irep gem_mrblib_mruby_string_ext_proc_irep_8 = {
  3,7,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_string_ext_proc_iseq_8,
  gem_mrblib_mruby_string_ext_proc_pool_8,gem_mrblib_mruby_string_ext_proc_syms_8,NULL,
  gem_mrblib_mruby_string_ext_proc_lv_8,
  &gem_mrblib_mruby_string_ext_proc_debug_8,
  62,1,5,0,0
};
static const mrb_pool_value gem_mrblib_mruby_string_ext_proc_pool_9[3] = {
{IREP_TT_STR|(15<<2), {"\x74\x79\x70\x65\x20\x6d\x69\x73\x6d\x61\x74\x63\x68\x3a\x20"}},
{IREP_TT_STR|(6<<2), {"\x20\x67\x69\x76\x65\x6e"}},
{IREP_TT_STR|(0<<2), {""}},
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_syms_9, 9, (MRB_SYM(String), MRB_SYM_Q(is_a), MRB_SYM(TypeError), MRB_SYM(class), MRB_SYM(raise), MRB_SYM(index), MRB_OPSYM(aref), MRB_SYM(size), MRB_SYM(slice), ), const);
static const mrb_code gem_mrblib_mruby_string_ext_proc_iseq_9[144] = {
0x33,0x04,0x00,0x00,0x01,0x05,0x01,0x1d,0x06,0x00,0x2d,0x05,0x01,0x01,0x24,0x05,0x00,0x1a,0x12,0x05,
0x1d,0x06,0x02,0x50,0x07,0x00,0x01,0x08,0x01,0x2d,0x08,0x03,0x00,0x51,0x07,0x50,0x08,0x01,0x51,0x07,
0x2d,0x05,0x04,0x02,0x12,0x05,0x01,0x06,0x01,0x2d,0x05,0x05,0x01,0x01,0x03,0x05,0x01,0x05,0x03,0x26,
0x05,0x00,0x03,0x23,0x00,0x18,0x12,0x05,0x06,0x06,0x05,0x07,0x58,0x06,0x2d,0x05,0x06,0x01,0x50,0x06,
0x02,0x50,0x07,0x02,0x46,0x05,0x03,0x23,0x00,0x34,0x01,0x05,0x03,0x01,0x06,0x01,0x2d,0x06,0x07,0x00,
0x3b,0x05,0x01,0x04,0x05,0x12,0x05,0x06,0x06,0x01,0x07,0x03,0x2d,0x05,0x08,0x02,0x01,0x06,0x01,0x12,
0x07,0x01,0x08,0x04,0x12,0x09,0x2d,0x09,0x07,0x00,0x01,0x0a,0x04,0x3d,0x09,0x2d,0x07,0x08,0x02,0x46,
0x05,0x03,0x37,0x05,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_lv_9, 4, (MRB_SYM(sep), MRB_OPSYM(and), MRB_SYM(n), MRB_SYM(m), ), const);
static char gem_mrblib_mruby_string_ext_proc_debug_lines_9[] = "\x00\x72\x04\x01\x28\x01\x0c\x01\x08\x04\x18\xfe\xff\xff\xff\x0f\x02\xff\xff\xff\xff\x0f\x0f\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_string_ext_proc_debug_file_9 = {
0, 173, 24, mrb_debug_line_packed_map, {gem_mrblib_mruby_string_ext_proc_debug_lines_9}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_string_ext_proc_debug_file_9_ = &gem_mrblib_mruby_string_ext_proc_debug_file_9;
static mrb_irep_debug_info gem_mrblib_mruby_string_ext_proc_debug_9 = {
144, 1, &gem_mrblib_mruby_string_ext_proc_debug_file_9_};
static const mrb_irep gem_mrblib_mruby_string_ext_proc_irep_9 = {
  5,12,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_string_ext_proc_iseq_9,
  gem_mrblib_mruby_string_ext_proc_pool_9,gem_mrblib_mruby_string_ext_proc_syms_9,NULL,
  gem_mrblib_mruby_string_ext_proc_lv_9,
  &gem_mrblib_mruby_string_ext_proc_debug_9,
  144,3,9,0,0
};
static const mrb_pool_value gem_mrblib_mruby_string_ext_proc_pool_10[3] = {
{IREP_TT_STR|(15<<2), {"\x74\x79\x70\x65\x20\x6d\x69\x73\x6d\x61\x74\x63\x68\x3a\x20"}},
{IREP_TT_STR|(6<<2), {"\x20\x67\x69\x76\x65\x6e"}},
{IREP_TT_STR|(0<<2), {""}},
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_syms_10, 8, (MRB_SYM(String), MRB_SYM_Q(is_a), MRB_SYM(TypeError), MRB_SYM(class), MRB_SYM(raise), MRB_SYM(rindex), MRB_SYM(size), MRB_SYM(slice), ), const);
static const mrb_code gem_mrblib_mruby_string_ext_proc_iseq_10[134] = {
0x33,0x04,0x00,0x00,0x01,0x05,0x01,0x1d,0x06,0x00,0x2d,0x05,0x01,0x01,0x24,0x05,0x00,0x1a,0x12,0x05,
0x1d,0x06,0x02,0x50,0x07,0x00,0x01,0x08,0x01,0x2d,0x08,0x03,0x00,0x51,0x07,0x50,0x08,0x01,0x51,0x07,
0x2d,0x05,0x04,0x02,0x12,0x05,0x01,0x06,0x01,0x2d,0x05,0x05,0x01,0x01,0x03,0x05,0x01,0x05,0x03,0x26,
0x05,0x00,0x03,0x23,0x00,0x0e,0x50,0x05,0x02,0x50,0x06,0x02,0x12,0x07,0x46,0x05,0x03,0x23,0x00,0x34,
0x01,0x05,0x03,0x01,0x06,0x01,0x2d,0x06,0x06,0x00,0x3b,0x05,0x01,0x04,0x05,0x12,0x05,0x06,0x06,0x01,
0x07,0x03,0x2d,0x05,0x07,0x02,0x01,0x06,0x01,0x12,0x07,0x01,0x08,0x04,0x12,0x09,0x2d,0x09,0x06,0x00,
0x01,0x0a,0x04,0x3d,0x09,0x2d,0x07,0x07,0x02,0x46,0x05,0x03,0x37,0x05,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_lv_10, 4, (MRB_SYM(sep), MRB_OPSYM(and), MRB_SYM(n), MRB_SYM(m), ), const);
static char gem_mrblib_mruby_string_ext_proc_debug_lines_10[] = "\x00\x7d\x04\x01\x28\x01\x0c\x01\x08\x04\x0e\xfe\xff\xff\xff\x0f\x02\xff\xff\xff\xff\x0f\x0f\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_string_ext_proc_debug_file_10 = {
0, 173, 24, mrb_debug_line_packed_map, {gem_mrblib_mruby_string_ext_proc_debug_lines_10}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_string_ext_proc_debug_file_10_ = &gem_mrblib_mruby_string_ext_proc_debug_file_10;
static mrb_irep_debug_info gem_mrblib_mruby_string_ext_proc_debug_10 = {
134, 1, &gem_mrblib_mruby_string_ext_proc_debug_file_10_};
static const mrb_irep gem_mrblib_mruby_string_ext_proc_irep_10 = {
  5,12,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_string_ext_proc_iseq_10,
  gem_mrblib_mruby_string_ext_proc_pool_10,gem_mrblib_mruby_string_ext_proc_syms_10,NULL,
  gem_mrblib_mruby_string_ext_proc_lv_10,
  &gem_mrblib_mruby_string_ext_proc_debug_10,
  134,3,8,0,0
};
static const mrb_pool_value gem_mrblib_mruby_string_ext_proc_pool_11[3] = {
{IREP_TT_STR|(26<<2), {"\x63\x61\x6e\x27\x74\x20\x6d\x6f\x64\x69\x66\x79\x20\x66\x72\x6f\x7a\x65\x6e\x20\x53\x74\x72\x69\x6e\x67"}},
{IREP_TT_STR|(36<<2), {"\x77\x72\x6f\x6e\x67\x20\x6e\x75\x6d\x62\x65\x72\x20\x6f\x66\x20\x61\x72\x67\x75\x6d\x65\x6e\x74\x73\x20\x28\x66\x6f\x72\x20\x31\x2e\x2e\x32\x29"}},
{IREP_TT_STR|(0<<2), {""}},
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_syms_11, 16, (MRB_SYM_Q(frozen), MRB_SYM(FrozenError), MRB_SYM(raise), MRB_SYM_Q(nil), MRB_OPSYM(not), MRB_SYM(size), MRB_OPSYM(aref), MRB_SYM(Range), MRB_SYM_Q(kind_of), MRB_SYM(begin), MRB_SYM(end), MRB_SYM_Q(exclude_end), MRB_SYM(String), MRB_SYM(to_s), MRB_SYM(index), MRB_SYM(replace), ), const);
static const mrb_code gem_mrblib_mruby_string_ext_proc_iseq_11[804] = {
0x33,0x04,0x20,0x00,0x23,0x00,0x03,0x23,0x00,0x02,0x11,0x02,0x12,0x0b,0x2d,0x0b,0x00,0x00,0x25,0x0b,
0x00,0x0c,0x12,0x0b,0x1d,0x0c,0x01,0x50,0x0d,0x00,0x2d,0x0b,0x02,0x02,0x01,0x0b,0x01,0x2d,0x0b,0x03,
0x00,0x25,0x0b,0x00,0x07,0x01,0x0b,0x02,0x2d,0x0b,0x03,0x00,0x25,0x0b,0x00,0x09,0x12,0x0b,0x50,0x0c,
0x01,0x2d,0x0b,0x02,0x01,0x01,0x0b,0x01,0x2d,0x0b,0x03,0x00,0x2d,0x0b,0x04,0x00,0x25,0x0b,0x00,0x0b,
0x01,0x0b,0x02,0x2d,0x0b,0x03,0x00,0x2d,0x0b,0x04,0x00,0x25,0x0b,0x00,0x5d,0x01,0x0b,0x01,0x01,0x04,
0x0b,0x01,0x0b,0x01,0x06,0x0c,0x42,0x0b,0x25,0x0b,0x00,0x0e,0x01,0x0b,0x04,0x12,0x0c,0x2d,0x0c,0x05,
0x00,0x3b,0x0b,0x01,0x04,0x0b,0x01,0x0b,0x04,0x06,0x0c,0x45,0x0b,0x25,0x0b,0x00,0x0b,0x01,0x0b,0x04,
0x12,0x0c,0x2d,0x0c,0x05,0x00,0x43,0x0b,0x25,0x0b,0x00,0x07,0x01,0x0b,0x02,0x06,0x0c,0x44,0x0b,0x25,
0x0b,0x00,0x12,0x12,0x0b,0x01,0x0c,0x04,0x01,0x0d,0x02,0x2d,0x0b,0x06,0x02,0x01,0x05,0x0b,0x23,0x00,
0x04,0x11,0x0b,0x37,0x0b,0x23,0x00,0xd1,0x14,0x06,0x01,0x0b,0x01,0x1d,0x0c,0x07,0x2d,0x0b,0x08,0x01,
0x25,0x0b,0x00,0x59,0x01,0x0b,0x01,0x2d,0x0b,0x09,0x00,0x01,0x07,0x0b,0x01,0x0b,0x01,0x2d,0x0b,0x0a,
0x00,0x01,0x08,0x0b,0x01,0x0b,0x07,0x06,0x0c,0x42,0x0b,0x25,0x0b,0x00,0x0e,0x01,0x0b,0x07,0x12,0x0c,
0x2d,0x0c,0x05,0x00,0x3b,0x0b,0x01,0x07,0x0b,0x01,0x0b,0x08,0x06,0x0c,0x42,0x0b,0x25,0x0b,0x00,0x0e,
0x01,0x0b,0x08,0x12,0x0c,0x2d,0x0c,0x05,0x00,0x3b,0x0b,0x01,0x08,0x0b,0x01,0x0b,0x01,0x2d,0x0b,0x0b,
0x00,0x25,0x0b,0x00,0x03,0x3e,0x08,0x01,0x13,0x06,0x23,0x00,0x4e,0x01,0x0b,0x01,0x1d,0x0c,0x0c,0x2d,
0x0b,0x08,0x01,0x25,0x0b,0x00,0x05,0x13,0x06,0x23,0x00,0x3b,0x01,0x0b,0x01,0x01,0x04,0x0b,0x01,0x0b,
0x01,0x06,0x0c,0x42,0x0b,0x25,0x0b,0x00,0x0e,0x01,0x0b,0x04,0x12,0x0c,0x2d,0x0c,0x05,0x00,0x3b,0x0b,
0x01,0x04,0x0b,0x01,0x0b,0x04,0x06,0x0c,0x45,0x0b,0x25,0x0b,0x00,0x0b,0x01,0x0b,0x01,0x12,0x0c,0x2d,
0x0c,0x05,0x00,0x42,0x0b,0x25,0x0b,0x00,0x02,0x13,0x06,0x01,0x0b,0x06,0x25,0x0b,0x00,0x0f,0x12,0x0b,
0x01,0x0c,0x01,0x2d,0x0b,0x06,0x01,0x01,0x05,0x0b,0x23,0x00,0x04,0x11,0x0b,0x37,0x0b,0x01,0x0b,0x05,
0x2d,0x0b,0x03,0x00,0x24,0x0b,0x00,0x08,0x01,0x0b,0x05,0x50,0x0c,0x02,0x41,0x0b,0x24,0x0b,0x01,0x7e,
0x01,0x0b,0x01,0x2d,0x0b,0x03,0x00,0x2d,0x0b,0x04,0x00,0x25,0x0b,0x00,0x0b,0x01,0x0b,0x02,0x2d,0x0b,
0x03,0x00,0x2d,0x0b,0x04,0x00,0x25,0x0b,0x00,0x4a,0x01,0x0b,0x01,0x06,0x0c,0x45,0x0b,0x25,0x0b,0x00,
0x06,0x01,0x0b,0x01,0x23,0x00,0x0b,0x12,0x0b,0x2d,0x0b,0x05,0x00,0x01,0x0c,0x01,0x3b,0x0b,0x01,0x04,
0x0b,0x12,0x0b,0x06,0x0c,0x01,0x0d,0x04,0x59,0x0c,0x2d,0x0b,0x06,0x01,0x12,0x0c,0x01,0x0d,0x04,0x01,
0x0e,0x02,0x3b,0x0d,0x05,0x0e,0x58,0x0d,0x2d,0x0c,0x06,0x01,0x2d,0x0c,0x0d,0x00,0x3b,0x0b,0x01,0x09,
0x0b,0x23,0x01,0x06,0x01,0x0b,0x01,0x1d,0x0c,0x07,0x2d,0x0b,0x08,0x01,0x25,0x0b,0x00,0x67,0x01,0x0b,
0x07,0x06,0x0c,0x45,0x0b,0x25,0x0b,0x00,0x06,0x01,0x0b,0x07,0x23,0x00,0x0b,0x12,0x0b,0x2d,0x0b,0x05,
0x00,0x01,0x0c,0x07,0x3b,0x0b,0x01,0x04,0x0b,0x01,0x0b,0x08,0x06,0x0c,0x45,0x0b,0x25,0x0b,0x00,0x06,
0x01,0x0b,0x08,0x23,0x00,0x0b,0x12,0x0b,0x2d,0x0b,0x05,0x00,0x01,0x0c,0x08,0x3b,0x0b,0x01,0x0a,0x0b,
0x12,0x0b,0x06,0x0c,0x01,0x0d,0x04,0x59,0x0c,0x2d,0x0b,0x06,0x01,0x12,0x0c,0x01,0x0d,0x0a,0x3c,0x0d,
0x01,0x05,0x0e,0x58,0x0d,0x2d,0x0c,0x06,0x01,0x2d,0x0c,0x0d,0x00,0x3b,0x0b,0x01,0x09,0x0b,0x23,0x00,
0x91,0x01,0x0b,0x01,0x1d,0x0c,0x0c,0x2d,0x0b,0x08,0x01,0x25,0x0b,0x00,0x3e,0x12,0x0b,0x01,0x0c,0x01,
0x2d,0x0b,0x0e,0x01,0x01,0x04,0x0b,0x01,0x0b,0x04,0x26,0x0b,0x00,0x28,0x12,0x0b,0x06,0x0c,0x01,0x0d,
0x04,0x59,0x0c,0x2d,0x0b,0x06,0x01,0x12,0x0c,0x01,0x0d,0x04,0x01,0x0e,0x01,0x2d,0x0e,0x05,0x00,0x3b,
0x0d,0x05,0x0e,0x58,0x0d,0x2d,0x0c,0x06,0x01,0x3b,0x0b,0x01,0x09,0x0b,0x23,0x00,0x45,0x01,0x0b,0x01,
0x06,0x0c,0x45,0x0b,0x25,0x0b,0x00,0x06,0x01,0x0b,0x01,0x23,0x00,0x0b,0x12,0x0b,0x2d,0x0b,0x05,0x00,
0x01,0x0c,0x01,0x3b,0x0b,0x01,0x04,0x0b,0x12,0x0b,0x06,0x0c,0x01,0x0d,0x04,0x59,0x0c,0x2d,0x0b,0x06,
0x01,0x12,0x0c,0x01,0x0d,0x04,0x3c,0x0d,0x01,0x05,0x0e,0x58,0x0d,0x2d,0x0c,0x06,0x01,0x2d,0x0c,0x0d,
0x00,0x3b,0x0b,0x01,0x09,0x0b,0x01,0x0b,0x09,0x26,0x0b,0x00,0x09,0x12,0x0b,0x01,0x0c,0x09,0x2d,0x0b,
0x0f,0x01,0x37,0x05,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_lv_11, 10, (MRB_SYM(arg1), MRB_SYM(arg2), MRB_OPSYM(and), MRB_SYM(idx), MRB_SYM(str), MRB_SYM(validated), MRB_SYM(beg), MRB_SYM(ed), MRB_SYM(str2), MRB_SYM(idx2), ), const);
static char gem_mrblib_mruby_string_ext_proc_debug_lines_11[] = "\x00\x98\x01\x0c\x01\x16\x01\x1f\x02\x1c\x06\x02\xfb\xff\xff\xff\x0f\x06\x01\x19\x01\x23\x01\x12\x02\x07\x15\x02\xee\xff\xff\xff\x0f\x02\x01\x0c\x06\x02\xfb\xff\xff\xff\x0f\x0a\x01\x0a\x01\x19\x01\x19\x01\x0e\x01\x03\x06\x02\xfb\xff\xff\xff\x0f\x0c\x01\x05\x04\x02\xfe\xff\xff\xff\x0f\x06\x01\x19\x01\x1c\x02\x05\x01\x0f\x02\x06\x03\x15\x11\x02\xf0\xff\xff\xff\x0f\x1c\x02\x02\xff\xff\xff\xff\x0f\x1f\x01\x29\x0b\x02\xf7\xff\xff\xff\x0f\x0c\x03\x02\xfe\xff\xff\xff\x0f\x1f\x01\x1f\x01\x27\x06\x02\xfb\xff\xff\xff\x0f\x0c\x02\x02\xff\xff\xff\xff\x0f\x0c\x01\x30\x03\x02\xff\xff\xff\xff\x0f\x1f\x01\x26\x03\x10\x02";
static mrb_irep_debug_info_file gem_mrblib_mruby_string_ext_proc_debug_file_11 = {
0, 173, 153, mrb_debug_line_packed_map, {gem_mrblib_mruby_string_ext_proc_debug_lines_11}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_string_ext_proc_debug_file_11_ = &gem_mrblib_mruby_string_ext_proc_debug_file_11;
static mrb_irep_debug_info gem_mrblib_mruby_string_ext_proc_debug_11 = {
804, 1, &gem_mrblib_mruby_string_ext_proc_debug_file_11_};
static const mrb_irep gem_mrblib_mruby_string_ext_proc_irep_11 = {
  11,16,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_string_ext_proc_iseq_11,
  gem_mrblib_mruby_string_ext_proc_pool_11,gem_mrblib_mruby_string_ext_proc_syms_11,NULL,
  gem_mrblib_mruby_string_ext_proc_lv_11,
  &gem_mrblib_mruby_string_ext_proc_debug_11,
  804,3,16,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_syms_12, 2, (MRB_OPSYM(lshift), MRB_OPSYM(aset), ), const);
static const mrb_code gem_mrblib_mruby_string_ext_proc_iseq_12[64] = {
0x33,0x08,0x00,0x00,0x01,0x04,0x01,0x05,0x05,0x41,0x04,0x25,0x04,0x00,0x0e,0x12,0x04,0x01,0x05,0x02,
0x2d,0x04,0x00,0x01,0x37,0x04,0x23,0x00,0x0e,0x01,0x04,0x01,0x06,0x05,0x42,0x04,0x25,0x04,0x00,0x03,
0x3c,0x01,0x01,0x01,0x04,0x02,0x12,0x05,0x01,0x06,0x01,0x06,0x07,0x01,0x08,0x04,0x2d,0x05,0x01,0x03,
0x12,0x04,0x37,0x04,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_lv_12, 3, (MRB_SYM(idx), MRB_SYM(str), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_string_ext_proc_debug_lines_12[] = "\x00\xe0\x01\x04\x01\x09\x01\x0e\x02\x02\xff\xff\xff\xff\x0f\x09\x01\x05\x02\x11\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_string_ext_proc_debug_file_12 = {
0, 173, 21, mrb_debug_line_packed_map, {gem_mrblib_mruby_string_ext_proc_debug_lines_12}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_string_ext_proc_debug_file_12_ = &gem_mrblib_mruby_string_ext_proc_debug_file_12;
static mrb_irep_debug_info gem_mrblib_mruby_string_ext_proc_debug_12 = {
64, 1, &gem_mrblib_mruby_string_ext_proc_debug_file_12_};
static const mrb_irep gem_mrblib_mruby_string_ext_proc_irep_12 = {
  4,10,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_string_ext_proc_iseq_12,
  NULL,gem_mrblib_mruby_string_ext_proc_syms_12,NULL,
  gem_mrblib_mruby_string_ext_proc_lv_12,
  &gem_mrblib_mruby_string_ext_proc_debug_12,
  64,0,2,0,0
};
static const mrb_pool_value gem_mrblib_mruby_string_ext_proc_pool_13[3] = {
{IREP_TT_STR|(1<<2), {"\x20"}},
{IREP_TT_STR|(0<<2), {""}},
{IREP_TT_STR|(18<<2), {"\x7a\x65\x72\x6f\x20\x77\x69\x64\x74\x68\x20\x70\x61\x64\x64\x69\x6e\x67"}},
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_syms_13, 4, (MRB_SYM(ArgumentError), MRB_SYM(raise), MRB_SYM(size), MRB_OPSYM(aref), ), const);
static const mrb_code gem_mrblib_mruby_string_ext_proc_iseq_13[108] = {
0x33,0x04,0x20,0x00,0x23,0x00,0x03,0x23,0x00,0x03,0x50,0x02,0x00,0x01,0x06,0x02,0x50,0x07,0x01,0x41,
0x06,0x25,0x06,0x00,0x0c,0x12,0x06,0x1d,0x07,0x00,0x50,0x08,0x02,0x2d,0x06,0x01,0x02,0x01,0x06,0x01,
0x12,0x07,0x2d,0x07,0x02,0x00,0x43,0x06,0x25,0x06,0x00,0x04,0x12,0x06,0x37,0x06,0x01,0x06,0x01,0x01,
0x07,0x02,0x2d,0x07,0x02,0x00,0x40,0x06,0x01,0x04,0x06,0x01,0x06,0x02,0x01,0x07,0x04,0x3f,0x06,0x06,
0x07,0x01,0x08,0x01,0x12,0x09,0x2d,0x09,0x02,0x00,0x3d,0x08,0x2d,0x06,0x03,0x02,0x01,0x05,0x06,0x12,
0x06,0x01,0x07,0x05,0x3b,0x06,0x37,0x06,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_lv_13, 5, (MRB_SYM(idx), MRB_SYM(padstr), MRB_OPSYM(and), MRB_SYM(pad_repetitions), MRB_SYM(padding), ), const);
static char gem_mrblib_mruby_string_ext_proc_debug_lines_13[] = "\x00\xf5\x01\x0d\x01\x18\x01\x13\x01\x0f\x01\x1c\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_string_ext_proc_debug_file_13 = {
0, 173, 13, mrb_debug_line_packed_map, {gem_mrblib_mruby_string_ext_proc_debug_lines_13}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_string_ext_proc_debug_file_13_ = &gem_mrblib_mruby_string_ext_proc_debug_file_13;
static mrb_irep_debug_info gem_mrblib_mruby_string_ext_proc_debug_13 = {
108, 1, &gem_mrblib_mruby_string_ext_proc_debug_file_13_};
static const mrb_irep gem_mrblib_mruby_string_ext_proc_irep_13 = {
  6,11,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_string_ext_proc_iseq_13,
  gem_mrblib_mruby_string_ext_proc_pool_13,gem_mrblib_mruby_string_ext_proc_syms_13,NULL,
  gem_mrblib_mruby_string_ext_proc_lv_13,
  &gem_mrblib_mruby_string_ext_proc_debug_13,
  108,3,4,0,0
};
static const mrb_pool_value gem_mrblib_mruby_string_ext_proc_pool_14[3] = {
{IREP_TT_STR|(1<<2), {"\x20"}},
{IREP_TT_STR|(0<<2), {""}},
{IREP_TT_STR|(18<<2), {"\x7a\x65\x72\x6f\x20\x77\x69\x64\x74\x68\x20\x70\x61\x64\x64\x69\x6e\x67"}},
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_syms_14, 4, (MRB_SYM(ArgumentError), MRB_SYM(raise), MRB_SYM(size), MRB_OPSYM(aref), ), const);
static const mrb_code gem_mrblib_mruby_string_ext_proc_iseq_14[108] = {
0x33,0x04,0x20,0x00,0x23,0x00,0x03,0x23,0x00,0x03,0x50,0x02,0x00,0x01,0x06,0x02,0x50,0x07,0x01,0x41,
0x06,0x25,0x06,0x00,0x0c,0x12,0x06,0x1d,0x07,0x00,0x50,0x08,0x02,0x2d,0x06,0x01,0x02,0x01,0x06,0x01,
0x12,0x07,0x2d,0x07,0x02,0x00,0x43,0x06,0x25,0x06,0x00,0x04,0x12,0x06,0x37,0x06,0x01,0x06,0x01,0x01,
0x07,0x02,0x2d,0x07,0x02,0x00,0x40,0x06,0x01,0x04,0x06,0x01,0x06,0x02,0x01,0x07,0x04,0x3f,0x06,0x06,
0x07,0x01,0x08,0x01,0x12,0x09,0x2d,0x09,0x02,0x00,0x3d,0x08,0x2d,0x06,0x03,0x02,0x01,0x05,0x06,0x01,
0x06,0x05,0x12,0x07,0x3b,0x06,0x37,0x06,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_lv_14, 5, (MRB_SYM(idx), MRB_SYM(padstr), MRB_OPSYM(and), MRB_SYM(pad_repetitions), MRB_SYM(padding), ), const);
static char gem_mrblib_mruby_string_ext_proc_debug_lines_14[] = "\x00\x88\x02\x0d\x01\x18\x01\x13\x01\x0f\x01\x1c\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_string_ext_proc_debug_file_14 = {
0, 173, 13, mrb_debug_line_packed_map, {gem_mrblib_mruby_string_ext_proc_debug_lines_14}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_string_ext_proc_debug_file_14_ = &gem_mrblib_mruby_string_ext_proc_debug_file_14;
static mrb_irep_debug_info gem_mrblib_mruby_string_ext_proc_debug_14 = {
108, 1, &gem_mrblib_mruby_string_ext_proc_debug_file_14_};
static const mrb_irep gem_mrblib_mruby_string_ext_proc_irep_14 = {
  6,11,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_string_ext_proc_iseq_14,
  gem_mrblib_mruby_string_ext_proc_pool_14,gem_mrblib_mruby_string_ext_proc_syms_14,NULL,
  gem_mrblib_mruby_string_ext_proc_lv_14,
  &gem_mrblib_mruby_string_ext_proc_debug_14,
  108,3,4,0,0
};
static const mrb_pool_value gem_mrblib_mruby_string_ext_proc_pool_15[3] = {
{IREP_TT_STR|(1<<2), {"\x20"}},
{IREP_TT_STR|(0<<2), {""}},
{IREP_TT_STR|(18<<2), {"\x7a\x65\x72\x6f\x20\x77\x69\x64\x74\x68\x20\x70\x61\x64\x64\x69\x6e\x67"}},
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_syms_15, 4, (MRB_SYM(ArgumentError), MRB_SYM(raise), MRB_SYM(size), MRB_OPSYM(aref), ), const);
static const mrb_code gem_mrblib_mruby_string_ext_proc_iseq_15[133] = {
0x33,0x04,0x20,0x00,0x23,0x00,0x03,0x23,0x00,0x03,0x50,0x02,0x00,0x01,0x06,0x02,0x50,0x07,0x01,0x41,
0x06,0x25,0x06,0x00,0x0c,0x12,0x06,0x1d,0x07,0x00,0x50,0x08,0x02,0x2d,0x06,0x01,0x02,0x01,0x06,0x01,
0x12,0x07,0x2d,0x07,0x02,0x00,0x43,0x06,0x25,0x06,0x00,0x04,0x12,0x06,0x37,0x06,0x01,0x06,0x01,0x12,
0x07,0x2d,0x07,0x02,0x00,0x3d,0x06,0x01,0x01,0x06,0x01,0x06,0x01,0x08,0x07,0x40,0x06,0x01,0x04,0x06,
0x01,0x06,0x01,0x01,0x07,0x04,0x3d,0x06,0x01,0x05,0x06,0x01,0x06,0x02,0x01,0x07,0x04,0x3f,0x06,0x06,
0x07,0x01,0x08,0x04,0x2d,0x06,0x03,0x02,0x12,0x07,0x3b,0x06,0x01,0x07,0x02,0x01,0x08,0x05,0x3f,0x07,
0x06,0x08,0x01,0x09,0x05,0x2d,0x07,0x03,0x02,0x3b,0x06,0x37,0x06,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_lv_15, 5, (MRB_SYM(width), MRB_SYM(padstr), MRB_OPSYM(and), MRB_SYM(pad1), MRB_SYM(pad2), ), const);
static char gem_mrblib_mruby_string_ext_proc_debug_lines_15[] = "\x00\x9b\x02\x0d\x01\x18\x01\x13\x01\x0e\x01\x0a\x01\x0b\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_string_ext_proc_debug_file_15 = {
0, 173, 15, mrb_debug_line_packed_map, {gem_mrblib_mruby_string_ext_proc_debug_lines_15}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_string_ext_proc_debug_file_15_ = &gem_mrblib_mruby_string_ext_proc_debug_file_15;
static mrb_irep_debug_info gem_mrblib_mruby_string_ext_proc_debug_15 = {
133, 1, &gem_mrblib_mruby_string_ext_proc_debug_file_15_};
static const mrb_irep gem_mrblib_mruby_string_ext_proc_irep_15 = {
  6,11,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_string_ext_proc_iseq_15,
  gem_mrblib_mruby_string_ext_proc_pool_15,gem_mrblib_mruby_string_ext_proc_syms_15,NULL,
  gem_mrblib_mruby_string_ext_proc_lv_15,
  &gem_mrblib_mruby_string_ext_proc_debug_15,
  133,3,4,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_syms_317, 1, (MRB_SYM(call), ), const);
static const mrb_code gem_mrblib_mruby_string_ext_proc_iseq_317[17] = {
0x33,0x04,0x00,0x00,0x21,0x03,0x01,0x00,0x01,0x04,0x01,0x2d,0x03,0x00,0x01,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_lv_317, 2, (MRB_SYM(i), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_string_ext_proc_debug_lines_317[] = "\x00\xa6\x02\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_string_ext_proc_debug_file_317 = {
0, 173, 5, mrb_debug_line_packed_map, {gem_mrblib_mruby_string_ext_proc_debug_lines_317}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_string_ext_proc_debug_file_317_ = &gem_mrblib_mruby_string_ext_proc_debug_file_317;
static mrb_irep_debug_info gem_mrblib_mruby_string_ext_proc_debug_317 = {
17, 1, &gem_mrblib_mruby_string_ext_proc_debug_file_317_};
static const mrb_irep gem_mrblib_mruby_string_ext_proc_irep_317 = {
  3,6,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_string_ext_proc_iseq_317,
  NULL,gem_mrblib_mruby_string_ext_proc_syms_317,NULL,
  gem_mrblib_mruby_string_ext_proc_lv_317,
  &gem_mrblib_mruby_string_ext_proc_debug_317,
  17,0,1,0,0
};
static const mrb_irep *gem_mrblib_mruby_string_ext_proc_reps_16[1] = {
  &gem_mrblib_mruby_string_ext_proc_irep_317,
};
static const mrb_pool_value gem_mrblib_mruby_string_ext_proc_pool_16[1] = {
{IREP_TT_STR|(0<<2), {""}},
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_syms_16, 3, (MRB_SYM_Q(block_given), MRB_SYM(split), MRB_SYM(each), ), const);
static const mrb_code gem_mrblib_mruby_string_ext_proc_iseq_16[46] = {
0x33,0x00,0x00,0x01,0x12,0x02,0x2d,0x02,0x00,0x00,0x25,0x02,0x00,0x15,0x12,0x02,0x50,0x03,0x00,0x2d,
0x02,0x01,0x01,0x56,0x03,0x00,0x2e,0x02,0x02,0x00,0x12,0x02,0x23,0x00,0x09,0x12,0x02,0x50,0x03,0x00,
0x2d,0x02,0x01,0x01,0x37,0x02,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_lv_16, 1, (MRB_SYM(block), ), const);
static char gem_mrblib_mruby_string_ext_proc_debug_lines_16[] = "\x00\xa4\x02\x04\x01\x08\x04\x02\xfd\xff\xff\xff\x0f\x10\x03\x03\x02";
static mrb_irep_debug_info_file gem_mrblib_mruby_string_ext_proc_debug_file_16 = {
0, 173, 17, mrb_debug_line_packed_map, {gem_mrblib_mruby_string_ext_proc_debug_lines_16}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_string_ext_proc_debug_file_16_ = &gem_mrblib_mruby_string_ext_proc_debug_file_16;
static mrb_irep_debug_info gem_mrblib_mruby_string_ext_proc_debug_16 = {
46, 1, &gem_mrblib_mruby_string_ext_proc_debug_file_16_};
static const mrb_irep gem_mrblib_mruby_string_ext_proc_irep_16 = {
  2,5,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_string_ext_proc_iseq_16,
  gem_mrblib_mruby_string_ext_proc_pool_16,gem_mrblib_mruby_string_ext_proc_syms_16,gem_mrblib_mruby_string_ext_proc_reps_16,
  gem_mrblib_mruby_string_ext_proc_lv_16,
  &gem_mrblib_mruby_string_ext_proc_debug_16,
  46,1,3,1,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_syms_17, 5, (MRB_SYM(each_char), MRB_SYM(to_enum), MRB_SYM(size), MRB_OPSYM(aref), MRB_SYM(call), ), const);
static const mrb_code gem_mrblib_mruby_string_ext_proc_iseq_17[65] = {
0x33,0x00,0x00,0x01,0x01,0x03,0x01,0x24,0x03,0x00,0x0b,0x12,0x03,0x10,0x04,0x00,0x2d,0x03,0x01,0x01,
0x37,0x03,0x06,0x02,0x01,0x03,0x02,0x12,0x04,0x2d,0x04,0x02,0x00,0x42,0x03,0x25,0x03,0x00,0x16,0x01,
0x03,0x01,0x12,0x04,0x01,0x05,0x02,0x2d,0x04,0x03,0x01,0x2d,0x03,0x04,0x01,0x3c,0x02,0x01,0x23,0xff,
0xdb,0x12,0x03,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_lv_17, 2, (MRB_SYM(block), MRB_SYM(pos), ), const);
static char gem_mrblib_mruby_string_ext_proc_debug_lines_17[] = "\x00\xb2\x02\x04\x01\x12\x01\x02\x01\x0d\x02\x02\xff\xff\xff\xff\x0f\x10\x01\x06\x02";
static mrb_irep_debug_info_file gem_mrblib_mruby_string_ext_proc_debug_file_17 = {
0, 173, 21, mrb_debug_line_packed_map, {gem_mrblib_mruby_string_ext_proc_debug_lines_17}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_string_ext_proc_debug_file_17_ = &gem_mrblib_mruby_string_ext_proc_debug_file_17;
static mrb_irep_debug_info gem_mrblib_mruby_string_ext_proc_debug_17 = {
65, 1, &gem_mrblib_mruby_string_ext_proc_debug_file_17_};
static const mrb_irep gem_mrblib_mruby_string_ext_proc_irep_17 = {
  3,7,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_string_ext_proc_iseq_17,
  NULL,gem_mrblib_mruby_string_ext_proc_syms_17,NULL,
  gem_mrblib_mruby_string_ext_proc_lv_17,
  &gem_mrblib_mruby_string_ext_proc_debug_17,
  65,0,5,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_syms_360, 2, (MRB_SYM(ord), MRB_SYM(call), ), const);
static const mrb_code gem_mrblib_mruby_string_ext_proc_iseq_360[21] = {
0x33,0x04,0x00,0x00,0x21,0x03,0x01,0x00,0x01,0x04,0x01,0x2d,0x04,0x00,0x00,0x2d,0x03,0x01,0x01,0x37,
0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_lv_360, 2, (MRB_SYM(x), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_string_ext_proc_debug_lines_360[] = "\x00\xbe\x02\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_string_ext_proc_debug_file_360 = {
0, 173, 5, mrb_debug_line_packed_map, {gem_mrblib_mruby_string_ext_proc_debug_lines_360}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_string_ext_proc_debug_file_360_ = &gem_mrblib_mruby_string_ext_proc_debug_file_360;
static mrb_irep_debug_info gem_mrblib_mruby_string_ext_proc_debug_360 = {
21, 1, &gem_mrblib_mruby_string_ext_proc_debug_file_360_};
static const mrb_irep gem_mrblib_mruby_string_ext_proc_irep_360 = {
  3,6,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_string_ext_proc_iseq_360,
  NULL,gem_mrblib_mruby_string_ext_proc_syms_360,NULL,
  gem_mrblib_mruby_string_ext_proc_lv_360,
  &gem_mrblib_mruby_string_ext_proc_debug_360,
  21,0,2,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_syms_361, 1, (MRB_SYM(ord), ), const);
static const mrb_code gem_mrblib_mruby_string_ext_proc_iseq_361[13] = {
0x33,0x04,0x00,0x00,0x01,0x03,0x01,0x2d,0x03,0x00,0x00,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_lv_361, 2, (MRB_SYM(x), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_string_ext_proc_debug_lines_361[] = "\x00\xc3\x02";
static mrb_irep_debug_info_file gem_mrblib_mruby_string_ext_proc_debug_file_361 = {
0, 173, 3, mrb_debug_line_packed_map, {gem_mrblib_mruby_string_ext_proc_debug_lines_361}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_string_ext_proc_debug_file_361_ = &gem_mrblib_mruby_string_ext_proc_debug_file_361;
static mrb_irep_debug_info gem_mrblib_mruby_string_ext_proc_debug_361 = {
13, 1, &gem_mrblib_mruby_string_ext_proc_debug_file_361_};
static const mrb_irep gem_mrblib_mruby_string_ext_proc_irep_361 = {
  3,5,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_string_ext_proc_iseq_361,
  NULL,gem_mrblib_mruby_string_ext_proc_syms_361,NULL,
  gem_mrblib_mruby_string_ext_proc_lv_361,
  &gem_mrblib_mruby_string_ext_proc_debug_361,
  13,0,1,0,0
};
static const mrb_irep *gem_mrblib_mruby_string_ext_proc_reps_18[2] = {
  &gem_mrblib_mruby_string_ext_proc_irep_360,
  &gem_mrblib_mruby_string_ext_proc_irep_361,
};
static const mrb_pool_value gem_mrblib_mruby_string_ext_proc_pool_18[1] = {
{IREP_TT_STR|(0<<2), {""}},
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_syms_18, 4, (MRB_SYM_Q(block_given), MRB_SYM(split), MRB_SYM(each), MRB_SYM(map), ), const);
static const mrb_code gem_mrblib_mruby_string_ext_proc_iseq_18[53] = {
0x33,0x00,0x00,0x01,0x12,0x02,0x2d,0x02,0x00,0x00,0x25,0x02,0x00,0x15,0x12,0x02,0x50,0x03,0x00,0x2d,
0x02,0x01,0x01,0x56,0x03,0x00,0x2e,0x02,0x02,0x00,0x12,0x02,0x23,0x00,0x10,0x12,0x02,0x50,0x03,0x00,
0x2d,0x02,0x01,0x01,0x56,0x03,0x01,0x2e,0x02,0x03,0x00,0x37,0x02,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_lv_18, 1, (MRB_SYM(block), ), const);
static char gem_mrblib_mruby_string_ext_proc_debug_lines_18[] = "\x00\xbc\x02\x04\x01\x08\x04\x02\xfd\xff\xff\xff\x0f\x10\x03\x03\x02";
static mrb_irep_debug_info_file gem_mrblib_mruby_string_ext_proc_debug_file_18 = {
0, 173, 17, mrb_debug_line_packed_map, {gem_mrblib_mruby_string_ext_proc_debug_lines_18}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_string_ext_proc_debug_file_18_ = &gem_mrblib_mruby_string_ext_proc_debug_file_18;
static mrb_irep_debug_info gem_mrblib_mruby_string_ext_proc_debug_18 = {
53, 1, &gem_mrblib_mruby_string_ext_proc_debug_file_18_};
static const mrb_irep gem_mrblib_mruby_string_ext_proc_irep_18 = {
  2,5,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_string_ext_proc_iseq_18,
  gem_mrblib_mruby_string_ext_proc_pool_18,gem_mrblib_mruby_string_ext_proc_syms_18,gem_mrblib_mruby_string_ext_proc_reps_18,
  gem_mrblib_mruby_string_ext_proc_lv_18,
  &gem_mrblib_mruby_string_ext_proc_debug_18,
  53,1,4,2,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_syms_19, 1, (MRB_OPSYM(aset), ), const);
static const mrb_code gem_mrblib_mruby_string_ext_proc_iseq_19[24] = {
0x33,0x04,0x00,0x00,0x01,0x03,0x01,0x12,0x04,0x06,0x05,0x06,0x06,0x01,0x07,0x03,0x2d,0x04,0x00,0x03,
0x12,0x03,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_lv_19, 2, (MRB_SYM(arg), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_string_ext_proc_debug_lines_19[] = "\x00\xd1\x02\x04\x01\x10\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_string_ext_proc_debug_file_19 = {
0, 173, 7, mrb_debug_line_packed_map, {gem_mrblib_mruby_string_ext_proc_debug_lines_19}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_string_ext_proc_debug_file_19_ = &gem_mrblib_mruby_string_ext_proc_debug_file_19;
static mrb_irep_debug_info gem_mrblib_mruby_string_ext_proc_debug_19 = {
24, 1, &gem_mrblib_mruby_string_ext_proc_debug_file_19_};
static const mrb_irep gem_mrblib_mruby_string_ext_proc_irep_19 = {
  3,9,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_string_ext_proc_iseq_19,
  NULL,gem_mrblib_mruby_string_ext_proc_syms_19,NULL,
  gem_mrblib_mruby_string_ext_proc_lv_19,
  &gem_mrblib_mruby_string_ext_proc_debug_19,
  24,0,1,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_syms_406, 1, (MRB_SYM(call), ), const);
static const mrb_code gem_mrblib_mruby_string_ext_proc_iseq_406[17] = {
0x33,0x04,0x00,0x00,0x21,0x03,0x01,0x00,0x01,0x04,0x01,0x2d,0x03,0x00,0x01,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_lv_406, 2, (MRB_SYM(line), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_string_ext_proc_debug_lines_406[] = "\x00\xe4\x02\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_string_ext_proc_debug_file_406 = {
0, 173, 5, mrb_debug_line_packed_map, {gem_mrblib_mruby_string_ext_proc_debug_lines_406}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_string_ext_proc_debug_file_406_ = &gem_mrblib_mruby_string_ext_proc_debug_file_406;
static mrb_irep_debug_info gem_mrblib_mruby_string_ext_proc_debug_406 = {
17, 1, &gem_mrblib_mruby_string_ext_proc_debug_file_406_};
static const mrb_irep gem_mrblib_mruby_string_ext_proc_irep_406 = {
  3,6,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_string_ext_proc_iseq_406,
  NULL,gem_mrblib_mruby_string_ext_proc_syms_406,NULL,
  gem_mrblib_mruby_string_ext_proc_lv_406,
  &gem_mrblib_mruby_string_ext_proc_debug_406,
  17,0,1,0,0
};
static const mrb_irep *gem_mrblib_mruby_string_ext_proc_reps_20[1] = {
  &gem_mrblib_mruby_string_ext_proc_irep_406,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_syms_20, 2, (MRB_SYM(__lines), MRB_SYM(each), ), const);
static const mrb_code gem_mrblib_mruby_string_ext_proc_iseq_20[32] = {
0x33,0x00,0x00,0x01,0x12,0x03,0x2d,0x03,0x00,0x00,0x01,0x02,0x03,0x01,0x03,0x01,0x25,0x03,0x00,0x0a,
0x01,0x03,0x02,0x56,0x04,0x00,0x2e,0x03,0x01,0x00,0x37,0x02,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_lv_20, 2, (MRB_SYM(blk), MRB_SYM(lines), ), const);
static char gem_mrblib_mruby_string_ext_proc_debug_lines_20[] = "\x00\xe1\x02\x04\x01\x09\x01\x05\x01\x0c\x04";
static mrb_irep_debug_info_file gem_mrblib_mruby_string_ext_proc_debug_file_20 = {
0, 173, 11, mrb_debug_line_packed_map, {gem_mrblib_mruby_string_ext_proc_debug_lines_20}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_string_ext_proc_debug_file_20_ = &gem_mrblib_mruby_string_ext_proc_debug_file_20;
static mrb_irep_debug_info gem_mrblib_mruby_string_ext_proc_debug_20 = {
32, 1, &gem_mrblib_mruby_string_ext_proc_debug_file_20_};
static const mrb_irep gem_mrblib_mruby_string_ext_proc_irep_20 = {
  3,5,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_string_ext_proc_iseq_20,
  NULL,gem_mrblib_mruby_string_ext_proc_syms_20,gem_mrblib_mruby_string_ext_proc_reps_20,
  gem_mrblib_mruby_string_ext_proc_lv_20,
  &gem_mrblib_mruby_string_ext_proc_debug_20,
  32,0,2,1,0
};
static const mrb_pool_value gem_mrblib_mruby_string_ext_proc_pool_21[4] = {
{IREP_TT_STR|(26<<2), {"\x6e\x6f\x20\x69\x6d\x70\x6c\x69\x63\x69\x74\x20\x63\x6f\x6e\x76\x65\x72\x73\x69\x6f\x6e\x20\x6f\x66\x20"}},
{IREP_TT_STR|(12<<2), {"\x20\x69\x6e\x74\x6f\x20\x53\x74\x72\x69\x6e\x67"}},
{IREP_TT_STR|(10<<2), {"\x41\x53\x43\x49\x49\x2d\x38\x42\x49\x54"}},
{IREP_TT_STR|(1<<2), {"\x30"}},
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_syms_21, 17, (MRB_SYM(upto), MRB_SYM(to_enum), MRB_SYM(String), MRB_SYM_Q(kind_of), MRB_SYM(TypeError), MRB_SYM(class), MRB_SYM(raise), MRB_SYM(length), MRB_SYM(ord), MRB_SYM(chr), MRB_SYM(call), MRB_SYM(to_i), MRB_SYM(to_s), MRB_SYM(rjust), MRB_OPSYM(cmp), MRB_SYM(size), MRB_SYM(succ), ), const);
static const mrb_code gem_mrblib_mruby_string_ext_proc_iseq_21[508] = {
0x33,0x04,0x20,0x01,0x23,0x00,0x03,0x23,0x00,0x02,0x14,0x02,0x01,0x0e,0x03,0x24,0x0e,0x00,0x11,0x12,
0x0e,0x10,0x0f,0x00,0x01,0x10,0x01,0x01,0x11,0x02,0x2d,0x0e,0x01,0x03,0x37,0x0e,0x01,0x0e,0x01,0x1d,
0x0f,0x02,0x2d,0x0e,0x03,0x01,0x24,0x0e,0x00,0x1a,0x12,0x0e,0x1d,0x0f,0x04,0x50,0x10,0x00,0x01,0x11,
0x01,0x2d,0x11,0x05,0x00,0x51,0x10,0x50,0x11,0x01,0x51,0x10,0x2d,0x0e,0x06,0x02,0x12,0x0e,0x2d,0x0e,
0x07,0x00,0x01,0x04,0x0e,0x01,0x0e,0x01,0x2d,0x0e,0x07,0x00,0x01,0x05,0x0e,0x01,0x0e,0x04,0x07,0x0f,
0x41,0x0e,0x25,0x0e,0x00,0x07,0x01,0x0e,0x05,0x07,0x0f,0x41,0x0e,0x25,0x0e,0x00,0x51,0x12,0x0e,0x2d,
0x0e,0x08,0x00,0x01,0x06,0x0e,0x01,0x0e,0x01,0x2d,0x0e,0x08,0x00,0x01,0x07,0x0e,0x01,0x0e,0x06,0x01,
0x0f,0x07,0x43,0x0e,0x25,0x0e,0x00,0x2e,0x01,0x0e,0x02,0x25,0x0e,0x00,0x08,0x01,0x0e,0x06,0x01,0x0f,
0x07,0x41,0x0e,0x25,0x0e,0x00,0x03,0x27,0x00,0x18,0x01,0x0f,0x06,0x50,0x10,0x02,0x2d,0x0f,0x09,0x01,
0x3a,0x0e,0x10,0x00,0x2d,0x0e,0x0a,0x01,0x3c,0x06,0x01,0x23,0xff,0xc6,0x12,0x0e,0x37,0x0e,0x12,0x0e,
0x03,0x0f,0x0a,0x2d,0x0e,0x0b,0x01,0x01,0x08,0x0e,0x01,0x0e,0x01,0x03,0x0f,0x0a,0x2d,0x0e,0x0b,0x01,
0x01,0x09,0x0e,0x01,0x0e,0x08,0x06,0x0f,0x44,0x0e,0x24,0x0e,0x00,0x0d,0x01,0x0e,0x08,0x50,0x0f,0x03,
0x01,0x10,0x04,0x3f,0x0f,0x41,0x0e,0x25,0x0e,0x00,0x18,0x01,0x0e,0x09,0x06,0x0f,0x44,0x0e,0x24,0x0e,
0x00,0x0d,0x01,0x0e,0x09,0x50,0x0f,0x03,0x01,0x10,0x05,0x3f,0x0f,0x41,0x0e,0x25,0x0e,0x00,0x61,0x01,
0x0e,0x08,0x01,0x0f,0x09,0x43,0x0e,0x25,0x0e,0x00,0x51,0x01,0x0e,0x02,0x25,0x0e,0x00,0x08,0x01,0x0e,
0x08,0x01,0x0f,0x09,0x41,0x0e,0x25,0x0e,0x00,0x03,0x27,0x00,0x3b,0x01,0x0e,0x08,0x2d,0x0e,0x0c,0x00,
0x01,0x0a,0x0e,0x01,0x0e,0x0a,0x2d,0x0e,0x07,0x00,0x01,0x0f,0x04,0x42,0x0e,0x25,0x0e,0x00,0x10,0x01,
0x0e,0x0a,0x01,0x0f,0x04,0x50,0x10,0x03,0x2d,0x0e,0x0d,0x02,0x01,0x0a,0x0e,0x01,0x0f,0x0a,0x3a,0x0e,
0x10,0x00,0x2d,0x0e,0x0a,0x01,0x3c,0x08,0x01,0x23,0xff,0xa3,0x12,0x0e,0x37,0x0e,0x12,0x0b,0x01,0x0e,
0x0b,0x01,0x0f,0x01,0x2d,0x0e,0x0e,0x01,0x01,0x0c,0x0e,0x01,0x0e,0x0c,0x06,0x0f,0x44,0x0e,0x25,0x0e,
0x00,0x03,0x27,0x00,0x63,0x01,0x0e,0x02,0x25,0x0e,0x00,0x07,0x01,0x0e,0x0c,0x06,0x0f,0x41,0x0e,0x25,
0x0e,0x00,0x03,0x27,0x00,0x4e,0x01,0x0f,0x0b,0x3a,0x0e,0x10,0x00,0x2d,0x0e,0x0a,0x01,0x01,0x0e,0x0c,
0x06,0x0f,0x41,0x0e,0x25,0x0e,0x00,0x03,0x27,0x00,0x35,0x01,0x0e,0x0b,0x2d,0x0e,0x0f,0x00,0x01,0x0d,
0x0e,0x01,0x0e,0x0d,0x01,0x0f,0x01,0x2d,0x0f,0x0f,0x00,0x44,0x0e,0x24,0x0e,0x00,0x07,0x01,0x0e,0x0d,
0x06,0x0f,0x41,0x0e,0x25,0x0e,0x00,0x03,0x27,0x00,0x0d,0x01,0x0e,0x0b,0x2d,0x0e,0x10,0x00,0x01,0x0b,
0x0e,0x23,0xff,0x82,0x12,0x0e,0x37,0x0e,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_lv_21, 13, (MRB_SYM(max), MRB_SYM(exclusive), MRB_SYM(block), MRB_SYM(len), MRB_SYM(maxlen), MRB_SYM(c), MRB_SYM(e), MRB_SYM(bi), MRB_SYM(ei), MRB_SYM(s), MRB_SYM(bs), MRB_SYM(n), MRB_SYM(bsiz), ), const);
static char gem_mrblib_mruby_string_ext_proc_debug_lines_21[] = "\x00\x89\x03\x0c\x01\x18\x01\x28\x02\x09\x01\x0a\x02\x14\x08\x02\xf9\xff\xff\xff\x0f\x09\x01\x0a\x01\x0a\x03\x02\xfe\xff\xff\xff\x0f\x14\x02\x02\xff\xff\xff\xff\x0f\x12\x01\x06\x02\x04\x03\x0c\x01\x0d\x01\x36\x08\x02\xf9\xff\xff\xff\x0f\x0a\x05\x02\xfc\xff\xff\xff\x0f\x14\x04\x02\xfd\xff\xff\xff\x0f\x0a\x01\x20\x01\x0b\x01\x06\x02\x04\x02\x02\x02\x0d\x01\x0c\x06\x02\xfb\xff\xff\xff\x0f\x13\x05\x02\xfc\xff\xff\xff\x0f\x0b\x01\x0c\x03\x02\xfe\xff\xff\xff\x0f\x0a\x01\x1c\x01\x0f\x02";
static mrb_irep_debug_info_file gem_mrblib_mruby_string_ext_proc_debug_file_21 = {
0, 173, 121, mrb_debug_line_packed_map, {gem_mrblib_mruby_string_ext_proc_debug_lines_21}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_string_ext_proc_debug_file_21_ = &gem_mrblib_mruby_string_ext_proc_debug_file_21;
static mrb_irep_debug_info gem_mrblib_mruby_string_ext_proc_debug_21 = {
508, 1, &gem_mrblib_mruby_string_ext_proc_debug_file_21_};
static const mrb_irep gem_mrblib_mruby_string_ext_proc_irep_21 = {
  14,19,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_string_ext_proc_iseq_21,
  gem_mrblib_mruby_string_ext_proc_pool_21,gem_mrblib_mruby_string_ext_proc_syms_21,NULL,
  gem_mrblib_mruby_string_ext_proc_lv_21,
  &gem_mrblib_mruby_string_ext_proc_debug_21,
  508,4,17,0,0
};
static const mrb_pool_value gem_mrblib_mruby_string_ext_proc_pool_22[1] = {
{IREP_TT_STR|(1<<2), {"\x30"}},
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_syms_22, 6, (MRB_SYM(length), MRB_SYM(to_i), MRB_SYM(to_s), MRB_SYM(rjust), MRB_SYM(call), MRB_SYM(succ), ), const);
static const mrb_code gem_mrblib_mruby_string_ext_proc_iseq_22[146] = {
0x33,0x00,0x00,0x01,0x12,0x06,0x2d,0x06,0x00,0x00,0x01,0x02,0x06,0x12,0x06,0x03,0x07,0x0a,0x2d,0x06,
0x01,0x01,0x01,0x03,0x06,0x01,0x06,0x03,0x06,0x07,0x44,0x06,0x24,0x06,0x00,0x0d,0x01,0x06,0x03,0x50,
0x07,0x00,0x01,0x08,0x02,0x3f,0x07,0x41,0x06,0x25,0x06,0x00,0x3f,0x01,0x06,0x03,0x2d,0x06,0x02,0x00,
0x01,0x04,0x06,0x01,0x06,0x04,0x2d,0x06,0x00,0x00,0x01,0x07,0x02,0x42,0x06,0x25,0x06,0x00,0x10,0x01,
0x06,0x04,0x01,0x07,0x02,0x50,0x08,0x00,0x2d,0x06,0x03,0x02,0x01,0x04,0x06,0x01,0x07,0x04,0x3a,0x06,
0x00,0x00,0x2d,0x06,0x04,0x01,0x3c,0x03,0x01,0x23,0xff,0xc5,0x12,0x06,0x37,0x06,0x12,0x05,0x01,0x07,
0x05,0x3a,0x06,0x00,0x00,0x2d,0x06,0x04,0x01,0x01,0x06,0x05,0x2d,0x06,0x05,0x00,0x01,0x05,0x06,0x23,
0xff,0xe8,0x12,0x06,0x37,0x06,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_lv_22, 5, (MRB_SYM(block), MRB_SYM(len), MRB_SYM(bi), MRB_SYM(s), MRB_SYM(bs), ), const);
static char gem_mrblib_mruby_string_ext_proc_debug_lines_22[] = "\x00\xb5\x03\x04\x01\x09\x02\x0c\x01\x1a\x07\x02\xfb\xff\xff\xff\x0f\x0a\x01\x20\x01\x0b\x01\x06\x02\x04\x02\x02\x02\x0b\x01\x0d\x02";
static mrb_irep_debug_info_file gem_mrblib_mruby_string_ext_proc_debug_file_22 = {
0, 173, 33, mrb_debug_line_packed_map, {gem_mrblib_mruby_string_ext_proc_debug_lines_22}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_string_ext_proc_debug_file_22_ = &gem_mrblib_mruby_string_ext_proc_debug_file_22;
static mrb_irep_debug_info gem_mrblib_mruby_string_ext_proc_debug_22 = {
146, 1, &gem_mrblib_mruby_string_ext_proc_debug_file_22_};
static const mrb_irep gem_mrblib_mruby_string_ext_proc_irep_22 = {
  6,10,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_string_ext_proc_iseq_22,
  gem_mrblib_mruby_string_ext_proc_pool_22,gem_mrblib_mruby_string_ext_proc_syms_22,NULL,
  gem_mrblib_mruby_string_ext_proc_lv_22,
  &gem_mrblib_mruby_string_ext_proc_debug_22,
  146,1,6,0,0
};
static const mrb_irep *gem_mrblib_mruby_string_ext_proc_reps_1[21] = {
  &gem_mrblib_mruby_string_ext_proc_irep_2,
  &gem_mrblib_mruby_string_ext_proc_irep_3,
  &gem_mrblib_mruby_string_ext_proc_irep_4,
  &gem_mrblib_mruby_string_ext_proc_irep_5,
  &gem_mrblib_mruby_string_ext_proc_irep_6,
  &gem_mrblib_mruby_string_ext_proc_irep_7,
  &gem_mrblib_mruby_string_ext_proc_irep_8,
  &gem_mrblib_mruby_string_ext_proc_irep_9,
  &gem_mrblib_mruby_string_ext_proc_irep_10,
  &gem_mrblib_mruby_string_ext_proc_irep_11,
  &gem_mrblib_mruby_string_ext_proc_irep_12,
  &gem_mrblib_mruby_string_ext_proc_irep_13,
  &gem_mrblib_mruby_string_ext_proc_irep_14,
  &gem_mrblib_mruby_string_ext_proc_irep_15,
  &gem_mrblib_mruby_string_ext_proc_irep_16,
  &gem_mrblib_mruby_string_ext_proc_irep_17,
  &gem_mrblib_mruby_string_ext_proc_irep_18,
  &gem_mrblib_mruby_string_ext_proc_irep_19,
  &gem_mrblib_mruby_string_ext_proc_irep_20,
  &gem_mrblib_mruby_string_ext_proc_irep_21,
  &gem_mrblib_mruby_string_ext_proc_irep_22,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_syms_1, 22, (MRB_SYM(clear), MRB_SYM(lstrip), MRB_SYM(rstrip), MRB_SYM(strip), MRB_SYM_B(lstrip), MRB_SYM_B(rstrip), MRB_SYM_B(strip), MRB_SYM(partition), MRB_SYM(rpartition), MRB_SYM_B(slice), MRB_SYM(insert), MRB_SYM(ljust), MRB_SYM(rjust), MRB_SYM(center), MRB_SYM(chars), MRB_SYM(each_char), MRB_SYM(codepoints), MRB_SYM(each_codepoint), MRB_SYM(prepend), MRB_SYM(lines), MRB_SYM(upto), MRB_SYM(__upto_endless), ), const);
static const mrb_code gem_mrblib_mruby_string_ext_proc_iseq_1[173] = {
0x62,0x01,0x57,0x02,0x00,0x5e,0x01,0x00,0x62,0x01,0x57,0x02,0x01,0x5e,0x01,0x01,0x62,0x01,0x57,0x02,
0x02,0x5e,0x01,0x02,0x62,0x01,0x57,0x02,0x03,0x5e,0x01,0x03,0x62,0x01,0x57,0x02,0x04,0x5e,0x01,0x04,
0x62,0x01,0x57,0x02,0x05,0x5e,0x01,0x05,0x62,0x01,0x57,0x02,0x06,0x5e,0x01,0x06,0x62,0x01,0x57,0x02,
0x07,0x5e,0x01,0x07,0x62,0x01,0x57,0x02,0x08,0x5e,0x01,0x08,0x62,0x01,0x57,0x02,0x09,0x5e,0x01,0x09,
0x62,0x01,0x57,0x02,0x0a,0x5e,0x01,0x0a,0x62,0x01,0x57,0x02,0x0b,0x5e,0x01,0x0b,0x62,0x01,0x57,0x02,
0x0c,0x5e,0x01,0x0c,0x62,0x01,0x57,0x02,0x0d,0x5e,0x01,0x0d,0x62,0x01,0x57,0x02,0x0e,0x5e,0x01,0x0e,
0x62,0x01,0x57,0x02,0x0f,0x5e,0x01,0x0f,0x62,0x01,0x57,0x02,0x10,0x5e,0x01,0x10,0x5f,0x11,0x10,0x62,
0x01,0x57,0x02,0x11,0x5e,0x01,0x12,0x62,0x01,0x57,0x02,0x12,0x5e,0x01,0x13,0x62,0x01,0x57,0x02,0x13,
0x5e,0x01,0x14,0x62,0x01,0x57,0x02,0x14,0x5e,0x01,0x15,0x37,0x01,};
static char gem_mrblib_mruby_string_ext_proc_debug_lines_1[] = "\x00\x0c\x08\x0e\x08\x11\x08\x10\x08\x13\x08\x11\x08\x0d\x08\x06\x08\x0b\x08\x1b\x08\x48\x08\x15\x08\x13\x08\x13\x08\x09\x08\x0e\x08\x0a\x08\x0a\x03\x0b\x08\x10\x08\x28\x08\x2c";
static mrb_irep_debug_info_file gem_mrblib_mruby_string_ext_proc_debug_file_1 = {
0, 173, 44, mrb_debug_line_packed_map, {gem_mrblib_mruby_string_ext_proc_debug_lines_1}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_string_ext_proc_debug_file_1_ = &gem_mrblib_mruby_string_ext_proc_debug_file_1;
static mrb_irep_debug_info gem_mrblib_mruby_string_ext_proc_debug_1 = {
173, 1, &gem_mrblib_mruby_string_ext_proc_debug_file_1_};
static const mrb_irep gem_mrblib_mruby_string_ext_proc_irep_1 = {
  1,3,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_string_ext_proc_iseq_1,
  NULL,gem_mrblib_mruby_string_ext_proc_syms_1,gem_mrblib_mruby_string_ext_proc_reps_1,
  NULL,					/* lv */
  &gem_mrblib_mruby_string_ext_proc_debug_1,
  173,0,22,21,0
};
static const mrb_irep *gem_mrblib_mruby_string_ext_proc_reps_0[1] = {
  &gem_mrblib_mruby_string_ext_proc_irep_1,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_string_ext_proc_syms_0, 1, (MRB_SYM(String), ), const);
static const mrb_code gem_mrblib_mruby_string_ext_proc_iseq_0[13] = {
0x11,0x01,0x11,0x02,0x5b,0x01,0x00,0x5d,0x01,0x00,0x37,0x01,0x68,};
static char gem_mrblib_mruby_string_ext_proc_debug_lines_0[] = "\x00\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_string_ext_proc_debug_file_0 = {
0, 173, 2, mrb_debug_line_packed_map, {gem_mrblib_mruby_string_ext_proc_debug_lines_0}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_string_ext_proc_debug_file_0_ = &gem_mrblib_mruby_string_ext_proc_debug_file_0;
static mrb_irep_debug_info gem_mrblib_mruby_string_ext_proc_debug_0 = {
13, 1, &gem_mrblib_mruby_string_ext_proc_debug_file_0_};
static const mrb_irep gem_mrblib_mruby_string_ext_proc_irep_0 = {
  1,3,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_string_ext_proc_iseq_0,
  NULL,gem_mrblib_mruby_string_ext_proc_syms_0,gem_mrblib_mruby_string_ext_proc_reps_0,
  NULL,					/* lv */
  &gem_mrblib_mruby_string_ext_proc_debug_0,
  13,0,1,1,0
};
static
const struct RProc gem_mrblib_mruby_string_ext_proc[] = {{
NULL,NULL,MRB_TT_PROC,MRB_GC_RED,0,{&gem_mrblib_mruby_string_ext_proc_irep_0},NULL,{NULL},
}};
static void
gem_mrblib_mruby_string_ext_proc_init_syms(mrb_state *mrb)
{
  gem_mrblib_mruby_string_ext_proc_debug_file_2.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-string-ext/mrblib/string.rb");
  gem_mrblib_mruby_string_ext_proc_debug_file_3.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-string-ext/mrblib/string.rb");
  gem_mrblib_mruby_string_ext_proc_debug_file_4.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-string-ext/mrblib/string.rb");
  gem_mrblib_mruby_string_ext_proc_debug_file_5.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-string-ext/mrblib/string.rb");
  gem_mrblib_mruby_string_ext_proc_debug_file_6.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-string-ext/mrblib/string.rb");
  gem_mrblib_mruby_string_ext_proc_debug_file_7.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-string-ext/mrblib/string.rb");
  gem_mrblib_mruby_string_ext_proc_debug_file_8.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-string-ext/mrblib/string.rb");
  gem_mrblib_mruby_string_ext_proc_debug_file_9.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-string-ext/mrblib/string.rb");
  gem_mrblib_mruby_string_ext_proc_debug_file_10.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-string-ext/mrblib/string.rb");
  gem_mrblib_mruby_string_ext_proc_debug_file_11.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-string-ext/mrblib/string.rb");
  gem_mrblib_mruby_string_ext_proc_debug_file_12.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-string-ext/mrblib/string.rb");
  gem_mrblib_mruby_string_ext_proc_debug_file_13.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-string-ext/mrblib/string.rb");
  gem_mrblib_mruby_string_ext_proc_debug_file_14.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-string-ext/mrblib/string.rb");
  gem_mrblib_mruby_string_ext_proc_debug_file_15.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-string-ext/mrblib/string.rb");
  gem_mrblib_mruby_string_ext_proc_debug_file_317.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-string-ext/mrblib/string.rb");
  gem_mrblib_mruby_string_ext_proc_debug_file_16.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-string-ext/mrblib/string.rb");
  gem_mrblib_mruby_string_ext_proc_debug_file_17.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-string-ext/mrblib/string.rb");
  gem_mrblib_mruby_string_ext_proc_debug_file_360.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-string-ext/mrblib/string.rb");
  gem_mrblib_mruby_string_ext_proc_debug_file_361.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-string-ext/mrblib/string.rb");
  gem_mrblib_mruby_string_ext_proc_debug_file_18.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-string-ext/mrblib/string.rb");
  gem_mrblib_mruby_string_ext_proc_debug_file_19.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-string-ext/mrblib/string.rb");
  gem_mrblib_mruby_string_ext_proc_debug_file_406.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-string-ext/mrblib/string.rb");
  gem_mrblib_mruby_string_ext_proc_debug_file_20.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-string-ext/mrblib/string.rb");
  gem_mrblib_mruby_string_ext_proc_debug_file_21.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-string-ext/mrblib/string.rb");
  gem_mrblib_mruby_string_ext_proc_debug_file_22.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-string-ext/mrblib/string.rb");
  gem_mrblib_mruby_string_ext_proc_debug_file_1.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-string-ext/mrblib/string.rb");
  gem_mrblib_mruby_string_ext_proc_debug_file_0.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-string-ext/mrblib/string.rb");
}
void mrb_mruby_string_ext_gem_init(mrb_state *mrb);
void mrb_mruby_string_ext_gem_final(mrb_state *mrb);

void GENERATED_TMP_mrb_mruby_string_ext_gem_init(mrb_state *mrb) {
  int ai = mrb_gc_arena_save(mrb);
  gem_mrblib_mruby_string_ext_proc_init_syms(mrb);
  mrb_mruby_string_ext_gem_init(mrb);
  mrb_load_proc(mrb, gem_mrblib_mruby_string_ext_proc);
  if (mrb->exc) {
    mrb_print_error(mrb);
    mrb_close(mrb);
    exit(EXIT_FAILURE);
  }
  struct REnv *e = mrb_vm_ci_env(mrb->c->cibase);
  mrb_vm_ci_env_set(mrb->c->cibase, NULL);
  mrb_env_unshare(mrb, e);
  mrb_gc_arena_restore(mrb, ai);
}

void GENERATED_TMP_mrb_mruby_string_ext_gem_final(mrb_state *mrb) {
  mrb_mruby_string_ext_gem_final(mrb);
}
