/*
 * This file is loading the irep
 * Ruby GEM code.
 *
 * IMPORTANT:
 *   This file was generated!
 *   All manual changes will get lost.
 */
#include <stdlib.h>
#include <mruby.h>
#include <mruby/irep.h>
#include <mruby/debug.h>
#include <mruby/proc.h>
#include <mruby/presym.h>

#define mrb_BRACED(...) {__VA_ARGS__}
#define mrb_DEFINE_SYMS_VAR(name, len, syms, qualifier) \
  static qualifier mrb_sym name[len] = mrb_BRACED syms

mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_file_stat_proc_syms_3, 3, (MRB_SYM(Stat), MRB_SYM(File), MRB_SYM(new), ), const);
static const mrb_code gem_mrblib_mruby_file_stat_proc_iseq_3[19] = {
0x33,0x04,0x00,0x00,0x1d,0x03,0x01,0x1f,0x03,0x00,0x01,0x04,0x01,0x2d,0x03,0x02,0x01,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_file_stat_proc_lv_3, 2, (MRB_SYM(fname), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_file_stat_proc_debug_lines_3[] = "\x00\x02\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_file_stat_proc_debug_file_3 = {
0, 173, 4, mrb_debug_line_packed_map, {gem_mrblib_mruby_file_stat_proc_debug_lines_3}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_file_stat_proc_debug_file_3_ = &gem_mrblib_mruby_file_stat_proc_debug_file_3;
static mrb_irep_debug_info gem_mrblib_mruby_file_stat_proc_debug_3 = {
19, 1, &gem_mrblib_mruby_file_stat_proc_debug_file_3_};
static const mrb_irep gem_mrblib_mruby_file_stat_proc_irep_3 = {
  3,6,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_file_stat_proc_iseq_3,
  NULL,gem_mrblib_mruby_file_stat_proc_syms_3,NULL,
  gem_mrblib_mruby_file_stat_proc_lv_3,
  &gem_mrblib_mruby_file_stat_proc_debug_3,
  19,0,3,0,0
};
static const mrb_irep *gem_mrblib_mruby_file_stat_proc_reps_1[1] = {
  &gem_mrblib_mruby_file_stat_proc_irep_3,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_file_stat_proc_syms_1, 1, (MRB_SYM(stat), ), const);
static const mrb_code gem_mrblib_mruby_file_stat_proc_iseq_1[12] = {
0x12,0x01,0x61,0x01,0x57,0x02,0x00,0x5e,0x01,0x00,0x37,0x01,};
static char gem_mrblib_mruby_file_stat_proc_debug_lines_1[] = "\x00\x02";
static mrb_irep_debug_info_file gem_mrblib_mruby_file_stat_proc_debug_file_1 = {
0, 173, 2, mrb_debug_line_packed_map, {gem_mrblib_mruby_file_stat_proc_debug_lines_1}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_file_stat_proc_debug_file_1_ = &gem_mrblib_mruby_file_stat_proc_debug_file_1;
static mrb_irep_debug_info gem_mrblib_mruby_file_stat_proc_debug_1 = {
12, 1, &gem_mrblib_mruby_file_stat_proc_debug_file_1_};
static const mrb_irep gem_mrblib_mruby_file_stat_proc_irep_1 = {
  1,2,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_file_stat_proc_iseq_1,
  NULL,gem_mrblib_mruby_file_stat_proc_syms_1,gem_mrblib_mruby_file_stat_proc_reps_1,
  NULL,					/* lv */
  &gem_mrblib_mruby_file_stat_proc_debug_1,
  12,0,1,1,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_file_stat_proc_syms_7, 4, (MRB_SYM(class), MRB_SYM_Q(kind_of), MRB_SYM(mtime), MRB_OPSYM(cmp), ), const);
static const mrb_code gem_mrblib_mruby_file_stat_proc_iseq_7[45] = {
0x33,0x04,0x00,0x00,0x01,0x03,0x01,0x12,0x04,0x2d,0x04,0x00,0x00,0x2d,0x03,0x01,0x01,0x25,0x03,0x00,
0x14,0x12,0x03,0x2d,0x03,0x02,0x00,0x01,0x04,0x01,0x2d,0x04,0x02,0x00,0x2d,0x03,0x03,0x01,0x23,0x00,
0x02,0x11,0x03,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_file_stat_proc_lv_7, 2, (MRB_SYM(other), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_file_stat_proc_debug_lines_7[] = "\x00\x05\x04\x01\x0f\x01\x14\x02";
static mrb_irep_debug_info_file gem_mrblib_mruby_file_stat_proc_debug_file_7 = {
0, 174, 8, mrb_debug_line_packed_map, {gem_mrblib_mruby_file_stat_proc_debug_lines_7}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_file_stat_proc_debug_file_7_ = &gem_mrblib_mruby_file_stat_proc_debug_file_7;
static mrb_irep_debug_info gem_mrblib_mruby_file_stat_proc_debug_7 = {
45, 1, &gem_mrblib_mruby_file_stat_proc_debug_file_7_};
static const mrb_irep gem_mrblib_mruby_file_stat_proc_irep_7 = {
  3,6,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_file_stat_proc_iseq_7,
  NULL,gem_mrblib_mruby_file_stat_proc_syms_7,NULL,
  gem_mrblib_mruby_file_stat_proc_lv_7,
  &gem_mrblib_mruby_file_stat_proc_debug_7,
  45,0,4,0,0
};
static const mrb_pool_value gem_mrblib_mruby_file_stat_proc_pool_15[2] = {
{IREP_TT_STR|(0<<2), {""}},
{IREP_TT_STR|(1<<2), {"\x3d"}},
};
static const mrb_code gem_mrblib_mruby_file_stat_proc_iseq_15[29] = {
0x33,0x08,0x00,0x00,0x50,0x04,0x00,0x01,0x05,0x01,0x51,0x04,0x50,0x05,0x01,0x51,0x04,0x01,0x05,0x02,
0x51,0x04,0x50,0x05,0x00,0x51,0x04,0x37,0x04,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_file_stat_proc_lv_15, 3, (MRB_SYM(k), MRB_SYM(v), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_file_stat_proc_debug_lines_15[] = "\x00\x2a";
static mrb_irep_debug_info_file gem_mrblib_mruby_file_stat_proc_debug_file_15 = {
0, 174, 2, mrb_debug_line_packed_map, {gem_mrblib_mruby_file_stat_proc_debug_lines_15}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_file_stat_proc_debug_file_15_ = &gem_mrblib_mruby_file_stat_proc_debug_file_15;
static mrb_irep_debug_info gem_mrblib_mruby_file_stat_proc_debug_15 = {
29, 1, &gem_mrblib_mruby_file_stat_proc_debug_file_15_};
static const mrb_irep gem_mrblib_mruby_file_stat_proc_irep_15 = {
  4,6,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_file_stat_proc_iseq_15,
  gem_mrblib_mruby_file_stat_proc_pool_15,NULL,NULL,
  gem_mrblib_mruby_file_stat_proc_lv_15,
  &gem_mrblib_mruby_file_stat_proc_debug_15,
  29,2,0,0,0
};
static const mrb_irep *gem_mrblib_mruby_file_stat_proc_reps_8[1] = {
  &gem_mrblib_mruby_file_stat_proc_irep_15,
};
static const mrb_pool_value gem_mrblib_mruby_file_stat_proc_pool_8[21] = {
{IREP_TT_STR|(2<<2), {"\x30\x78"}},
{IREP_TT_STR|(0<<2), {""}},
{IREP_TT_STR|(1<<2), {"\x30"}},
{IREP_TT_STR|(3<<2), {"\x64\x65\x76"}},
{IREP_TT_STR|(3<<2), {"\x69\x6e\x6f"}},
{IREP_TT_STR|(4<<2), {"\x6d\x6f\x64\x65"}},
{IREP_TT_STR|(5<<2), {"\x6e\x6c\x69\x6e\x6b"}},
{IREP_TT_STR|(3<<2), {"\x75\x69\x64"}},
{IREP_TT_STR|(3<<2), {"\x67\x69\x64"}},
{IREP_TT_STR|(4<<2), {"\x72\x64\x65\x76"}},
{IREP_TT_STR|(4<<2), {"\x73\x69\x7a\x65"}},
{IREP_TT_STR|(7<<2), {"\x62\x6c\x6b\x73\x69\x7a\x65"}},
{IREP_TT_STR|(6<<2), {"\x62\x6c\x6f\x63\x6b\x73"}},
{IREP_TT_STR|(5<<2), {"\x61\x74\x69\x6d\x65"}},
{IREP_TT_STR|(5<<2), {"\x6d\x74\x69\x6d\x65"}},
{IREP_TT_STR|(5<<2), {"\x63\x74\x69\x6d\x65"}},
{IREP_TT_STR|(9<<2), {"\x62\x69\x72\x74\x68\x74\x69\x6d\x65"}},
{IREP_TT_STR|(2<<2), {"\x23\x3c"}},
{IREP_TT_STR|(1<<2), {"\x20"}},
{IREP_TT_STR|(2<<2), {"\x2c\x20"}},
{IREP_TT_STR|(1<<2), {"\x3e"}},
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_file_stat_proc_syms_8, 22, (MRB_SYM(dev), MRB_SYM(Fixnum), MRB_SYM_Q(kind_of), MRB_SYM(to_s), MRB_SYM(mode), MRB_SYM(rdev), MRB_SYM(ino), MRB_SYM(nlink), MRB_SYM(uid), MRB_SYM(gid), MRB_SYM(size), MRB_SYM(blksize), MRB_SYM(blocks), MRB_SYM(atime), MRB_SYM(mtime), MRB_SYM(ctime), MRB_SYM(birthtime), MRB_OPSYM(aset), MRB_SYM(NotImplementedError), MRB_SYM(class), MRB_SYM(map), MRB_SYM(join), ), const);
static const mrb_code gem_mrblib_mruby_file_stat_proc_iseq_8[360] = {
0x33,0x00,0x00,0x00,0x12,0x06,0x2d,0x06,0x00,0x00,0x01,0x02,0x06,0x01,0x06,0x02,0x1d,0x07,0x01,0x2d,
0x06,0x02,0x01,0x25,0x06,0x00,0x17,0x50,0x06,0x00,0x01,0x07,0x02,0x03,0x08,0x10,0x2d,0x07,0x03,0x01,
0x51,0x06,0x50,0x07,0x01,0x51,0x06,0x01,0x02,0x06,0x12,0x06,0x2d,0x06,0x04,0x00,0x01,0x03,0x06,0x01,
0x06,0x03,0x1d,0x07,0x01,0x2d,0x06,0x02,0x01,0x25,0x06,0x00,0x17,0x50,0x06,0x02,0x01,0x07,0x03,0x03,
0x08,0x08,0x2d,0x07,0x03,0x01,0x51,0x06,0x50,0x07,0x01,0x51,0x06,0x01,0x03,0x06,0x12,0x06,0x2d,0x06,
0x05,0x00,0x01,0x04,0x06,0x01,0x06,0x04,0x1d,0x07,0x01,0x2d,0x06,0x02,0x01,0x25,0x06,0x00,0x17,0x50,
0x06,0x00,0x01,0x07,0x04,0x03,0x08,0x10,0x2d,0x07,0x03,0x01,0x51,0x06,0x50,0x07,0x01,0x51,0x06,0x01,
0x04,0x06,0x50,0x06,0x03,0x01,0x07,0x02,0x50,0x08,0x04,0x12,0x09,0x2d,0x09,0x06,0x00,0x50,0x0a,0x05,
0x01,0x0b,0x03,0x50,0x0c,0x06,0x12,0x0d,0x2d,0x0d,0x07,0x00,0x50,0x0e,0x07,0x12,0x0f,0x2d,0x0f,0x08,
0x00,0x50,0x10,0x08,0x12,0x11,0x2d,0x11,0x09,0x00,0x50,0x12,0x09,0x01,0x13,0x04,0x50,0x14,0x0a,0x12,
0x15,0x2d,0x15,0x0a,0x00,0x50,0x16,0x0b,0x12,0x17,0x2d,0x17,0x0b,0x00,0x50,0x18,0x0c,0x12,0x19,0x2d,
0x19,0x0c,0x00,0x50,0x1a,0x0d,0x12,0x1b,0x2d,0x1b,0x0d,0x00,0x50,0x1c,0x0e,0x12,0x1d,0x2d,0x1d,0x0e,
0x00,0x50,0x1e,0x0f,0x12,0x1f,0x2d,0x1f,0x0f,0x00,0x52,0x06,0x0d,0x01,0x05,0x06,0x12,0x06,0x2d,0x06,
0x10,0x00,0x01,0x07,0x05,0x50,0x08,0x10,0x01,0x09,0x06,0x2d,0x07,0x11,0x02,0x01,0x06,0x06,0x23,0x00,
0x14,0x28,0x06,0x1d,0x07,0x12,0x29,0x06,0x07,0x24,0x07,0x00,0x03,0x23,0x00,0x03,0x23,0x00,0x02,0x2a,
0x06,0x50,0x06,0x11,0x12,0x07,0x2d,0x07,0x13,0x00,0x2d,0x07,0x03,0x00,0x51,0x06,0x50,0x07,0x12,0x51,
0x06,0x01,0x07,0x05,0x56,0x08,0x00,0x2e,0x07,0x14,0x00,0x50,0x08,0x13,0x2d,0x07,0x15,0x01,0x51,0x06,
0x50,0x07,0x14,0x51,0x06,0x37,0x06,0x00,0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x16,0x00,0x00,0x01,0x19,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_file_stat_proc_lv_8, 5, (MRB_OPSYM(and), MRB_SYM(_dev), MRB_SYM(_mode), MRB_SYM(_rdev), MRB_SYM(stats), ), const);
static char gem_mrblib_mruby_file_stat_proc_debug_lines_8[] = "\x00\x0d\x04\x01\x09\x01\x25\x01\x09\x01\x25\x01\x09\x01\x25\x03\x06\x01\x09\x01\x06\x01\x09\x01\x09\x01\x09\x01\x06\x01\x09\x01\x09\x01\x09\x01\x09\x01\x09\x01\x0f\x03\x17\x03\x02\xfd\xff\xff\xff\x0f\x02\x01\x0b\x02\x07\x02";
static mrb_irep_debug_info_file gem_mrblib_mruby_file_stat_proc_debug_file_8 = {
0, 174, 56, mrb_debug_line_packed_map, {gem_mrblib_mruby_file_stat_proc_debug_lines_8}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_file_stat_proc_debug_file_8_ = &gem_mrblib_mruby_file_stat_proc_debug_file_8;
static mrb_irep_debug_info gem_mrblib_mruby_file_stat_proc_debug_8 = {
347, 1, &gem_mrblib_mruby_file_stat_proc_debug_file_8_};
static const mrb_irep gem_mrblib_mruby_file_stat_proc_irep_8 = {
  6,33,1,
  MRB_IREP_STATIC,gem_mrblib_mruby_file_stat_proc_iseq_8,
  gem_mrblib_mruby_file_stat_proc_pool_8,gem_mrblib_mruby_file_stat_proc_syms_8,gem_mrblib_mruby_file_stat_proc_reps_8,
  gem_mrblib_mruby_file_stat_proc_lv_8,
  &gem_mrblib_mruby_file_stat_proc_debug_8,
  347,21,22,1,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_file_stat_proc_syms_9, 1, (MRB_SYM(size), ), const);
static const mrb_code gem_mrblib_mruby_file_stat_proc_iseq_9[34] = {
0x33,0x00,0x00,0x00,0x12,0x03,0x2d,0x03,0x00,0x00,0x01,0x02,0x03,0x01,0x03,0x02,0x06,0x04,0x41,0x03,
0x25,0x03,0x00,0x05,0x11,0x03,0x23,0x00,0x03,0x01,0x03,0x02,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_file_stat_proc_lv_9, 2, (MRB_OPSYM(and), MRB_SYM(s), ), const);
static char gem_mrblib_mruby_file_stat_proc_debug_lines_9[] = "\x00\x2d\x04\x01\x09\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_file_stat_proc_debug_file_9 = {
0, 174, 6, mrb_debug_line_packed_map, {gem_mrblib_mruby_file_stat_proc_debug_lines_9}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_file_stat_proc_debug_file_9_ = &gem_mrblib_mruby_file_stat_proc_debug_file_9;
static mrb_irep_debug_info gem_mrblib_mruby_file_stat_proc_debug_9 = {
34, 1, &gem_mrblib_mruby_file_stat_proc_debug_file_9_};
static const mrb_irep gem_mrblib_mruby_file_stat_proc_irep_9 = {
  3,6,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_file_stat_proc_iseq_9,
  NULL,gem_mrblib_mruby_file_stat_proc_syms_9,NULL,
  gem_mrblib_mruby_file_stat_proc_lv_9,
  &gem_mrblib_mruby_file_stat_proc_debug_9,
  34,0,1,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_file_stat_proc_syms_10, 1, (MRB_SYM(size), ), const);
static const mrb_code gem_mrblib_mruby_file_stat_proc_iseq_10[16] = {
0x33,0x00,0x00,0x00,0x12,0x02,0x2d,0x02,0x00,0x00,0x06,0x03,0x41,0x02,0x37,0x02,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_file_stat_proc_lv_10, 1, (MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_file_stat_proc_debug_lines_10[] = "\x00\x32\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_file_stat_proc_debug_file_10 = {
0, 174, 4, mrb_debug_line_packed_map, {gem_mrblib_mruby_file_stat_proc_debug_lines_10}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_file_stat_proc_debug_file_10_ = &gem_mrblib_mruby_file_stat_proc_debug_file_10;
static mrb_irep_debug_info gem_mrblib_mruby_file_stat_proc_debug_10 = {
16, 1, &gem_mrblib_mruby_file_stat_proc_debug_file_10_};
static const mrb_irep gem_mrblib_mruby_file_stat_proc_irep_10 = {
  2,5,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_file_stat_proc_iseq_10,
  NULL,gem_mrblib_mruby_file_stat_proc_syms_10,NULL,
  gem_mrblib_mruby_file_stat_proc_lv_10,
  &gem_mrblib_mruby_file_stat_proc_debug_10,
  16,0,1,0,0
};
static const mrb_irep *gem_mrblib_mruby_file_stat_proc_reps_6[4] = {
  &gem_mrblib_mruby_file_stat_proc_irep_7,
  &gem_mrblib_mruby_file_stat_proc_irep_8,
  &gem_mrblib_mruby_file_stat_proc_irep_9,
  &gem_mrblib_mruby_file_stat_proc_irep_10,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_file_stat_proc_syms_6, 6, (MRB_SYM(Comparable), MRB_SYM(include), MRB_OPSYM(cmp), MRB_SYM(inspect), MRB_SYM_Q(size), MRB_SYM_Q(zero), ), const);
static const mrb_code gem_mrblib_mruby_file_stat_proc_iseq_6[43] = {
0x12,0x01,0x1d,0x02,0x00,0x2d,0x01,0x01,0x01,0x62,0x01,0x57,0x02,0x00,0x5e,0x01,0x02,0x62,0x01,0x57,
0x02,0x01,0x5e,0x01,0x03,0x62,0x01,0x57,0x02,0x02,0x5e,0x01,0x04,0x62,0x01,0x57,0x02,0x03,0x5e,0x01,
0x05,0x37,0x01,};
static char gem_mrblib_mruby_file_stat_proc_debug_lines_6[] = "\x00\x03\x09\x02\x08\x08\x08\x20\x08\x05";
static mrb_irep_debug_info_file gem_mrblib_mruby_file_stat_proc_debug_file_6 = {
0, 174, 10, mrb_debug_line_packed_map, {gem_mrblib_mruby_file_stat_proc_debug_lines_6}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_file_stat_proc_debug_file_6_ = &gem_mrblib_mruby_file_stat_proc_debug_file_6;
static mrb_irep_debug_info gem_mrblib_mruby_file_stat_proc_debug_6 = {
43, 1, &gem_mrblib_mruby_file_stat_proc_debug_file_6_};
static const mrb_irep gem_mrblib_mruby_file_stat_proc_irep_6 = {
  1,4,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_file_stat_proc_iseq_6,
  NULL,gem_mrblib_mruby_file_stat_proc_syms_6,gem_mrblib_mruby_file_stat_proc_reps_6,
  NULL,					/* lv */
  &gem_mrblib_mruby_file_stat_proc_debug_6,
  43,0,6,4,0
};
static const mrb_irep *gem_mrblib_mruby_file_stat_proc_reps_2[1] = {
  &gem_mrblib_mruby_file_stat_proc_irep_6,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_file_stat_proc_syms_2, 1, (MRB_SYM(Stat), ), const);
static const mrb_code gem_mrblib_mruby_file_stat_proc_iseq_2[12] = {
0x11,0x01,0x11,0x02,0x5b,0x01,0x00,0x5d,0x01,0x00,0x37,0x01,};
static char gem_mrblib_mruby_file_stat_proc_debug_lines_2[] = "\x00\x02";
static mrb_irep_debug_info_file gem_mrblib_mruby_file_stat_proc_debug_file_2 = {
0, 174, 2, mrb_debug_line_packed_map, {gem_mrblib_mruby_file_stat_proc_debug_lines_2}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_file_stat_proc_debug_file_2_ = &gem_mrblib_mruby_file_stat_proc_debug_file_2;
static mrb_irep_debug_info gem_mrblib_mruby_file_stat_proc_debug_2 = {
12, 1, &gem_mrblib_mruby_file_stat_proc_debug_file_2_};
static const mrb_irep gem_mrblib_mruby_file_stat_proc_irep_2 = {
  1,3,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_file_stat_proc_iseq_2,
  NULL,gem_mrblib_mruby_file_stat_proc_syms_2,gem_mrblib_mruby_file_stat_proc_reps_2,
  NULL,					/* lv */
  &gem_mrblib_mruby_file_stat_proc_debug_2,
  12,0,1,1,0
};
static const mrb_irep *gem_mrblib_mruby_file_stat_proc_reps_0[2] = {
  &gem_mrblib_mruby_file_stat_proc_irep_1,
  &gem_mrblib_mruby_file_stat_proc_irep_2,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_file_stat_proc_syms_0, 1, (MRB_SYM(File), ), const);
static const mrb_code gem_mrblib_mruby_file_stat_proc_iseq_0[23] = {
0x11,0x01,0x11,0x02,0x5b,0x01,0x00,0x5d,0x01,0x00,0x11,0x01,0x11,0x02,0x5b,0x01,0x00,0x5d,0x01,0x01,
0x37,0x01,0x68,};
static const mrb_irep gem_mrblib_mruby_file_stat_proc_irep_0 = {
  1,3,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_file_stat_proc_iseq_0,
  NULL,gem_mrblib_mruby_file_stat_proc_syms_0,gem_mrblib_mruby_file_stat_proc_reps_0,
  NULL,					/* lv */
  NULL,					/* debug_info */
  23,0,1,2,0
};
static
const struct RProc gem_mrblib_mruby_file_stat_proc[] = {{
NULL,NULL,MRB_TT_PROC,MRB_GC_RED,0,{&gem_mrblib_mruby_file_stat_proc_irep_0},NULL,{NULL},
}};
static void
gem_mrblib_mruby_file_stat_proc_init_syms(mrb_state *mrb)
{
  gem_mrblib_mruby_file_stat_proc_debug_file_3.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-file-stat/mrblib/ext.rb");
  gem_mrblib_mruby_file_stat_proc_debug_file_1.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-file-stat/mrblib/ext.rb");
  gem_mrblib_mruby_file_stat_proc_debug_file_7.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-file-stat/mrblib/file-stat.rb");
  gem_mrblib_mruby_file_stat_proc_debug_file_15.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-file-stat/mrblib/file-stat.rb");
  gem_mrblib_mruby_file_stat_proc_debug_file_8.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-file-stat/mrblib/file-stat.rb");
  gem_mrblib_mruby_file_stat_proc_debug_file_9.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-file-stat/mrblib/file-stat.rb");
  gem_mrblib_mruby_file_stat_proc_debug_file_10.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-file-stat/mrblib/file-stat.rb");
  gem_mrblib_mruby_file_stat_proc_debug_file_6.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-file-stat/mrblib/file-stat.rb");
  gem_mrblib_mruby_file_stat_proc_debug_file_2.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-file-stat/mrblib/file-stat.rb");
}
void mrb_mruby_file_stat_gem_init(mrb_state *mrb);
void mrb_mruby_file_stat_gem_final(mrb_state *mrb);

void GENERATED_TMP_mrb_mruby_file_stat_gem_init(mrb_state *mrb) {
  int ai = mrb_gc_arena_save(mrb);
  gem_mrblib_mruby_file_stat_proc_init_syms(mrb);
  mrb_mruby_file_stat_gem_init(mrb);
  mrb_load_proc(mrb, gem_mrblib_mruby_file_stat_proc);
  if (mrb->exc) {
    mrb_print_error(mrb);
    mrb_close(mrb);
    exit(EXIT_FAILURE);
  }
  struct REnv *e = mrb_vm_ci_env(mrb->c->cibase);
  mrb_vm_ci_env_set(mrb->c->cibase, NULL);
  mrb_env_unshare(mrb, e);
  mrb_gc_arena_restore(mrb, ai);
}

void GENERATED_TMP_mrb_mruby_file_stat_gem_final(mrb_state *mrb) {
  mrb_mruby_file_stat_gem_final(mrb);
}
