/*
 * This file is loading the irep
 * Ruby GEM code.
 *
 * IMPORTANT:
 *   This file was generated!
 *   All manual changes will get lost.
 */
#include <stdlib.h>
#include <mruby.h>
#include <mruby/irep.h>
#include <mruby/debug.h>
#include <mruby/proc.h>
#include <mruby/presym.h>

#define mrb_BRACED(...) {__VA_ARGS__}
#define mrb_DEFINE_SYMS_VAR(name, len, syms, qualifier) \
  static qualifier mrb_sym name[len] = mrb_BRACED syms

mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_time_proc_syms_2, 1, (MRB_SYM(wday), ), const);
static const mrb_code gem_mrblib_mruby_time_proc_iseq_2[16] = {
0x33,0x00,0x00,0x00,0x12,0x02,0x2d,0x02,0x00,0x00,0x06,0x03,0x41,0x02,0x37,0x02,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_time_proc_lv_2, 1, (MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_time_proc_debug_lines_2[] = "\x00\x02";
static mrb_irep_debug_info_file gem_mrblib_mruby_time_proc_debug_file_2 = {
0, 173, 2, mrb_debug_line_packed_map, {gem_mrblib_mruby_time_proc_debug_lines_2}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_time_proc_debug_file_2_ = &gem_mrblib_mruby_time_proc_debug_file_2;
static mrb_irep_debug_info gem_mrblib_mruby_time_proc_debug_2 = {
16, 1, &gem_mrblib_mruby_time_proc_debug_file_2_};
static const mrb_irep gem_mrblib_mruby_time_proc_irep_2 = {
  2,5,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_time_proc_iseq_2,
  NULL,gem_mrblib_mruby_time_proc_syms_2,NULL,
  gem_mrblib_mruby_time_proc_lv_2,
  &gem_mrblib_mruby_time_proc_debug_2,
  16,0,1,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_time_proc_syms_3, 1, (MRB_SYM(wday), ), const);
static const mrb_code gem_mrblib_mruby_time_proc_iseq_3[16] = {
0x33,0x00,0x00,0x00,0x12,0x02,0x2d,0x02,0x00,0x00,0x07,0x03,0x41,0x02,0x37,0x02,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_time_proc_lv_3, 1, (MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_time_proc_debug_lines_3[] = "\x00\x03";
static mrb_irep_debug_info_file gem_mrblib_mruby_time_proc_debug_file_3 = {
0, 173, 2, mrb_debug_line_packed_map, {gem_mrblib_mruby_time_proc_debug_lines_3}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_time_proc_debug_file_3_ = &gem_mrblib_mruby_time_proc_debug_file_3;
static mrb_irep_debug_info gem_mrblib_mruby_time_proc_debug_3 = {
16, 1, &gem_mrblib_mruby_time_proc_debug_file_3_};
static const mrb_irep gem_mrblib_mruby_time_proc_irep_3 = {
  2,5,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_time_proc_iseq_3,
  NULL,gem_mrblib_mruby_time_proc_syms_3,NULL,
  gem_mrblib_mruby_time_proc_lv_3,
  &gem_mrblib_mruby_time_proc_debug_3,
  16,0,1,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_time_proc_syms_4, 1, (MRB_SYM(wday), ), const);
static const mrb_code gem_mrblib_mruby_time_proc_iseq_4[16] = {
0x33,0x00,0x00,0x00,0x12,0x02,0x2d,0x02,0x00,0x00,0x08,0x03,0x41,0x02,0x37,0x02,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_time_proc_lv_4, 1, (MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_time_proc_debug_lines_4[] = "\x00\x04";
static mrb_irep_debug_info_file gem_mrblib_mruby_time_proc_debug_file_4 = {
0, 173, 2, mrb_debug_line_packed_map, {gem_mrblib_mruby_time_proc_debug_lines_4}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_time_proc_debug_file_4_ = &gem_mrblib_mruby_time_proc_debug_file_4;
static mrb_irep_debug_info gem_mrblib_mruby_time_proc_debug_4 = {
16, 1, &gem_mrblib_mruby_time_proc_debug_file_4_};
static const mrb_irep gem_mrblib_mruby_time_proc_irep_4 = {
  2,5,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_time_proc_iseq_4,
  NULL,gem_mrblib_mruby_time_proc_syms_4,NULL,
  gem_mrblib_mruby_time_proc_lv_4,
  &gem_mrblib_mruby_time_proc_debug_4,
  16,0,1,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_time_proc_syms_5, 1, (MRB_SYM(wday), ), const);
static const mrb_code gem_mrblib_mruby_time_proc_iseq_5[16] = {
0x33,0x00,0x00,0x00,0x12,0x02,0x2d,0x02,0x00,0x00,0x09,0x03,0x41,0x02,0x37,0x02,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_time_proc_lv_5, 1, (MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_time_proc_debug_lines_5[] = "\x00\x05";
static mrb_irep_debug_info_file gem_mrblib_mruby_time_proc_debug_file_5 = {
0, 173, 2, mrb_debug_line_packed_map, {gem_mrblib_mruby_time_proc_debug_lines_5}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_time_proc_debug_file_5_ = &gem_mrblib_mruby_time_proc_debug_file_5;
static mrb_irep_debug_info gem_mrblib_mruby_time_proc_debug_5 = {
16, 1, &gem_mrblib_mruby_time_proc_debug_file_5_};
static const mrb_irep gem_mrblib_mruby_time_proc_irep_5 = {
  2,5,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_time_proc_iseq_5,
  NULL,gem_mrblib_mruby_time_proc_syms_5,NULL,
  gem_mrblib_mruby_time_proc_lv_5,
  &gem_mrblib_mruby_time_proc_debug_5,
  16,0,1,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_time_proc_syms_6, 1, (MRB_SYM(wday), ), const);
static const mrb_code gem_mrblib_mruby_time_proc_iseq_6[16] = {
0x33,0x00,0x00,0x00,0x12,0x02,0x2d,0x02,0x00,0x00,0x0a,0x03,0x41,0x02,0x37,0x02,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_time_proc_lv_6, 1, (MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_time_proc_debug_lines_6[] = "\x00\x06";
static mrb_irep_debug_info_file gem_mrblib_mruby_time_proc_debug_file_6 = {
0, 173, 2, mrb_debug_line_packed_map, {gem_mrblib_mruby_time_proc_debug_lines_6}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_time_proc_debug_file_6_ = &gem_mrblib_mruby_time_proc_debug_file_6;
static mrb_irep_debug_info gem_mrblib_mruby_time_proc_debug_6 = {
16, 1, &gem_mrblib_mruby_time_proc_debug_file_6_};
static const mrb_irep gem_mrblib_mruby_time_proc_irep_6 = {
  2,5,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_time_proc_iseq_6,
  NULL,gem_mrblib_mruby_time_proc_syms_6,NULL,
  gem_mrblib_mruby_time_proc_lv_6,
  &gem_mrblib_mruby_time_proc_debug_6,
  16,0,1,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_time_proc_syms_7, 1, (MRB_SYM(wday), ), const);
static const mrb_code gem_mrblib_mruby_time_proc_iseq_7[16] = {
0x33,0x00,0x00,0x00,0x12,0x02,0x2d,0x02,0x00,0x00,0x0b,0x03,0x41,0x02,0x37,0x02,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_time_proc_lv_7, 1, (MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_time_proc_debug_lines_7[] = "\x00\x07";
static mrb_irep_debug_info_file gem_mrblib_mruby_time_proc_debug_file_7 = {
0, 173, 2, mrb_debug_line_packed_map, {gem_mrblib_mruby_time_proc_debug_lines_7}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_time_proc_debug_file_7_ = &gem_mrblib_mruby_time_proc_debug_file_7;
static mrb_irep_debug_info gem_mrblib_mruby_time_proc_debug_7 = {
16, 1, &gem_mrblib_mruby_time_proc_debug_file_7_};
static const mrb_irep gem_mrblib_mruby_time_proc_irep_7 = {
  2,5,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_time_proc_iseq_7,
  NULL,gem_mrblib_mruby_time_proc_syms_7,NULL,
  gem_mrblib_mruby_time_proc_lv_7,
  &gem_mrblib_mruby_time_proc_debug_7,
  16,0,1,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_time_proc_syms_8, 1, (MRB_SYM(wday), ), const);
static const mrb_code gem_mrblib_mruby_time_proc_iseq_8[16] = {
0x33,0x00,0x00,0x00,0x12,0x02,0x2d,0x02,0x00,0x00,0x0c,0x03,0x41,0x02,0x37,0x02,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_time_proc_lv_8, 1, (MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_time_proc_debug_lines_8[] = "\x00\x08";
static mrb_irep_debug_info_file gem_mrblib_mruby_time_proc_debug_file_8 = {
0, 173, 2, mrb_debug_line_packed_map, {gem_mrblib_mruby_time_proc_debug_lines_8}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_time_proc_debug_file_8_ = &gem_mrblib_mruby_time_proc_debug_file_8;
static mrb_irep_debug_info gem_mrblib_mruby_time_proc_debug_8 = {
16, 1, &gem_mrblib_mruby_time_proc_debug_file_8_};
static const mrb_irep gem_mrblib_mruby_time_proc_irep_8 = {
  2,5,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_time_proc_iseq_8,
  NULL,gem_mrblib_mruby_time_proc_syms_8,NULL,
  gem_mrblib_mruby_time_proc_lv_8,
  &gem_mrblib_mruby_time_proc_debug_8,
  16,0,1,0,0
};
static const mrb_irep *gem_mrblib_mruby_time_proc_reps_1[7] = {
  &gem_mrblib_mruby_time_proc_irep_2,
  &gem_mrblib_mruby_time_proc_irep_3,
  &gem_mrblib_mruby_time_proc_irep_4,
  &gem_mrblib_mruby_time_proc_irep_5,
  &gem_mrblib_mruby_time_proc_irep_6,
  &gem_mrblib_mruby_time_proc_irep_7,
  &gem_mrblib_mruby_time_proc_irep_8,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_time_proc_syms_1, 7, (MRB_SYM_Q(sunday), MRB_SYM_Q(monday), MRB_SYM_Q(tuesday), MRB_SYM_Q(wednesday), MRB_SYM_Q(thursday), MRB_SYM_Q(friday), MRB_SYM_Q(saturday), ), const);
static const mrb_code gem_mrblib_mruby_time_proc_iseq_1[58] = {
0x62,0x01,0x57,0x02,0x00,0x5e,0x01,0x00,0x62,0x01,0x57,0x02,0x01,0x5e,0x01,0x01,0x62,0x01,0x57,0x02,
0x02,0x5e,0x01,0x02,0x62,0x01,0x57,0x02,0x03,0x5e,0x01,0x03,0x62,0x01,0x57,0x02,0x04,0x5e,0x01,0x04,
0x62,0x01,0x57,0x02,0x05,0x5e,0x01,0x05,0x62,0x01,0x57,0x02,0x06,0x5e,0x01,0x06,0x37,0x01,};
static char gem_mrblib_mruby_time_proc_debug_lines_1[] = "\x00\x02\x08\x01\x08\x01\x08\x01\x08\x01\x08\x01\x08\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_time_proc_debug_file_1 = {
0, 173, 14, mrb_debug_line_packed_map, {gem_mrblib_mruby_time_proc_debug_lines_1}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_time_proc_debug_file_1_ = &gem_mrblib_mruby_time_proc_debug_file_1;
static mrb_irep_debug_info gem_mrblib_mruby_time_proc_debug_1 = {
58, 1, &gem_mrblib_mruby_time_proc_debug_file_1_};
static const mrb_irep gem_mrblib_mruby_time_proc_irep_1 = {
  1,3,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_time_proc_iseq_1,
  NULL,gem_mrblib_mruby_time_proc_syms_1,gem_mrblib_mruby_time_proc_reps_1,
  NULL,					/* lv */
  &gem_mrblib_mruby_time_proc_debug_1,
  58,0,7,7,0
};
static const mrb_irep *gem_mrblib_mruby_time_proc_reps_0[1] = {
  &gem_mrblib_mruby_time_proc_irep_1,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_time_proc_syms_0, 1, (MRB_SYM(Time), ), const);
static const mrb_code gem_mrblib_mruby_time_proc_iseq_0[13] = {
0x11,0x01,0x11,0x02,0x5b,0x01,0x00,0x5d,0x01,0x00,0x37,0x01,0x68,};
static char gem_mrblib_mruby_time_proc_debug_lines_0[] = "\x00\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_time_proc_debug_file_0 = {
0, 173, 2, mrb_debug_line_packed_map, {gem_mrblib_mruby_time_proc_debug_lines_0}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_time_proc_debug_file_0_ = &gem_mrblib_mruby_time_proc_debug_file_0;
static mrb_irep_debug_info gem_mrblib_mruby_time_proc_debug_0 = {
13, 1, &gem_mrblib_mruby_time_proc_debug_file_0_};
static const mrb_irep gem_mrblib_mruby_time_proc_irep_0 = {
  1,3,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_time_proc_iseq_0,
  NULL,gem_mrblib_mruby_time_proc_syms_0,gem_mrblib_mruby_time_proc_reps_0,
  NULL,					/* lv */
  &gem_mrblib_mruby_time_proc_debug_0,
  13,0,1,1,0
};
static
const struct RProc gem_mrblib_mruby_time_proc[] = {{
NULL,NULL,MRB_TT_PROC,MRB_GC_RED,0,{&gem_mrblib_mruby_time_proc_irep_0},NULL,{NULL},
}};
static void
gem_mrblib_mruby_time_proc_init_syms(mrb_state *mrb)
{
  gem_mrblib_mruby_time_proc_debug_file_2.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-time/mrblib/time.rb");
  gem_mrblib_mruby_time_proc_debug_file_3.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-time/mrblib/time.rb");
  gem_mrblib_mruby_time_proc_debug_file_4.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-time/mrblib/time.rb");
  gem_mrblib_mruby_time_proc_debug_file_5.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-time/mrblib/time.rb");
  gem_mrblib_mruby_time_proc_debug_file_6.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-time/mrblib/time.rb");
  gem_mrblib_mruby_time_proc_debug_file_7.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-time/mrblib/time.rb");
  gem_mrblib_mruby_time_proc_debug_file_8.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-time/mrblib/time.rb");
  gem_mrblib_mruby_time_proc_debug_file_1.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-time/mrblib/time.rb");
  gem_mrblib_mruby_time_proc_debug_file_0.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-time/mrblib/time.rb");
}
void mrb_mruby_time_gem_init(mrb_state *mrb);
void mrb_mruby_time_gem_final(mrb_state *mrb);

void GENERATED_TMP_mrb_mruby_time_gem_init(mrb_state *mrb) {
  int ai = mrb_gc_arena_save(mrb);
  gem_mrblib_mruby_time_proc_init_syms(mrb);
  mrb_mruby_time_gem_init(mrb);
  mrb_load_proc(mrb, gem_mrblib_mruby_time_proc);
  if (mrb->exc) {
    mrb_print_error(mrb);
    mrb_close(mrb);
    exit(EXIT_FAILURE);
  }
  struct REnv *e = mrb_vm_ci_env(mrb->c->cibase);
  mrb_vm_ci_env_set(mrb->c->cibase, NULL);
  mrb_env_unshare(mrb, e);
  mrb_gc_arena_restore(mrb, ai);
}

void GENERATED_TMP_mrb_mruby_time_gem_final(mrb_state *mrb) {
  mrb_mruby_time_gem_final(mrb);
}
