/*
 * This file is loading the irep
 * Ruby GEM code.
 *
 * IMPORTANT:
 *   This file was generated!
 *   All manual changes will get lost.
 */
#include <stdlib.h>
#include <mruby.h>
#include <mruby/irep.h>
#include <mruby/debug.h>
#include <mruby/proc.h>
#include <mruby/presym.h>

#define mrb_BRACED(...) {__VA_ARGS__}
#define mrb_DEFINE_SYMS_VAR(name, len, syms, qualifier) \
  static qualifier mrb_sym name[len] = mrb_BRACED syms

mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_8, 4, (MRB_SYM_E(r), MRB_SYM_E(g), MRB_SYM_E(b), MRB_SYM_E(a), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_8[56] = {
0x33,0x10,0x00,0x00,0x01,0x06,0x01,0x01,0x07,0x02,0x01,0x08,0x03,0x01,0x09,0x04,0x12,0x0b,0x01,0x0c,
0x06,0x2d,0x0b,0x00,0x01,0x12,0x0b,0x01,0x0c,0x07,0x2d,0x0b,0x01,0x01,0x12,0x0b,0x01,0x0c,0x08,0x2d,
0x0b,0x02,0x01,0x12,0x0b,0x01,0x0c,0x09,0x2d,0x0b,0x03,0x01,0x12,0x06,0x37,0x06,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_lv_8, 5, (MRB_SYM(r), MRB_SYM(g), MRB_SYM(b), MRB_SYM(a), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_nanovg_proc_debug_lines_8[] = "\x00\x0d\x04\x01\x30\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_8 = {
0, 173, 6, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_8}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_8_ = &gem_mrblib_mruby_nanovg_proc_debug_file_8;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_8 = {
56, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_8_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_8 = {
  6,14,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_8,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_8,NULL,
  gem_mrblib_mruby_nanovg_proc_lv_8,
  &gem_mrblib_mruby_nanovg_proc_debug_8,
  56,0,4,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_9, 1, (MRB_SYM(rgb), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_9[21] = {
0x33,0x04,0x00,0x00,0x12,0x03,0x01,0x04,0x01,0x01,0x05,0x01,0x01,0x06,0x01,0x2d,0x03,0x00,0x03,0x37,
0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_lv_9, 2, (MRB_SYM(value), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_nanovg_proc_debug_lines_9[] = "\x00\x19\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_9 = {
0, 173, 4, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_9}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_9_ = &gem_mrblib_mruby_nanovg_proc_debug_file_9;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_9 = {
21, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_9_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_9 = {
  3,8,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_9,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_9,NULL,
  gem_mrblib_mruby_nanovg_proc_lv_9,
  &gem_mrblib_mruby_nanovg_proc_debug_9,
  21,0,1,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_10, 1, (MRB_SYM(rgbf), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_10[21] = {
0x33,0x04,0x00,0x00,0x12,0x03,0x01,0x04,0x01,0x01,0x05,0x01,0x01,0x06,0x01,0x2d,0x03,0x00,0x03,0x37,
0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_lv_10, 2, (MRB_SYM(value), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_nanovg_proc_debug_lines_10[] = "\x00\x24\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_10 = {
0, 173, 4, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_10}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_10_ = &gem_mrblib_mruby_nanovg_proc_debug_file_10;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_10 = {
21, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_10_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_10 = {
  3,8,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_10,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_10,NULL,
  gem_mrblib_mruby_nanovg_proc_lv_10,
  &gem_mrblib_mruby_nanovg_proc_debug_10,
  21,0,1,0,0
};
static const mrb_irep *gem_mrblib_mruby_nanovg_proc_reps_7[3] = {
  &gem_mrblib_mruby_nanovg_proc_irep_8,
  &gem_mrblib_mruby_nanovg_proc_irep_9,
  &gem_mrblib_mruby_nanovg_proc_irep_10,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_7, 3, (MRB_SYM(set), MRB_SYM(mono), MRB_SYM(monof), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_7[30] = {
0x62,0x01,0x57,0x02,0x00,0x5e,0x01,0x00,0x12,0x01,0x61,0x01,0x57,0x02,0x01,0x5e,0x01,0x01,0x12,0x01,
0x61,0x01,0x57,0x02,0x02,0x5e,0x01,0x02,0x37,0x01,};
static char gem_mrblib_mruby_nanovg_proc_debug_lines_7[] = "\x00\x0d\x08\x0c\x0a\x0b";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_7 = {
0, 173, 6, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_7}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_7_ = &gem_mrblib_mruby_nanovg_proc_debug_file_7;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_7 = {
30, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_7_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_7 = {
  1,3,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_7,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_7,gem_mrblib_mruby_nanovg_proc_reps_7,
  NULL,					/* lv */
  &gem_mrblib_mruby_nanovg_proc_debug_7,
  30,0,3,3,0
};
static const mrb_irep *gem_mrblib_mruby_nanovg_proc_reps_1[1] = {
  &gem_mrblib_mruby_nanovg_proc_irep_7,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_1, 1, (MRB_SYM(Color), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_1[12] = {
0x11,0x01,0x11,0x02,0x5b,0x01,0x00,0x5d,0x01,0x00,0x37,0x01,};
static char gem_mrblib_mruby_nanovg_proc_debug_lines_1[] = "\x00\x02";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_1 = {
0, 173, 2, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_1}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_1_ = &gem_mrblib_mruby_nanovg_proc_debug_file_1;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_1 = {
12, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_1_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_1 = {
  1,3,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_1,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_1,gem_mrblib_mruby_nanovg_proc_reps_1,
  NULL,					/* lv */
  &gem_mrblib_mruby_nanovg_proc_debug_1,
  12,0,1,1,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_24, 3, (MRB_SYM_Q(empty), MRB_SYM(NO_FLAG), MRB_SYM(initialize_w_flags), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_24[41] = {
0x33,0x00,0x10,0x00,0x01,0x03,0x01,0x2d,0x03,0x00,0x00,0x25,0x03,0x00,0x0c,0x12,0x03,0x1d,0x04,0x01,
0x2d,0x03,0x02,0x01,0x23,0x00,0x0c,0x12,0x03,0x11,0x04,0x01,0x05,0x01,0x48,0x04,0x2b,0x03,0x02,0x37,
0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_lv_24, 2, (MRB_SYM(args), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_nanovg_proc_debug_lines_24[] = "\x00\x11\x04\x01\x09\x01\x0c\x02";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_24 = {
0, 175, 8, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_24}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_24_ = &gem_mrblib_mruby_nanovg_proc_debug_file_24;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_24 = {
41, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_24_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_24 = {
  3,6,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_24,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_24,NULL,
  gem_mrblib_mruby_nanovg_proc_lv_24,
  &gem_mrblib_mruby_nanovg_proc_debug_24,
  41,0,3,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_25, 4, (MRB_SYM(Transform), MRB_SYM(Nanovg), MRB_SYM(new), MRB_SYM(copy_current_transform_to), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_25[43] = {
0x33,0x00,0x20,0x00,0x23,0x00,0x03,0x23,0x00,0x02,0x11,0x01,0x01,0x03,0x01,0x24,0x03,0x00,0x0d,0x1d,
0x03,0x01,0x1f,0x03,0x00,0x2d,0x03,0x02,0x00,0x01,0x01,0x03,0x12,0x03,0x01,0x04,0x01,0x2d,0x03,0x03,
0x01,0x37,0x01,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_lv_25, 2, (MRB_SYM(t), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_nanovg_proc_debug_lines_25[] = "\x00\x24\x0c\x01\x14\x01\x09\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_25 = {
0, 175, 8, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_25}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_25_ = &gem_mrblib_mruby_nanovg_proc_debug_file_25;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_25 = {
43, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_25_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_25 = {
  3,6,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_25,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_25,NULL,
  gem_mrblib_mruby_nanovg_proc_lv_25,
  &gem_mrblib_mruby_nanovg_proc_debug_25,
  43,0,4,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_26, 4, (MRB_SYM(ImageHandle), MRB_SYM(Nanovg), MRB_OPSYM(eqq), MRB_SYM(new), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_26[44] = {
0x33,0x04,0x00,0x00,0x1d,0x03,0x01,0x1f,0x03,0x00,0x01,0x04,0x01,0x2d,0x03,0x02,0x01,0x25,0x03,0x00,
0x06,0x01,0x03,0x01,0x23,0x00,0x0f,0x1d,0x03,0x01,0x1f,0x03,0x00,0x12,0x04,0x01,0x05,0x01,0x2d,0x03,
0x03,0x02,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_lv_26, 2, (MRB_SYM(image), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_nanovg_proc_debug_lines_26[] = "\x00\x32\x04\x01\x0f\x01\x06\x02";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_26 = {
0, 175, 8, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_26}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_26_ = &gem_mrblib_mruby_nanovg_proc_debug_file_26;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_26 = {
44, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_26_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_26 = {
  3,7,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_26,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_26,NULL,
  gem_mrblib_mruby_nanovg_proc_lv_26,
  &gem_mrblib_mruby_nanovg_proc_debug_26,
  44,0,4,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_27, 4, (MRB_SYM(FontHandle), MRB_SYM(Nanovg), MRB_OPSYM(eqq), MRB_SYM(new), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_27[44] = {
0x33,0x04,0x00,0x00,0x1d,0x03,0x01,0x1f,0x03,0x00,0x01,0x04,0x01,0x2d,0x03,0x02,0x01,0x25,0x03,0x00,
0x06,0x01,0x03,0x01,0x23,0x00,0x0f,0x1d,0x03,0x01,0x1f,0x03,0x00,0x12,0x04,0x01,0x05,0x01,0x2d,0x03,
0x03,0x02,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_lv_27, 2, (MRB_SYM(font), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_nanovg_proc_debug_lines_27[] = "\x00\x40\x04\x01\x0f\x01\x06\x02";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_27 = {
0, 175, 8, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_27}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_27_ = &gem_mrblib_mruby_nanovg_proc_debug_file_27;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_27 = {
44, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_27_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_27 = {
  3,7,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_27,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_27,NULL,
  gem_mrblib_mruby_nanovg_proc_lv_27,
  &gem_mrblib_mruby_nanovg_proc_debug_27,
  44,0,4,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_28, 3, (MRB_SYM(cancel_frame), MRB_SYM(AbortFrame), MRB_SYM(raise), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_28[21] = {
0x33,0x00,0x00,0x00,0x12,0x02,0x2d,0x02,0x00,0x00,0x12,0x02,0x1d,0x03,0x01,0x2d,0x02,0x02,0x01,0x37,
0x02,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_lv_28, 1, (MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_nanovg_proc_debug_lines_28[] = "\x00\x4e\x04\x01\x06\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_28 = {
0, 175, 6, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_28}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_28_ = &gem_mrblib_mruby_nanovg_proc_debug_file_28;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_28 = {
21, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_28_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_28 = {
  2,5,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_28,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_28,NULL,
  gem_mrblib_mruby_nanovg_proc_lv_28,
  &gem_mrblib_mruby_nanovg_proc_debug_28,
  21,0,3,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_29, 4, (MRB_SYM(begin_frame), MRB_SYM(call), MRB_SYM(end_frame), MRB_SYM(AbortFrame), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_29[72] = {
0x33,0x00,0x10,0x00,0x12,0x03,0x11,0x04,0x01,0x05,0x01,0x48,0x04,0x2b,0x03,0x00,0x12,0x04,0x3a,0x03,
0x04,0x00,0x2d,0x03,0x01,0x01,0x12,0x03,0x2d,0x03,0x02,0x00,0x23,0x00,0x16,0x28,0x03,0x1d,0x04,0x03,
0x29,0x03,0x04,0x24,0x04,0x00,0x03,0x23,0x00,0x05,0x11,0x03,0x23,0x00,0x02,0x2a,0x03,0x37,0x03,0x00,
0x00,0x00,0x00,0x04,0x00,0x00,0x00,0x20,0x00,0x00,0x00,0x23,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_lv_29, 2, (MRB_SYM(args), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_nanovg_proc_debug_lines_29[] = "\x00\x58\x04\x01\x0c\x01\x0a\x01\x07\x02\x02\xfe\xff\xff\xff\x0f\x02\x01\x0b\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_29 = {
0, 175, 20, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_29}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_29_ = &gem_mrblib_mruby_nanovg_proc_debug_file_29;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_29 = {
59, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_29_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_29 = {
  3,6,1,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_29,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_29,NULL,
  gem_mrblib_mruby_nanovg_proc_lv_29,
  &gem_mrblib_mruby_nanovg_proc_debug_29,
  59,0,4,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_30, 2, (MRB_SYM(begin_path), MRB_SYM(call), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_30[22] = {
0x33,0x00,0x00,0x00,0x12,0x02,0x2d,0x02,0x00,0x00,0x12,0x03,0x3a,0x02,0x00,0x00,0x2d,0x02,0x01,0x01,
0x37,0x02,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_lv_30, 1, (MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_nanovg_proc_debug_lines_30[] = "\x00\x66\x04\x01\x06\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_30 = {
0, 175, 6, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_30}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_30_ = &gem_mrblib_mruby_nanovg_proc_debug_file_30;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_30 = {
22, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_30_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_30 = {
  2,5,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_30,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_30,NULL,
  gem_mrblib_mruby_nanovg_proc_lv_30,
  &gem_mrblib_mruby_nanovg_proc_debug_30,
  22,0,2,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_31, 3, (MRB_SYM(save), MRB_SYM(call), MRB_SYM(restore), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_31[28] = {
0x33,0x00,0x00,0x00,0x12,0x02,0x2d,0x02,0x00,0x00,0x12,0x03,0x3a,0x02,0x00,0x00,0x2d,0x02,0x01,0x01,
0x12,0x02,0x2d,0x02,0x02,0x00,0x37,0x02,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_lv_31, 1, (MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_nanovg_proc_debug_lines_31[] = "\x00\x72\x04\x01\x06\x01\x0a\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_31 = {
0, 175, 8, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_31}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_31_ = &gem_mrblib_mruby_nanovg_proc_debug_file_31;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_31 = {
28, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_31_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_31 = {
  2,5,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_31,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_31,NULL,
  gem_mrblib_mruby_nanovg_proc_lv_31,
  &gem_mrblib_mruby_nanovg_proc_debug_31,
  28,0,3,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_105, 1, (MRB_OPSYM(lshift), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_105[26] = {
0x33,0x10,0x00,0x00,0x21,0x06,0x05,0x00,0x01,0x07,0x02,0x01,0x08,0x03,0x01,0x09,0x04,0x46,0x07,0x03,
0x2d,0x06,0x00,0x01,0x37,0x06,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_lv_105, 5, (MRB_SYM(_), MRB_SYM(x), MRB_SYM(min_x), MRB_SYM(max_x), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_nanovg_proc_debug_lines_105[] = "\x00\x86\x01\x04\xff\xff\xff\xff\x0f";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_105 = {
0, 175, 9, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_105}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_105_ = &gem_mrblib_mruby_nanovg_proc_debug_file_105;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_105 = {
26, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_105_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_105 = {
  6,10,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_105,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_105,NULL,
  gem_mrblib_mruby_nanovg_proc_lv_105,
  &gem_mrblib_mruby_nanovg_proc_debug_105,
  26,0,1,0,0
};
static const mrb_irep *gem_mrblib_mruby_nanovg_proc_reps_32[1] = {
  &gem_mrblib_mruby_nanovg_proc_irep_105,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_32, 1, (MRB_SYM(text_glyph_positions_yielded), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_32[30] = {
0x33,0x0c,0x00,0x00,0x46,0x06,0x00,0x01,0x05,0x06,0x12,0x06,0x01,0x07,0x01,0x01,0x08,0x02,0x01,0x09,
0x03,0x56,0x0a,0x00,0x2e,0x06,0x00,0x03,0x37,0x05,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_lv_32, 5, (MRB_SYM(x), MRB_SYM(y), MRB_SYM(text), MRB_OPSYM(and), MRB_SYM(result), ), const);
static char gem_mrblib_mruby_nanovg_proc_debug_lines_32[] = "\x00\x82\x01\x04\x01\x06\x01\x0b\x02\x07\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_32 = {
0, 175, 11, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_32}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_32_ = &gem_mrblib_mruby_nanovg_proc_debug_file_32;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_32 = {
30, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_32_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_32 = {
  6,11,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_32,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_32,gem_mrblib_mruby_nanovg_proc_reps_32,
  gem_mrblib_mruby_nanovg_proc_lv_32,
  &gem_mrblib_mruby_nanovg_proc_debug_32,
  30,0,1,1,0
};
static const mrb_irep *gem_mrblib_mruby_nanovg_proc_reps_23[9] = {
  &gem_mrblib_mruby_nanovg_proc_irep_24,
  &gem_mrblib_mruby_nanovg_proc_irep_25,
  &gem_mrblib_mruby_nanovg_proc_irep_26,
  &gem_mrblib_mruby_nanovg_proc_irep_27,
  &gem_mrblib_mruby_nanovg_proc_irep_28,
  &gem_mrblib_mruby_nanovg_proc_irep_29,
  &gem_mrblib_mruby_nanovg_proc_irep_30,
  &gem_mrblib_mruby_nanovg_proc_irep_31,
  &gem_mrblib_mruby_nanovg_proc_irep_32,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_23, 49, (MRB_SYM(IndexError), MRB_SYM(AbortFrame), MRB_SYM(initialize_w_flags), MRB_SYM(initialize), MRB_SYM(private), MRB_SYM(copy_current_transform_to), MRB_SYM(current_transform), MRB_SYM(image_handle), MRB_SYM(font_handle), MRB_SYM(abort_frame), MRB_SYM(draw), MRB_SYM(path), MRB_SYM(spork), MRB_SYM(text_glyph_positions_yielded), MRB_SYM(text_glyph_positions), MRB_SYM_E(fill_color), MRB_SYM(fill_color), MRB_SYM_E(fill_paint), MRB_SYM(fill_paint), MRB_SYM_E(font_blur), MRB_SYM(font_blur), MRB_SYM_E(font_size), MRB_SYM(font_size), MRB_SYM_E(global_alpha), MRB_SYM(global_alpha), MRB_SYM_E(line_cap), MRB_SYM(line_cap), MRB_SYM_E(line_join), MRB_SYM(line_join), MRB_SYM_E(miter_limit), MRB_SYM(miter_limit), MRB_SYM_E(path_winding), MRB_SYM(path_winding), MRB_SYM_E(stroke_color), MRB_SYM(stroke_color), MRB_SYM_E(stroke_paint), MRB_SYM(stroke_paint), MRB_SYM_E(stroke_width), MRB_SYM(stroke_width), MRB_SYM_E(text_letter_spacing), MRB_SYM(text_letter_spacing), MRB_SYM_E(text_line_height), MRB_SYM(text_line_height), MRB_SYM_E(text_align), MRB_SYM(text_align), MRB_SYM_E(font_face_id), MRB_SYM(font_face_id), MRB_SYM_E(font_face), MRB_SYM(font_face), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_23[155] = {
0x11,0x01,0x1d,0x02,0x00,0x5b,0x01,0x01,0x11,0x01,0x5f,0x02,0x03,0x12,0x01,0x10,0x02,0x02,0x2d,0x01,
0x04,0x01,0x62,0x01,0x57,0x02,0x00,0x5e,0x01,0x03,0x5f,0x05,0x06,0x62,0x01,0x57,0x02,0x01,0x5e,0x01,
0x06,0x62,0x01,0x57,0x02,0x02,0x5e,0x01,0x07,0x62,0x01,0x57,0x02,0x03,0x5e,0x01,0x08,0x62,0x01,0x57,
0x02,0x04,0x5e,0x01,0x09,0x62,0x01,0x57,0x02,0x05,0x5e,0x01,0x0a,0x62,0x01,0x57,0x02,0x06,0x5e,0x01,
0x0b,0x62,0x01,0x57,0x02,0x07,0x5e,0x01,0x0c,0x5f,0x0d,0x0e,0x62,0x01,0x57,0x02,0x08,0x5e,0x01,0x0e,
0x5f,0x0f,0x10,0x5f,0x11,0x12,0x5f,0x13,0x14,0x5f,0x15,0x16,0x5f,0x17,0x18,0x5f,0x19,0x1a,0x5f,0x1b,
0x1c,0x5f,0x1d,0x1e,0x5f,0x1f,0x20,0x5f,0x21,0x22,0x5f,0x23,0x24,0x5f,0x25,0x26,0x5f,0x27,0x28,0x5f,
0x29,0x2a,0x5f,0x2b,0x2c,0x5f,0x2d,0x2e,0x5f,0x2f,0x30,0x11,0x01,0x37,0x01,};
static char gem_mrblib_mruby_nanovg_proc_debug_lines_23[] = "\x00\x06\x0a\x04\x03\x01\x09\x06\x08\x0b\x03\x08\x08\x0e\x08\x0e\x08\x0e\x08\x0a\x08\x0e\x08\x0c\x08\x07\x03\x09\x08\x09\x03\x01\x03\x01\x03\x01\x03\x01\x03\x01\x03\x01\x03\x01\x03\x01\x03\x01\x03\x01\x03\x01\x03\x01\x03\x01\x03\x01\x03\x01\x03\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_23 = {
0, 175, 62, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_23}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_23_ = &gem_mrblib_mruby_nanovg_proc_debug_file_23;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_23 = {
155, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_23_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_23 = {
  1,4,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_23,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_23,gem_mrblib_mruby_nanovg_proc_reps_23,
  NULL,					/* lv */
  &gem_mrblib_mruby_nanovg_proc_debug_23,
  155,0,49,9,0
};
static const mrb_irep *gem_mrblib_mruby_nanovg_proc_reps_2[1] = {
  &gem_mrblib_mruby_nanovg_proc_irep_23,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_2, 1, (MRB_SYM(Context), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_2[12] = {
0x11,0x01,0x11,0x02,0x5b,0x01,0x00,0x5d,0x01,0x00,0x37,0x01,};
static char gem_mrblib_mruby_nanovg_proc_debug_lines_2[] = "\x00\x02";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_2 = {
0, 175, 2, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_2}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_2_ = &gem_mrblib_mruby_nanovg_proc_debug_file_2;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_2 = {
12, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_2_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_2 = {
  1,3,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_2,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_2,gem_mrblib_mruby_nanovg_proc_reps_2,
  NULL,					/* lv */
  &gem_mrblib_mruby_nanovg_proc_debug_2,
  12,0,1,1,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_113, 2, (MRB_IVSYM(context), MRB_IVSYM(font), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_113[15] = {
0x33,0x08,0x00,0x00,0x1a,0x01,0x00,0x01,0x04,0x02,0x1a,0x04,0x01,0x37,0x04,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_lv_113, 3, (MRB_SYM(context), MRB_SYM(font), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_nanovg_proc_debug_lines_113[] = "\x00\x0e\x04\x01\x03\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_113 = {
0, 232, 6, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_113}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_113_ = &gem_mrblib_mruby_nanovg_proc_debug_file_113;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_113 = {
15, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_113_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_113 = {
  4,5,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_113,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_113,NULL,
  gem_mrblib_mruby_nanovg_proc_lv_113,
  &gem_mrblib_mruby_nanovg_proc_debug_113,
  15,0,2,0,0
};
static const mrb_irep *gem_mrblib_mruby_nanovg_proc_reps_112[1] = {
  &gem_mrblib_mruby_nanovg_proc_irep_113,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_112, 6, (MRB_SYM(context), MRB_SYM(attr_reader), MRB_SYM(font), MRB_SYM(initialize), MRB_SYM(to_int), MRB_SYM(to_i), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_112[36] = {
0x12,0x01,0x10,0x02,0x00,0x2d,0x01,0x01,0x01,0x12,0x01,0x10,0x02,0x02,0x2d,0x01,0x01,0x01,0x62,0x01,
0x57,0x02,0x00,0x5e,0x01,0x03,0x5f,0x04,0x02,0x5f,0x05,0x02,0x11,0x01,0x37,0x01,};
static char gem_mrblib_mruby_nanovg_proc_debug_lines_112[] = "\x00\x07\x09\x03\x09\x04\x08\x06\x03\x02";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_112 = {
0, 232, 10, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_112}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_112_ = &gem_mrblib_mruby_nanovg_proc_debug_file_112;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_112 = {
36, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_112_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_112 = {
  1,4,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_112,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_112,gem_mrblib_mruby_nanovg_proc_reps_112,
  NULL,					/* lv */
  &gem_mrblib_mruby_nanovg_proc_debug_112,
  36,0,6,1,0
};
static const mrb_irep *gem_mrblib_mruby_nanovg_proc_reps_3[1] = {
  &gem_mrblib_mruby_nanovg_proc_irep_112,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_3, 1, (MRB_SYM(FontHandle), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_3[12] = {
0x11,0x01,0x11,0x02,0x5b,0x01,0x00,0x5d,0x01,0x00,0x37,0x01,};
static char gem_mrblib_mruby_nanovg_proc_debug_lines_3[] = "\x00\x04";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_3 = {
0, 232, 2, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_3}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_3_ = &gem_mrblib_mruby_nanovg_proc_debug_file_3;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_3 = {
12, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_3_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_3 = {
  1,3,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_3,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_3,gem_mrblib_mruby_nanovg_proc_reps_3,
  NULL,					/* lv */
  &gem_mrblib_mruby_nanovg_proc_debug_3,
  12,0,1,1,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_121, 2, (MRB_IVSYM(context), MRB_IVSYM(image), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_121[15] = {
0x33,0x08,0x00,0x00,0x1a,0x01,0x00,0x01,0x04,0x02,0x1a,0x04,0x01,0x37,0x04,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_lv_121, 3, (MRB_SYM(context), MRB_SYM(image), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_nanovg_proc_debug_lines_121[] = "\x00\x10\x04\x01\x03\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_121 = {
0, 236, 6, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_121}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_121_ = &gem_mrblib_mruby_nanovg_proc_debug_file_121;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_121 = {
15, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_121_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_121 = {
  4,5,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_121,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_121,NULL,
  gem_mrblib_mruby_nanovg_proc_lv_121,
  &gem_mrblib_mruby_nanovg_proc_debug_121,
  15,0,2,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_122, 3, (MRB_IVSYM(context), MRB_IVSYM(image), MRB_SYM(image_size), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_122[16] = {
0x33,0x00,0x00,0x00,0x19,0x02,0x00,0x19,0x03,0x01,0x2d,0x02,0x02,0x01,0x37,0x02,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_lv_122, 1, (MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_nanovg_proc_debug_lines_122[] = "\x00\x18\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_122 = {
0, 236, 4, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_122}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_122_ = &gem_mrblib_mruby_nanovg_proc_debug_file_122;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_122 = {
16, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_122_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_122 = {
  2,5,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_122,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_122,NULL,
  gem_mrblib_mruby_nanovg_proc_lv_122,
  &gem_mrblib_mruby_nanovg_proc_debug_122,
  16,0,3,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_123, 3, (MRB_IVSYM(context), MRB_IVSYM(image), MRB_SYM(delete_image), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_123[16] = {
0x33,0x00,0x00,0x00,0x19,0x02,0x00,0x19,0x03,0x01,0x2d,0x02,0x02,0x01,0x37,0x02,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_lv_123, 1, (MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_nanovg_proc_debug_lines_123[] = "\x00\x1f\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_123 = {
0, 236, 4, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_123}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_123_ = &gem_mrblib_mruby_nanovg_proc_debug_file_123;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_123 = {
16, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_123_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_123 = {
  2,5,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_123,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_123,NULL,
  gem_mrblib_mruby_nanovg_proc_lv_123,
  &gem_mrblib_mruby_nanovg_proc_debug_123,
  16,0,3,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_124, 3, (MRB_IVSYM(context), MRB_IVSYM(image), MRB_SYM(update_image), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_124[23] = {
0x33,0x00,0x10,0x00,0x19,0x03,0x00,0x19,0x04,0x01,0x46,0x04,0x01,0x01,0x05,0x01,0x48,0x04,0x2b,0x03,
0x02,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_lv_124, 2, (MRB_SYM(args), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_nanovg_proc_debug_lines_124[] = "\x00\x27\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_124 = {
0, 236, 4, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_124}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_124_ = &gem_mrblib_mruby_nanovg_proc_debug_file_124;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_124 = {
23, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_124_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_124 = {
  3,6,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_124,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_124,NULL,
  gem_mrblib_mruby_nanovg_proc_lv_124,
  &gem_mrblib_mruby_nanovg_proc_debug_124,
  23,0,3,0,0
};
static const mrb_irep *gem_mrblib_mruby_nanovg_proc_reps_120[4] = {
  &gem_mrblib_mruby_nanovg_proc_irep_121,
  &gem_mrblib_mruby_nanovg_proc_irep_122,
  &gem_mrblib_mruby_nanovg_proc_irep_123,
  &gem_mrblib_mruby_nanovg_proc_irep_124,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_120, 9, (MRB_SYM(context), MRB_SYM(attr_reader), MRB_SYM(image), MRB_SYM(initialize), MRB_SYM(dimensions), MRB_SYM(delete), MRB_SYM(update), MRB_SYM(to_int), MRB_SYM(to_i), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_120[60] = {
0x12,0x01,0x10,0x02,0x00,0x2d,0x01,0x01,0x01,0x12,0x01,0x10,0x02,0x02,0x2d,0x01,0x01,0x01,0x62,0x01,
0x57,0x02,0x00,0x5e,0x01,0x03,0x62,0x01,0x57,0x02,0x01,0x5e,0x01,0x04,0x62,0x01,0x57,0x02,0x02,0x5e,
0x01,0x05,0x62,0x01,0x57,0x02,0x03,0x5e,0x01,0x06,0x5f,0x07,0x02,0x5f,0x08,0x02,0x11,0x01,0x37,0x01,};
static char gem_mrblib_mruby_nanovg_proc_debug_lines_120[] = "\x00\x09\x09\x03\x09\x04\x08\x08\x08\x07\x08\x08\x08\x05\x03\x02";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_120 = {
0, 236, 16, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_120}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_120_ = &gem_mrblib_mruby_nanovg_proc_debug_file_120;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_120 = {
60, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_120_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_120 = {
  1,4,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_120,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_120,gem_mrblib_mruby_nanovg_proc_reps_120,
  NULL,					/* lv */
  &gem_mrblib_mruby_nanovg_proc_debug_120,
  60,0,9,4,0
};
static const mrb_irep *gem_mrblib_mruby_nanovg_proc_reps_4[1] = {
  &gem_mrblib_mruby_nanovg_proc_irep_120,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_4, 1, (MRB_SYM(ImageHandle), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_4[12] = {
0x11,0x01,0x11,0x02,0x5b,0x01,0x00,0x5d,0x01,0x00,0x37,0x01,};
static char gem_mrblib_mruby_nanovg_proc_debug_lines_4[] = "\x00\x06";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_4 = {
0, 236, 2, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_4}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_4_ = &gem_mrblib_mruby_nanovg_proc_debug_file_4;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_4 = {
12, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_4_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_4 = {
  1,3,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_4,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_4,gem_mrblib_mruby_nanovg_proc_reps_4,
  NULL,					/* lv */
  &gem_mrblib_mruby_nanovg_proc_debug_4,
  12,0,1,1,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_143, 3, (MRB_SYM(Color), MRB_SYM(Nanovg), MRB_SYM(hsl), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_143[25] = {
0x33,0x0c,0x00,0x00,0x1d,0x05,0x01,0x1f,0x05,0x00,0x01,0x06,0x01,0x01,0x07,0x02,0x01,0x08,0x03,0x2d,
0x05,0x02,0x03,0x37,0x05,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_lv_143, 4, (MRB_SYM(h), MRB_SYM(s), MRB_SYM(l), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_nanovg_proc_debug_lines_143[] = "\x00\x0e\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_143 = {
0, 244, 4, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_143}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_143_ = &gem_mrblib_mruby_nanovg_proc_debug_file_143;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_143 = {
25, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_143_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_143 = {
  5,10,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_143,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_143,NULL,
  gem_mrblib_mruby_nanovg_proc_lv_143,
  &gem_mrblib_mruby_nanovg_proc_debug_143,
  25,0,3,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_144, 3, (MRB_SYM(Color), MRB_SYM(Nanovg), MRB_SYM(hsla), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_144[28] = {
0x33,0x10,0x00,0x00,0x1d,0x06,0x01,0x1f,0x06,0x00,0x01,0x07,0x01,0x01,0x08,0x02,0x01,0x09,0x03,0x01,
0x0a,0x04,0x2d,0x06,0x02,0x04,0x37,0x06,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_lv_144, 5, (MRB_SYM(h), MRB_SYM(s), MRB_SYM(l), MRB_SYM(a), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_nanovg_proc_debug_lines_144[] = "\x00\x14\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_144 = {
0, 244, 4, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_144}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_144_ = &gem_mrblib_mruby_nanovg_proc_debug_file_144;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_144 = {
28, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_144_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_144 = {
  6,12,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_144,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_144,NULL,
  gem_mrblib_mruby_nanovg_proc_lv_144,
  &gem_mrblib_mruby_nanovg_proc_debug_144,
  28,0,3,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_145, 3, (MRB_SYM(Color), MRB_SYM(Nanovg), MRB_SYM(rgb), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_145[25] = {
0x33,0x0c,0x00,0x00,0x1d,0x05,0x01,0x1f,0x05,0x00,0x01,0x06,0x01,0x01,0x07,0x02,0x01,0x08,0x03,0x2d,
0x05,0x02,0x03,0x37,0x05,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_lv_145, 4, (MRB_SYM(r), MRB_SYM(g), MRB_SYM(b), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_nanovg_proc_debug_lines_145[] = "\x00\x19\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_145 = {
0, 244, 4, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_145}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_145_ = &gem_mrblib_mruby_nanovg_proc_debug_file_145;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_145 = {
25, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_145_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_145 = {
  5,10,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_145,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_145,NULL,
  gem_mrblib_mruby_nanovg_proc_lv_145,
  &gem_mrblib_mruby_nanovg_proc_debug_145,
  25,0,3,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_146, 3, (MRB_SYM(Color), MRB_SYM(Nanovg), MRB_SYM(rgbf), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_146[25] = {
0x33,0x0c,0x00,0x00,0x1d,0x05,0x01,0x1f,0x05,0x00,0x01,0x06,0x01,0x01,0x07,0x02,0x01,0x08,0x03,0x2d,
0x05,0x02,0x03,0x37,0x05,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_lv_146, 4, (MRB_SYM(r), MRB_SYM(g), MRB_SYM(b), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_nanovg_proc_debug_lines_146[] = "\x00\x1e\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_146 = {
0, 244, 4, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_146}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_146_ = &gem_mrblib_mruby_nanovg_proc_debug_file_146;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_146 = {
25, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_146_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_146 = {
  5,10,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_146,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_146,NULL,
  gem_mrblib_mruby_nanovg_proc_lv_146,
  &gem_mrblib_mruby_nanovg_proc_debug_146,
  25,0,3,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_147, 3, (MRB_SYM(Color), MRB_SYM(Nanovg), MRB_SYM(rgba), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_147[28] = {
0x33,0x10,0x00,0x00,0x1d,0x06,0x01,0x1f,0x06,0x00,0x01,0x07,0x01,0x01,0x08,0x02,0x01,0x09,0x03,0x01,
0x0a,0x04,0x2d,0x06,0x02,0x04,0x37,0x06,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_lv_147, 5, (MRB_SYM(r), MRB_SYM(g), MRB_SYM(b), MRB_SYM(a), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_nanovg_proc_debug_lines_147[] = "\x00\x23\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_147 = {
0, 244, 4, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_147}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_147_ = &gem_mrblib_mruby_nanovg_proc_debug_file_147;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_147 = {
28, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_147_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_147 = {
  6,12,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_147,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_147,NULL,
  gem_mrblib_mruby_nanovg_proc_lv_147,
  &gem_mrblib_mruby_nanovg_proc_debug_147,
  28,0,3,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_148, 3, (MRB_SYM(Color), MRB_SYM(Nanovg), MRB_SYM(rgbaf), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_148[28] = {
0x33,0x10,0x00,0x00,0x1d,0x06,0x01,0x1f,0x06,0x00,0x01,0x07,0x01,0x01,0x08,0x02,0x01,0x09,0x03,0x01,
0x0a,0x04,0x2d,0x06,0x02,0x04,0x37,0x06,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_lv_148, 5, (MRB_SYM(r), MRB_SYM(g), MRB_SYM(b), MRB_SYM(a), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_nanovg_proc_debug_lines_148[] = "\x00\x28\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_148 = {
0, 244, 4, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_148}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_148_ = &gem_mrblib_mruby_nanovg_proc_debug_file_148;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_148 = {
28, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_148_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_148 = {
  6,12,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_148,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_148,NULL,
  gem_mrblib_mruby_nanovg_proc_lv_148,
  &gem_mrblib_mruby_nanovg_proc_debug_148,
  28,0,3,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_149, 3, (MRB_SYM(Color), MRB_SYM(Nanovg), MRB_SYM(mono), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_149[19] = {
0x33,0x04,0x00,0x00,0x1d,0x03,0x01,0x1f,0x03,0x00,0x01,0x04,0x01,0x2d,0x03,0x02,0x01,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_lv_149, 2, (MRB_SYM(value), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_nanovg_proc_debug_lines_149[] = "\x00\x2d\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_149 = {
0, 244, 4, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_149}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_149_ = &gem_mrblib_mruby_nanovg_proc_debug_file_149;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_149 = {
19, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_149_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_149 = {
  3,6,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_149,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_149,NULL,
  gem_mrblib_mruby_nanovg_proc_lv_149,
  &gem_mrblib_mruby_nanovg_proc_debug_149,
  19,0,3,0,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_150, 3, (MRB_SYM(Color), MRB_SYM(Nanovg), MRB_SYM(monof), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_150[19] = {
0x33,0x04,0x00,0x00,0x1d,0x03,0x01,0x1f,0x03,0x00,0x01,0x04,0x01,0x2d,0x03,0x02,0x01,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_lv_150, 2, (MRB_SYM(value), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_nanovg_proc_debug_lines_150[] = "\x00\x32\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_150 = {
0, 244, 4, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_150}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_150_ = &gem_mrblib_mruby_nanovg_proc_debug_file_150;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_150 = {
19, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_150_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_150 = {
  3,6,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_150,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_150,NULL,
  gem_mrblib_mruby_nanovg_proc_lv_150,
  &gem_mrblib_mruby_nanovg_proc_debug_150,
  19,0,3,0,0
};
static const mrb_irep *gem_mrblib_mruby_nanovg_proc_reps_5[8] = {
  &gem_mrblib_mruby_nanovg_proc_irep_143,
  &gem_mrblib_mruby_nanovg_proc_irep_144,
  &gem_mrblib_mruby_nanovg_proc_irep_145,
  &gem_mrblib_mruby_nanovg_proc_irep_146,
  &gem_mrblib_mruby_nanovg_proc_irep_147,
  &gem_mrblib_mruby_nanovg_proc_irep_148,
  &gem_mrblib_mruby_nanovg_proc_irep_149,
  &gem_mrblib_mruby_nanovg_proc_irep_150,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_5, 16, (MRB_SYM(Winding), MRB_SYM(include), MRB_SYM(Solidity), MRB_SYM(LineCap), MRB_SYM(Align), MRB_SYM(ImageFlags), MRB_SYM(CreateFlags), MRB_SYM(NO_FLAG), MRB_SYM(hsl), MRB_SYM(hsla), MRB_SYM(rgb), MRB_SYM(rgbf), MRB_SYM(rgba), MRB_SYM(rgbaf), MRB_SYM(mono), MRB_SYM(monof), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_5[141] = {
0x12,0x01,0x1d,0x02,0x00,0x2d,0x01,0x01,0x01,0x12,0x01,0x1d,0x02,0x02,0x2d,0x01,0x01,0x01,0x12,0x01,
0x1d,0x02,0x03,0x2d,0x01,0x01,0x01,0x12,0x01,0x1d,0x02,0x04,0x2d,0x01,0x01,0x01,0x12,0x01,0x1d,0x02,
0x05,0x2d,0x01,0x01,0x01,0x12,0x01,0x1d,0x02,0x06,0x2d,0x01,0x01,0x01,0x06,0x01,0x1e,0x01,0x07,0x12,
0x01,0x61,0x01,0x57,0x02,0x00,0x5e,0x01,0x08,0x12,0x01,0x61,0x01,0x57,0x02,0x01,0x5e,0x01,0x09,0x12,
0x01,0x61,0x01,0x57,0x02,0x02,0x5e,0x01,0x0a,0x12,0x01,0x61,0x01,0x57,0x02,0x03,0x5e,0x01,0x0b,0x12,
0x01,0x61,0x01,0x57,0x02,0x04,0x5e,0x01,0x0c,0x12,0x01,0x61,0x01,0x57,0x02,0x05,0x5e,0x01,0x0d,0x12,
0x01,0x61,0x01,0x57,0x02,0x06,0x5e,0x01,0x0e,0x12,0x01,0x61,0x01,0x57,0x02,0x07,0x5e,0x01,0x0f,0x37,
0x01,};
static char gem_mrblib_mruby_nanovg_proc_debug_lines_5[] = "\x00\x02\x09\x01\x09\x01\x09\x01\x09\x01\x09\x01\x09\x03\x05\x04\x0a\x06\x0a\x05\x0a\x05\x0a\x05\x0a\x05\x0a\x05\x0a\x05";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_5 = {
0, 244, 30, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_5}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_5_ = &gem_mrblib_mruby_nanovg_proc_debug_file_5;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_5 = {
141, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_5_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_5 = {
  1,4,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_5,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_5,gem_mrblib_mruby_nanovg_proc_reps_5,
  NULL,					/* lv */
  &gem_mrblib_mruby_nanovg_proc_debug_5,
  141,0,16,8,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_222, 1, (MRB_SYM_B(translate), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_222[21] = {
0x33,0x04,0x00,0x00,0x01,0x03,0x01,0x21,0x04,0x01,0x00,0x21,0x05,0x02,0x00,0x2d,0x03,0x00,0x02,0x37,
0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_lv_222, 2, (MRB_SYM(t), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_nanovg_proc_debug_lines_222[] = "\x00\x0a";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_222 = {
0, 251, 2, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_222}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_222_ = &gem_mrblib_mruby_nanovg_proc_debug_file_222;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_222 = {
21, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_222_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_222 = {
  3,7,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_222,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_222,NULL,
  gem_mrblib_mruby_nanovg_proc_lv_222,
  &gem_mrblib_mruby_nanovg_proc_debug_222,
  21,0,1,0,0
};
static const mrb_irep *gem_mrblib_mruby_nanovg_proc_reps_214[1] = {
  &gem_mrblib_mruby_nanovg_proc_irep_222,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_214, 2, (MRB_SYM(dup), MRB_SYM(tap), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_214[19] = {
0x33,0x08,0x00,0x00,0x12,0x04,0x2d,0x04,0x00,0x00,0x56,0x05,0x00,0x2e,0x04,0x01,0x00,0x37,0x04,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_lv_214, 3, (MRB_SYM(x), MRB_SYM(y), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_nanovg_proc_debug_lines_214[] = "\x00\x09\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_214 = {
0, 251, 4, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_214}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_214_ = &gem_mrblib_mruby_nanovg_proc_debug_file_214;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_214 = {
19, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_214_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_214 = {
  4,6,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_214,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_214,gem_mrblib_mruby_nanovg_proc_reps_214,
  gem_mrblib_mruby_nanovg_proc_lv_214,
  &gem_mrblib_mruby_nanovg_proc_debug_214,
  19,0,2,1,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_231, 1, (MRB_SYM_B(scale), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_231[21] = {
0x33,0x04,0x00,0x00,0x01,0x03,0x01,0x21,0x04,0x01,0x00,0x21,0x05,0x02,0x00,0x2d,0x03,0x00,0x02,0x37,
0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_lv_231, 2, (MRB_SYM(t), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_nanovg_proc_debug_lines_231[] = "\x00\x14";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_231 = {
0, 251, 2, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_231}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_231_ = &gem_mrblib_mruby_nanovg_proc_debug_file_231;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_231 = {
21, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_231_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_231 = {
  3,7,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_231,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_231,NULL,
  gem_mrblib_mruby_nanovg_proc_lv_231,
  &gem_mrblib_mruby_nanovg_proc_debug_231,
  21,0,1,0,0
};
static const mrb_irep *gem_mrblib_mruby_nanovg_proc_reps_215[1] = {
  &gem_mrblib_mruby_nanovg_proc_irep_231,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_215, 2, (MRB_SYM(dup), MRB_SYM(tap), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_215[19] = {
0x33,0x08,0x00,0x00,0x12,0x04,0x2d,0x04,0x00,0x00,0x56,0x05,0x00,0x2e,0x04,0x01,0x00,0x37,0x04,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_lv_215, 3, (MRB_SYM(x), MRB_SYM(y), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_nanovg_proc_debug_lines_215[] = "\x00\x13\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_215 = {
0, 251, 4, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_215}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_215_ = &gem_mrblib_mruby_nanovg_proc_debug_file_215;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_215 = {
19, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_215_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_215 = {
  4,6,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_215,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_215,gem_mrblib_mruby_nanovg_proc_reps_215,
  gem_mrblib_mruby_nanovg_proc_lv_215,
  &gem_mrblib_mruby_nanovg_proc_debug_215,
  19,0,2,1,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_240, 1, (MRB_SYM_B(rotate), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_240[17] = {
0x33,0x04,0x00,0x00,0x01,0x03,0x01,0x21,0x04,0x01,0x00,0x2d,0x03,0x00,0x01,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_lv_240, 2, (MRB_SYM(t), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_nanovg_proc_debug_lines_240[] = "\x00\x1d";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_240 = {
0, 251, 2, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_240}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_240_ = &gem_mrblib_mruby_nanovg_proc_debug_file_240;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_240 = {
17, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_240_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_240 = {
  3,6,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_240,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_240,NULL,
  gem_mrblib_mruby_nanovg_proc_lv_240,
  &gem_mrblib_mruby_nanovg_proc_debug_240,
  17,0,1,0,0
};
static const mrb_irep *gem_mrblib_mruby_nanovg_proc_reps_216[1] = {
  &gem_mrblib_mruby_nanovg_proc_irep_240,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_216, 2, (MRB_SYM(dup), MRB_SYM(tap), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_216[19] = {
0x33,0x04,0x00,0x00,0x12,0x03,0x2d,0x03,0x00,0x00,0x56,0x04,0x00,0x2e,0x03,0x01,0x00,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_lv_216, 2, (MRB_SYM(angle), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_nanovg_proc_debug_lines_216[] = "\x00\x1c\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_216 = {
0, 251, 4, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_216}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_216_ = &gem_mrblib_mruby_nanovg_proc_debug_file_216;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_216 = {
19, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_216_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_216 = {
  3,5,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_216,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_216,gem_mrblib_mruby_nanovg_proc_reps_216,
  gem_mrblib_mruby_nanovg_proc_lv_216,
  &gem_mrblib_mruby_nanovg_proc_debug_216,
  19,0,2,1,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_249, 1, (MRB_SYM_B(skew_x), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_249[17] = {
0x33,0x04,0x00,0x00,0x01,0x03,0x01,0x21,0x04,0x01,0x00,0x2d,0x03,0x00,0x01,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_lv_249, 2, (MRB_SYM(t), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_nanovg_proc_debug_lines_249[] = "\x00\x26";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_249 = {
0, 251, 2, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_249}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_249_ = &gem_mrblib_mruby_nanovg_proc_debug_file_249;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_249 = {
17, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_249_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_249 = {
  3,6,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_249,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_249,NULL,
  gem_mrblib_mruby_nanovg_proc_lv_249,
  &gem_mrblib_mruby_nanovg_proc_debug_249,
  17,0,1,0,0
};
static const mrb_irep *gem_mrblib_mruby_nanovg_proc_reps_217[1] = {
  &gem_mrblib_mruby_nanovg_proc_irep_249,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_217, 2, (MRB_SYM(dup), MRB_SYM(tap), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_217[19] = {
0x33,0x04,0x00,0x00,0x12,0x03,0x2d,0x03,0x00,0x00,0x56,0x04,0x00,0x2e,0x03,0x01,0x00,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_lv_217, 2, (MRB_SYM(amount), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_nanovg_proc_debug_lines_217[] = "\x00\x25\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_217 = {
0, 251, 4, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_217}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_217_ = &gem_mrblib_mruby_nanovg_proc_debug_file_217;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_217 = {
19, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_217_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_217 = {
  3,5,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_217,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_217,gem_mrblib_mruby_nanovg_proc_reps_217,
  gem_mrblib_mruby_nanovg_proc_lv_217,
  &gem_mrblib_mruby_nanovg_proc_debug_217,
  19,0,2,1,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_258, 1, (MRB_SYM_B(skew_y), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_258[17] = {
0x33,0x04,0x00,0x00,0x01,0x03,0x01,0x21,0x04,0x01,0x00,0x2d,0x03,0x00,0x01,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_lv_258, 2, (MRB_SYM(t), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_nanovg_proc_debug_lines_258[] = "\x00\x2f";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_258 = {
0, 251, 2, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_258}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_258_ = &gem_mrblib_mruby_nanovg_proc_debug_file_258;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_258 = {
17, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_258_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_258 = {
  3,6,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_258,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_258,NULL,
  gem_mrblib_mruby_nanovg_proc_lv_258,
  &gem_mrblib_mruby_nanovg_proc_debug_258,
  17,0,1,0,0
};
static const mrb_irep *gem_mrblib_mruby_nanovg_proc_reps_218[1] = {
  &gem_mrblib_mruby_nanovg_proc_irep_258,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_218, 2, (MRB_SYM(dup), MRB_SYM(tap), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_218[19] = {
0x33,0x04,0x00,0x00,0x12,0x03,0x2d,0x03,0x00,0x00,0x56,0x04,0x00,0x2e,0x03,0x01,0x00,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_lv_218, 2, (MRB_SYM(amount), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_nanovg_proc_debug_lines_218[] = "\x00\x2e\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_218 = {
0, 251, 4, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_218}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_218_ = &gem_mrblib_mruby_nanovg_proc_debug_file_218;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_218 = {
19, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_218_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_218 = {
  3,5,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_218,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_218,gem_mrblib_mruby_nanovg_proc_reps_218,
  gem_mrblib_mruby_nanovg_proc_lv_218,
  &gem_mrblib_mruby_nanovg_proc_debug_218,
  19,0,2,1,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_267, 1, (MRB_SYM_B(multiply), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_267[17] = {
0x33,0x04,0x00,0x00,0x01,0x03,0x01,0x21,0x04,0x01,0x00,0x2d,0x03,0x00,0x01,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_lv_267, 2, (MRB_SYM(t), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_nanovg_proc_debug_lines_267[] = "\x00\x38";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_267 = {
0, 251, 2, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_267}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_267_ = &gem_mrblib_mruby_nanovg_proc_debug_file_267;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_267 = {
17, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_267_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_267 = {
  3,6,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_267,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_267,NULL,
  gem_mrblib_mruby_nanovg_proc_lv_267,
  &gem_mrblib_mruby_nanovg_proc_debug_267,
  17,0,1,0,0
};
static const mrb_irep *gem_mrblib_mruby_nanovg_proc_reps_219[1] = {
  &gem_mrblib_mruby_nanovg_proc_irep_267,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_219, 2, (MRB_SYM(dup), MRB_SYM(tap), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_219[19] = {
0x33,0x04,0x00,0x00,0x12,0x03,0x2d,0x03,0x00,0x00,0x56,0x04,0x00,0x2e,0x03,0x01,0x00,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_lv_219, 2, (MRB_SYM(trans), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_nanovg_proc_debug_lines_219[] = "\x00\x37\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_219 = {
0, 251, 4, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_219}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_219_ = &gem_mrblib_mruby_nanovg_proc_debug_file_219;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_219 = {
19, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_219_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_219 = {
  3,5,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_219,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_219,gem_mrblib_mruby_nanovg_proc_reps_219,
  gem_mrblib_mruby_nanovg_proc_lv_219,
  &gem_mrblib_mruby_nanovg_proc_debug_219,
  19,0,2,1,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_276, 1, (MRB_SYM_B(pre_multiply), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_276[17] = {
0x33,0x04,0x00,0x00,0x01,0x03,0x01,0x21,0x04,0x01,0x00,0x2d,0x03,0x00,0x01,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_lv_276, 2, (MRB_SYM(t), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_nanovg_proc_debug_lines_276[] = "\x00\x41";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_276 = {
0, 251, 2, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_276}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_276_ = &gem_mrblib_mruby_nanovg_proc_debug_file_276;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_276 = {
17, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_276_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_276 = {
  3,6,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_276,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_276,NULL,
  gem_mrblib_mruby_nanovg_proc_lv_276,
  &gem_mrblib_mruby_nanovg_proc_debug_276,
  17,0,1,0,0
};
static const mrb_irep *gem_mrblib_mruby_nanovg_proc_reps_220[1] = {
  &gem_mrblib_mruby_nanovg_proc_irep_276,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_220, 2, (MRB_SYM(dup), MRB_SYM(tap), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_220[19] = {
0x33,0x04,0x00,0x00,0x12,0x03,0x2d,0x03,0x00,0x00,0x56,0x04,0x00,0x2e,0x03,0x01,0x00,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_lv_220, 2, (MRB_SYM(trans), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_nanovg_proc_debug_lines_220[] = "\x00\x40\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_220 = {
0, 251, 4, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_220}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_220_ = &gem_mrblib_mruby_nanovg_proc_debug_file_220;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_220 = {
19, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_220_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_220 = {
  3,5,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_220,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_220,gem_mrblib_mruby_nanovg_proc_reps_220,
  gem_mrblib_mruby_nanovg_proc_lv_220,
  &gem_mrblib_mruby_nanovg_proc_debug_220,
  19,0,2,1,0
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_285, 1, (MRB_SYM_B(inverse), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_285[17] = {
0x33,0x04,0x00,0x00,0x01,0x03,0x01,0x21,0x04,0x01,0x00,0x2d,0x03,0x00,0x01,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_lv_285, 2, (MRB_SYM(t), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_nanovg_proc_debug_lines_285[] = "\x00\x4a";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_285 = {
0, 251, 2, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_285}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_285_ = &gem_mrblib_mruby_nanovg_proc_debug_file_285;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_285 = {
17, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_285_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_285 = {
  3,6,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_285,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_285,NULL,
  gem_mrblib_mruby_nanovg_proc_lv_285,
  &gem_mrblib_mruby_nanovg_proc_debug_285,
  17,0,1,0,0
};
static const mrb_irep *gem_mrblib_mruby_nanovg_proc_reps_221[1] = {
  &gem_mrblib_mruby_nanovg_proc_irep_285,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_221, 2, (MRB_SYM(dup), MRB_SYM(tap), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_221[19] = {
0x33,0x04,0x00,0x00,0x12,0x03,0x2d,0x03,0x00,0x00,0x56,0x04,0x00,0x2e,0x03,0x01,0x00,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_lv_221, 2, (MRB_SYM(trans), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_nanovg_proc_debug_lines_221[] = "\x00\x49\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_221 = {
0, 251, 4, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_221}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_221_ = &gem_mrblib_mruby_nanovg_proc_debug_file_221;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_221 = {
19, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_221_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_221 = {
  3,5,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_221,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_221,gem_mrblib_mruby_nanovg_proc_reps_221,
  gem_mrblib_mruby_nanovg_proc_lv_221,
  &gem_mrblib_mruby_nanovg_proc_debug_221,
  19,0,2,1,0
};
static const mrb_irep *gem_mrblib_mruby_nanovg_proc_reps_213[8] = {
  &gem_mrblib_mruby_nanovg_proc_irep_214,
  &gem_mrblib_mruby_nanovg_proc_irep_215,
  &gem_mrblib_mruby_nanovg_proc_irep_216,
  &gem_mrblib_mruby_nanovg_proc_irep_217,
  &gem_mrblib_mruby_nanovg_proc_irep_218,
  &gem_mrblib_mruby_nanovg_proc_irep_219,
  &gem_mrblib_mruby_nanovg_proc_irep_220,
  &gem_mrblib_mruby_nanovg_proc_irep_221,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_213, 8, (MRB_SYM(translate), MRB_SYM(scale), MRB_SYM(rotate), MRB_SYM(skew_x), MRB_SYM(skew_y), MRB_SYM(multiply), MRB_SYM(pre_multiply), MRB_SYM(inverse), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_213[66] = {
0x62,0x01,0x57,0x02,0x00,0x5e,0x01,0x00,0x62,0x01,0x57,0x02,0x01,0x5e,0x01,0x01,0x62,0x01,0x57,0x02,
0x02,0x5e,0x01,0x02,0x62,0x01,0x57,0x02,0x03,0x5e,0x01,0x03,0x62,0x01,0x57,0x02,0x04,0x5e,0x01,0x04,
0x62,0x01,0x57,0x02,0x05,0x5e,0x01,0x05,0x62,0x01,0x57,0x02,0x06,0x5e,0x01,0x06,0x62,0x01,0x57,0x02,
0x07,0x5e,0x01,0x07,0x37,0x01,};
static char gem_mrblib_mruby_nanovg_proc_debug_lines_213[] = "\x00\x09\x08\x0a\x08\x09\x08\x09\x08\x09\x08\x09\x08\x09\x08\x09";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_213 = {
0, 251, 16, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_213}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_213_ = &gem_mrblib_mruby_nanovg_proc_debug_file_213;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_213 = {
66, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_213_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_213 = {
  1,3,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_213,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_213,gem_mrblib_mruby_nanovg_proc_reps_213,
  NULL,					/* lv */
  &gem_mrblib_mruby_nanovg_proc_debug_213,
  66,0,8,8,0
};
static const mrb_irep *gem_mrblib_mruby_nanovg_proc_reps_6[1] = {
  &gem_mrblib_mruby_nanovg_proc_irep_213,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_6, 1, (MRB_SYM(Transform), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_6[12] = {
0x11,0x01,0x11,0x02,0x5b,0x01,0x00,0x5d,0x01,0x00,0x37,0x01,};
static char gem_mrblib_mruby_nanovg_proc_debug_lines_6[] = "\x00\x02";
static mrb_irep_debug_info_file gem_mrblib_mruby_nanovg_proc_debug_file_6 = {
0, 251, 2, mrb_debug_line_packed_map, {gem_mrblib_mruby_nanovg_proc_debug_lines_6}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_nanovg_proc_debug_file_6_ = &gem_mrblib_mruby_nanovg_proc_debug_file_6;
static mrb_irep_debug_info gem_mrblib_mruby_nanovg_proc_debug_6 = {
12, 1, &gem_mrblib_mruby_nanovg_proc_debug_file_6_};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_6 = {
  1,3,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_6,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_6,gem_mrblib_mruby_nanovg_proc_reps_6,
  NULL,					/* lv */
  &gem_mrblib_mruby_nanovg_proc_debug_6,
  12,0,1,1,0
};
static const mrb_irep *gem_mrblib_mruby_nanovg_proc_reps_0[6] = {
  &gem_mrblib_mruby_nanovg_proc_irep_1,
  &gem_mrblib_mruby_nanovg_proc_irep_2,
  &gem_mrblib_mruby_nanovg_proc_irep_3,
  &gem_mrblib_mruby_nanovg_proc_irep_4,
  &gem_mrblib_mruby_nanovg_proc_irep_5,
  &gem_mrblib_mruby_nanovg_proc_irep_6,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_nanovg_proc_syms_0, 2, (MRB_SYM(Nanovg), MRB_SYM(NVG), ), const);
static const mrb_code gem_mrblib_mruby_nanovg_proc_iseq_0[57] = {
0x11,0x01,0x5c,0x01,0x00,0x5d,0x01,0x00,0x11,0x01,0x5c,0x01,0x00,0x5d,0x01,0x01,0x11,0x01,0x5c,0x01,
0x00,0x5d,0x01,0x02,0x11,0x01,0x5c,0x01,0x00,0x5d,0x01,0x03,0x11,0x01,0x5c,0x01,0x00,0x5d,0x01,0x04,
0x1d,0x01,0x00,0x1e,0x01,0x01,0x11,0x01,0x5c,0x01,0x00,0x5d,0x01,0x05,0x37,0x01,0x68,};
static const mrb_irep gem_mrblib_mruby_nanovg_proc_irep_0 = {
  1,2,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_nanovg_proc_iseq_0,
  NULL,gem_mrblib_mruby_nanovg_proc_syms_0,gem_mrblib_mruby_nanovg_proc_reps_0,
  NULL,					/* lv */
  NULL,					/* debug_info */
  57,0,2,6,0
};
static
const struct RProc gem_mrblib_mruby_nanovg_proc[] = {{
NULL,NULL,MRB_TT_PROC,MRB_GC_RED,0,{&gem_mrblib_mruby_nanovg_proc_irep_0},NULL,{NULL},
}};
static void
gem_mrblib_mruby_nanovg_proc_init_syms(mrb_state *mrb)
{
  gem_mrblib_mruby_nanovg_proc_debug_file_8.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/color.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_9.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/color.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_10.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/color.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_7.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/color.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_1.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/color.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_24.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/context.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_25.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/context.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_26.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/context.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_27.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/context.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_28.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/context.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_29.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/context.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_30.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/context.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_31.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/context.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_105.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/context.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_32.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/context.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_23.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/context.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_2.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/context.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_113.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/font_handle.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_112.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/font_handle.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_3.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/font_handle.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_121.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/image_handle.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_122.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/image_handle.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_123.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/image_handle.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_124.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/image_handle.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_120.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/image_handle.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_4.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/image_handle.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_143.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/nanovg.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_144.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/nanovg.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_145.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/nanovg.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_146.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/nanovg.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_147.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/nanovg.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_148.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/nanovg.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_149.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/nanovg.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_150.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/nanovg.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_5.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/nanovg.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_222.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/transform.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_214.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/transform.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_231.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/transform.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_215.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/transform.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_240.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/transform.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_216.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/transform.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_249.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/transform.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_217.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/transform.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_258.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/transform.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_218.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/transform.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_267.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/transform.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_219.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/transform.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_276.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/transform.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_220.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/transform.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_285.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/transform.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_221.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/transform.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_213.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/transform.rb");
  gem_mrblib_mruby_nanovg_proc_debug_file_6.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/deps/mruby-nanovg/mrblib/transform.rb");
}
void mrb_mruby_nanovg_gem_init(mrb_state *mrb);
void mrb_mruby_nanovg_gem_final(mrb_state *mrb);

void GENERATED_TMP_mrb_mruby_nanovg_gem_init(mrb_state *mrb) {
  int ai = mrb_gc_arena_save(mrb);
  gem_mrblib_mruby_nanovg_proc_init_syms(mrb);
  mrb_mruby_nanovg_gem_init(mrb);
  mrb_load_proc(mrb, gem_mrblib_mruby_nanovg_proc);
  if (mrb->exc) {
    mrb_print_error(mrb);
    mrb_close(mrb);
    exit(EXIT_FAILURE);
  }
  struct REnv *e = mrb_vm_ci_env(mrb->c->cibase);
  mrb_vm_ci_env_set(mrb->c->cibase, NULL);
  mrb_env_unshare(mrb, e);
  mrb_gc_arena_restore(mrb, ai);
}

void GENERATED_TMP_mrb_mruby_nanovg_gem_final(mrb_state *mrb) {
  mrb_mruby_nanovg_gem_final(mrb);
}
