/*
 * This file is loading the irep
 * Ruby GEM code.
 *
 * IMPORTANT:
 *   This file was generated!
 *   All manual changes will get lost.
 */
#include <stdlib.h>
#include <mruby.h>
#include <mruby/irep.h>
#include <mruby/debug.h>
#include <mruby/proc.h>
#include <mruby/presym.h>

#define mrb_BRACED(...) {__VA_ARGS__}
#define mrb_DEFINE_SYMS_VAR(name, len, syms, qualifier) \
  static qualifier mrb_sym name[len] = mrb_BRACED syms

mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_toplevel_ext_proc_syms_1, 2, (MRB_SYM(class), MRB_SYM(include), ), const);
static const mrb_code gem_mrblib_mruby_toplevel_ext_proc_iseq_1[22] = {
0x33,0x00,0x10,0x00,0x12,0x03,0x2d,0x03,0x00,0x00,0x11,0x04,0x01,0x05,0x01,0x48,0x04,0x2b,0x03,0x01,
0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_toplevel_ext_proc_lv_1, 2, (MRB_SYM(modules), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_toplevel_ext_proc_debug_lines_1[] = "\x00\x02\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_toplevel_ext_proc_debug_file_1 = {
0, 173, 4, mrb_debug_line_packed_map, {gem_mrblib_mruby_toplevel_ext_proc_debug_lines_1}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_toplevel_ext_proc_debug_file_1_ = &gem_mrblib_mruby_toplevel_ext_proc_debug_file_1;
static mrb_irep_debug_info gem_mrblib_mruby_toplevel_ext_proc_debug_1 = {
22, 1, &gem_mrblib_mruby_toplevel_ext_proc_debug_file_1_};
static const mrb_irep gem_mrblib_mruby_toplevel_ext_proc_irep_1 = {
  3,6,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_toplevel_ext_proc_iseq_1,
  NULL,gem_mrblib_mruby_toplevel_ext_proc_syms_1,NULL,
  gem_mrblib_mruby_toplevel_ext_proc_lv_1,
  &gem_mrblib_mruby_toplevel_ext_proc_debug_1,
  22,0,2,0,0
};
static const mrb_code gem_mrblib_mruby_toplevel_ext_proc_iseq_2[8] = {
0x33,0x00,0x10,0x00,0x11,0x03,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_toplevel_ext_proc_lv_2, 2, (MRB_SYM(methods), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_toplevel_ext_proc_debug_lines_2[] = "\x00\x06\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_toplevel_ext_proc_debug_file_2 = {
0, 173, 4, mrb_debug_line_packed_map, {gem_mrblib_mruby_toplevel_ext_proc_debug_lines_2}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_toplevel_ext_proc_debug_file_2_ = &gem_mrblib_mruby_toplevel_ext_proc_debug_file_2;
static mrb_irep_debug_info gem_mrblib_mruby_toplevel_ext_proc_debug_2 = {
8, 1, &gem_mrblib_mruby_toplevel_ext_proc_debug_file_2_};
static const mrb_irep gem_mrblib_mruby_toplevel_ext_proc_irep_2 = {
  3,4,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_toplevel_ext_proc_iseq_2,
  NULL,NULL,NULL,
  gem_mrblib_mruby_toplevel_ext_proc_lv_2,
  &gem_mrblib_mruby_toplevel_ext_proc_debug_2,
  8,0,0,0,0
};
static const mrb_code gem_mrblib_mruby_toplevel_ext_proc_iseq_3[8] = {
0x33,0x00,0x10,0x00,0x11,0x03,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_toplevel_ext_proc_lv_3, 2, (MRB_SYM(methods), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_toplevel_ext_proc_debug_lines_3[] = "\x00\x08\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_toplevel_ext_proc_debug_file_3 = {
0, 173, 4, mrb_debug_line_packed_map, {gem_mrblib_mruby_toplevel_ext_proc_debug_lines_3}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_toplevel_ext_proc_debug_file_3_ = &gem_mrblib_mruby_toplevel_ext_proc_debug_file_3;
static mrb_irep_debug_info gem_mrblib_mruby_toplevel_ext_proc_debug_3 = {
8, 1, &gem_mrblib_mruby_toplevel_ext_proc_debug_file_3_};
static const mrb_irep gem_mrblib_mruby_toplevel_ext_proc_irep_3 = {
  3,4,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_toplevel_ext_proc_iseq_3,
  NULL,NULL,NULL,
  gem_mrblib_mruby_toplevel_ext_proc_lv_3,
  &gem_mrblib_mruby_toplevel_ext_proc_debug_3,
  8,0,0,0,0
};
static const mrb_code gem_mrblib_mruby_toplevel_ext_proc_iseq_4[8] = {
0x33,0x00,0x10,0x00,0x11,0x03,0x37,0x03,};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_toplevel_ext_proc_lv_4, 2, (MRB_SYM(methods), MRB_OPSYM(and), ), const);
static char gem_mrblib_mruby_toplevel_ext_proc_debug_lines_4[] = "\x00\x0a\x04\x01";
static mrb_irep_debug_info_file gem_mrblib_mruby_toplevel_ext_proc_debug_file_4 = {
0, 173, 4, mrb_debug_line_packed_map, {gem_mrblib_mruby_toplevel_ext_proc_debug_lines_4}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_toplevel_ext_proc_debug_file_4_ = &gem_mrblib_mruby_toplevel_ext_proc_debug_file_4;
static mrb_irep_debug_info gem_mrblib_mruby_toplevel_ext_proc_debug_4 = {
8, 1, &gem_mrblib_mruby_toplevel_ext_proc_debug_file_4_};
static const mrb_irep gem_mrblib_mruby_toplevel_ext_proc_irep_4 = {
  3,4,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_toplevel_ext_proc_iseq_4,
  NULL,NULL,NULL,
  gem_mrblib_mruby_toplevel_ext_proc_lv_4,
  &gem_mrblib_mruby_toplevel_ext_proc_debug_4,
  8,0,0,0,0
};
static const mrb_irep *gem_mrblib_mruby_toplevel_ext_proc_reps_0[4] = {
  &gem_mrblib_mruby_toplevel_ext_proc_irep_1,
  &gem_mrblib_mruby_toplevel_ext_proc_irep_2,
  &gem_mrblib_mruby_toplevel_ext_proc_irep_3,
  &gem_mrblib_mruby_toplevel_ext_proc_irep_4,
};
mrb_DEFINE_SYMS_VAR(gem_mrblib_mruby_toplevel_ext_proc_syms_0, 4, (MRB_SYM(include), MRB_SYM(private), MRB_SYM(protected), MRB_SYM(public), ), const);
static const mrb_code gem_mrblib_mruby_toplevel_ext_proc_iseq_0[43] = {
0x12,0x01,0x61,0x01,0x57,0x02,0x00,0x5e,0x01,0x00,0x12,0x01,0x61,0x01,0x57,0x02,0x01,0x5e,0x01,0x01,
0x12,0x01,0x61,0x01,0x57,0x02,0x02,0x5e,0x01,0x02,0x12,0x01,0x61,0x01,0x57,0x02,0x03,0x5e,0x01,0x03,
0x37,0x01,0x68,};
static char gem_mrblib_mruby_toplevel_ext_proc_debug_lines_0[] = "\x00\x02\x0a\x04\x0a\x02\x0a\x02";
static mrb_irep_debug_info_file gem_mrblib_mruby_toplevel_ext_proc_debug_file_0 = {
0, 173, 8, mrb_debug_line_packed_map, {gem_mrblib_mruby_toplevel_ext_proc_debug_lines_0}};
static mrb_irep_debug_info_file *gem_mrblib_mruby_toplevel_ext_proc_debug_file_0_ = &gem_mrblib_mruby_toplevel_ext_proc_debug_file_0;
static mrb_irep_debug_info gem_mrblib_mruby_toplevel_ext_proc_debug_0 = {
43, 1, &gem_mrblib_mruby_toplevel_ext_proc_debug_file_0_};
static const mrb_irep gem_mrblib_mruby_toplevel_ext_proc_irep_0 = {
  1,2,0,
  MRB_IREP_STATIC,gem_mrblib_mruby_toplevel_ext_proc_iseq_0,
  NULL,gem_mrblib_mruby_toplevel_ext_proc_syms_0,gem_mrblib_mruby_toplevel_ext_proc_reps_0,
  NULL,					/* lv */
  &gem_mrblib_mruby_toplevel_ext_proc_debug_0,
  43,0,4,4,0
};
static
const struct RProc gem_mrblib_mruby_toplevel_ext_proc[] = {{
NULL,NULL,MRB_TT_PROC,MRB_GC_RED,0,{&gem_mrblib_mruby_toplevel_ext_proc_irep_0},NULL,{NULL},
}};
static void
gem_mrblib_mruby_toplevel_ext_proc_init_syms(mrb_state *mrb)
{
  gem_mrblib_mruby_toplevel_ext_proc_debug_file_1.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-toplevel-ext/mrblib/toplevel.rb");
  gem_mrblib_mruby_toplevel_ext_proc_debug_file_2.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-toplevel-ext/mrblib/toplevel.rb");
  gem_mrblib_mruby_toplevel_ext_proc_debug_file_3.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-toplevel-ext/mrblib/toplevel.rb");
  gem_mrblib_mruby_toplevel_ext_proc_debug_file_4.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-toplevel-ext/mrblib/toplevel.rb");
  gem_mrblib_mruby_toplevel_ext_proc_debug_file_0.filename_sym = mrb_intern_lit(mrb,"/build/zynaddsubfx/src/zynaddsubfx-mruby-zest-build/mruby/mrbgems/mruby-toplevel-ext/mrblib/toplevel.rb");
}
void mrb_mruby_toplevel_ext_gem_init(mrb_state *mrb);
void mrb_mruby_toplevel_ext_gem_final(mrb_state *mrb);

void GENERATED_TMP_mrb_mruby_toplevel_ext_gem_init(mrb_state *mrb) {
  int ai = mrb_gc_arena_save(mrb);
  gem_mrblib_mruby_toplevel_ext_proc_init_syms(mrb);
  mrb_load_proc(mrb, gem_mrblib_mruby_toplevel_ext_proc);
  if (mrb->exc) {
    mrb_print_error(mrb);
    mrb_close(mrb);
    exit(EXIT_FAILURE);
  }
  struct REnv *e = mrb_vm_ci_env(mrb->c->cibase);
  mrb_vm_ci_env_set(mrb->c->cibase, NULL);
  mrb_env_unshare(mrb, e);
  mrb_gc_arena_restore(mrb, ai);
}

void GENERATED_TMP_mrb_mruby_toplevel_ext_gem_final(mrb_state *mrb) {
}
